/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.applications.morphing;

import java.awt.*;
import javax.swing.*;
import javax.swing.GroupLayout;
import com.frojasg1.general.desktop.view.about.GenericAboutJDialog;
import com.frojasg1.general.desktop.view.pdf.ImageJPanel;
import com.frojasg1.applications.morphing.configuration.ApplicationConfiguration;
import com.frojasg1.applications.morphing.constants.Colors;
import com.frojasg1.applications.common.components.internationalization.window.InternationalizationInitializationEndCallback;
import com.frojasg1.general.desktop.files.DesktopResourceFunctions;
import com.frojasg1.general.desktop.image.ImageFunctions;
import com.frojasg1.general.lib3d.components.api.about.animation.AnimationForAboutFactory;
import com.frojasg1.general.number.IntegerFunctions;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Point;
import java.util.function.Consumer;
import javax.swing.SwingUtilities;

/**
 *
 * @author Usuario
 */
public class AboutJDialog extends GenericAboutJDialog
{
	public static final String CONFIGURATION_BASE_FILE_NAME = "JDial_about";
	public static final String RESOURCE_FOR_ABOUT_ICON = null;
//	public static final String RESOURCE_FOR_ABOUT_ICON = "com/frojasg1/applications/jfe/resources/puzzle.redimensionado.png";
	public static final String ABOUT_RTF_FILE_NAME = "AboutText.rtf";

	/**
	 * Creates new form JDialAbout
	 */
	public AboutJDialog(java.awt.Frame parent, boolean modal,
						Consumer<InternationalizationInitializationEndCallback> initializationEndCallback,
						AnimationForAboutFactory animationFactory )
	{
		super(parent, modal, ApplicationConfiguration.instance(),
				ImageFunctions.instance().changeAlpha(
					DesktopResourceFunctions.instance().loadResourceImage(
						ApplicationConfiguration.instance().getResourceNameForApplicationIcon() ),
					48 ),
				CONFIGURATION_BASE_FILE_NAME, ABOUT_RTF_FILE_NAME,
				animationFactory,
				initializationEndCallback);

//		initComponents();

		initOwnComponents();

		setColors();
	}

	protected void setColors()
	{
		setImageBackground( Color.WHITE );
		setForegroundColorForTextLines( Colors.LIGHT_BLUE_GREEN );

		setForegroundColorOfHomeWebPage( Colors.ORANGE.brighter() );
	}

	protected void initOwnComponents()
	{
		SwingUtilities.invokeLater( () -> initContents() );

		setRtfDocument( getAppliConf().getLanguage() );
	}

	protected void initContents()
	{
//		String authorEmailAddress = getAppliConf().getAuthorEmailAddress();
//		setData( authorEmailAddress );

		Dimension imageDimensions = getSizeOfOriginalImageJPanel();
		int xx = (int) (imageDimensions.getWidth() / 2);
		int yyIncrement = (int) (imageDimensions.getHeight() / 7);

		addTextLine( new ImageJPanel.TextLine( "Morphing", new Point( xx, 2 * yyIncrement ) ) );
		addTextLine( new ImageJPanel.TextLine( "editor", new Point( xx, IntegerFunctions.zoomValueInt( yyIncrement, 3.5D ) ) ) );
		addTextLine(new ImageJPanel.TextLine( getVersion(), new Point( xx, yyIncrement * 5 ) ) );

//		setImageBackground( Color.WHITE );
		setOriginalFontForTextLines( new Font("Calibri", Font.BOLD, 16) );
//		setForegroundColorForTextLines( Colors.LIGHT_BLUE_GREEN );

//		setForegroundColorOfHomeWebPage( Colors.ORANGE.brighter() );
	}

	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	@SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    // Generated using JFormDesigner non-commercial license
    private void initComponents() {

        //======== this ========
        setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
        setName("AboutJDialog");
        Container contentPane = getContentPane();

        GroupLayout contentPaneLayout = new GroupLayout(contentPane);
        contentPane.setLayout(contentPaneLayout);
        contentPaneLayout.setHorizontalGroup(
            contentPaneLayout.createParallelGroup()
                .addGap(0, 400, Short.MAX_VALUE)
        );
        contentPaneLayout.setVerticalGroup(
            contentPaneLayout.createParallelGroup()
                .addGap(0, 300, Short.MAX_VALUE)
        );
        pack();
        setLocationRelativeTo(getOwner());
    }// </editor-fold>//GEN-END:initComponents

    // Variables declaration - do not modify//GEN-BEGIN:variables
    // Generated using JFormDesigner non-commercial license
    // End of variables declaration//GEN-END:variables
}
