/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.applications.morphing;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

import com.frojasg1.general.desktop.view.associatedcomponents.value.impl.integer.JSpinnerWrapperDefault;
import com.frojasg1.general.desktop.view.panels.AcceptCancelRevertControllerInterface;
import com.frojasg1.general.desktop.view.panels.AcceptCancelRevertPanel;
import com.frojasg1.applications.common.components.internationalization.window.InternationalizedJDialog;
import com.frojasg1.general.desktop.view.panels.InformerInterface;
import com.frojasg1.general.number.DoubleFunctions;
import com.frojasg1.general.number.IntegerFunctions;
import com.frojasg1.general.threads.ThreadFunctions;
import com.frojasg1.generic.GenericFunctions;
import com.frojasg1.general.view.internationalization.window.InternException;
import com.frojasg1.applications.common.components.internationalization.window.InternationalizedJFrame;
import com.frojasg1.applications.common.components.resizecomp.MapResizeRelocateComponentItem;
import com.frojasg1.applications.common.components.resizecomp.ResizeRelocateItem;
import com.frojasg1.applications.common.configuration.application.ConfigurationForDialogsImp;
import com.frojasg1.generic.zoom.ZoomFactorsAvailable;
import com.frojasg1.applications.morphing.configuration.ApplicationConfiguration;
import com.frojasg1.applications.common.components.internationalization.window.InternationalizationInitializationEndCallback;
import com.frojasg1.applications.common.components.internationalization.window.exceptions.ValidationException;
import com.frojasg1.general.desktop.view.panels.languages.ConfigurationServerForLanguage;
import com.frojasg1.general.desktop.view.panels.languages.LanguagesConfigurationJPanel;
import com.frojasg1.general.desktop.view.zoom.mapper.ComponentMapper;

import java.awt.Component;
import java.util.Locale;
import java.util.function.Consumer;
import javax.swing.ComboBoxModel;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JTextField;

/**
 *
 * @author Usuario
 */
public class JDial_applicationConfiguration extends InternationalizedJDialog
										implements AcceptCancelRevertControllerInterface,
													ConfigurationServerForLanguage
{
	protected static final String CONF_CONFIGURATION_ERROR = "CONFIGURATION_ERROR";
	protected static final String CONF_NOT_A_VALID_INT = "NOT_A_VALID_INT";
	protected static final String CONF_VALUE_MUST_BE_GREATER_OR_EQUAL_THAN_ONE = "VALUE_MUST_BE_GREATER_OR_EQUAL_THAN_ONE";

	protected AcceptCancelRevertPanel _acceptPanel = null;

	protected boolean a_initializing = true;
	protected boolean a_userHasPressedOK = false;
	protected String a_languageInConstructor = null;
	protected final static String a_configurationBaseFileName = "JDial_applicationConfiguration";

	protected boolean _modifiedByProgram = false;
	protected boolean _localeSelectionChangedByApplication = false;

	protected String[] _arrayOfLanguages = null;

	protected boolean _changedByProgram = false;

	protected boolean _ignoreCheckBoxChange = false;

	protected ConfigurationForDialogsImp _tmpConf = new ConfigurationForDialogsImp();

	protected InternationalizedJFrame _mainWindowParent = null;

	protected LanguagesConfigurationJPanel _languagesConfigurationJPanel = null;

	protected JSpinnerWrapperDefault threadsSpinnerWrapper;

	/**
	 * Creates new form JDial_applicationConfiguration
	 */
	public JDial_applicationConfiguration(java.awt.Frame parent, boolean modal,
											Consumer<InternationalizationInitializationEndCallback> initializationEndCallBack
		)
	{
		super(parent, modal, ApplicationConfiguration.instance(), null,
			initializationEndCallBack, true);
		_tmpConf.init( _appliConf );

		initComponents();
		initOwnComponents();

		JCheckBox cb;

		a_languageInConstructor = _appliConf.getLanguage();

		M_initializeComponentContents();

		setWindowConfiguration();

		boolean initialConfiguration = true;

		if( modal )
			setAlwaysOnTop( true );

		if( parent instanceof InternationalizedJFrame )
			_mainWindowParent = (InternationalizedJFrame) parent;
	}

	protected void initOwnComponents()
	{
		System.out.println( "jDial_applicationConfiguration" );

		threadsSpinnerWrapper = new JSpinnerWrapperDefault(jSpThreads, this);
        threadsSpinnerWrapper.setValue(getAppliConf().getNumThreads());
		threadsSpinnerWrapper.setBounds(2, IntegerFunctions.max(2, ThreadFunctions.instance().getNumMaxThreads() - 1));

		_languagesConfigurationJPanel = new LanguagesConfigurationJPanel( this, getAppliConf() );
		jP_language.add( _languagesConfigurationJPanel );
		_languagesConfigurationJPanel.setBounds( 0, 0,
												jP_language.getWidth(),
												jP_language.getHeight() );

		_acceptPanel = new AcceptCancelRevertPanel( this );
		jPanel5.add( _acceptPanel );
		_acceptPanel.setBounds( 0, 0, jPanel5.getWidth(), jPanel5.getHeight() );
	}

    private void jCBzoom(ActionEvent e) {
        // TODO add your code here

		if( ! _changedByProgram )
		{
			double newZoomFactor = getSelectedZoomFactor();
			_tmpConf.setZoomFactor( newZoomFactor );
			setNewZoomFactor( newZoomFactor );
		}
    }

	protected void setWindowConfiguration( )
	{
		MapResizeRelocateComponentItem mapRRCI = new MapResizeRelocateComponentItem();
		try
		{
//			mapRRCI.putResizeRelocateComponentItem( jMenuBar1, ResizeRelocateItem.RESIZE_TO_RIGHT );

			mapRRCI.putResizeRelocateComponentItem( jPanel1, ResizeRelocateItem.RESIZE_TO_RIGHT + ResizeRelocateItem.RESIZE_TO_BOTTOM );
			mapRRCI.putResizeRelocateComponentItem( jTP_tabbedPane, ResizeRelocateItem.RESIZE_TO_RIGHT + ResizeRelocateItem.RESIZE_TO_BOTTOM );
			mapRRCI.putResizeRelocateComponentItem( jPanel5, ResizeRelocateItem.RESIZE_TO_RIGHT + ResizeRelocateItem.MOVE_TO_BOTTOM );
			mapRRCI.putResizeRelocateComponentItem( _acceptPanel, ResizeRelocateItem.RESIZE_TO_RIGHT + ResizeRelocateItem.RESIZE_TO_BOTTOM );

			mapRRCI.putAll( _acceptPanel.getResizeRelocateInfo() );
		}
		catch( Throwable th )
		{
			th.printStackTrace();
		}

		createInternationalization(	getAppliConf().getConfigurationMainFolder(),
									getAppliConf().getApplicationNameFolder(),
									getAppliConf().getApplicationGroup(),
									getAppliConf().getInternationalPropertiesPathInJar(),
									a_configurationBaseFileName,
									this,
									getParent(),
									null,
									true,
									mapRRCI );

		registerInternationalString( CONF_CONFIGURATION_ERROR, "Configuration error" );
		registerInternationalString( CONF_NOT_A_VALID_INT, "Not a valid int" );
		registerInternationalString( CONF_VALUE_MUST_BE_GREATER_OR_EQUAL_THAN_ONE, "Value must be greater or equal than 1" );

		unregisterFromChangeZoomFactorAsObserver();
	}

	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	@SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    // Generated using JFormDesigner non-commercial license
    private void initComponents() {
        jPanel1 = new JPanel();
        jPanel5 = new JPanel();
        jTP_tabbedPane = new JTabbedPane();
        jP_language = new JPanel();
        jPView = new JPanel();
        jP_zoom = new JPanel();
        jLbl_EncryptingConfigurationPriority = new JLabel();
        jCBzoom = new JComboBox<>();
        jPThreads = new JPanel();
        jPThreads2 = new JPanel();
        jLbl_Threads = new JLabel();
        jSpThreads = new JSpinner();

        //======== this ========
        setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
        setTitle("Configuration window");
        setMinimumSize(new Dimension(590, 455));
        setName("ConfigurationDialog");
        setResizable(false);
        addWindowListener(new WindowAdapter() {
            @Override
            public void windowClosing(WindowEvent e) {
                formWindowClosing(e);
            }
        });
        Container contentPane = getContentPane();
        contentPane.setLayout(null);

        //======== jPanel1 ========
        {
            jPanel1.setName("Panel1");
            jPanel1.setLayout(null);

            //======== jPanel5 ========
            {
                jPanel5.setBorder(new LineBorder(new Color(0x3366ff)));
                jPanel5.setMinimumSize(new Dimension(130, 50));
                jPanel5.setName("");
                jPanel5.setLayout(null);

                {
                    // compute preferred size
                    Dimension preferredSize = new Dimension();
                    for(int i = 0; i < jPanel5.getComponentCount(); i++) {
                        Rectangle bounds = jPanel5.getComponent(i).getBounds();
                        preferredSize.width = Math.max(bounds.x + bounds.width, preferredSize.width);
                        preferredSize.height = Math.max(bounds.y + bounds.height, preferredSize.height);
                    }
                    Insets insets = jPanel5.getInsets();
                    preferredSize.width += insets.right;
                    preferredSize.height += insets.bottom;
                    jPanel5.setMinimumSize(preferredSize);
                    jPanel5.setPreferredSize(preferredSize);
                }
            }
            jPanel1.add(jPanel5);
            jPanel5.setBounds(220, 360, 130, 50);

            //======== jTP_tabbedPane ========
            {
                jTP_tabbedPane.setName("jTP_tabbedPane");

                //======== jP_language ========
                {
                    jP_language.setName("jP_language");
                    jP_language.setLayout(null);

                    {
                        // compute preferred size
                        Dimension preferredSize = new Dimension();
                        for(int i = 0; i < jP_language.getComponentCount(); i++) {
                            Rectangle bounds = jP_language.getComponent(i).getBounds();
                            preferredSize.width = Math.max(bounds.x + bounds.width, preferredSize.width);
                            preferredSize.height = Math.max(bounds.y + bounds.height, preferredSize.height);
                        }
                        Insets insets = jP_language.getInsets();
                        preferredSize.width += insets.right;
                        preferredSize.height += insets.bottom;
                        jP_language.setMinimumSize(preferredSize);
                        jP_language.setPreferredSize(preferredSize);
                    }
                }
                jTP_tabbedPane.addTab("Language", jP_language);

                //======== jPView ========
                {
                    jPView.setName("jPView");
                    jPView.setPreferredSize(new Dimension(130, 50));
                    jPView.setLayout(null);

                    //======== jP_zoom ========
                    {
                        jP_zoom.setBorder(new TitledBorder(LineBorder.createBlackLineBorder(), "Zoom"));
                        jP_zoom.setName("jP_zoom");
                        jP_zoom.setLayout(null);

                        //---- jLbl_EncryptingConfigurationPriority ----
                        jLbl_EncryptingConfigurationPriority.setFont(new Font("Tahoma", Font.BOLD, 12));
                        jLbl_EncryptingConfigurationPriority.setText("Zoom");
                        jLbl_EncryptingConfigurationPriority.setName("JDial_applicationFontSize");
                        jP_zoom.add(jLbl_EncryptingConfigurationPriority);
                        jLbl_EncryptingConfigurationPriority.setBounds(180, 60, 370, 20);

                        //---- jCBzoom ----
                        jCBzoom.setMinimumSize(new Dimension(84, 20));
                        jCBzoom.setPreferredSize(new Dimension(79, 20));
                        jCBzoom.setModel(new DefaultComboBoxModel<>(new String[] {
                            "75%",
                            "100%",
                            "133%",
                            "175%",
                            "250%"
                        }));
                        jCBzoom.setSelectedIndex(1);
                        jCBzoom.addActionListener(e -> jCBzoom(e));
                        jP_zoom.add(jCBzoom);
                        jCBzoom.setBounds(new Rectangle(new Point(90, 60), jCBzoom.getPreferredSize()));

                        {
                            // compute preferred size
                            Dimension preferredSize = new Dimension();
                            for(int i = 0; i < jP_zoom.getComponentCount(); i++) {
                                Rectangle bounds = jP_zoom.getComponent(i).getBounds();
                                preferredSize.width = Math.max(bounds.x + bounds.width, preferredSize.width);
                                preferredSize.height = Math.max(bounds.y + bounds.height, preferredSize.height);
                            }
                            Insets insets = jP_zoom.getInsets();
                            preferredSize.width += insets.right;
                            preferredSize.height += insets.bottom;
                            jP_zoom.setMinimumSize(preferredSize);
                            jP_zoom.setPreferredSize(preferredSize);
                        }
                    }
                    jPView.add(jP_zoom);
                    jP_zoom.setBounds(10, 10, 570, 140);

                    {
                        // compute preferred size
                        Dimension preferredSize = new Dimension();
                        for(int i = 0; i < jPView.getComponentCount(); i++) {
                            Rectangle bounds = jPView.getComponent(i).getBounds();
                            preferredSize.width = Math.max(bounds.x + bounds.width, preferredSize.width);
                            preferredSize.height = Math.max(bounds.y + bounds.height, preferredSize.height);
                        }
                        Insets insets = jPView.getInsets();
                        preferredSize.width += insets.right;
                        preferredSize.height += insets.bottom;
                        jPView.setMinimumSize(preferredSize);
                        jPView.setPreferredSize(preferredSize);
                    }
                }
                jTP_tabbedPane.addTab("View", jPView);

                //======== jPThreads ========
                {
                    jPThreads.setName("jPThreads");
                    jPThreads.setLayout(null);

                    //======== jPThreads2 ========
                    {
                        jPThreads2.setBorder(new TitledBorder(LineBorder.createBlackLineBorder(), "Threads"));
                        jPThreads2.setName("jPThreads2");
                        jPThreads2.setLayout(null);

                        //---- jLbl_Threads ----
                        jLbl_Threads.setFont(new Font("Tahoma", Font.BOLD, 12));
                        jLbl_Threads.setText("Threads");
                        jLbl_Threads.setName("jLbl_Threads");
                        jPThreads2.add(jLbl_Threads);
                        jLbl_Threads.setBounds(180, 60, 370, 20);

                        //---- jSpThreads ----
                        jSpThreads.setMinimumSize(new Dimension(50, 20));
                        jSpThreads.setPreferredSize(new Dimension(50, 20));
                        jPThreads2.add(jSpThreads);
                        jSpThreads.setBounds(120, 60, 53, jSpThreads.getPreferredSize().height);

                        {
                            // compute preferred size
                            Dimension preferredSize = new Dimension();
                            for(int i = 0; i < jPThreads2.getComponentCount(); i++) {
                                Rectangle bounds = jPThreads2.getComponent(i).getBounds();
                                preferredSize.width = Math.max(bounds.x + bounds.width, preferredSize.width);
                                preferredSize.height = Math.max(bounds.y + bounds.height, preferredSize.height);
                            }
                            Insets insets = jPThreads2.getInsets();
                            preferredSize.width += insets.right;
                            preferredSize.height += insets.bottom;
                            jPThreads2.setMinimumSize(preferredSize);
                            jPThreads2.setPreferredSize(preferredSize);
                        }
                    }
                    jPThreads.add(jPThreads2);
                    jPThreads2.setBounds(0, 0, 570, 140);

                    {
                        // compute preferred size
                        Dimension preferredSize = new Dimension();
                        for(int i = 0; i < jPThreads.getComponentCount(); i++) {
                            Rectangle bounds = jPThreads.getComponent(i).getBounds();
                            preferredSize.width = Math.max(bounds.x + bounds.width, preferredSize.width);
                            preferredSize.height = Math.max(bounds.y + bounds.height, preferredSize.height);
                        }
                        Insets insets = jPThreads.getInsets();
                        preferredSize.width += insets.right;
                        preferredSize.height += insets.bottom;
                        jPThreads.setMinimumSize(preferredSize);
                        jPThreads.setPreferredSize(preferredSize);
                    }
                }
                jTP_tabbedPane.addTab("Threads", jPThreads);
            }
            jPanel1.add(jTP_tabbedPane);
            jTP_tabbedPane.setBounds(0, 0, 590, 350);

            {
                // compute preferred size
                Dimension preferredSize = new Dimension();
                for(int i = 0; i < jPanel1.getComponentCount(); i++) {
                    Rectangle bounds = jPanel1.getComponent(i).getBounds();
                    preferredSize.width = Math.max(bounds.x + bounds.width, preferredSize.width);
                    preferredSize.height = Math.max(bounds.y + bounds.height, preferredSize.height);
                }
                Insets insets = jPanel1.getInsets();
                preferredSize.width += insets.right;
                preferredSize.height += insets.bottom;
                jPanel1.setMinimumSize(preferredSize);
                jPanel1.setPreferredSize(preferredSize);
            }
        }
        contentPane.add(jPanel1);
        jPanel1.setBounds(0, 0, 590, 420);

        {
            // compute preferred size
            Dimension preferredSize = new Dimension();
            for(int i = 0; i < contentPane.getComponentCount(); i++) {
                Rectangle bounds = contentPane.getComponent(i).getBounds();
                preferredSize.width = Math.max(bounds.x + bounds.width, preferredSize.width);
                preferredSize.height = Math.max(bounds.y + bounds.height, preferredSize.height);
            }
            Insets insets = contentPane.getInsets();
            preferredSize.width += insets.right;
            preferredSize.height += insets.bottom;
            contentPane.setMinimumSize(preferredSize);
            contentPane.setPreferredSize(preferredSize);
        }
        pack();
        setLocationRelativeTo(getOwner());
    }// </editor-fold>//GEN-END:initComponents

    private void formWindowClosing(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_formWindowClosing
        // TODO add your handling code here:

		cancel( null );

    }//GEN-LAST:event_formWindowClosing

    private void jCBzoomActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jCBzoomActionPerformed
        // TODO add your handling code here:

        if( ! _changedByProgram )
        {
            double newZoomFactor = getSelectedZoomFactor();
            _tmpConf.setZoomFactor( newZoomFactor );
            setNewZoomFactor( newZoomFactor );
        }
    }//GEN-LAST:event_jCBzoomActionPerformed


    // Variables declaration - do not modify//GEN-BEGIN:variables
    // Generated using JFormDesigner non-commercial license
    private JPanel jPanel1;
    private JPanel jPanel5;
    private JTabbedPane jTP_tabbedPane;
    private JPanel jP_language;
    private JPanel jPView;
    private JPanel jP_zoom;
    private JLabel jLbl_EncryptingConfigurationPriority;
    private JComboBox<String> jCBzoom;
    private JPanel jPThreads;
    private JPanel jPThreads2;
    private JLabel jLbl_Threads;
    private JSpinner jSpThreads;
    // End of variables declaration//GEN-END:variables

	protected String getSelectedLanguage()
	{
		return( _languagesConfigurationJPanel.getSelectedLanguage() );
	}

	protected void applyChanges()
	{
		String language = getSelectedLanguage();

		ApplicationConfiguration appConf = getAppliConf();

		appConf.setNumThreads(threadsSpinnerWrapper.getValue());

		try
		{
			if( language != null )
				appConf.changeLanguage(language);
		}
		catch( Throwable th )
		{
			th.printStackTrace();
		}

		try
		{
			_tmpConf.updateGlobalConfiguration(getAppliConf());
		}
		catch( Throwable th )
		{
			th.printStackTrace();
		}

		try
		{
			GenericFunctions.instance().getObtainAvailableLanguages().updateLocaleLanguagesToDisk();

			if( _mainWindowParent != null )
				_mainWindowParent.updateRadioButtonMenus();
		}
		catch( Exception ex )
		{
			ex.printStackTrace();
		}
	}

	protected void M_saveInternationalization()
	{
		try
		{
			if( a_intern != null ) a_intern.saveConfiguration();
		}
		catch( Throwable th )
		{
			th.printStackTrace();
		}
	}

	protected void validateIntFieldGreaterThanZero( Component comp, Integer value ) throws ValidationException
	{
		if( value == null )
			throw( new ValidationException( getInternationalString( CONF_NOT_A_VALID_INT ), comp ) );
		else if( value < 1 )
			throw( new ValidationException( getInternationalString( CONF_VALUE_MUST_BE_GREATER_OR_EQUAL_THAN_ONE ), comp ) );
	}

	public void validateChanges() throws ValidationException
	{
	}

	@Override
	public void accept(InformerInterface panel)
	{
		boolean proceed = false;
/*
		try
		{
			validateChanges();
			proceed = true;
		}
		catch( ValidateException ve )
		{
			Component comp = ve.getComponentNotValidated();
			ViewComponent vc = GenericFunctions.instance().getViewFacilities().createViewComponent(comp);

			focusAndHighlightComponent( vc );

			GenericFunctions.instance().getDialogsWrapper().showMessageDialog( this, ve.getMessage(),
											getInternationalString( CONF_CONFIGURATION_ERROR ),
											DialogsWrapper.ERROR_MESSAGE );

		}
*/
		validateForm();
		proceed = wasSuccessful();

		if( proceed )
		{
			applyChanges();

			try
			{
				ApplicationConfiguration.instance().M_saveConfiguration();
			}
			catch( Throwable th )
			{
				th.printStackTrace();
			}
/*
			if( M_hasToCreateAdditionalLanguageFolder() )
			{
				M_createFolderAndCopyLanguageConfigurationFilesFromEnglish();
			}
*/
			M_saveInternationalization();
			a_userHasPressedOK = true;
			setVisible(false);
		}
	}

	@Override
	public void cancel(InformerInterface panel)
	{
		// we load again the Strings configuration with the language which with was called the dialogue
		try
		{
//			StringsConfiguration.instance().changeLanguage( a_languageInConstructor );
		}
		catch( Throwable th )
		{
			th.printStackTrace();
		}

		try
		{
			GenericFunctions.instance().getObtainAvailableLanguages().reloadLanguages();
		}
		catch( Exception ex )
		{
			ex.printStackTrace();
		}

		M_saveInternationalization();
        setVisible(false);
	}

	@Override
	public void revert(InformerInterface panel)
	{
		try
		{
			a_intern.changeLanguage( getAppliConf().getLanguage() );
		}
		catch( Throwable th )
		{
			th.printStackTrace();
		}

		M_initializeComponentContents();

		boolean initialConfiguration = true;
	}

	public boolean getUserHasPressedOK()
	{
		return( a_userHasPressedOK );
	}

	public void releaseResources()
	{
		a_intern=null;	// for the garbage collector to free the memory of the internationallization object and after the memory of this form
	}

	@Override
	protected void translateMappedComponents(ComponentMapper compMapper)
	{
		jCBzoom = compMapper.mapComponent( jCBzoom );
		jLbl_EncryptingConfigurationPriority = compMapper.mapComponent( jLbl_EncryptingConfigurationPriority );
		jPView = compMapper.mapComponent( jPView );
		jP_language = compMapper.mapComponent( jP_language );
		jP_zoom = compMapper.mapComponent( jP_zoom );
		jPanel1 = compMapper.mapComponent( jPanel1 );
		jPanel5 = compMapper.mapComponent( jPanel5 );
		jTP_tabbedPane = compMapper.mapComponent( jTP_tabbedPane );

		jPThreads = compMapper.mapComponent(jPThreads);
		jPThreads2 = compMapper.mapComponent(jPThreads2);
		jSpThreads = compMapper.mapComponent(jSpThreads);
		jLbl_Threads = compMapper.mapComponent(jLbl_Threads);

	}

	@Override
	public void localChangeLanguage(String language) {
		try
		{
//			StringsConfiguration.instance().changeLanguage( language );
		}
		catch( Exception ce )
		{
			ce.printStackTrace();
		}

		try
		{
			if( a_intern != null )
			{
				a_intern.changeLanguage( language );
			}
		}
		catch( InternException ie )
		{
			ie.printStackTrace();
		}
	}

	@Override
	public void addLanguageActionPerformed(String language)
	{
        if( getParent() instanceof MainWindow )
        {
            MainWindow mw = (MainWindow) getParent();
            mw.addLanguageToMenu(language);
        }
	}

	protected static class ComboBoxOfChessLanguageData
	{
		protected JComboBox _own = null;
		protected JComboBox _linked = null;
		protected JTextField _linkedTextField = null;

		public ComboBoxOfChessLanguageData( JComboBox own, JComboBox linked, JTextField linkedTextField )
		{
			_own = own;
			_linked = linked;
			_linkedTextField = linkedTextField;
		}
		
		public JComboBox getOwn()
		{
			return( _own );
		}
		
		public JComboBox getLinked()
		{
			return( _linked );
		}
		
		public JTextField getLinkedTextField()
		{
			return( _linkedTextField );
		}
	}
	
	@Override
	public ApplicationConfiguration getAppliConf()
	{
		return( (ApplicationConfiguration) super.getAppliConf() );
	}

	protected void M_initializeComponentContents()
	{
		a_initializing = true;

		_languagesConfigurationJPanel.M_initializeComponentContents();

		ApplicationConfiguration appConf = getAppliConf();

		initComboBox();

		threadsSpinnerWrapper.setValue(appConf.getNumThreads());

		a_initializing=false;
	}

	protected String M_getAdditionalLanguageFolder( String language )
	{
		return( ( (language==null) || (language.length()==0 ) ) ? "" : getAppliConf().getDefaultLanguageConfigurationFolder( language ) );
	}

	protected double getSelectedZoomFactor()
	{
		String selectedItem = (String) jCBzoom.getSelectedItem();
		double result = getZoomFactorFromComboBoxItem( selectedItem );

		return( result );
	}

	protected void setNewZoomFactor( double zoomFactor )
	{
		changeZoomFactor_centerMousePointer( zoomFactor );
	}

	protected double getZoomFactorFromComboBoxItem( String item )
	{
		double result = -1.0D;
		
		if( ( item != null ) && ( item.length() > 0 ) &&
			( item.charAt( item.length() - 1 ) == '%' ) )
		{
			try
			{
				String numberStr = item.substring( 0, item.length() - 1 );
				result = Double.valueOf( numberStr ) / 100;
			}
			catch( Throwable th )
			{
				th.printStackTrace();
			}
		}

		return( result );
	}
	
	protected int selectMostSuitableItemIndex( double zoomFactor )
	{
		ComboBoxModel<String> dcbm = jCBzoom.getModel();

		double minDiff = 100;
		int result = -1;
		for( int ii=0; ii<dcbm.getSize(); ii++ )
		{
			double currentZoomFactor = getZoomFactorFromComboBoxItem( dcbm.getElementAt(ii) );
			double currentDiff = DoubleFunctions.instance().abs( currentZoomFactor - zoomFactor );

			if( currentDiff < minDiff )
			{
				minDiff = currentDiff;
				result = ii;
			}
		}

		return( result );
	}

	protected void fillInZoomComboBox()
	{
		DefaultComboBoxModel dcm = new DefaultComboBoxModel( ZoomFactorsAvailable.instance().getZoomFactorsAvailable().toArray( new String[0] ) );
		jCBzoom.setModel( dcm );
	}

	protected void initComboBox()
	{
//		_changedByProgram = true;

		fillInZoomComboBox();

		int index = selectMostSuitableItemIndex( getAppliConf().getZoomFactor() );

		jCBzoom.setSelectedIndex(index);

//		_changedByProgram = false;
	}

	public boolean M_getUserHasPressedOK()
	{
		return( a_userHasPressedOK );
	}

	public void M_releaseResources()
	{
		a_intern=null;	// for the garbage collector to free the memory of the internationallization object and after the memory of this form
	}

	protected String getLocaleCode( String language )
	{
		String result = null;

		Locale locale = GenericFunctions.instance().getObtainAvailableLanguages().getLocaleOfLanguage(language);
		if( locale != null )
			result = locale.toString();

		return( result );
	}
}
