/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.applications.morphing;

import java.awt.*;
import javax.swing.*;
import com.frojasg1.applications.common.configuration.application.BaseApplicationConfigurationInterface;
import com.frojasg1.general.desktop.view.pdf.ImageJPanel;
import com.frojasg1.general.desktop.view.splash.GenericBasicSplash;
import com.frojasg1.applications.morphing.configuration.ApplicationConfiguration;
import com.frojasg1.applications.morphing.constants.Colors;
import com.frojasg1.general.desktop.image.ImageFunctions;
import com.frojasg1.general.number.IntegerFunctions;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Point;
import java.awt.image.BufferedImage;

/**
 *
 * @author Usuario
 */
public class Splash extends GenericBasicSplash
{
	protected static Splash _instance = null;
	public static final String sa_configurationBaseFileName = "Splash";

	public static final String CONF_OPENING_CONFIGURATION = "OPENING_CONFIGURATION";
	public static final String CONF_SHOWING_LICENSES = "SHOWING_LICENSES";
	public static final String CONF_IMPORTING_CONFIGURATION = "IMPORTING_CONFIGURATION";
	public static final String CONF_CREATING_MAIN_WINDOW = "CREATING_MAIN_WINDOW";
	public static final String CONF_PROBLEM_LAUNCHING_THE_APPLICATION = "PROBLEM_LAUNCHING_THE_APPLICATION";

	public static final String CONF_HAS_BEEN_IMPORTED = "HAS_BEEN_IMPORTED";
	public static final String CONF_INDEPENDENT_CONF = "INDEPENDENT_CONF";
	public static final String CONF_NOTICE = "NOTICE";

	public static Splash instance()
	{
		if( _instance == null )
			_instance = new Splash( ApplicationConfiguration.instance() );

		return( _instance );
	}

	/**
	 * Creates new form Splash
	 */
	protected Splash( BaseApplicationConfigurationInterface applicationConfiguration )
	{
		super(applicationConfiguration);
//		initComponents();

		BufferedImage image = getImage( applicationConfiguration.getResourceNameForApplicationIcon() );
		image = ImageFunctions.instance().changeAlpha( image, 48 );
		setImage( image );

		setCenteredLocation();
//		activateProgress(false );

		setListenersRecursive( this );

		initOwnComponents();

		setWindowConfiguration( );
	}

	protected void setListenersRecursive( Component comp )
	{
		comp.addMouseMotionListener(this);
		comp.addMouseListener(this);

		if( comp instanceof Container )
		{
			Container cont = (Container) comp;

			for( int ii=0; ii<cont.getComponentCount(); ii++ )
				setListenersRecursive( cont.getComponent(ii) );
		}
	}

	protected void initOwnComponents()
	{
		Dimension imageDimensions = getSizeOfOriginalImageJPanel();
		int xx = (int) (imageDimensions.getWidth() / 2);
		int yyIncrement = (int) (imageDimensions.getHeight() / 7);

		addTextLine( new ImageJPanel.TextLine( "Morphing", new Point( xx, 2 * yyIncrement ) ) );
		addTextLine( new ImageJPanel.TextLine( "editor", new Point( xx, IntegerFunctions.zoomValueInt( yyIncrement, 3.5D ) ) ) );
		addTextLine(new ImageJPanel.TextLine( ApplicationConfiguration.VERSION, new Point( xx, yyIncrement * 5 ) ) );

		setImageBackground( Color.WHITE );
		setOriginalFontForTextLines( new Font("Calibri", Font.BOLD, 16) );
		setForegroundColorForTextLines( Colors.LIGHT_BLUE_GREEN );	// light blue-green

//		zoomImage();
	}

	@Override
	protected String getBaseConfigurationFileName()
	{
		return( sa_configurationBaseFileName );
	}

	@Override
	protected void setWindowConfiguration( )
	{
		super.setWindowConfiguration();
	}

	@Override
	public void internationalizationInitializationEndCallback()
	{
		super.internationalizationInitializationEndCallback();

		this.registerInternationalString(CONF_OPENING_CONFIGURATION, "Opening configuration ..." );
		this.registerInternationalString(CONF_SHOWING_LICENSES,	"Showing licenses ..." );
		this.registerInternationalString(CONF_PROBLEM_LAUNCHING_THE_APPLICATION, "Problem launching the application" );
		this.registerInternationalString(CONF_CREATING_MAIN_WINDOW, "Creating main window ..." );
		this.registerInternationalString(CONF_IMPORTING_CONFIGURATION, "Importing configuration ..." );

		this.registerInternationalString(CONF_HAS_BEEN_IMPORTED, "Existing configuration has been imported ..." );
		this.registerInternationalString(CONF_INDEPENDENT_CONF, "From now on, these configurations will be independent." );
		this.registerInternationalString(CONF_NOTICE, "NOTICE" );
	}

	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	@SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    // Generated using JFormDesigner non-commercial license
    private void initComponents() {

        //======== this ========
        setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
        setAutoRequestFocus(false);
        Container contentPane = getContentPane();
        contentPane.setLayout(null);

        {
            // compute preferred size
            Dimension preferredSize = new Dimension();
            for(int i = 0; i < contentPane.getComponentCount(); i++) {
                Rectangle bounds = contentPane.getComponent(i).getBounds();
                preferredSize.width = Math.max(bounds.x + bounds.width, preferredSize.width);
                preferredSize.height = Math.max(bounds.y + bounds.height, preferredSize.height);
            }
            Insets insets = contentPane.getInsets();
            preferredSize.width += insets.right;
            preferredSize.height += insets.bottom;
            contentPane.setMinimumSize(preferredSize);
            contentPane.setPreferredSize(preferredSize);
        }
        pack();
        setLocationRelativeTo(getOwner());
    }// </editor-fold>//GEN-END:initComponents

	/**
	 * @param args the command line arguments
	 */
	public static void main(String args[]) {
		/* Set the Nimbus look and feel */
        //<editor-fold defaultstate="collapsed" desc=" Look and feel setting code (optional) ">
        /* If Nimbus (introduced in Java SE 6) is not available, stay with the default look and feel.
		 * For details see http://download.oracle.com/javase/tutorial/uiswing/lookandfeel/plaf.html 
		 */
		try {
			for (javax.swing.UIManager.LookAndFeelInfo info : javax.swing.UIManager.getInstalledLookAndFeels()) {
				if ("Nimbus".equals(info.getName())) {
					javax.swing.UIManager.setLookAndFeel(info.getClassName());
					break;
				}
			}
		} catch (ClassNotFoundException ex) {
			java.util.logging.Logger.getLogger(Splash.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		} catch (InstantiationException ex) {
			java.util.logging.Logger.getLogger(Splash.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		} catch (IllegalAccessException ex) {
			java.util.logging.Logger.getLogger(Splash.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		} catch (javax.swing.UnsupportedLookAndFeelException ex) {
			java.util.logging.Logger.getLogger(Splash.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		}
        //</editor-fold>

		/* Create and display the form */
/*		java.awt.EventQueue.invokeLater(new Runnable() {
			public void run() {
				new Splash().setVisible(true);
			}
		});
*/
	}

    // Variables declaration - do not modify//GEN-BEGIN:variables
    // Generated using JFormDesigner non-commercial license
    // End of variables declaration//GEN-END:variables


	public static void free()
	{
		if( _instance != null )
		{
			_instance.getAppliConf().unregisterZoomFactorObserver(_instance);
			_instance.setVisible( false );
			_instance.dispose();
			_instance = null;
		}
	}
}
