/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.applications.morphing.configuration;

import com.frojasg1.applications.common.configuration.application.BaseApplicationConfiguration;
import com.frojasg1.applications.libmorphing.configuration.MorphingApplicationConfiguration;
import com.frojasg1.applications.libmorphing.constants.MorphingConstants;
import com.frojasg1.general.number.DoubleReference;
import com.frojasg1.general.number.IntegerFunctions;
import com.frojasg1.general.threads.ThreadFunctions;
import com.frojasg1.generic.GenericFunctions;
import com.frojasg1.generic.languages.ObtainAvailableLanguages_base;
import java.io.File;
import java.util.Locale;
import java.util.Properties;
import javax.swing.JComponent;

/**
 *
 * @author Usuario
 */
public class ApplicationConfiguration extends BaseApplicationConfiguration implements MorphingApplicationConfiguration {
	private static ApplicationConfiguration a_instance = null;

	public static final String CONF_LAST_DIRECTORY = "LAST_DIRECTORY";
	public static final String CONF_LANGUAGE = "LANGUAGE";
	public static final String CONF_ADDITIONAL_LANGUAGE = "ADDITIONAL_LANGUAGE";

	public static final String CONF_LICENSES_HAVE_BEEN_ACCEPTED = "LICENSES_HAVE_BEEN_ACCEPTED";
//	public static final String CONF_HOME_WEB_PAGE_URL = "HOME_WEB_PAGE_URL";
//	public static final String CONF_AUTHOR_EMAIL_ADDRESS = "AUTHOR_EMAIL_ADDRESS";

//	public static final String CONF_LATEST_PROJECT_XML_FILENAME = "LATEST_PROJECT_XML_FILENAME";
	protected static final String CONF_TABLE_COLUMNS_STRING = "TABLE_COLUMNS_STRING";
	protected static final String CONF_NUM_THREADS = MorphingConstants.CONF_NUM_THREADS;

//	protected static final String APPLICATION_GROUP = "general";
	protected static final String GLOBAL_CONF_FILE_NAME = "GlobalConfiguration.properties";
/*
	protected static final String sa_PATH_PROPERTIES_IN_JAR =	"com" + ConfigurationParent.sa_dirSeparator + 
																"hotmail" + ConfigurationParent.sa_dirSeparator +
																"frojasg1" + ConfigurationParent.sa_dirSeparator + 
																"aplicaciones" + ConfigurationParent.sa_dirSeparator +
																"fileencoderapplication" + ConfigurationParent.sa_dirSeparator +
																"view" + ConfigurationParent.sa_dirSeparator +
																"internationalization" + ConfigurationParent.sa_dirSeparator +
																"properties";
*/
	protected static final String RESOURCE_FOR_APPLICATION_ICON="com/frojasg1/morphing/resources/icons/App.icon.png";
	public static final String sa_PROPERTIES_PATH_IN_JAR="com/frojasg1/app/morphing/inter";
	public static final String sa_MAIN_FOLDER = ".frojasg1.apps";
	public static String sa_APPLICATION_NAME = null;
	public static final String sa_CONFIGURATION_GROUP = "Configuration";

	public static final String ES_LANGUAGE = "ES";	 // Spanish language
	public static final String EN_LANGUAGE = "EN";	 // English language
	protected static final String URL_FOR_FACE_DETECTION_QUERY = "https://frojasg1.com:8443/downloads_web";

	public static String VERSION = "v1.1";

	private ApplicationConfiguration()
	{
//		super( sa_MAIN_FOLDER, sa_APPLICATION_NAME, APPLICATION_GROUP, null, GLOBAL_CONF_FILE_NAME );
		this( sa_MAIN_FOLDER, sa_APPLICATION_NAME, VERSION, sa_CONFIGURATION_GROUP, GLOBAL_CONF_FILE_NAME );
	}

	public ApplicationConfiguration( String mainFolder, String applicationName, String version, String group, String globalConfFileName )
	{
		super( mainFolder, applicationName + "/" + version, group, null, globalConfFileName );
	}

	public static ApplicationConfiguration instance()
	{
		return( a_instance );
	}

	public static ApplicationConfiguration create( String applicationName )
	{
		sa_APPLICATION_NAME = applicationName;
		a_instance = new ApplicationConfiguration();

		return( a_instance );
	}

	@Override
	public Properties M_getDefaultProperties( String language )
	{
		Properties result = super.M_getDefaultProperties( language );

		result.setProperty(CONF_LANGUAGE, EN_LANGUAGE );
		result.setProperty(CONF_ADDITIONAL_LANGUAGE, "CAT" );

		result.setProperty(CONF_NUM_THREADS, "16");
		result.setProperty(CONF_TABLE_COLUMNS_STRING, "com.frojasg1.general.desktop.view.componentwrapper.jtable.column.JTableArrangeableColumnsContext{com.frojasg1.general.desktop.view.componentwrapper.jtable.column.TableColumnData{Label:java.lang.String{\"INDEX\"},Width:int{-1}};com.frojasg1.general.desktop.view.componentwrapper.jtable.column.TableColumnData{Label:java.lang.String{\"NAME\"},Width:int{115}};com.frojasg1.general.desktop.view.componentwrapper.jtable.column.TableColumnData{Label:java.lang.String{\"IS_MASTER\"},Width:int{-1}};com.frojasg1.general.desktop.view.componentwrapper.jtable.column.TableColumnData{Label:java.lang.String{\"SIZE\"},Width:int{87}};com.frojasg1.general.desktop.view.componentwrapper.jtable.column.TableColumnData{Label:java.lang.String{\"FILE_NAME\"},Width:int{508}}}" );

		result.setProperty(CONF_LICENSES_HAVE_BEEN_ACCEPTED, "0" );
//		result.setProperty(CONF_HOME_WEB_PAGE_URL, "https://frojasg1.com" );
//		result.setProperty(CONF_AUTHOR_EMAIL_ADDRESS, "frojasg1@hotmail.com" );

		return( result );
	}

/*
	public float getFontSizeFactor()
	{
		String factorStr = M_getStrParamConfiguration(ApplicationConfiguration.CONF_APPLICATION_FONT_SIZE);
		float factor = 1.0F;
		try
		{
			factor = Float.valueOf( factorStr );
		}
		catch( Throwable th )
		{
			th.printStackTrace();
		}
		return( factor );
	}
*/
	@Override
	public String getConfigurationMainFolder()
	{
		return( sa_MAIN_FOLDER );
	}

	@Override
	public String getApplicationNameFolder()
	{
		return( getApplicationName() + File.separator + getApplicationVersion() );
	}

	@Override
	public String getApplicationName()
	{
		return( sa_APPLICATION_NAME );
	}

	@Override
	public String getApplicationVersion()
	{
		return( VERSION );
	}

	@Override
	public String getApplicationGroup()
	{
		return( sa_CONFIGURATION_GROUP );
	}

	@Override
	public String getInternationalPropertiesPathInJar()
	{
		return( sa_PROPERTIES_PATH_IN_JAR );
	}
	
	@Override
	public String getResourceNameForApplicationIcon()
	{
		return( RESOURCE_FOR_APPLICATION_ICON );
	}

	@Override
	public boolean getLicensesHaveBeenAccepted()
	{
		return( this.M_getBoolParamConfiguration(CONF_LICENSES_HAVE_BEEN_ACCEPTED) );
	}

	@Override
	public void setLicensesHaveBeenAccepted( boolean value )
	{
		this.M_setBoolParamConfiguration( CONF_LICENSES_HAVE_BEEN_ACCEPTED, value );
	}
	
	@Override
	public String getAdditionalLanguage()
	{
		return( this.M_getStrParamConfiguration(CONF_ADDITIONAL_LANGUAGE) );
	}

	@Override
	public void setAdditionalLanguage( String value )
	{
		this.M_setStrParamConfiguration( CONF_ADDITIONAL_LANGUAGE, value );
	}

//	@Override
//	public String getLatestProjectXmlFileName() {
//		return( this.M_getStrParamConfiguration(CONF_LATEST_PROJECT_XML_FILENAME) );
//	}
//
//	@Override
//	public void setLatestProjectXmlFileName(String latestProjectXmlFileName) {
//		M_setStrParamConfiguration(CONF_LATEST_PROJECT_XML_FILENAME, latestProjectXmlFileName);
//	}

	@Override
	public String getDefaultLanguageBaseConfigurationFolder()
	{
		String result = System.getProperty("user.home") + File.separator  +
						getConfigurationMainFolder() + File.separator +
						getApplicationNameFolder() + File.separator +
						getApplicationGroup() + File.separator;

		return( result );
	}

	@Override
	public String getDefaultLanguageConfigurationFolder( String language )
	{
		String result = getDefaultLanguageBaseConfigurationFolder() + language;

		return( result );
	}

	@Override
	public String getDefaultLanguage()
	{
		return( GenericFunctions.instance().getObtainAvailableLanguages().getDefaultLanguage() );
	}

	@Override
	public void setZoomFactorReference(DoubleReference zoomFactor) {
		throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
	}

	@Override
	public DoubleReference getZoomFactorReference() {
		throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
	}

	@Override
	public int getNumThreads() {
		return IntegerFunctions.limit(M_getIntParamConfiguration(CONF_NUM_THREADS),
				2,
				ThreadFunctions.instance().getNumMaxThreads() - 1);
	}

	@Override
	public void setNumThreads(int value) {
		M_setIntParamConfiguration(CONF_NUM_THREADS, value);
	}

	@Override
	public String getTableColumnsString()
	{
		return( M_getStrParamConfiguration(CONF_TABLE_COLUMNS_STRING) );
	}

	@Override
	public void setTableColumnsString(String value)
	{
		this.M_setStrParamConfiguration( CONF_TABLE_COLUMNS_STRING, value );
	}
/*
	@Override
	public String getHomePageUrl()
	{
		return( this.M_getStrParamConfiguration( CONF_HOME_WEB_PAGE_URL ) );
	}

	@Override
	public String getAuthorEmailAddress()
	{
		return( this.M_getStrParamConfiguration( CONF_AUTHOR_EMAIL_ADDRESS ) );
	}
*/

	@Override
	public String getUrlForFaceDetectionQuery()
	{
		return( URL_FOR_FACE_DETECTION_QUERY );
	//		return( M_getStrParamConfiguration( CONF_URL_FOR_NEW_VERSION_QUERY ) );
	}

	@Override
	public void setLanguage( String language )
	{
		if( language != null )
		{
			super.setLanguage( language );

			try
			{
				Locale locale = ObtainAvailableLanguages_base.instance().getLocaleOfLanguage( language );

				JComponent.setDefaultLocale( locale );
			}
			catch( Exception ex )
			{}
		}
	}
}
