/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.applications.morphing.imagelisttable;

import com.frojasg1.applications.common.configuration.InternationalizedStringConf;
import com.frojasg1.applications.libmorphing.model.images.image.MorphingImageModel;
import com.frojasg1.general.FileFunctions;
import com.frojasg1.general.desktop.view.componentwrapper.jtable.column.RecordToTableCellStringsBase;
import com.frojasg1.general.desktop.view.componentwrapper.jtable.column.TableColumnData;
import java.io.File;
import java.util.Map;
import javax.swing.JTable;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class MorphingImageModelToTableCellStrings extends RecordToTableCellStringsBase<MorphingImageModel>
{
	public static final String INDEX_LABEL = "INDEX";
	public static final String NAME_LABEL = "NAME";
	public static final String IS_MASTER_LABEL = "IS_MASTER";
	public static final String SIZE_LABEL = "SIZE";
	public static final String FILE_NAME_LABEL = "FILE_NAME";
	public static final String PATH_LABEL = "PATH";

	public static final String CONF_TRUE = "TRUE";
	public static final String CONF_FALSE = "FALSE";

	public MorphingImageModelToTableCellStrings( JTable table, InternationalizedStringConf internationalStrings )
	{
		super( table, internationalStrings::getInternationalString );
	}

	@Override
	public Map<String, TableColumnData<MorphingImageModel>> fillColumnDataMapChild(Map<String, TableColumnData<MorphingImageModel>> result)
	{
		put( result, INDEX_LABEL, (ind, ple) -> String.valueOf(ind), true, false, null );
		put( result, NAME_LABEL, this::getName, true, true, createGenericComparator( this::getName ) );
		put( result, IS_MASTER_LABEL, this::getIsMaster, true, true, createGenericComparator( this::getIsMaster, this::getFileName ) );
		put( result, SIZE_LABEL, this::getFileSize, true, true, createGenericComparator( this::getFileSize, this::getFileName ) );
		put(result, FILE_NAME_LABEL, this::getFileBaseName, true, true, createGenericComparator( this::getFileBaseName, this::getFileName ) );
		put( result, PATH_LABEL, this::getFileName, false, true, createGenericComparator( this::getFileName ) );

		return( result );
	}

	protected String getName( int index, MorphingImageModel record )
	{
		return( getName( record ) );
	}

	protected String getName( MorphingImageModel record )
	{
		String result = null;
		if( record != null )
			result = record.getName();

		return( result );
	}

	protected String getIsMaster( int index, MorphingImageModel record )
	{
		return( getIsMaster( record ) );
	}

	protected String getIsMaster( MorphingImageModel record )
	{
		String result = null;
		if( record != null )
		{
			if( record.isMaster() )
				result = getInternationalString(CONF_TRUE);
			else
				result = getInternationalString(CONF_FALSE);
		}

		return( result );
	}

	protected String formatFileSize( long sizeInBytes )
	{
		return( FileFunctions.instance().formatFileSize( sizeInBytes, getOutputLocale() ) );
	}

	protected String getFileSize( int index, MorphingImageModel record )
	{
		return( formatFileSize( getFileSize( record ) ) );
	}

	protected Long getFileSize( MorphingImageModel record )
	{
		return( new File( getFileName(record) ).length() );
	}

	protected String getFileBaseName( int index, MorphingImageModel record )
	{
		return( getFileBaseName( record ) );
	}

	protected String getFileBaseName( MorphingImageModel record )
	{
		return( ( record == null ) ? "" :
			FileFunctions.instance().getBaseName( record.getFileName() ) );
	}

	protected String getFileName( int index, MorphingImageModel record )
	{
		return( getFileName( record ) );
	}

	protected String getFileName( MorphingImageModel record )
	{
		return( ( record == null ) ? "" : record.getFileName() );
	}
}
