/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.applications.morphing.language;

import com.frojasg1.applications.common.configuration.imp.InternationalizedStringConfImp;
import com.frojasg1.applications.morphing.configuration.ApplicationConfiguration;
import com.frojasg1.applications.morphing.controller.MorphingControllerImpl;
import com.frojasg1.applications.morphing.imagelisttable.MorphingImageModelListTableWrapper;
import com.frojasg1.applications.morphing.imagelisttable.MorphingImageModelToTableCellStrings;
import com.frojasg1.general.ExecutionFunctions;
import com.frojasg1.general.desktop.view.componentwrapper.jtable.labels.JTableWrapperInternationalLabels;
import com.frojasg1.applications.libmorphing.api.facedetector.runnable.FaceDetectionQueryExecution;
/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class MorphingApplicationInternationalTexts extends InternationalizedStringConfImp {
	public static final String GLOBAL_CONF_FILE_NAME = "MorphingApplicationInternationalTexts.properties";

	public static final String CONF_DEFAULT_PROJECT_NAME = MorphingControllerImpl.CONF_DEFAULT_PROJECT_NAME;

	protected static class LazyHolder
	{
		protected static final MorphingApplicationInternationalTexts INSTANCE = new MorphingApplicationInternationalTexts();
	}

	public static MorphingApplicationInternationalTexts instance()
	{
		return( LazyHolder.INSTANCE );
	}

	protected MorphingApplicationInternationalTexts()
	{
		super( GLOBAL_CONF_FILE_NAME, ApplicationConfiguration.instance().getInternationalPropertiesPathInJar() );

		registerInternationalizedStrings();

		ExecutionFunctions.instance().safeMethodExecution( () -> changeLanguage( getLanguage() ) );
	}

	protected void registerInternationalizedStrings()
	{
/*
		registerInternationalString(ControllerInternationalLabels.CONF_M3U_PLAYLIST, "m3u playlist" );
		registerInternationalString(ControllerInternationalLabels.CONF_CURRENT_PLAY_LIST_HAS_BEEN_MODIFIED, "Current play list has been modified. Do you want to save it?" );
		registerInternationalString(ControllerInternationalLabels.CONF_ERROR_LOADING_PLAYLIST_FILE, "Error when loading playlist file: $1" );
		registerInternationalString(ControllerInternationalLabels.CONF_ERROR_SAVING_PLAYLIST_FILE, "Error when saving playlist: $1" );
		registerInternationalString(ControllerInternationalLabels.CONF_PLAYLIST_FILE_SAVED_OK, "Play list saved successfully" );
		registerInternationalString(ControllerInternationalLabels.CONF_ARE_YOU_SURE_YOU_WANT_TO_ERASE_THE_SELECTED_ROWS, "Are you sure you want to erase the selected rows ?" );
		registerInternationalString(ControllerInternationalLabels.CONF_CURRENT_PLAY_LIST_IS_EMPTY, "The play list is empty. Are you sure you want to save it ?" );
		registerInternationalString(ControllerInternationalLabels.CONF_LOADING_IN_PROGRESS, "Loading in progress" );
		registerInternationalString(ControllerInternationalLabels.CONF_ERROR_PROCESSING_FILE, "Error processing file $1" );
*/

		registerInternationalString( MorphingImageModelToTableCellStrings.INDEX_LABEL, "#" );
		registerInternationalString( MorphingImageModelToTableCellStrings.NAME_LABEL, "Name" );
		registerInternationalString( MorphingImageModelToTableCellStrings.IS_MASTER_LABEL, "Is master" );
		registerInternationalString( MorphingImageModelToTableCellStrings.SIZE_LABEL, "File size" );
		registerInternationalString( MorphingImageModelToTableCellStrings.FILE_NAME_LABEL, "File name" );
		registerInternationalString( MorphingImageModelToTableCellStrings.PATH_LABEL, "Long file name" );
//		registerInternationalString( PlayListEntryToTableCellStrings.CONF_FORMAT_NOT_RECOGNIZED, "Format not recognized" );
		registerInternationalString( MorphingImageModelListTableWrapper.CONF_DRAG_AND_DROP_FILES, "Drag and drop files" );
		registerInternationalString( MorphingImageModelListTableWrapper.CONF_CLONE_SELECTED_ROWS, "Clone selected rows" );

		registerInternationalString( JTableWrapperInternationalLabels.CONF_AUTORESIZE_ALL_COLUMNS, "Autoresize all columns" );
		registerInternationalString( JTableWrapperInternationalLabels.CONF_AUTORESIZE_THIS_COLUMN, "Autoresize this column" );
		registerInternationalString( JTableWrapperInternationalLabels.CONF_AUTORESIZE_ALWAYS_ALL_COLUMNS, "Autoresize always alls columns" );
		registerInternationalString( JTableWrapperInternationalLabels.CONF_SORT_ASCENTING_BY_THIS_COLUMN, "Order ascending by this column" );
		registerInternationalString( JTableWrapperInternationalLabels.CONF_SORT_DESCENTING_BY_THIS_COLUMN, "Order descending by this column" );
		registerInternationalString( JTableWrapperInternationalLabels.CONF_DELETE_SELECTED_ROWS, "Remove songs from play list" );
		registerInternationalString( JTableWrapperInternationalLabels.CONF_RESET_SELECTION, "Clean selection" );
		registerInternationalString( JTableWrapperInternationalLabels.CONF_ERROR, "- Error -" );
		registerInternationalString( JTableWrapperInternationalLabels.CONF_SHOW_CURRENT_ROW, "Show current row" );

		registerInternationalString( MorphingControllerImpl.CONF_ERROR_WHEN_LOADING_MORPHING_PROJECT, "Error when loading project: $1" );
		registerInternationalString( MorphingControllerImpl.CONF_ERROR_WHEN_SAVING_MORPHING_PROJECT, "Error when saving project: $1" );
		registerInternationalString( MorphingControllerImpl.CONF_ERROR_WHEN_ERASING_IMAGES, "Error when erasing images: $1" );
		registerInternationalString( MorphingControllerImpl.CONF_CONFIRM_RECALCULATION_OF_ALL_MESHES, "This action cannot be reverted.\n Are you sure you want to recalculate all meshes ?" );
		registerInternationalString( CONF_DEFAULT_PROJECT_NAME, "Project Name" );
		registerInternationalString( MorphingControllerImpl.CONF_PROJECT_SAVED_SUCCESSFULLY, "Project saved successfully" );
		registerInternationalString( MorphingControllerImpl.CONF_SUCCESS_IMPORTING_FILE, "Successfully imported the detected faces of $1 image files" );
		registerInternationalString( MorphingControllerImpl.CONF_SUCCESS_EXPORTING_FILE, "Successfully exported face detection file\n $1" );
		registerInternationalString( MorphingControllerImpl.CONF_ERROR_IMPORTING_FILE, "Error importing face detection file: $1" );
		registerInternationalString( MorphingControllerImpl.CONF_ERROR_EXPORTING_FILE, "Error exporting face detection file: $1" );
		registerInternationalString( JTableWrapperInternationalLabels.CONF_ARE_YOU_SURE_YOU_WANT_TO_ERASE_THE_SELECTED_ROWS, "Are you sure you want to erase the ($1) selected rows ?" );
	}
}

