/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.applications.morphing.mvc;

import com.frojasg1.applications.libmorphing.model.images.image.MorphingImageModel;
import com.frojasg1.applications.libmorphing.mvc.DelaunayTriangulatorView;
import com.frojasg1.applications.libmorphing.mvc.DelaunayTriangulatorWindowView;
import com.frojasg1.applications.libmorphing.view.mainwindow.evt.MainWindowEvent;
import com.frojasg1.libgeometry.types.GeometricRectangle;
import com.frojasg1.general.collection.functions.ListStateContextBase;
import com.frojasg1.general.desktop.view.panels.impl.file.FileHandlingPanelEvent;
import com.frojasg1.general.view.ViewComponent;
import com.frojasg1.general.view.evt.wrapper.GenericWrapperEvent;
import com.frojasg1.general.view.evt.wrapper.impl.table.impl.GenTableWrapperRecordsModifiedEventBase;
import java.util.function.Consumer;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public interface MorphingView extends DelaunayTriangulatorView
{
	public void addImageListTableListener( Consumer<? super GenericWrapperEvent> listener,
											Long ... eventTypes );
	public void removeImageListTableListener( Consumer<? super GenericWrapperEvent> listener );

	public void addFileHandlingListener( Consumer<FileHandlingPanelEvent> listener );
	public void removeFileHandlingListener( Consumer<FileHandlingPanelEvent> listener );
	public void setMainWindowListener(Consumer<MainWindowEvent> listener);


//	public void setTriangulation(String imageName,
//								 MorphingProjectWholeFileModelManager morphingProjectManager);
	public void updateTriangulation(String imageName);

	public void updateCurrentTableList( ListStateContextBase<MorphingImageModel> modifiedListState );

	public String getFileName();
	public void setFileName(String fileName);

	public String getProjectName();
	public void setProjectName( String projectName );

	public ViewComponent getMainWindow();
	public DelaunayTriangulatorWindowView getDelaunayTriangulatorView();

	public void newImageChosen( MorphingImageModel mim );

	public void askNewForANewBoundary( MorphingImageModel mim,
										GeometricRectangle masterBoundary,
										Consumer<GeometricRectangle> callback );

	public GeometricRectangle getFittingImageSizeForJPanel();

	public boolean confirmErasureOfSelectedRecords(
		GenTableWrapperRecordsModifiedEventBase<MorphingImageModel,
                                                    ?,
                                                    ListStateContextBase<MorphingImageModel>> evt );
}
