/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.applications.morphing.startapp;

import com.frojasg1.applications.morphing.LicenseJDialog;
import com.frojasg1.applications.morphing.Splash;
import com.frojasg1.applications.morphing.configuration.ApplicationConfiguration;
import com.frojasg1.applications.common.components.internationalization.window.InternationalizationInitializationEndCallback;
import com.frojasg1.applications.morphing.context.ApplicationContextImp;
import com.frojasg1.applications.morphing.controller.MorphingControllerImpl;
import com.frojasg1.applications.morphing.mvc.MorphingController;
import com.frojasg1.applications.morphing.view.MorphingViewManager;
import com.frojasg1.delaunay.triangulation.quadedge.builder.QuadEdgeDelaunayBuilder;
import com.frojasg1.delaunay.triangulation.quadedge.builder.impl.QuadEdgeDelaunayBuilderImpl;
import com.frojasg1.general.desktop.startapp.StartApplicationBase;
import com.frojasg1.general.desktop.view.license.GenericLicenseJDialog;
import com.frojasg1.general.exceptions.ConfigurationException;
import java.util.function.Consumer;
import javax.swing.JDialog;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class StartApplication extends StartApplicationBase<ApplicationContextImp>
{
//	protected static final int MINIMUM_NUMBER_OF_STEPS = 3;

	protected OpenConfiguration _oc = null;

	@Override
	protected Splash createSplash()
	{
		return( Splash.instance() );
	}

	@Override
	protected OpenConfiguration createOpenConfiguration() throws ConfigurationException 
	{
		_oc = new OpenConfiguration( getApplicationContext().getApplicationName() );
		return( _oc );
	}

	@Override
	protected ApplicationConfiguration getAppliConf()
	{
		return( ApplicationConfiguration.instance() );
	}

	@Override
	protected GenericLicenseJDialog createLicenseJDialog(Consumer<InternationalizationInitializationEndCallback> initializationCallback)
	{
		return( new LicenseJDialog( (JDialog)null, getAppliConf(), initializationCallback ) );
	}

	@Override
	protected void saveApplicationConfiguration() throws ConfigurationException
	{
		getAppliConf().M_saveConfiguration();
	}
/*
	@Override
	protected MainWindow createMainWindow() throws ConfigurationException
	{
		MainWindow result = new MainWindow("Morphing effect creator application", getApplicationContext() );

		return( result );
	}
*/
	@Override
	protected MorphingViewManager createMainWindow() throws ConfigurationException
	{
		getApplicationContext().init();
		MorphingViewManager result = new MorphingViewManager( getApplicationContext() );

		return( result );
	}

	@Override
	public MorphingViewManager getMainWindow()
	{
		return( (MorphingViewManager) super.getMainWindow() );
	}

	protected MorphingController createController()
	{
		return( new MorphingControllerImpl( getApplicationContext() ) );
	}

	protected MorphingController getController()
	{
		return( getIfNotNull( getApplicationContext(),
							ApplicationContextImp::getController ) );
	}

	protected QuadEdgeDelaunayBuilder createDelaunayBuilder()
	{
		return( new QuadEdgeDelaunayBuilderImpl() );
	}

	@Override
	protected void postMainWindowCreation()
	{
		getApplicationContext().setView( getMainWindow() );
		getApplicationContext().getController().setView( getMainWindow() );
	}

	@Override
	protected ApplicationContextImp refineApplicationContext( ApplicationContextImp applicationContext )
	{
		MorphingController controller = createController();
		applicationContext.setDelaunayBuilder( createDelaunayBuilder() );
		applicationContext.setController( controller );

		return( applicationContext );
	}

/*
	@Override
	protected int getMinimumNumberOfSteps()
	{
		return( MINIMUM_NUMBER_OF_STEPS );
	}
*/
}
