/* 
 * Copyright (C) 2022 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/gpl-3.0.txt
 *
 */

package com.frojasg1.applications.morphing.videobuilder.configuration;

import com.frojasg1.applications.common.configuration.application.BaseApplicationConfiguration;
import com.frojasg1.applications.libmorphing.configuration.MorphingApplicationConfiguration;
import com.frojasg1.applications.libmorphing.constants.MorphingConstants;
import com.frojasg1.applications.morphing.videobuilder.MainWindow;
import com.frojasg1.general.ExecutionFunctions;
import com.frojasg1.general.exceptions.IllegalCallException;
import com.frojasg1.general.number.DoubleReference;
import com.frojasg1.general.number.IntegerFunctions;
import com.frojasg1.general.threads.ThreadFunctions;
import com.frojasg1.generic.GenericFunctions;
import com.frojasg1.generic.languages.ObtainAvailableLanguages_base;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.util.Locale;
import java.util.Properties;
import javax.swing.JComponent;

/**
 *
 * @author Usuario
 */
public class ApplicationConfiguration extends BaseApplicationConfiguration implements MorphingApplicationConfiguration
{
	private static final Logger LOGGER = LoggerFactory.getLogger(ApplicationConfiguration.class);

	private static ApplicationConfiguration a_instance = null;

	public static final String CONF_LAST_DIRECTORY = "LAST_DIRECTORY";
	public static final String CONF_LANGUAGE = "LANGUAGE";
	public static final String CONF_ADDITIONAL_LANGUAGE = "ADDITIONAL_LANGUAGE";

	public static final String CONF_LICENSES_HAVE_BEEN_ACCEPTED = "LICENSES_HAVE_BEEN_ACCEPTED";
//	public static final String CONF_HOME_WEB_PAGE_URL = "HOME_WEB_PAGE_URL";
//	public static final String CONF_AUTHOR_EMAIL_ADDRESS = "AUTHOR_EMAIL_ADDRESS";

	public static final String CONF_LATEST_PROJECT_XML_FILENAME = "LATEST_PROJECT_XML_FILENAME";
	public static final String CONF_FFMPEG_BINARY_FILENAME = "FFMPEG_BINARY_FILENAME";

//	protected static final String APPLICATION_GROUP = "general";
	protected static final String GLOBAL_CONF_FILE_NAME = "GlobalConfiguration.properties";
/*
	protected static final String sa_PATH_PROPERTIES_IN_JAR =	"com" + ConfigurationParent.sa_dirSeparator + 
																"hotmail" + ConfigurationParent.sa_dirSeparator +
																"frojasg1" + ConfigurationParent.sa_dirSeparator + 
																"aplicaciones" + ConfigurationParent.sa_dirSeparator +
																"fileencoderapplication" + ConfigurationParent.sa_dirSeparator +
																"view" + ConfigurationParent.sa_dirSeparator +
																"internationalization" + ConfigurationParent.sa_dirSeparator +
																"properties";
*/
	protected static final String RESOURCE_FOR_APPLICATION_ICON="com/frojasg1/morphing/videobuilder/resources/icons/App.icon.png";
	public static final String sa_PROPERTIES_PATH_IN_JAR="com/frojasg1/app/morphing/videobuilder/inter";
	public static final String sa_MAIN_FOLDER = ".frojasg1.apps";
	public static String sa_APPLICATION_NAME = null;
	public static final String sa_CONFIGURATION_GROUP = "Configuration";
	protected static final String CONF_NUM_THREADS = MorphingConstants.CONF_NUM_THREADS;

	public static final String ES_LANGUAGE = "ES";	 // Spanish language
	public static final String EN_LANGUAGE = "EN";	 // English language

	public static String VERSION = "v1.1";

	protected Integer numThreads;

	private ApplicationConfiguration()
	{
//		super( sa_MAIN_FOLDER, sa_APPLICATION_NAME, APPLICATION_GROUP, null, GLOBAL_CONF_FILE_NAME );
		this( sa_MAIN_FOLDER, sa_APPLICATION_NAME, VERSION, sa_CONFIGURATION_GROUP, GLOBAL_CONF_FILE_NAME );
	}

	public ApplicationConfiguration( String mainFolder, String applicationName, String version, String group, String globalConfFileName )
	{
		super( mainFolder, applicationName + "/" + version, group, null, globalConfFileName );
	}

	public static ApplicationConfiguration instance()
	{
		return( a_instance );
	}

	public static ApplicationConfiguration create( String applicationName )
	{
		sa_APPLICATION_NAME = applicationName;
		a_instance = new ApplicationConfiguration();

		return( a_instance );
	}

	@Override
	public Properties M_getDefaultProperties( String language )
	{
		Properties result = super.M_getDefaultProperties( language );

		result.setProperty(CONF_LANGUAGE, EN_LANGUAGE );
		result.setProperty(CONF_ADDITIONAL_LANGUAGE, "CAT" );

		result.setProperty(CONF_LICENSES_HAVE_BEEN_ACCEPTED, "0" );
//		result.setProperty(CONF_HOME_WEB_PAGE_URL, "https://frojasg1.com" );
//		result.setProperty(CONF_AUTHOR_EMAIL_ADDRESS, "frojasg1@hotmail.com" );

		return( result );
	}

/*
	public float getFontSizeFactor()
	{
		String factorStr = M_getStrParamConfiguration(ApplicationConfiguration.CONF_APPLICATION_FONT_SIZE);
		float factor = 1.0F;
		try
		{
			factor = Float.valueOf( factorStr );
		}
		catch( Throwable th )
		{
			th.printStackTrace();
		}
		return( factor );
	}
*/
	@Override
	public String getConfigurationMainFolder()
	{
		return( sa_MAIN_FOLDER );
	}

	@Override
	public String getApplicationNameFolder()
	{
		return( getApplicationName() + File.separator + getApplicationVersion() );
	}

	@Override
	public String getApplicationName()
	{
		return( sa_APPLICATION_NAME );
	}

	@Override
	public String getApplicationVersion()
	{
		return( VERSION );
	}

	@Override
	public String getApplicationGroup()
	{
		return( sa_CONFIGURATION_GROUP );
	}

	@Override
	public String getInternationalPropertiesPathInJar()
	{
		return( sa_PROPERTIES_PATH_IN_JAR );
	}
	
	@Override
	public String getResourceNameForApplicationIcon()
	{
		return( RESOURCE_FOR_APPLICATION_ICON );
	}

	@Override
	public boolean getLicensesHaveBeenAccepted()
	{
		return( this.M_getBoolParamConfiguration(CONF_LICENSES_HAVE_BEEN_ACCEPTED) );
	}

	@Override
	public void setLicensesHaveBeenAccepted( boolean value )
	{
		this.M_setBoolParamConfiguration( CONF_LICENSES_HAVE_BEEN_ACCEPTED, value );
	}

	@Override
	public synchronized int getNumThreads() {
		if(numThreads == null) {
			numThreads = ExecutionFunctions.instance().safeFunctionExecution(() -> getEditorNumThreads());
		}

		if(numThreads == null) {
			numThreads = 16;
		}

		numThreads = IntegerFunctions.limit(numThreads,
				2,
				ThreadFunctions.instance().getNumMaxThreads() - 1);

		return numThreads;
	}

	protected String getEditorConfigurationFileName() {
		return getConfigurationFileName(
				getRootApplicationConfigurationPath(
						a_mainFolder,
						MorphingConstants.EDITOR_APPLICATION_NAME, getApplicationVersion(),
						getApplicationGroup()),
				null, a_configurationFileName);
	}

	protected Integer getEditorNumThreads() {
		Properties prop = openProperties(getEditorConfigurationFileName());
		Integer result = getInteger(prop, CONF_NUM_THREADS);

		LOGGER.info("Loaded editor num threads: {}", result);
		return result;
	}

	@Override
	public void setNumThreads(int value) {
//		M_setIntParamConfiguration(CONF_NUM_THREADS, value);
		throw new IllegalCallException("Not allowed call. That parameter will be retrieved from the Editor configuration");
	}

	protected Integer getInteger(Properties prop, String label) {
		String value = prop.getProperty(label);
		return IntegerFunctions.parseInt(value);
	}

	@Override
	public String getAdditionalLanguage()
	{
		return( this.M_getStrParamConfiguration(CONF_ADDITIONAL_LANGUAGE) );
	}

	@Override
	public void setAdditionalLanguage( String value )
	{
		this.M_setStrParamConfiguration( CONF_ADDITIONAL_LANGUAGE, value );
	}

	public String getLatestProjectXmlFileName() {
		return( this.M_getStrParamConfiguration(CONF_LATEST_PROJECT_XML_FILENAME) );
	}

	public void setLatestProjectXmlFileName(String latestProjectXmlFileName) {
		M_setStrParamConfiguration(CONF_LATEST_PROJECT_XML_FILENAME, latestProjectXmlFileName);
	}

	@Override
	public String getTableColumnsString() {
		return null;
	}

	@Override
	public void setTableColumnsString(String value) {

	}

	@Override
	public String getUrlForFaceDetectionQuery() {
		return null;
	}

	public String getFfmpegBinaryFileName() {
		return( this.M_getStrParamConfiguration(CONF_FFMPEG_BINARY_FILENAME) );
	}

	public void setFfmpegBinaryFileName(String value) {
		M_setStrParamConfiguration(CONF_FFMPEG_BINARY_FILENAME, value);
	}

	@Override
	public String getDefaultLanguageBaseConfigurationFolder()
	{
		String result = System.getProperty("user.home") + File.separator  +
						getConfigurationMainFolder() + File.separator +
						getApplicationNameFolder() + File.separator +
						getApplicationGroup() + File.separator;

		return( result );
	}

	@Override
	public String getDefaultLanguageConfigurationFolder( String language )
	{
		String result = getDefaultLanguageBaseConfigurationFolder() + language;

		return( result );
	}

	@Override
	public String getDefaultLanguage()
	{
		return( GenericFunctions.instance().getObtainAvailableLanguages().getDefaultLanguage() );
	}

	@Override
	public void setZoomFactorReference(DoubleReference zoomFactor) {
		throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
	}

	@Override
	public DoubleReference getZoomFactorReference() {
		throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
	}
/*
	@Override
	public String getHomePageUrl()
	{
		return( this.M_getStrParamConfiguration( CONF_HOME_WEB_PAGE_URL ) );
	}

	@Override
	public String getAuthorEmailAddress()
	{
		return( this.M_getStrParamConfiguration( CONF_AUTHOR_EMAIL_ADDRESS ) );
	}
*/
	@Override
	public void setLanguage( String language )
	{
		if( language != null )
		{
			super.setLanguage( language );

			try
			{
				Locale locale = ObtainAvailableLanguages_base.instance().getLocaleOfLanguage( language );

				JComponent.setDefaultLocale( locale );
			}
			catch( Exception ex )
			{}
		}
	}
}
