/* 
 * Copyright (C) 2022 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/gpl-3.0.txt
 *
 */

package com.frojasg1.applications.morphing.videobuilder.startapp;

import com.frojasg1.applications.libmorphing.init.LibMorphingInit;
import com.frojasg1.applications.morphing.videobuilder.configuration.ApplicationConfiguration;
import com.frojasg1.applications.morphing.videobuilder.configuration.ApplicationConfigurationFactory;
import com.frojasg1.general.FileFunctions;
import com.frojasg1.general.desktop.startapp.OpenConfigurationDesktopBase;
import com.frojasg1.general.exceptions.ConfigurationException;
import java.io.IOException;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class OpenConfiguration extends OpenConfigurationDesktopBase<ApplicationConfiguration>
{
	protected static final int INDEX_MAIN_CONFIGURATION = 0;
	protected static final int NUMBER_OF_SETS_OF_CONFIGURATIONS_TO_IMPORT = 1;

	protected static final String MAIN_APP_CONF_FOLDER = FileFunctions.instance().convertFolderSeparator( ".frojasg1.apps/morphingEffectCreator/v1.0" );

//	protected ApplicationContextImp _appCtx = null;

	protected String _applicationName = null;

	protected static final String[] PARAMETERS_FOR_APPLICATION_CONFIGURATION = 	{
																					"Configuration",
																					"GlobalConfiguration.properties"
																				};

	public OpenConfiguration( String applicationName ) throws ConfigurationException
	{
		super();
		_applicationName = applicationName;
	}

	@Override
	protected int getNumberOfSetsOfConfigurationsToImport()
	{
		return( NUMBER_OF_SETS_OF_CONFIGURATIONS_TO_IMPORT );
	}

	@Override
	protected PairCurrentToArrayOfPossibleImports createImportElement(int index)
	{
		PairCurrentToArrayOfPossibleImports result = null;
		if( index == INDEX_MAIN_CONFIGURATION )
		{
			result = createPairForMainConfiguration();
		}

		return( result );
	}

	protected PairCurrentToArrayOfPossibleImports createPairForMainConfiguration()
	{
		return( new PairCurrentToArrayOfPossibleImports( MAIN_APP_CONF_FOLDER,
								createArray(
									new VersionToImportEntry(
											new String[] { ".frojasg1.apps", "morphingEffectCreator" },
											"v1.0",
											PARAMETERS_FOR_APPLICATION_CONFIGURATION
									)
								),
								(existingFolder, newFolder) -> createOrCopyMainFolder(existingFolder, newFolder)
														)
				);
/*
		return( new PairCurrentToArrayOfPossibleImports( MAIN_APP_CONF_FOLDER,
								createArray( 
									new VersionToImportEntry(
											new String[] { ".frojasg1.apps", "multiprecisionCalculator" },
											"v1.1",
											PARAMETERS_FOR_APPLICATION_CONFIGURATION
									)
								),
								(existingFolder, newFolder) -> createOrCopyMainFolder(existingFolder, newFolder)
														)
				);
*/
	}

	@Override
	protected ApplicationConfiguration createApplicationConfiguration() {
		ApplicationConfiguration result = ApplicationConfiguration.create( _applicationName );

		return( result );
	}

	@Override
	protected String[] getBasicLanguages()
	{
		String[] basicLanguages = new String[]{ "EN", "ES", "CAT" };

		return( basicLanguages );
	}

	@Override
	protected String[] getAvailableLanguagesInJar()
	{
		String[] availableLanguagesInJar = new String[]{ "EN", "ES", "CAT" };

		return( availableLanguagesInJar );
	}

	@Override
	protected String[] getWebLanguages()
	{
		String[] webLanguages = new String[]{ "Espanyol", "English", "Catala" };

		return( webLanguages );
	}

	@Override
	public void openOtherConfiguration() throws ConfigurationException
	{
		super.openOtherConfiguration();

		try
		{
//			StringsConfiguration.createInstance( ApplicationConfiguration.instance() );
//			StringsConfiguration.instance().M_openConfiguration();
//			StringsConfiguration.instance().changeLanguage( ApplicationConfiguration.instance().getLanguage() );
		}
		catch( Exception ce )
		{
			ce.printStackTrace();
			throw( ce );
		}

//		WarningsOfEncryptingConfigurations_desktop.createInstance( ApplicationConfiguration.instance() );
	}
	
	@Override
	protected void initLibraries()
	{
		super.initLibraries();

		try
		{
//			_appCtx = Init.init();
			LibMorphingInit.init();
			Init.init();
		}
		catch( Exception ex )
		{
			ex.printStackTrace();
			throw( new RuntimeException( ex ) );
		}
	}
/*
	public ApplicationContextImp getAppCtx() {
		return _appCtx;
	}
*/
	@Override
	public void eraseConfiguration() throws IOException
	{
		FileFunctions.instance().eraseDirCompletely( getFolderName( MAIN_APP_CONF_FOLDER ) );
	}

	@Override
	protected ApplicationConfigurationFactory createApplicationConfigurationFactory()
	{
		return( new ApplicationConfigurationFactory() );
	}

	@Override
	protected void preUpdateMinorReleaseChange() {
		// intentionally left blank
	}
}
