/* 
 * Copyright (C) 2022 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/gpl-3.0.txt
 *
 */

package com.frojasg1.applications.morphing.videobuilder.startapp;

import com.frojasg1.applications.common.components.internationalization.window.InternationalizationInitializationEndCallback;
import com.frojasg1.applications.morphing.videobuilder.LicenseJDialog;
import com.frojasg1.applications.morphing.videobuilder.MainWindow;
import com.frojasg1.applications.morphing.videobuilder.Splash;
import com.frojasg1.applications.morphing.videobuilder.configuration.ApplicationConfiguration;
import com.frojasg1.applications.morphing.videobuilder.context.ApplicationContextImp;
import com.frojasg1.delaunay.triangulation.quadedge.builder.QuadEdgeDelaunayBuilder;
import com.frojasg1.delaunay.triangulation.quadedge.builder.impl.QuadEdgeDelaunayBuilderImpl;
import com.frojasg1.general.desktop.startapp.StartApplicationBase;
import com.frojasg1.general.desktop.view.license.GenericLicenseJDialog;
import com.frojasg1.general.exceptions.ConfigurationException;
import java.util.function.Consumer;
import javax.swing.JDialog;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class StartApplication extends StartApplicationBase<ApplicationContextImp>
{
//	protected static final int MINIMUM_NUMBER_OF_STEPS = 3;

	protected OpenConfiguration _oc = null;

	@Override
	protected Splash createSplash()
	{
		return( Splash.instance() );
	}

	@Override
	protected OpenConfiguration createOpenConfiguration() throws ConfigurationException 
	{
		_oc = new OpenConfiguration( getApplicationContext().getApplicationName() );
		return( _oc );
	}

	@Override
	protected ApplicationConfiguration getAppliConf()
	{
		return( ApplicationConfiguration.instance() );
	}

	@Override
	protected GenericLicenseJDialog createLicenseJDialog(Consumer<InternationalizationInitializationEndCallback> initializationCallback)
	{
		return( new LicenseJDialog( (JDialog)null, getAppliConf(), initializationCallback ) );
	}

	@Override
	protected void saveApplicationConfiguration() throws ConfigurationException
	{
		getAppliConf().M_saveConfiguration();
	}
/*
	@Override
	protected MainWindow createMainWindow() throws ConfigurationException
	{
		MainWindow result = new MainWindow("Morphing effect creator application", getApplicationContext() );

		return( result );
	}
*/
	@Override
	protected MainWindow createMainWindow() throws ConfigurationException
	{
		getApplicationContext().init();
		MainWindow result = new MainWindow("Morphing video creator application", getApplicationContext() );

		return( result );
	}

	@Override
	public MainWindow getMainWindow()
	{
		return( (MainWindow) super.getMainWindow() );
	}

	protected QuadEdgeDelaunayBuilder createDelaunayBuilder()
	{
		return( new QuadEdgeDelaunayBuilderImpl() );
	}

	@Override
	protected void postMainWindowCreation()
	{
	}

	@Override
	protected ApplicationContextImp refineApplicationContext( ApplicationContextImp applicationContext )
	{
		return( applicationContext );
	}

/*
	@Override
	protected int getMinimumNumberOfSteps()
	{
		return( MINIMUM_NUMBER_OF_STEPS );
	}
*/
}
