/* 
 * Copyright (C) 2021 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/gpl-3.0.txt
 *
 */
package com.frojasg1.applications.libmorphing.api.facedetector.jersey;

import com.frojasg1.applications.common.configuration.application.BaseApplicationConfigurationInterface;
import com.frojasg1.applications.libmorphing.api.facedetector.FaceDetectionQueryResult;
import com.frojasg1.general.desktop.queries.InetQueryException;
import com.frojasg1.generic.GenericFunctions;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 * @author fjavier.rojas
 */
public class JerseyFaceDetectionInetQueryForApplications extends JerseyFaceDetectionInetQuery
{
	private static final Logger LOGGER = LoggerFactory.getLogger(JerseyFaceDetectionInetQueryForApplications.class);

	// usage:
	// curl -i -X POST https://frojasg1.com:8443/downloads_web/restful2/faceDetector/20221204.TextCrypt.v1.6.7z -F "file=@./fotos/Fran.202106.143x143.png" -o out.json

	@Override
	public void init( String url )
	{
		super.init(url, RELATIVE_RESTFUL_URL + getDownloadFile());
	}

	@Override
	public FaceDetectionQueryResult queryForApplication(String imageFileName) throws InetQueryException
	{
		FaceDetectionQueryResult result = null;

		LOGGER.info("New face_detection api function invoked, with imageFileName: {}", imageFileName);
		if(imageFileName != null)
		{
			result = query(imageFileName);
			if( result != null )
			{
				LOGGER.info( "JerseyFaceDetectionInetQueryForApplications isSuccessful: " + result.isSuccessful() );
				if( ! result.isSuccessful() )
					LOGGER.error( "JerseyFaceDetectionInetQueryForApplications Error: " + result.getErrorMessage() );
			}
		}
		else
		{
			LOGGER.warn( "JerseyFaceDetectionInetQueryForApplications imageFileName was null" );
		}

		return( result );
	}

	protected String getDownloadFile()
	{
		return( GenericFunctions.instance().getApplicationFacilities().getApplicationVersion());
	}

	protected BaseApplicationConfigurationInterface getAppliConf()
	{
		return( GenericFunctions.instance().getAppliConf() );
	}
}
