/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.delaunay.triangulation.quadedge;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
// http://karlchenofhell.org/cppswp/lischinski.pdf
public class DelaunayQuadEdgeTopologicalOperators {
	protected static DelaunayQuadEdgeTopologicalOperators INSTANCE = new DelaunayQuadEdgeTopologicalOperators();

	public static DelaunayQuadEdgeTopologicalOperators instance()
	{
		return( INSTANCE );
	}

	/*********************** Basic Topological Operators ************************/
	public Edge MakeEdge()
	{
		QuadEdge ql = new QuadEdge();
		return ql.get(0);
	}

	public void Splice(Edge a, Edge b)
	// This operator affects the two edge rings around the origins of a and b,
	// and, independently, the two edge rings around the left faces of a and b.
	// In each case, (i) if the two rings are distinct, Splice will combine
	// them into one; (ii) if the two are the same ring, Splice will break it
	// into two separate pieces.
	// Thus, Splice can be used both to attach the two edges together, and
	// to break them apart. See Guibas and Stolfi (1985) p.96 for more details
	// and illustrations.
	{
		Edge alpha = a.Onext().Rot();
		Edge beta = b.Onext().Rot();

		Edge t1 = b.Onext();
		Edge t2 = a.Onext();
		Edge t3 = beta.Onext();
		Edge t4 = alpha.Onext();
		a.setNext( t1 );
		b.setNext( t2 );
		alpha.setNext( t3 );
		beta.setNext( t4 );
	}

	public void DeleteEdge(Edge e)
	{
		Splice(e, e.Oprev());
		Splice(e.Sym(), e.Sym().Oprev());
		e.Qedge().release();
	}
}
