/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.delaunay.triangulation.quadedge;

import com.frojasg1.libgeometry.types.Point2d;
import java.util.Objects;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class EdgeView {
	protected Point2d _orig;
	protected Point2d _dest;

	public EdgeView( Point2d orig, Point2d dest )
	{
		_orig = orig;
		_dest = dest;
	}

	@Override
	public int hashCode() {
		int hash = 7;
		// making orig and dest be interchangeable
		hash = 67 * hash + Objects.hashCode(this._orig) +
			Objects.hashCode(this._dest);

		return hash;
	}

	protected boolean equals( Object obj11, Object obj12,
								Object obj21, Object obj22)
	{
		return( Objects.equals(obj11, obj12) &&
				Objects.equals(obj21, obj22) );
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		final EdgeView other = (EdgeView) obj;

		return( equals( this._orig, other._orig, this._dest, other._dest ) ||
			equals( this._orig, other._dest, this._dest, other._orig ) );
	}

	
}
