/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.delaunay.triangulation.quadedge;

import com.frojasg1.general.changestate.ChangeableState;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
// http://karlchenofhell.org/cppswp/lischinski.pdf
public class QuadEdge implements ChangeableState<QuadEdge>
{
//	friend Edge *MakeEdge();
	protected EdgeArray _edges;

	public QuadEdge()
	{
		_edges = new EdgeArray();

		Edge e0 = add();
		Edge e1 = add();
		Edge e2 = add();
		Edge e3 = add();
		
		e0.setNext( e0 );
		e1.setNext( e3 );
		e2.setNext( e2 );
		e3.setNext( e1 );
	}

	public EdgeArray getEdges() {
		return _edges;
	}

	protected Edge get( int index )
	{
		return( _edges.get(index) );
	}
	
	protected Edge create()
	{
		return( new Edge(this) );
	}

	protected Edge add()
	{
		Edge result = create();
		_edges.add( result );

		return( result );
	}

	public void release()
	{
		for( Edge edge: _edges.getArray() )
			edge.release();
		_edges = null;
	}

	@Override
	public void changeState(QuadEdge that) {
		_edges = that._edges;
	}

	protected QuadEdge createEmptyCopy() {
		return( new QuadEdge() );
	}

	@Override
	public QuadEdge copyState() {
		QuadEdge result = createEmptyCopy();
		result.changeState(this);

		return( result );
	}
}
