/*
 * Copyright (c) 2023. Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 3.0 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */

/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.frojasg1.delaunay.triangulation.quadedge.builder;

import com.frojasg1.libgeometry.types.GeometricRectangle;
import com.frojasg1.delaunay.triangulation.quadedge.DelaunaySubdivision;
import com.frojasg1.delaunay.triangulation.quadedge.Edge;
import com.frojasg1.libgeometry.types.Point2d;
import java.util.function.BiConsumer;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public interface QuadEdgeDelaunayBuilder {
	public GeometricRectangle createRectangle(double x, double y, double width, double height );
	public DelaunaySubdivision createDelaunaySubdivision(GeometricRectangle bounds);
	public Point2d createPoint( double xx, double yy );
	public Point2d createPoint( Point2d that );

	public boolean intersectsWithBoundary( Edge edge, GeometricRectangle bounds );

	public void browseEdges( DelaunaySubdivision delaunaySubdivision,
							GeometricRectangle bounds,
							BiConsumer<Point2d, Point2d> edgeConsumer );

	public Point2d getIntersectionWithBoundary( GeometricRectangle bounds,
												Point2d pt1,
												Point2d pt2 );

	public void round( Point2d pt );
}
