/*
 * Copyright (c) 2023. Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 3.0 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */

package com.frojasg1.applications.libmorphing.api.facedetector.helpers;

import com.frojasg1.applications.libmorphing.api.facedetector.pojo.FaceDetection;
import com.frojasg1.applications.libmorphing.api.facedetector.pojo.FaceDetectionMeshData;
import com.frojasg1.applications.libmorphing.api.facedetector.pojo.ImageFaceDetection;
import com.frojasg1.applications.libmorphing.model.images.vertex.map.impl.control.IndexedVerticesMap;

import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

public class FaceDetectionMeshDataBuilderAndUpdater extends Control88meshCreator {

    protected static FaceDetectionMeshDataBuilderAndUpdater INSTANCE = new FaceDetectionMeshDataBuilderAndUpdater();

    public static FaceDetectionMeshDataBuilderAndUpdater instance() {
        return INSTANCE;
    }

    protected FaceDetectionMeshData createFaceDetectionMeshData() {
        return new FaceDetectionMeshData();
    }

    public void updateFaceDetectionMeshData(FaceDetectionMeshData result) {
        if(result != null) {
            String normalizedName = "normalized";
            String idealName = "ideal";
            IndexedVerticesMap vertices = createIdealAndNormalizedControlMeshVerticesMap(result.getOriginalFaceDetection(),
                    idealName, normalizedName);

            result.setName(idealName);

//            result.setOriginalFaceDetection(faceDetection);
//            result.resetModifiedFaceDetection();
            result.setLandmark80points(vertices);
            result.setFaceDetectionFrame(getFaceDetectionFrame(result.getOriginalFaceDetection()));

            int[][] triangles = createMeshTriangles(vertices, normalizedName);
            result.setTriangles(triangles);
        }
    }

    public FaceDetectionMeshData createFaceDetectionMeshData(FaceDetection faceDetection) {
        FaceDetectionMeshData result = null;
        if(faceDetection != null) {
            result = createFaceDetectionMeshData();

            String normalizedName = "normalized";
            String idealName = "ideal";
            IndexedVerticesMap vertices = createIdealAndNormalizedControlMeshVerticesMap(faceDetection, idealName, normalizedName);

            result.setName(idealName);

            result.setOriginalFaceDetection(faceDetection);
            result.resetModifiedFaceDetection();
            result.setLandmark80points(vertices);
            result.setFaceDetectionFrame(getFaceDetectionFrame(faceDetection));

            int[][] triangles = createMeshTriangles(vertices, normalizedName);
            result.setTriangles(triangles);
        }

        return result;
    }

    public void updateFaceDetectionMeshDataList(List<FaceDetectionMeshData> detectedFaceMeshes) {
        List<FaceDetectionMeshData> result = null;
        if(detectedFaceMeshes != null) {
            for(FaceDetectionMeshData faceDetectionMeshData: detectedFaceMeshes) {
                updateFaceDetectionMeshData(faceDetectionMeshData);
            }
        }
    }

    public List<FaceDetectionMeshData> createFaceDetectionMeshDataList(ImageFaceDetection imageFaceDetection) {
        List<FaceDetectionMeshData> result = null;
        if(imageFaceDetection != null) {
            result = imageFaceDetection.getFaceDetections().stream()
                    .map(this::createFaceDetectionMeshData)
                    .collect(Collectors.toList());
        }

        if(result == null) {
            result = Collections.emptyList();
        }
        return result;
    }
}
