/*
 * Copyright (c) 2023. Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 3.0 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */

package com.frojasg1.applications.libmorphing.api.facedetector.helpers.facedetectionframe;

import com.frojasg1.applications.libmorphing.api.facedetector.pojo.FaceDetection;
import com.frojasg1.applications.libmorphing.api.facedetector.pojo.FaceDetectionFrame;

public class FaceDetectionFrameCalculatorContext {

    protected static final double THIRTY_DEGREES = 30d / 180d * Math.PI;

    protected double appliedFactor = 1.0d;
    protected double appliedAngle = 0d;
    protected double maxAngleCorrection = THIRTY_DEGREES;

    protected FaceDetectionFrame result;
    protected FaceDetection faceDetection;

    public FaceDetectionFrame getResult() {
        return result;
    }

    public void setResult(FaceDetectionFrame result) {
        this.result = result;
    }

    public FaceDetection getFaceDetection() {
        return faceDetection;
    }

    public void setFaceDetection(FaceDetection faceDetection) {
        this.faceDetection = faceDetection;
    }

    public double getAppliedFactor() {
        return appliedFactor;
    }

    public double getAppliedAngle() {
        return appliedAngle;
    }

    public void addAngle(double angleDelta) {
        appliedAngle += angleDelta;
    }

    public void applyFactor(double factor) {
        appliedFactor *= factor;
    }

    /**
     *
     * @return - It returns the maximum allowed angle correction in radians
     */
    public double getMaxAngleCorrectionInRadians() {
        return maxAngleCorrection;
    }

    public void setMaxAngleCorrectionInDegrees(double maxAngleCorrectionInDegrees) {
        this.maxAngleCorrection = maxAngleCorrectionInDegrees / 180d * Math.PI;
    }
}
