/*
 * Copyright (c) 2023. Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 3.0 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */

package com.frojasg1.applications.libmorphing.api.facedetector.pojo;

import com.frojasg1.libgeometry.types.GeometricRectangle;
import com.frojasg1.libgeometry.types.Point2d;

import java.util.ArrayList;
import java.util.List;

public class FaceDetection {
    protected GeometricRectangle boundingBox;
    protected List<Point2d> points = new ArrayList<>();

    public GeometricRectangle getBoundingBox() {
        return boundingBox;
    }

    public void setBoundingBox(GeometricRectangle boundingBox) {
        this.boundingBox = boundingBox;
    }

    public List<Point2d> getPoints() {
        return points;
    }

    public void addPoint(Point2d point) {
        getPoints().add(point);
    }

    public void setPoint(int index, Point2d point) {
        getPoints().set(index, point.createCopy());
    }

    public FaceDetection createCopy() {
        FaceDetection result = new FaceDetection();
        result.copy(this);

        return result;
    }

    public void copy(FaceDetection that) {
        if(this != that) {
            boundingBox = that.boundingBox.createCopy();

            getPoints().clear();
            for(Point2d pt: that.getPoints()) {
                getPoints().add(pt.createCopy());
            }
        }
    }
}
