/*
 * Copyright (c) 2023. Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 3.0 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */

package com.frojasg1.applications.libmorphing.contexts;

import com.frojasg1.applications.libmorphing.configuration.MorphingApplicationConfiguration;
import com.frojasg1.applications.libmorphing.controller.MorphingVerticesMapManager;
import com.frojasg1.applications.libmorphing.model.facedetector.ImageFaceDetectionPersistenceModelMap;
import com.frojasg1.applications.libmorphing.model.images.configuration.MorphingProjectConfigurationModel;
import com.frojasg1.applications.libmorphing.model.images.image.MorphingImageModel;
import com.frojasg1.applications.libmorphing.xml.facedetector.FaceDetectionWholeFileModel;
import com.frojasg1.applications.libmorphing.xml.whole.MorphingProjectWholeFileModel;
import com.frojasg1.delaunay.triangulation.quadedge.builder.QuadEdgeDelaunayBuilder;
import com.frojasg1.libgeometry.types.GeometricRectangle;
import com.frojasg1.general.desktop.startapp.GenericDesktopInitContext;
import org.xml.sax.SAXException;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;
import java.io.IOException;
import java.util.List;

public interface MorphingEditorApplicationContext extends GenericDesktopInitContext {
    public QuadEdgeDelaunayBuilder getDelaunayBuilder();
    public MorphingProjectWholeFileModel getMorphingProjectModel();
//    public void setMorphingProjectWholeFileModel(MorphingProjectWholeFileModel model);

    public FaceDetectionWholeFileModel getFaceDetectionWholeFileModel();
//    public void setFaceDetectionWholeFileModel(FaceDetectionWholeFileModel model);

    @Override
    public MorphingApplicationConfiguration getAppliConf();

//    public MorphingProjectWholeFileModelManager getProjectModelManager();

    public void saveMorphingProjectModelAs(String fileName) throws IOException, TransformerException;

    MorphingProjectWholeFileModel loadMorphingProjectModel(String fileName) throws IOException, SAXException, ParserConfigurationException;

    MorphingVerticesMapManager getVerticesManager();

    public void saveFaceDetections();

    public List<MorphingImageModel> getImageModelList();
    public List<MorphingImageModel> getImageModelListCopy();

    public MorphingImageModel getImageModel(String imageName);

    public MorphingImageModel addImage(String imageName, String imageFileName,
                                       GeometricRectangle bounds,
                                       MorphingProjectConfigurationModel configurationModel,
                                       boolean isMaster,
                                       int index);

    public void relocateImages(List<MorphingImageModel> resultingList);

    public void deleteImages(int[] indicesToBeDeleted, List<MorphingImageModel> resultingList);

    public ImageFaceDetectionPersistenceModelMap getImageFaceDetectionPersistenceModelMap();

    public void exportFaceDetections(String fileName);
    public int importFaceDetections(String fileName);

    public boolean allHaveFaceDetection();
}
