/*
 * Copyright (c) 2023. Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 3.0 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */

package com.frojasg1.applications.libmorphing.controller;

import com.frojasg1.applications.libmorphing.api.facedetector.pojo.FaceDetection;
import com.frojasg1.applications.libmorphing.api.facedetector.pojo.FaceDetectionMeshData;
import com.frojasg1.applications.libmorphing.model.images.color.types.MultiTriangle;
import com.frojasg1.applications.libmorphing.model.images.image.MorphingImageModel;
import com.frojasg1.applications.libmorphing.model.images.image.MorphingImageModelList;
import com.frojasg1.applications.libmorphing.model.images.vertex.map.impl.normal.MorphingVerticesMap;
import com.frojasg1.applications.libmorphing.mvc.DelaunayTriangulatorView;
import com.frojasg1.applications.libmorphing.view.panels.evt.VertexEvent;
import com.frojasg1.libgeometry.types.GeometricRectangle;
import com.frojasg1.libgeometry.types.Point2d;
import com.frojasg1.general.collection.functions.ListStateContextBase;

import java.io.File;
import java.util.List;

public interface MorphingVerticesMapManager<PP> {
//    void setMorphingProjectWholeFileModel(MorphingProjectWholeFileModelBase morphingProjectWholeFileModel);

    DelaunayTriangulatorController getDelaunayController();

    MorphingVerticesMap getVerticesMap();

    String getMasterImageName();

    MorphingImageModel getMasterImageModel();

    void setMasterImageName(String masterImageName);

    void clearTriangulation();

    GeometricRectangle getBounds(String imageName);

    /*
            public void relocateImage( String imageName, int toIndex )
            {
                relocateImage( getImageIndex( imageName ), toIndex );
            }

            public void relocateImage( int fromIndex, int toIndex )
            {
                MorphingImageModel fromImageModel = getImageModelList().getList().remove(fromIndex);
                addImageModel(toIndex, fromImageModel);
            }
        */
    void relocateImages(List<MorphingImageModel> resultingList);

    void deleteImages(int[] indicesToBeDeleted, List<MorphingImageModel> resultingList);

    ControlMeshManager getControlMeshManager();

    int getImageIndex(String imageName);

    int getImageIndex(MorphingImageModel imageModel);

    MorphingImageModel getImageModel(int index);

    /*
            public boolean removeImageModel( int index )
            {
                return( removeImageModel( getMasterImageModel( index ) ) );
            }

            public boolean removeImageModel( String imageName )
            {
                return( removeImageModel( getMasterImageModel(imageName) ) );
            }

            protected boolean removeImageModel( MorphingImageModel imageModel )
            {
                getVerticesMap().removeEntryByName( imageModel.getName() );

                return( getImageModelList().remove(imageModel) );
            }
        */
    void addImageModel(int index, MorphingImageModel imageModel);

    MorphingImageModel getImageModel(String imageName);

    MultiTriangle getMultiTriangle(Point2d pt, String imageName);

    void processNewPoint(DelaunayTriangulatorView view,
                         VertexEvent<PP> vertexEvent);

    void setCurrentImageName(String _currentImageName);

    void setCurrentImageIndex(int index);

    MorphingImageModelList getImageModelList();

    void setImageModelList(MorphingImageModelList _imageModelList,
                           String masterImageName);

    Point2d initialControlMap(Point2d masterPt, String imageName);

    void cloneImages(ListStateContextBase<MorphingImageModel> previousListState);

    void droppedFiles(List<File> droppedFileList, int indexWhereToInsert);

    String getCurrentImageModelName();

    void imageStart();

    void imageEnd();

    void imagePrevious();

    void imageNext();

    MorphingImageModel getCurrentImageModel();

    void setNewBounds(MorphingImageModel imageModel,
                      GeometricRectangle newBounds);

    List<FaceDetectionMeshData> getAllFaces(String imageName);

    String getImageFileName(String imageName);

    void reAutoCalculateVertices(String imageName);

    void reCalculateVerticesForAllImagesDifferentFromMaster();
//
//    public void setMorphingProjectWholeFileModel(MorphingProjectWholeFileModel morphingProjectWholeFileModel);

    public void updateImageModelFaceDetectionMeshes();

    public void setImageFaceDetection(MorphingImageModel imageModel, FaceDetection modifiedFaceDetection);

    public void updateVerticesModel();

    public void resetModifiedDetectedFaceMesh(String imageName);

    public void resetDetectedFaceMeshFrame(String imageName);
    public void resetDetectedFaceMeshFrame(String imageName, double maxFactor);

    public boolean insertControlVerticesToTriangulation(String imageName);
    public boolean isDelaunayTriangulationEmpty();
}
