/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.applications.libmorphing.frames.framebuilder;

import com.frojasg1.libgeometry.algebra.helpers.GeometryHelper;
import com.frojasg1.libgeometry.algebra.helpers.Point2dDiscarder;
import com.frojasg1.applications.libmorphing.contexts.MorphingEditorApplicationContext;
import com.frojasg1.applications.libmorphing.contexts.holder.ApplicationCtxHolder;
import com.frojasg1.applications.libmorphing.controller.MorphingVerticesMapManager;
import com.frojasg1.applications.libmorphing.helpers.MorphingHelper;
import com.frojasg1.applications.libmorphing.model.images.image.MorphingImageModel;
import com.frojasg1.applications.libmorphing.model.images.vertex.VertexModel;
import com.frojasg1.delaunay.triangulation.quadedge.DelaunaySubdivision;
import com.frojasg1.delaunay.triangulation.quadedge.helpers.DelaunayTriangleBrowser;
import com.frojasg1.libgeometry.types.GeometricRectangle;
import com.frojasg1.libgeometry.types.GeometricTriangle;
import com.frojasg1.libgeometry.types.Point2d;
import com.frojasg1.libgeometry.types.factory.GeometryCommonFactory;
import com.frojasg1.general.desktop.image.ImageFunctions;
import com.frojasg1.general.desktop.view.ViewFunctions;
import com.frojasg1.general.map.multimap.dynamic.deductible.DeductibleDynamicMultiMapElementBase;
import com.frojasg1.general.number.DoubleFunctions;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.awt.*;
import java.awt.image.BufferedImage;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.function.Supplier;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class MorphingFrameCreator_OLD extends ApplicationCtxHolder {
	private static final Logger LOGGER = LoggerFactory.getLogger(MorphingFrameCreator_OLD.class);

//	protected MorphingVerticesMapManager _morphingVerticesManager;

	public MorphingFrameCreator_OLD(MorphingEditorApplicationContext appliCtx) {
		super(appliCtx);
	}

	//	public MorphingFrameCreator( MorphingVerticesMapManager morphingVerticesManager )
//	{
//		_morphingVerticesManager = morphingVerticesManager;
//	}
/*
	protected double getAspectRatio()
	{
		return( getAspectRatio( getMasterImageModel().getBounds() ) );
	}
*/
	protected BufferedImage createEmptyImage(GeometricRectangle imageSize)
	{
		return( createEmptyImage( (int) imageSize.width, (int) imageSize.height ) );
	}

	protected BufferedImage createEmptyImage(int width, int height )
	{
		return( ImageFunctions.instance().createImage(width, height) );
	}

	protected MorphingHelper getMorphingHelper()
	{
		return( MorphingHelper.instance() );
	}
/*
	protected double getAspectRatio( double width, double height )
	{
		return( getMorphingHelper().getAspectRatio( width, height ) );
	}

	protected double getAspectRatio( BufferedImage image )
	{
		return( getMorphingHelper().getAspectRatio(image) );
	}

	protected double getAspectRatio(DelaunayRectangle originalBounds)
	{
		return( getMorphingHelper().getAspectRatio(originalBounds) );
	}

	protected int zoom( double value, double zoomFactor )
	{
		return( IntegerFunctions.zoomValueRound( value, zoomFactor ) );
	}
*/
	protected GeometricRectangle calculateBounds(GeometricRectangle rectangle )
	{
		return( calculateBounds( rectangle.width, rectangle.height ) );
	}

	protected GeometricRectangle calculateBounds(BufferedImage image )
	{
		return( calculateBounds( image.getWidth(), image.getHeight() ) );
	}

	public GeometricRectangle createDelaunayRectangle(Rectangle rectangle )
	{
		return( GeometryCommonFactory.instance().createDelaunayRectangle(rectangle) );
	}

	protected GeometricRectangle calculateBounds(double width, double height )
	{
		GeometricRectangle masterBounds = getMasterBounds();
		Rectangle rect = ViewFunctions.instance().calculateFittingBounds(
																masterBounds.width,
																masterBounds.height,
																width, height );

		return( createDelaunayRectangle( rect ) );
/*
		DelaunayRectangle masterBounds = getMasterBounds();

		DelaunayRectangle result = masterBounds.createCopy();
		double masterAspectRatio = getAspectRatio( masterBounds );
		double imageAspectRatio = getAspectRatio( width, height );

		if( masterAspectRatio > imageAspectRatio )
		{
			result.width = width;
			result.height = zoom( result.width , 1 / masterAspectRatio );
		}
		else
		{
			result.height = height;
			result.width = zoom( result.height , masterAspectRatio );
		}

		result.x = intHalfDiff( width, result.width );
		result.y = intHalfDiff( height, result.height );

		return( result );
*/
	}
/*
	protected int intHalfDiff( double val1, double val2 )
	{
		return( (int) ( val1 - val2 ) / 2 );
	}
*/
	public BufferedImage createFrame( String imageName1, String imageName2,
									double fadingFraction, double triangulationFraction,
									AtomicBoolean hasToCancel,
									Supplier<GeometricRectangle> frameSizeSupplier)
	{
		if( hasToCancel.get() )
			return( null );

		GeometricRectangle imageSize = frameSizeSupplier.get();
		GeometricRectangle imageBounds = calculateBounds(imageSize);
//		imageBounds.x = 0;
//		imageBounds.y = 0;

//		imageSize = imageBounds;

		return( createFrame( imageName1, imageName2, fadingFraction, triangulationFraction,
							hasToCancel, imageSize, imageBounds, null ) );
	}

	public BufferedImage createFrame(String imageName1, String imageName2,
                                     double fadingFraction, double triangulationFraction,
                                     AtomicBoolean hasToCancel, GeometricRectangle imageSize,
                                     GeometricRectangle bounds, GeometricRectangle clip)
	{
		double[] resultArgbArray = new double[4];
		BufferedImage tmpResult = createEmptyImage(imageSize);

		MorphingImageModel mim1 = getImageModel( imageName1 );
		MorphingImageModel mim2 = getImageModel( imageName2 );

		DelaunayTriangleBrowser.instance().browseTriangles(getDelaunaySubdivision(),
			getMasterBounds(),
			triangle -> processTriangle(triangle, tmpResult, bounds,
						resultArgbArray, mim1, mim2, fadingFraction, triangulationFraction,
						hasToCancel, clip ) );

		BufferedImage result = tmpResult;
		if( hasToCancel.get() )
			result = null;

		return( result );
	}

	protected MorphingImageModel getImageModel( String imageName )
	{
		return( getVerticesManager().getImageModel(imageName) );
	}

	protected MorphingImageModel getMasterImageModel()
	{
		return( getVerticesManager().getMasterImageModel() );
	}

	protected MorphingVerticesMapManager getVerticesManager() {
		return getAppliCtx().getVerticesManager();
	}

	protected GeometricRectangle getMasterBounds()
	{
		return( getVerticesManager().getMasterImageModel().getBounds() );
	}

	protected DelaunaySubdivision getDelaunaySubdivision()
	{
		return( getVerticesManager().getDelaunayController().getTriangulation() );
	}

	protected GeometricTriangle createTriangle(Point2d v1, Point2d v2, Point2d v3 )
	{
		return( GeometryCommonFactory.instance().createDelaunayTriangle( v1, v2, v3, false ) );
	}

	protected String getMasterName()
	{
		return( getVerticesManager().getMasterImageName() );
	}

	protected Point2d mapPoint( Point2d masterPoint, String imageName )
	{
		DeductibleDynamicMultiMapElementBase<VertexModel> multiVertex =
			getVerticesManager().getVerticesMap().get(getMasterName(), masterPoint);

		return( multiVertex.get(imageName).getPoint() );
	}

	protected GeometricTriangle createImageModelTriangle(String imageName,
														 GeometricTriangle masterTriangle )
	{
		return( createTriangle( mapPoint( masterTriangle.getVertex1(), imageName ),
					mapPoint( masterTriangle.getVertex2(), imageName ),
					mapPoint( masterTriangle.getVertex3(), imageName ) ) );
	}

	protected double weight( double value1, double value2, double fraction )
	{
		return( value1 * ( 1 - fraction ) + value2 * fraction );
	}

	protected Point2d weightPoint(Point2d masterPoint, String imageName1, String imageName2,
                                  GeometricRectangle imageBounds, double fraction )
	{
		DeductibleDynamicMultiMapElementBase<VertexModel> multiVertex =
			getVerticesManager().getVerticesMap().get(getMasterName(), masterPoint);

		Point2d normalizedPoint1 = multiVertex.get(imageName1).getNormalizedPoint();
		Point2d normalizedPoint2 = multiVertex.get(imageName2).getNormalizedPoint();

		Point2d normalizedResultPoint = normalizedPoint1.createCopy();
		normalizedResultPoint.x = weight( normalizedPoint1.x, normalizedPoint2.x, fraction );
		normalizedResultPoint.y = weight( normalizedPoint1.y, normalizedPoint2.y, fraction );

		return( round( fromNormalizedPt( normalizedResultPoint, imageBounds ) ) );
	}

	protected Point2d round( Point2d pt )
	{
		return( getMorphingHelper().toIntCoordinates( pt ) );
	}

	protected Point2d fromNormalizedPt( Point2d pt, GeometricRectangle bounds )
	{
		return( getMorphingHelper().fromNormalized(pt, bounds ) );
	}

	protected GeometricTriangle createResultImageTriangle(GeometricTriangle masterTriangle,
														  String imageName1, String imageName2, GeometricRectangle imageBounds,
														  double fraction)
	{
		return( createTriangle( weightPoint( masterTriangle.getVertex1(), imageName1, imageName2, imageBounds, fraction ),
								weightPoint( masterTriangle.getVertex2(), imageName1, imageName2, imageBounds, fraction ),
								weightPoint( masterTriangle.getVertex3(), imageName1, imageName2, imageBounds, fraction ) ) );
	}

	protected boolean processTriangle( GeometricTriangle masterTriangle,
		BufferedImage result, GeometricRectangle resultImageBounds,
		double[] resultArgbArray,
		MorphingImageModel mim1, MorphingImageModel mim2,
		double fadingFraction, double triangulationFraction,
		AtomicBoolean hasToCancel, GeometricRectangle clip)
	{
		if( hasToCancel.get() )
			return false;

		TriangleMapperContext context = new TriangleMapperContext();
		context.setFraction(fadingFraction);
		context.setImage1( mim1.getImage() );
		GeometricTriangle triangle1 = createImageModelTriangle( mim1.getName(), masterTriangle );
		context.setTriangle1( triangle1 );
		context.setImage2(mim2.getImage());
		GeometricTriangle triangle2 = createImageModelTriangle( mim2.getName(), masterTriangle );
		context.setTriangle2( triangle2 );

		context.setResultImage(result);
		GeometricTriangle resultTriangle = createResultImageTriangle(masterTriangle,
			mim1.getName(), mim2.getName(), resultImageBounds, triangulationFraction );
		context.setResultTriangle( resultTriangle );

		context.setResultArgbArray(resultArgbArray);

		if( !canDiscard( resultTriangle, clip ) )
			GeometryHelper.instance().browseTriangle(resultTriangle.getVertex1(),
				resultTriangle.getVertex2(), resultTriangle.getVertex3(), clip,
				context::processPoint );
		return true;
	}

	protected boolean canDiscard(GeometricTriangle triangle, GeometricRectangle clip )
	{
		return( Point2dDiscarder.instance().canDiscard(clip,
			triangle.getVertex1(), triangle.getVertex2(), triangle.getVertex3() ) );
	}

	protected static class TriangleMapperContext
	{
		protected double[] _resultArgbArray;

		protected double _fraction;

		protected BufferedImage _image1;
		protected GeometricTriangle _triangle1;

		protected BufferedImage _image2;
		protected GeometricTriangle _triangle2;

		protected BufferedImage _resultImage;
		protected GeometricRectangle _resultImageBounds;
		protected GeometricTriangle _resultTriangle;

		public void processPoint( Point2d resultImagePoint )
		{
			Point2d pt1 = mapTrianglePt( resultImagePoint, _resultTriangle, _triangle1 );
			Point2d pt2 = mapTrianglePt( resultImagePoint, _resultTriangle, _triangle2 );

			int xx = resultImagePoint.getIntX();
			int yy = resultImagePoint.getIntY();

			if( ( xx >= 0 ) && ( xx < _resultImage.getWidth() ) &&
				( yy >= 0 ) && ( yy < _resultImage.getHeight() ) )
			{
				_resultImage.setRGB( xx, yy, calculateColor(pt1, pt2) );
			}
		}

		protected Point2d mapTrianglePt( Point2d pt, GeometricTriangle originTriangle,
										GeometricTriangle destinationTriangle )
		{
			return( mapTrianglePt(pt,
						originTriangle.getVertex1(), originTriangle.getVertex2(), originTriangle.getVertex3(),
						destinationTriangle.getVertex1(), destinationTriangle.getVertex2(), destinationTriangle.getVertex3() )
			);
		}

		protected void resetArgbArray()
		{
			_resultArgbArray[0] = 0;
			_resultArgbArray[1] = 0;
			_resultArgbArray[2] = 0;
			_resultArgbArray[3] = 0;
		}

		protected double limit( double value, double lower, double upper )
		{
			return( DoubleFunctions.instance().limit(value, lower, upper) );
		}

		protected int getRgb( BufferedImage image, Point2d pt )
		{
			int xx = (int) limit( pt.x, 0, image.getWidth() - 1 );
			int yy = (int) limit( pt.y, 0, image.getHeight() - 1 );

			return( image.getRGB( xx, yy ) );
		}

		protected int calculateColor( Point2d pt1, Point2d pt2 )
		{
			resetArgbArray();

			ImageFunctions.instance().addARGB(_resultArgbArray,
											getRgb( _image1, pt1 ),
											1 - _fraction);
			ImageFunctions.instance().addARGB(_resultArgbArray,
											getRgb( _image2, pt2 ),
											_fraction);

			return( ImageFunctions.instance().getARGB(_resultArgbArray) );
		}

		public Point2d mapTrianglePt( Point2d pt,
			Point2d pt11, Point2d pt12, Point2d pt13,
			Point2d pt21, Point2d pt22, Point2d pt23 )
		{
			return( GeometryHelper.instance().mapTrianglePt(pt, pt11, pt12, pt13, pt21, pt22, pt23) );
		}

		public double[] getResultArgbArray() {
			return _resultArgbArray;
		}

		public void setResultArgbArray(double[] _resultArgbArray) {
			this._resultArgbArray = _resultArgbArray;
		}

		public GeometricRectangle getResultImageBounds() {
			return _resultImageBounds;
		}

		public void setResultImageBounds(GeometricRectangle _resultImageBounds) {
			this._resultImageBounds = _resultImageBounds;
		}

		public double getFraction() {
			return _fraction;
		}

		public void setFraction(double _fraction) {
			this._fraction = _fraction;
		}

		public BufferedImage getImage1() {
			return _image1;
		}

		public void setImage1(BufferedImage _image1) {
			this._image1 = _image1;
		}

		public BufferedImage getImage2() {
			return _image2;
		}

		public void setImage2(BufferedImage _image2) {
			this._image2 = _image2;
		}

		public BufferedImage getResultImage() {
			return _resultImage;
		}

		public void setResultImage(BufferedImage _resultImage) {
			this._resultImage = _resultImage;
		}

		public GeometricTriangle getTriangle1() {
			return _triangle1;
		}

		public void setTriangle1(GeometricTriangle _triangle1) {
			this._triangle1 = _triangle1;
		}

		public GeometricTriangle getTriangle2() {
			return _triangle2;
		}

		public void setTriangle2(GeometricTriangle _triangle2) {
			this._triangle2 = _triangle2;
		}

		public GeometricTriangle getResultTriangle() {
			return _resultTriangle;
		}

		public void setResultTriangle(GeometricTriangle _resultTriangle) {
			this._resultTriangle = _resultTriangle;
		}
	}
}
