/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.applications.libmorphing.language;

import com.frojasg1.applications.common.configuration.imp.InternationalizedStringConfImp;
import com.frojasg1.applications.libmorphing.api.facedetector.runnable.FaceDetectionQueryExecution;
import com.frojasg1.applications.libmorphing.controller.MorphingVerticesMapManagerBase;
import com.frojasg1.applications.libmorphing.view.painters.DelaunayTriangulationJPanelControllerBase;
import com.frojasg1.applications.libmorphing.view.painters.control.ControlDelaunayTriangulationJPanelController;
import com.frojasg1.applications.libmorphing.view.painters.delaunay.DelaunayTriangulationJPanelController;
import com.frojasg1.applications.libmorphing.view.painters.faceselector.FaceDetectionSelectorJPanelController;
import com.frojasg1.general.ExecutionFunctions;
/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class LibMorphingInternationalTexts extends InternationalizedStringConfImp {
	public static final String GLOBAL_CONF_FILE_NAME = "LibMorphingInternationalTexts.properties";


	protected static class LazyHolder
	{
		protected static final LibMorphingInternationalTexts INSTANCE = new LibMorphingInternationalTexts();
	}

	public static LibMorphingInternationalTexts instance()
	{
		return( LazyHolder.INSTANCE );
	}

	protected LibMorphingInternationalTexts()
	{
		super( GLOBAL_CONF_FILE_NAME, LibMorphingLanguageResources.sa_PROPERTIES_PATH_IN_JAR );

		registerInternationalizedStrings();

		ExecutionFunctions.instance().safeMethodExecution( () -> changeLanguage( getLanguage() ) );
	}

	protected void registerInternationalizedStrings()
	{
		registerInternationalString( MorphingVerticesMapManagerBase.CONF_COULD_NOT_CREATE_IMAGES, "Could not create $1 of the image models" );
		registerInternationalString(FaceDetectionSelectorJPanelController.CONF_CANCEL_FACE_DETECTION_SELECTION, "Do not select any face" );
		registerInternationalString(ControlDelaunayTriangulationJPanelController.CONF_RESET_MODIFIED_DETECTED_MESH, "Reset modified detected face mesh\n(it will affect to all projects that make use of that face)" );
		registerInternationalString(ControlDelaunayTriangulationJPanelController.CONF_RESET_FACE_MESH_FRAME, "Reset face mesh frame" );
		registerInternationalString(DelaunayTriangulationJPanelControllerBase.CONF_DELAUNAY_TRIANGULATION, "Triangulation layer" );
		registerInternationalString(DelaunayTriangulationJPanelControllerBase.CONF_FACE_DETECTION_SELECTION, "Face selection layer" );
		registerInternationalString(DelaunayTriangulationJPanelControllerBase.CONF_FACE_DETECTION_VERTEX_MODIFICATION, "Face detection vertex modification layer" );
		registerInternationalString(DelaunayTriangulationJPanelControllerBase.CONF_RUN_FACE_DETECTION, "Run face detection" );
		registerInternationalString(DelaunayTriangulationJPanelController.CONF_INSERT_68_DETECTED_POINTS, "Insert 68 + 12 detected points" );

		registerInternationalString(FaceDetectionQueryExecution.FACE_DETECTION_QUERY_SERVICE_NAME, "Face detection" );
		registerInternationalString(DelaunayTriangulationJPanelControllerBase.CONF_REAUTOCALCULATE_TRIANGULATION, "Reautocalculate triangulation" );
		registerInternationalString(DelaunayTriangulationJPanelControllerBase.CONF_REAUTOCALCULATE_ALL_TRIANGULATIONS, "Reautocalculate all triangulations" );
	}
}
