/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.applications.libmorphing.model.facedetector;

import com.frojasg1.applications.libmorphing.api.facedetector.helpers.FaceDetectionMeshDataBuilderAndUpdater;
import com.frojasg1.applications.libmorphing.api.facedetector.pojo.FaceDetectionMeshData;
import com.frojasg1.applications.libmorphing.api.facedetector.pojo.ImageFaceDetection;
import com.frojasg1.general.xml.model.KeyModelBase;

import java.util.List;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class ImageFaceDetectionPersistenceModel extends KeyModelBase<ImageFaceDetectionPersistenceModelKey>
{
	protected ImageFaceDetection imageFaceDetection;
	protected List<FaceDetectionMeshData> detectedFaceMeshes;

	public ImageFaceDetectionPersistenceModel(ImageFaceDetectionPersistenceModelKey key) {
		super(key);
	}

	public ImageFaceDetection getImageFaceDetection() {
		return imageFaceDetection;
	}

	public void setImageFaceDetection(ImageFaceDetection imageFaceDetection) {
		if(getImageFaceDetection() != imageFaceDetection) {
			updateDetectedFaceMeshes(imageFaceDetection);
		}
		this.imageFaceDetection = imageFaceDetection;
	}

	protected List<FaceDetectionMeshData> createDetectedFaceMeshes(ImageFaceDetection imageFaceDetection) {
		return FaceDetectionMeshDataBuilderAndUpdater.instance().createFaceDetectionMeshDataList(imageFaceDetection);
	}

	protected void updateDetectedFaceMeshes(ImageFaceDetection imageFaceDetection) {
		setDetectedFaceMeshesInternal(createDetectedFaceMeshes(imageFaceDetection));
	}

	protected void updateDetectedFaceMeshes(List<FaceDetectionMeshData> detectedFaceMeshes) {
		FaceDetectionMeshDataBuilderAndUpdater.instance().updateFaceDetectionMeshDataList(detectedFaceMeshes);
	}

	public List<FaceDetectionMeshData> getDetectedFaceMeshes() {
		return detectedFaceMeshes;
	}

	public void setDetectedFaceMeshes(List<FaceDetectionMeshData> detectedFaceMeshes) {
		if(getDetectedFaceMeshes() != detectedFaceMeshes) {
			updateDetectedFaceMeshes(detectedFaceMeshes);
		}
		setDetectedFaceMeshesInternal(detectedFaceMeshes);
	}

	protected void setDetectedFaceMeshesInternal(List<FaceDetectionMeshData> detectedFaceMeshes) {
		this.detectedFaceMeshes = detectedFaceMeshes;
	}
}
