/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.applications.libmorphing.model.facedetector;

import com.frojasg1.general.FileFunctions;
import com.frojasg1.general.HexadecimalFunctions;

import java.io.File;
import java.util.Objects;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class ImageFaceDetectionPersistenceModelKey {
	protected String fileName; // it is only descriptive. Normalized fileName
	protected long fileLength;
	protected String hexMd5;

	protected static String getHexMd5(String fileName) {
		return HexadecimalFunctions.instance().M_convertByteArrayToHexadecimalString(
				FileFunctions.instance().getMD5(FileFunctions.instance().convertFolderSeparator(fileName)));
	}

	public ImageFaceDetectionPersistenceModelKey(String fileName) {
		this(fileName, new File(fileName).length(), getHexMd5(fileName));
	}

	public ImageFaceDetectionPersistenceModelKey(String fileName, long fileLength, String hexMd5) {
		this.fileName = FileFunctions.instance().normalizeFolderSeparator(fileName);
		this.fileLength = fileLength;
		this.hexMd5 = hexMd5;
	}

	public long getFileLength() {
		return fileLength;
	}

	public String getHexMd5() {
		return hexMd5;
	}

	public String getFileName() {
		return fileName;
	}

	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		ImageFaceDetectionPersistenceModelKey that = (ImageFaceDetectionPersistenceModelKey) o;
		return fileLength == that.fileLength && Objects.equals(hexMd5, that.hexMd5);
	}

	@Override
	public int hashCode() {
		return Objects.hash(fileLength, hexMd5);
	}

	@Override
	public String toString() {
		return "ImageFaceDetectionConfigurationModelKey{" +
				"fileLength=" + fileLength +
				", hexMd5='" + hexMd5 + '\'' +
				'}';
	}
}
