/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.applications.libmorphing.model.facedetector;

import com.frojasg1.general.map.multimap.MultiMap;
import com.frojasg1.general.map.multimap.MultiMapMapEntry;

import java.util.HashMap;
import java.util.Map;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class ImageFaceDetectionPersistenceModelMap extends MultiMap<ImageFaceDetectionPersistenceModel>
{
	public static final String MAIN_MAP = "MAIN_MAP";

	public ImageFaceDetectionPersistenceModelMap()
	{
		this( true );
	}

	protected ImageFaceDetectionPersistenceModelMap(boolean init )
	{
		if( init )
			init();
	}

	@Override
	protected Map<String, MultiMapMapEntry<?, ImageFaceDetectionPersistenceModel>> fillMultiMapMapEntryMap() {
		Map<String, MultiMapMapEntry<?, ImageFaceDetectionPersistenceModel>> result = new HashMap<>();

		addMultiMapMapEntry( result, MAIN_MAP, ImageFaceDetectionPersistenceModelKey.class,
							ImageFaceDetectionPersistenceModel::getKey );

		return( result );
	}

	public ImageFaceDetectionPersistenceModelKey createKey(String fileName)
	{
		return new ImageFaceDetectionPersistenceModelKey(fileName);
	}

	public ImageFaceDetectionPersistenceModelKey createKey(String fileName, long fileLength, String hexMd5)
	{
		return new ImageFaceDetectionPersistenceModelKey(fileName, fileLength, hexMd5);
	}

	public ImageFaceDetectionPersistenceModel get(long fileLength, String hexMd5)
	{
		ImageFaceDetectionPersistenceModelKey key = createKey("", fileLength, hexMd5);

		return( get( MAIN_MAP, key ) );
	}

	public ImageFaceDetectionPersistenceModel get(String filename)
	{
		ImageFaceDetectionPersistenceModelKey key = createKey(filename);

		return( get( MAIN_MAP, key ) );
	}

	public Map<ImageFaceDetectionPersistenceModelKey, ImageFaceDetectionPersistenceModel> getMap() {
		return getMapByName(MAIN_MAP, ImageFaceDetectionPersistenceModelKey.class);
	}
}
