/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.applications.libmorphing.model.images.color;

import com.frojasg1.general.map.MapWrapper;
import java.util.Set;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class ColorsMap
{
	protected MapWrapper<String, ColorModel> _mapWrapper;

	public ColorsMap()
	{
		this( true );
	}

	public ColorsMap( boolean init )
	{
		super();

		if( init )
			init();
	}

	protected void init()
	{
		_mapWrapper = createMapWrapper();
	}

	protected <K, V> MapWrapper<K, V> createMapWrapper()
	{
		return( new MapWrapper<>() );
	}

	public ColorModel get( String key )
	{
		return( _mapWrapper.get(key) );
	}

	public void put( ColorModel value )
	{
		_mapWrapper.put(value.getName(), value);
	}

	public Set<String> keySet()
	{
		return( _mapWrapper.keySet() );
	}

	public ColorModel remove( String key )
	{
		return( _mapWrapper.remove(key) );
	}

	public void clear()
	{
		_mapWrapper.clear();
	}

	public int size()
	{
		return( _mapWrapper.size() );
	}
}
