/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.applications.libmorphing.model.images.color.types;

import com.frojasg1.libgeometry.algebra.helpers.VectorFunctions;
import com.frojasg1.applications.libmorphing.model.images.vertex.VertexModel;
import com.frojasg1.libgeometry.types.Point2d;
import com.frojasg1.general.map.multimap.dynamic.deductible.DeductibleDynamicMultiMapElementBase;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class MultiTriangle
{
	protected DeductibleDynamicMultiMapElementBase<VertexModel> _multiVertex1;
	protected DeductibleDynamicMultiMapElementBase<VertexModel> _multiVertex2;
	protected DeductibleDynamicMultiMapElementBase<VertexModel> _multiVertex3;

	public DeductibleDynamicMultiMapElementBase<VertexModel> getMultiVertex1() {
		return _multiVertex1;
	}

	public void setMultiVertex1(DeductibleDynamicMultiMapElementBase<VertexModel> _multiVertex1) {
		this._multiVertex1 = _multiVertex1;
	}

	public DeductibleDynamicMultiMapElementBase<VertexModel> getMultiVertex2() {
		return _multiVertex2;
	}

	public void setMultiVertex2(DeductibleDynamicMultiMapElementBase<VertexModel> _multiVertex2) {
		this._multiVertex2 = _multiVertex2;
	}

	public DeductibleDynamicMultiMapElementBase<VertexModel> getMultiVertex3() {
		return _multiVertex3;
	}

	public void setMultiVertex3(DeductibleDynamicMultiMapElementBase<VertexModel> _multiVertex3) {
		this._multiVertex3 = _multiVertex3;
	}

	public Point2d getVertex( DeductibleDynamicMultiMapElementBase<VertexModel> multiVertex,
							String imageName )
	{
		return( multiVertex.get( imageName ).getPoint() );
	}

	public Point2d getVertex1( String imageName ) {
		return( getVertex( getMultiVertex1(), imageName ) );
	}

	public Point2d getVertex2( String imageName ) {
		return( getVertex( getMultiVertex2(), imageName ) );
	}

	public Point2d getVertex3( String imageName ) {
		return( getVertex( getMultiVertex3(), imageName ) );
	}

	public Point2d getNormalizedBisector1( String imageName )
	{
		return( getNormalizedBisectorVector( getVertex1( imageName ),
											getVertex2( imageName ),
											getVertex3( imageName ) ) );
	}

	public Point2d getNormalizedBisector2( String imageName )
	{
		return( getNormalizedBisectorVector( getVertex2( imageName ),
											getVertex1( imageName ),
											getVertex3( imageName ) ) );
	}

	public Point2d getNormalizedBisector3( String imageName )
	{
		return( getNormalizedBisectorVector( getVertex3( imageName ),
											getVertex2( imageName ),
											getVertex1( imageName ) ) );
	}

	protected Point2d getNormalizedBisectorVector( Point2d p1, Point2d p2, Point2d p3 )
	{
		return( VectorFunctions.instance().getNormalizedBisectorVector(p1, p2, p3) );
	}
}
