/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.applications.libmorphing.model.images.configuration;

import com.frojasg1.applications.libmorphing.model.images.color.ColorModel;
import com.frojasg1.general.copy.CopyFacilities;
import com.frojasg1.general.getters.GetterDelegator;
import com.frojasg1.general.map.MapWrapper;
import java.util.Set;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class ColorsConfigurationModel
	extends GetterDelegator<ColorsConfigurationModel>
	implements CopyFacilities<ColorsConfigurationModel>
{
	public static final String EDGE = "edge";
	public static final String VERTEX = "vertex";
	public static final String SELECTED_VERTEX = "selectedvertex";
	public static final String ERRONEUS_VERTEX = "erroneusvertex";

	protected MapWrapper<String, ColorModel> _mapWrapper;

	public ColorsConfigurationModel(ColorsConfigurationModel parent)
	{
		this( parent, true );
	}

	public ColorsConfigurationModel( ColorsConfigurationModel parent, boolean init )
	{
		super(parent);

		if( init )
			init();
	}

	protected void init()
	{
		_mapWrapper = createMapWrapper();
	}

	@Override
	public ColorsConfigurationModel createCopy()
	{
		ColorsConfigurationModel result = new ColorsConfigurationModel( getParentToDelegateTo(), false );
		result.copy(this);

		return( result );
	}

	@Override
	public void copy( ColorsConfigurationModel that )
	{
		setParentToDelegateTo(that.getParentToDelegateTo());
		_mapWrapper = copy( _mapWrapper, that._mapWrapper );
	}

	protected <K, V> MapWrapper<K, V> createMapWrapper()
	{
		return( new MapWrapper<>() );
	}

	public ColorModel getInternal( String key )
	{
		return( _mapWrapper.get(key) );
	}

	public ColorModel get( String key ) {
		return getDelegating( obj -> obj.getInternal(key),
							obj -> obj.get(key) );
	}

	public void put( String key, ColorModel value )
	{
		_mapWrapper.put(key, value);
	}

	public Set<String> keySet()
	{
		return( _mapWrapper.keySet() );
	}

	public ColorModel remove( String key )
	{
		return( _mapWrapper.remove(key) );
	}

	public void clear()
	{
		_mapWrapper.clear();
	}

	public int size()
	{
		return( _mapWrapper.size() );
	}
}
