/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.applications.libmorphing.model.images.configuration;

import com.frojasg1.general.copy.CopyFacilities;
import com.frojasg1.general.getters.GetterDelegator;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class MeshAspectConfigurationModel
	extends GetterDelegator<MeshAspectConfigurationModel>
	implements CopyFacilities<MeshAspectConfigurationModel>
{
	protected Float _vertexRadius;
	protected Float _edgeWidth;

	protected Boolean _showMesh;
	protected Boolean _showVertices;
	protected Boolean _showContents;

	public MeshAspectConfigurationModel(MeshAspectConfigurationModel _parentToDelegateTo) {
		super(_parentToDelegateTo);
	}

	@Override
	public MeshAspectConfigurationModel createCopy()
	{
		MeshAspectConfigurationModel result = new MeshAspectConfigurationModel( getParentToDelegateTo() );
		result.copy(this);

		return( result );
	}

	@Override
	public void copy( MeshAspectConfigurationModel that )
	{
		setParentToDelegateTo(that.getParentToDelegateTo());
		_vertexRadius = that._vertexRadius;
		_edgeWidth = that._edgeWidth;
		_showMesh = that._showMesh;
		_showVertices = that._showVertices;
		_showContents = that._showContents;
	}


	public Float getVertexRadiusInternal() {
		return _vertexRadius;
	}

	public Float getVertexRadius() {
		return getDelegating(MeshAspectConfigurationModel::getVertexRadiusInternal,
							MeshAspectConfigurationModel::getVertexRadius);
	}

	public void setVertexRadius(Float _vertexRadius) {
		this._vertexRadius = _vertexRadius;
	}

	public Float getEdgeWidthInternal() {
		return _edgeWidth;
	}

	public Float getEdgeWidth() {
		return getDelegating(MeshAspectConfigurationModel::getEdgeWidthInternal,
							MeshAspectConfigurationModel::getEdgeWidth);
	}

	public void setEdgeWidth(Float _edgeWidth) {
		this._edgeWidth = _edgeWidth;
	}

	public Boolean getShowMeshInternal() {
		return _showMesh;
	}

	public Boolean getShowMesh() {
		return getDelegating(MeshAspectConfigurationModel::getShowMeshInternal,
							MeshAspectConfigurationModel::getShowMesh);
	}

	public void setShowMesh(Boolean _showMesh) {
		this._showMesh = _showMesh;
	}

	public Boolean getShowVerticesInternal() {
		return _showVertices;
	}

	public Boolean getShowVertices() {
		return getDelegating(MeshAspectConfigurationModel::getShowVerticesInternal,
							MeshAspectConfigurationModel::getShowVertices);
	}

	public void setShowVertices(Boolean _showVertices) {
		this._showVertices = _showVertices;
	}

	public Boolean getShowContentsInternal() {
		return _showContents;
	}

	public Boolean getShowContents() {
		return getDelegating(MeshAspectConfigurationModel::getShowContentsInternal,
							MeshAspectConfigurationModel::getShowContents);
	}

	public void setShowContents(Boolean _showContents) {
		this._showContents = _showContents;
	}
}
