/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.applications.libmorphing.model.images.configuration;

import com.frojasg1.general.copy.CopyFacilities;
import com.frojasg1.general.getters.GetterDelegator;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class MorphingProjectConfigurationModel
	extends GetterDelegator<MorphingProjectConfigurationModel>
	implements CopyFacilities<MorphingProjectConfigurationModel>
{

	protected SizeMode _sizeMode;
	protected Integer _transitionTimeMs;

	protected ColorsConfigurationModel _colorsConfiguration;
	protected MeshAspectConfigurationModel _meshAspectConfiguration;

	public MorphingProjectConfigurationModel( MorphingProjectConfigurationModel parent )
	{
		super( parent );
	}

	@Override
	public void copy( MorphingProjectConfigurationModel that )
	{
		setParentToDelegateTo(that.getParentToDelegateTo());
		_sizeMode = that._sizeMode;
		_transitionTimeMs = that._transitionTimeMs;
		_colorsConfiguration = copy( _colorsConfiguration, that._colorsConfiguration );
		_meshAspectConfiguration = copy( _meshAspectConfiguration,
										getIfNotNull( that._meshAspectConfiguration,
											MeshAspectConfigurationModel::createCopy ) );
	}

	@Override
	public MorphingProjectConfigurationModel createCopy()
	{
		MorphingProjectConfigurationModel result = new MorphingProjectConfigurationModel( getParentToDelegateTo() );
		result.copy(this);

		return( result );
	}


	public SizeMode getSizeModeInternal() {
		return _sizeMode;
	}

	public SizeMode getSizeMode() {
		return getDelegating(MorphingProjectConfigurationModel::getSizeModeInternal,
							MorphingProjectConfigurationModel::getSizeMode);
	}

	public void setSizeMode(SizeMode _sizeMode) {
		this._sizeMode = _sizeMode;
	}

	public Integer getTransitionTimeMsInternal() {
		return _transitionTimeMs;
	}

	public Integer getTransitionTimeMs() {
		return getDelegating(MorphingProjectConfigurationModel::getTransitionTimeMsInternal,
							MorphingProjectConfigurationModel::getTransitionTimeMs);
	}

	public void setTransitionTimeMs(Integer _transitionTimeMs) {
		this._transitionTimeMs = _transitionTimeMs;
	}

	public ColorsConfigurationModel getColorsConfigurationInternal() {
		return _colorsConfiguration;
	}

	public ColorsConfigurationModel getColorsConfiguration() {
		return getDelegating(MorphingProjectConfigurationModel::getColorsConfigurationInternal,
							MorphingProjectConfigurationModel::getColorsConfiguration);
	}

	public void setColorsConfiguration(ColorsConfigurationModel _colorsConfiguration) {
		this._colorsConfiguration = _colorsConfiguration;
	}

	public MeshAspectConfigurationModel getMeshAspectConfigurationInternal() {
		return _meshAspectConfiguration;
	}

	public MeshAspectConfigurationModel getMeshAspectConfiguration() {
		return getDelegating(MorphingProjectConfigurationModel::getMeshAspectConfigurationInternal,
							MorphingProjectConfigurationModel::getMeshAspectConfiguration);
	}

	public void setMeshAspectConfiguration(MeshAspectConfigurationModel _meshAspectConfiguration) {
		this._meshAspectConfiguration = _meshAspectConfiguration;
	}
}
