/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.applications.libmorphing.model.images.vertex.map.impl.control;

import com.frojasg1.applications.libmorphing.model.images.vertex.VertexModel;
import com.frojasg1.libgeometry.types.Point2d;
import com.frojasg1.general.map.multimap.dynamic.DynamicMultiMap;
import com.frojasg1.general.map.multimap.dynamic.DynamicMultiMapMapEntry;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Map;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class IndexedVerticesMap
		extends DynamicMultiMap<VertexModel,
		DynamicMultimapVertexWithIndex,
		Object>
{
	private static final Logger LOGGER = LoggerFactory.getLogger(IndexedVerticesMap.class);

	public static final String MAP_BY_INDEX = "__INDEX__";

	public IndexedVerticesMap() {
		super(VertexModel.class);
		init();
	}

	@Override
	protected Map<String, DynamicMultiMapMapEntry<VertexModel,
			DynamicMultimapVertexWithIndex,
			Object>> fillMultiMapMapEntryMap() {
		Map<String, DynamicMultiMapMapEntry<VertexModel,
				DynamicMultimapVertexWithIndex,
				Object>> mapByName = createMap();

		addMultiMapMapEntry(mapByName, MAP_BY_INDEX, Integer.class, DynamicMultimapVertexWithIndex::getIndex);

		return mapByName;
	}

	protected DynamicMultimapVertexWithIndex createEmptyMultiValue(int index) {
		DynamicMultimapVertexWithIndex result = new DynamicMultimapVertexWithIndex();
		result.setIndex(index);

		return(result);
	}

	public void addMapByName(String name)
	{
		DynamicMultiMapMapEntry<VertexModel,
				DynamicMultimapVertexWithIndex,
				Object> dmme = addMultiMapMapEntry(getMapByName(), name, getKeyClass(), elem -> elem.get(name));
	}

	public DynamicMultimapVertexWithIndex get(int index) {
		return get(MAP_BY_INDEX, index);
	}

	protected void remove(String mapName, DynamicMultimapVertexWithIndex elem) {
		VertexModel vm = elem.get(mapName);
		if(vm != null) {
			remove(mapName, vm.getPoint());
		}
	}

	public void set(String mapName, int index, VertexModel value) {
		DynamicMultimapVertexWithIndex elem = get(index);
//		VertexModel value = createElementValue(pt);
		if(elem == null) {
			elem = createEmptyMultiValue(index);
			elem.put(mapName, value);
			put(elem);
		} else {
			Map<VertexModel, DynamicMultimapVertexWithIndex> myParticularMap = getMapByName(mapName, getKeyClass());
			VertexModel oldKey = elem.get(mapName);
			if(oldKey != null) {
				myParticularMap.remove(oldKey);
			}
			elem.put(mapName, value);
			myParticularMap.put(value, elem);
		}
	}

//	public void set(String mapName, int index, Point2d pt) {
//		DynamicMultimapVertexWithIndex elem = get(index);
//		VertexModel value = createElementValue(pt);
//		if(elem == null) {
//			elem = createEmptyMultiValue(index);
//			elem.put(mapName, value);
//			put(elem);
//		} else {
//			Map<VertexModel, DynamicMultimapVertexWithIndex> myParticularMap = getMapByName(mapName, getKeyClass());
//			VertexModel oldKey = elem.get(mapName);
//			if(oldKey != null) {
//				myParticularMap.remove(oldKey);
//			}
//			elem.put(mapName, value);
//			myParticularMap.put(value, elem);
//		}
//	}

	protected VertexModel createElementValue( Point2d pt )
	{
		VertexModel result = new VertexModel(null);
		result.setPoint(pt);

		return( result );
	}

	public DynamicMultimapVertexWithIndex get( String mapName, Point2d pt )
	{
		return( get( mapName, createElementValue( pt ) ) );
	}

	public DynamicMultimapVertexWithIndex remove( String mapName, Point2d pt )
	{
		DynamicMultimapVertexWithIndex result = null;
		result = get( mapName, pt );
		if( result != null )
			remove( result );

		return( result );
	}

	protected boolean hasToPut(DynamicMultimapVertexWithIndex value, String mapName) {
		return MAP_BY_INDEX.equals(mapName) || (value.get(mapName) != null);
	}

	@Override
	public void put(DynamicMultimapVertexWithIndex value) {
		if (value != null) {
			String mapName;
			DynamicMultiMapMapEntry<VertexModel,
					DynamicMultimapVertexWithIndex,
					Object> dynamicEntryMap;
			for (Map.Entry<String, DynamicMultiMapMapEntry<VertexModel,
					DynamicMultimapVertexWithIndex,
					Object>> entry : mapByName.entrySet()) {
				dynamicEntryMap = entry.getValue();
				mapName = entry.getKey();
				if(hasToPut(value, mapName)) {
					entry.getValue().put(value);
				}
			}
		} else {
			throw new IllegalArgumentException("value was null, cannot put");
		}
	}
}
