/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.applications.libmorphing.model.images.vertex.map.impl.normal;

import com.frojasg1.applications.libmorphing.model.images.vertex.VertexModel;
import com.frojasg1.applications.libmorphing.model.images.vertex.map.MorphingVerticesMapMapEntryContext;
import com.frojasg1.libgeometry.types.Point2d;
import com.frojasg1.general.map.multimap.dynamic.DynamicMultiMapMapEntry;
import com.frojasg1.general.map.multimap.dynamic.deductible.DeductibleDynamicMultiMap;
import com.frojasg1.general.map.multimap.dynamic.deductible.DeductibleDynamicMultiMapElementBase;
import java.util.function.UnaryOperator;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class MorphingVerticesMap extends DeductibleDynamicMultiMap<VertexModel,
																	DeductibleDynamicMultiMapElementBase<VertexModel>,
		MorphingVerticesMapMapEntryContext>
{
	private static final Logger LOGGER = LoggerFactory.getLogger(MorphingVerticesMap.class);

	public MorphingVerticesMap(String masterName) {
		super(VertexModel.class, masterName);

		init();
	}

	protected UnaryOperator<VertexModel> getFromMasterDirectTranslator(DynamicMultiMapMapEntry<VertexModel,
																		DeductibleDynamicMultiMapElementBase<VertexModel>,
																		MorphingVerticesMapMapEntryContext> dmme) {
		return( dmme.getElementContext().getFromMasterDirectTranslator() );
	}

	@Override
	protected DeductibleDynamicMultiMapElementBase<VertexModel> createEmptyMultiValue() {
		return( new DeductibleDynamicMultiMapElementBase<>() );
	}

	@Override
	protected VertexModel initialTranslationFromMasterKey(DynamicMultiMapMapEntry<VertexModel,
													DeductibleDynamicMultiMapElementBase<VertexModel>,
													MorphingVerticesMapMapEntryContext> dmme,
													VertexModel masterElement,
													DeductibleDynamicMultiMapElementBase<VertexModel> multiElementValue) {
		return( getFromMasterDirectTranslator( dmme ).apply( masterElement ));
	}

	public VertexModel createElementValue( Point2d pt )
	{
		VertexModel result = new VertexModel(null);
		result.setPoint(pt);

		return( result );
	}

	public DeductibleDynamicMultiMapElementBase<VertexModel> getMasterElement( Point2d masterPt )
	{
		return( getMasterElement( createElementValue( masterPt ) ) );
	}

	public DeductibleDynamicMultiMapElementBase<VertexModel> get( String mapName, Point2d pt )
	{
		return( get( mapName, createElementValue( pt ) ) );
	}

	public DeductibleDynamicMultiMapElementBase<VertexModel> remove( String mapName, Point2d pt )
	{
		DeductibleDynamicMultiMapElementBase<VertexModel> result = null;
		result = get( mapName, pt );
		if( result != null )
			remove( result );

		return( result );
	}

	@Override
	protected void logInsertion( VertexModel masterElement,
								DeductibleDynamicMultiMapElementBase<VertexModel> result )
	{
		LOGGER.info( "{} added master element: {}  ---> {}", nowString(),
			masterElement, result.get(getMasterName()) );
	}
}
