/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.applications.libmorphing.mvc;

import com.frojasg1.applications.libmorphing.api.facedetector.pojo.FaceDetectionMeshData;
import com.frojasg1.applications.libmorphing.model.images.color.types.MultiTriangle;
import com.frojasg1.applications.libmorphing.view.painters.TriangulationPainterLayerTypeEnum;
import com.frojasg1.libgeometry.types.GeometricRectangle;
import com.frojasg1.libgeometry.types.Point2d;
import com.frojasg1.general.view.ViewComponent;
import com.frojasg1.general.view.evt.wrapper.GenericWrapperEvent;

import java.util.List;
import java.util.function.Consumer;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public interface DelaunayTriangulatorView extends ViewComponent
{
	public void addDelaunayTriangulatorViewListener( Consumer<? super GenericWrapperEvent> listener,
													Long ... eventTypes );
	public void removeDelaunayTriangulatorViewListener( Consumer<? super GenericWrapperEvent> listener );

	public void setPointInProgress( String imageName, Point2d pt );
	public void setClosestPoint( String imageName, Point2d pt );
	public void setControlMeshClosestPoint( String imageName, Point2d pt );

	public void updateTriangulation(String imageName,
//									MorphingVerticesMapManager morphingVerticesManager,
									Point2d normalizedCenterPoint,
									boolean madatoryUpdate,
									Double newImageZoomFactor);

	public void setNewOnTheFlyBounds( String imageName, GeometricRectangle newOnTheFlyBounds );

	public double getFadingFraction();
	public void fadeToMaster( double fadingFraction );
	public void resetFadedToMasterImage();
	public Point2d getNormalizedPointImageCenterShown();

	public void setSelectedMultiTriangle( MultiTriangle multiTriangle );
	public MultiTriangle getSelectedMultiTriangle();
	public void resetSelectedMultiTriangle();

	public double getImageZoomFactor();

	public void setAllFaces(String imageName, List<FaceDetectionMeshData> allFaces);
	public void setLayerType(String imageName, TriangulationPainterLayerTypeEnum layerType);
	public void setFaceDetectionMeshData(String imageName, FaceDetectionMeshData faceDetectionMeshData);
}
