/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.applications.libmorphing.view.newboundschooser;

import com.frojasg1.libgeometry.types.GeometricRectangle;
import com.frojasg1.generic.GenericFunctions;
import java.awt.Component;
import java.awt.Point;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class NewBoundsChooserController
	extends MouseAdapter
{
	protected NewBoundsChooserContext _context;
	protected Point _firstPoint;


	public NewBoundsChooserController( NewBoundsChooserContext context )
	{
		_context = context;
	}

	public NewBoundsChooserContext getContext() {
		return _context;
	}

	public Component getParent() {
		return getContext().getParent();
	}

	protected double getApplicationZoomFactor()
	{
		return( GenericFunctions.instance().getAppliConf().getZoomFactor() );
	}

	protected GeometricRectangle getBounds()
	{
		return( getContext().getBounds() );
	}

	protected Point viewToModel( Point pt )
	{
		return( getContext().viewToModel( pt ) );
	}

	@Override
    public void mouseMoved(MouseEvent e)
	{
		_firstPoint = null;
		setNewPoint( e );
	}

	@Override
	public void mouseExited(MouseEvent e)
	{
		setNewPoint( e );
	}

	@Override
	public void mousePressed(MouseEvent e)
	{
		Point imagePt = setNewPoint( e );
		if( imagePt != null )
			_firstPoint = imagePt;
	}

	@Override
	public void mouseReleased(MouseEvent e)
	{
		_firstPoint = null;
		setNewPoint( e );
	}

	@Override
	public void mouseDragged(MouseEvent e)
	{
		Point firstPoint = _firstPoint;
		if( firstPoint != null )
			getContext().updateBounds( firstPoint, viewToModel( e.getPoint() ) );

		setNewPoint( e );
	}

	protected Point setNewPoint( MouseEvent e )
	{
		Point result = viewToModel( e.getPoint() );
		if( !getContext().isModelPointInsideImage(result) )
			result = null;

		getContext().setPoint(result);

		return( result );
	}
}
