/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.applications.libmorphing.view.newboundschooser;

import com.frojasg1.applications.common.components.internationalization.window.InternationalizationInitializationEndCallback;
import com.frojasg1.applications.common.components.internationalization.window.InternationalizedJFrame;
import com.frojasg1.applications.libmorphing.model.images.image.MorphingImageModel;
import com.frojasg1.libgeometry.types.GeometricRectangle;
import java.util.function.Consumer;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class NewBoundsChooserLauncher {

	protected static NewBoundsChooserLauncher INSTANCE = new NewBoundsChooserLauncher();

	public static NewBoundsChooserLauncher instance()
	{
		return( INSTANCE );
	}

	public void askForANewBoundary( InternationalizedJFrame parent,
									MorphingImageModel mim,
									GeometricRectangle masterBounds,
									Consumer<GeometricRectangle> callback )
	{
		NewBoundsChooserJDialog dialog = new NewBoundsChooserJDialog( parent, true,
									parent.getAppliConf(),
									dial -> processDialog( dial, callback ),
									mim, masterBounds );
	}

	protected void processDialog( InternationalizationInitializationEndCallback dial,
								Consumer<GeometricRectangle> callback )
	{
		NewBoundsChooserJDialog dialog = (NewBoundsChooserJDialog) dial;
		dialog.setVisible(true);

		if( dialog.wasSuccessful() )
		{
			GeometricRectangle newBounds = dialog.getNewBounds();
			callback.accept(newBounds);
		}
	}
}
