/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.applications.libmorphing.view.newboundschooser;

import java.util.function.Predicate;
import javax.swing.SpinnerNumberModel;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class SpinnerNumberModelWithGenericLimits extends SpinnerNumberModel
{
	protected Predicate<Integer> _isValidFunction;

	public SpinnerNumberModelWithGenericLimits(int value, int minimum, int maximum, int stepSize,
											Predicate<Integer> isValidFunction) {
		super(value, minimum, maximum, stepSize);
		_isValidFunction = isValidFunction;
	}

	protected boolean isValid( Integer value )
	{
		return( ( _isValidFunction == null ) || _isValidFunction.test(value) );
	}

	@Override
	public void setValue( Object value )
	{
		if( ( value instanceof Integer ) && isValid( (Integer) value ) )
			super.setValue( value );
	}
}
