/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.applications.libmorphing.view.painters.control;

import com.frojasg1.applications.common.configuration.InternationalizedStringConf;
import com.frojasg1.applications.libmorphing.contexts.MorphingEditorApplicationContext;
import com.frojasg1.applications.libmorphing.view.painters.DelaunayTriangulationPanelWrapperLayerBase;
import com.frojasg1.applications.libmorphing.view.painters.TriangulationPainterLayerTypeEnum;
import com.frojasg1.applications.libmorphing.view.panels.evt.DelaunayJPanelsEventBuilderBase;
import com.frojasg1.delaunay.triangulation.quadedge.builder.QuadEdgeDelaunayBuilder;
import com.frojasg1.general.desktop.view.pdf.ImageJPanel;
import com.frojasg1.general.view.evt.wrapper.ListOfGenComponentWrapperEventContextListenersBase;
import com.frojasg1.general.view.evt.wrapper.impl.GenComponentWrapperEvent;

import javax.swing.*;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class ControlDelaunayTriangulationPanelWrapperLayer extends DelaunayTriangulationPanelWrapperLayerBase<ControlDelaunayTriangulationJPanelController, ControlDelaunayTriangulationPainter> {

	public ControlDelaunayTriangulationPanelWrapperLayer(MorphingEditorApplicationContext appliCtx,
														 ImageJPanel imageJPanel, TriangulationPainterLayerTypeEnum layerType,
//														 MorphingApplicationConfiguration appliConf,
														 QuadEdgeDelaunayBuilder delaunayBuilder,
														 InternationalizedStringConf internationalStrings,
														 ListOfGenComponentWrapperEventContextListenersBase<JPanel, GenComponentWrapperEvent> listenersList,
														 DelaunayJPanelsEventBuilderBase eventBuilder) {
		super(appliCtx, imageJPanel, layerType,// appliConf,
				delaunayBuilder, internationalStrings, listenersList, eventBuilder);
	}

	@Override
	public ControlDelaunayTriangulationPanelWrapperLayer init()
	{
		super.init();

		return this;
	}

	@Override
	protected ControlDelaunayTriangulationPainter createPainter()
	{
		return( new ControlDelaunayTriangulationPainter( getAppliCtx(), getImageJPanel(), getBuilder()) );
	}

	@Override
	protected void addListeners()
	{
		getImageJPanel().addMouseListener(_delaunayListeners);
		getImageJPanel().addMouseMotionListener(_delaunayListeners);
//		getImageJPanel().addMouseWheelListener(_delaunayListeners);
		getViewPort().addMouseMotionListener(_delaunayListeners);
	}

	@Override
	protected void removeListeners()
	{
		getImageJPanel().removeMouseListener(_delaunayListeners);
		getImageJPanel().removeMouseMotionListener(_delaunayListeners);
//		getImageJPanel().removeMouseWheelListener(_delaunayListeners);
		getViewPort().removeMouseMotionListener(_delaunayListeners);
	}

	@Override
	protected ControlDelaunayTriangulationJPanelController createDelaunayListeners()
	{
		return( new ControlDelaunayTriangulationJPanelController( getImageJPanel(), getLayerType(),
				null,
				getEventBuilder(),
				getListenersList(),
				getBuilder(),
				getInternationalStrings())
				.init());
	}

//	public void setTriangulation(String imageName,
//								MorphingVerticesMapManager morphingVerticesManager,
//								Point2d normalizedCenterPoint,
//								Double newImageZoomFactor)
//	{
//		DoubleReference zoomFactorRef = getImageJPanel().getZoomFactor();
//		MorphingImageModel imageModel = getImageModel( morphingVerticesManager, imageName );
//		BufferedImage image = ( imageModel == null ) ? null : imageModel.getImage();
//////		getImageJPanel().setNotZoomedImage(image, zoomFactorRef,
//////										InitialVerticalPosition.NOTHING);
////		if(isActive()) {
////			getImageJPanel().setImageToBePrezoomed(image,
////					previousCenter(normalizedCenterPoint, imageModel),
////					newImageZoomFactor);
////		}
//
//		getPainter().setTriangulation(imageName, morphingVerticesManager);
//		getDelaunayListeners().setImageModel(imageModel);
//	}
}
