/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.applications.libmorphing.view.painters.delaunay;

import com.frojasg1.applications.common.configuration.InternationalizedStringConf;
import com.frojasg1.applications.libmorphing.contexts.MorphingEditorApplicationContext;
import com.frojasg1.applications.libmorphing.model.images.color.types.MultiTriangle;
import com.frojasg1.applications.libmorphing.view.painters.DelaunayTriangulationPanelWrapperLayerBase;
import com.frojasg1.applications.libmorphing.view.painters.TriangulationPainterLayerTypeEnum;
import com.frojasg1.applications.libmorphing.view.panels.evt.DelaunayJPanelsEventBuilderBase;
import com.frojasg1.delaunay.triangulation.quadedge.builder.QuadEdgeDelaunayBuilder;
import com.frojasg1.libgeometry.types.GeometricRectangle;
import com.frojasg1.libgeometry.types.Point2d;
import com.frojasg1.general.view.evt.wrapper.ListOfGenComponentWrapperEventContextListenersBase;
import com.frojasg1.general.view.evt.wrapper.impl.GenComponentWrapperEvent;
import com.frojasg1.general.desktop.view.pdf.ImageJPanel;

import javax.swing.JPanel;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class DelaunayTriangulationPanelWrapperLayer extends DelaunayTriangulationPanelWrapperLayerBase<DelaunayTriangulationJPanelController, DelaunayTriangulationPainter> {

	public DelaunayTriangulationPanelWrapperLayer(MorphingEditorApplicationContext appliCtx,
												  ImageJPanel imageJPanel, TriangulationPainterLayerTypeEnum layerType,
//												  MorphingApplicationConfiguration appliConf,
												  QuadEdgeDelaunayBuilder delaunayBuilder,
												  InternationalizedStringConf internationalStrings,
												  ListOfGenComponentWrapperEventContextListenersBase<JPanel, GenComponentWrapperEvent> listenersList,
												  DelaunayJPanelsEventBuilderBase eventBuilder) {
		super(appliCtx, imageJPanel, layerType,// appliConf,
				delaunayBuilder, internationalStrings, listenersList, eventBuilder);
	}

	@Override
	public DelaunayTriangulationPanelWrapperLayer init()
	{
		super.init();
		return this;
	}

	@Override
	protected DelaunayTriangulationPainter createPainter()
	{
		return( new DelaunayTriangulationPainter( getAppliCtx(), getImageJPanel(), getBuilder()) );
	}

	@Override
	protected void addListeners()
	{
		addKeyListeners(_delaunayListeners);
		getImageJPanel().addMouseListener(_delaunayListeners);
		getImageJPanel().addMouseMotionListener(_delaunayListeners);
		getImageJPanel().addMouseWheelListener(_delaunayListeners);
		getViewPort().addMouseMotionListener(_delaunayListeners);
	}

	@Override
	protected void removeListeners()
	{
		removeKeyListeners(_delaunayListeners);
		getImageJPanel().removeMouseListener(_delaunayListeners);
		getImageJPanel().removeMouseMotionListener(_delaunayListeners);
		getImageJPanel().removeMouseWheelListener(_delaunayListeners);
		getViewPort().removeMouseMotionListener(_delaunayListeners);
	}

	@Override
	protected DelaunayTriangulationJPanelController createDelaunayListeners()
	{
		return( new DelaunayTriangulationJPanelController( getImageJPanel(), getLayerType(),
				null,
				getEventBuilder(),
				getListenersList(),
				getBuilder(),
				getInternationalStrings())
				.init());
	}

//	public void setTriangulation(String imageName,
//								MorphingVerticesMapManager morphingVerticesManager,
//								Point2d normalizedCenterPoint,
//								Double newImageZoomFactor)
//	{
//		DoubleReference zoomFactorRef = getImageJPanel().getZoomFactor();
//		MorphingImageModel imageModel = getImageModel( morphingVerticesManager, imageName );
//		BufferedImage image = ( imageModel == null ) ? null : imageModel.getImage();
////		getImageJPanel().setNotZoomedImage(image, zoomFactorRef,
////										InitialVerticalPosition.NOTHING);
//		getImageJPanel().setImageToBePrezoomed(image,
//				previousCenter( normalizedCenterPoint, imageModel ),
//				newImageZoomFactor );
//
//		getPainter().setTriangulation(imageName, morphingVerticesManager);
//		getDelaunayListeners().setImageModel(imageModel);
//	}

	public void setNewOnTheFlyBounds( String imageName, GeometricRectangle bounds )
	{
		getPainter().setNewOnTheFlyBounds( imageName, bounds );
	}

	public void fadeToMaster(double fadingFraction)
	{
		getPainter().fadeToMaster(fadingFraction);
	}

	public void resetFadedToMasterImage()
	{
		getPainter().resetFadedToMasterImage();
	}

	public Point2d getNormalizedPointImageCenterShown()
	{
		return( getPainter().getNormalizedPointImageCenterShown() );
	}	

	public void setSelectedMultiTriangle( MultiTriangle multiTriangle )
	{
		getPainter().setSelectedMultiTriangle(multiTriangle);

		getImageJPanel().repaint();
	}

	public MultiTriangle getSelectedMultiTriangle()
	{
		return( getPainter().getSelectedMultiTriangle() );
	}

	public void resetSelectedMultiTriangle()
	{
		getPainter().resetSelectedMultiTriangle();
	}
}
