/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.applications.libmorphing.view.panels;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import com.frojasg1.applications.common.components.resizecomp.MapResizeRelocateComponentItem;
import com.frojasg1.applications.common.components.resizecomp.ResizeRelocateItem;
import com.frojasg1.applications.common.configuration.application.BaseApplicationConfigurationInterface;
import com.frojasg1.applications.libmorphing.configuration.MorphingApplicationConfiguration;
import com.frojasg1.applications.libmorphing.frames.framebuilder.impl.MorphingTransition;
import com.frojasg1.applications.libmorphing.view.panels.evt.ChangeFollowingTransitionEvt;
import com.frojasg1.general.desktop.view.combobox.utils.ComboBoxFunctions;
import com.frojasg1.general.desktop.view.panels.InformerInterface;
import com.frojasg1.general.desktop.view.panels.NavigatorControllerInterface;
import com.frojasg1.general.desktop.view.panels.NavigatorJPanel;
import com.frojasg1.general.desktop.view.zoom.mapper.ComponentMapper;
import com.frojasg1.general.desktop.view.zoom.mapper.ComposedComponent;
import com.frojasg1.general.number.DoubleReference;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.util.List;
import java.util.Vector;
import javax.swing.DefaultComboBoxModel;
import com.frojasg1.applications.libmorphing.view.api.DelaunayViewerControlView;
import com.frojasg1.applications.libmorphing.view.panels.evt.ClearTriangulationEvent;
import com.frojasg1.applications.libmorphing.view.panels.evt.DelaunayJPanelsEventBuilderBase;
import com.frojasg1.applications.libmorphing.view.panels.evt.FadedMasterImageButtonPressedEvt;
import com.frojasg1.applications.libmorphing.view.panels.evt.MakeMasterImageEvt;
import com.frojasg1.applications.libmorphing.view.panels.evt.NavigatorEvt;
import com.frojasg1.applications.libmorphing.view.panels.evt.NavigatorEvtType;
import com.frojasg1.applications.libmorphing.view.panels.evt.NewBoundaryButtonPressedEvt;
import com.frojasg1.applications.libmorphing.view.panels.evt.ZoomEvt;
import com.frojasg1.general.view.evt.wrapper.ListOfGenComponentWrapperEventContextListenersBase;
import com.frojasg1.general.view.evt.wrapper.impl.GenComponentWrapperEvent;
import com.frojasg1.general.view.evt.wrapper.impl.GenComponentWrapperEventBuilder;
import com.frojasg1.general.desktop.view.panels.evt.CustomJPanelBaseWithJComponentEvt;
import javax.swing.JPanel;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class DelaunayViewerControlPanel extends CustomJPanelBaseWithJComponentEvt
	implements ComposedComponent, DelaunayViewerControlView,
				NavigatorControllerInterface, InformerInterface
{
//	protected PdfViewerMaster _controller = null;

	protected NavigatorJPanel _navigatorPanel = null;

	protected boolean _flagComboBoxItemSelectedByProgram = false;

	protected MapResizeRelocateComponentItem _resizeRelocateInfo = null;

	protected double _previousZoomFactor = 1.0d;
	protected String _currentImageName;

	public DelaunayViewerControlPanel(MorphingApplicationConfiguration appliConf,
									  ListOfGenComponentWrapperEventContextListenersBase<JPanel, GenComponentWrapperEvent> listenersList,
									  DelaunayJPanelsEventBuilderBase eventBuilder)
	{
		this( appliConf, listenersList, eventBuilder, true );
	}

	/**
	 * Creates new form PdfViewerControlPanel
	 */
	protected DelaunayViewerControlPanel(MorphingApplicationConfiguration appliConf,
									ListOfGenComponentWrapperEventContextListenersBase<JPanel, GenComponentWrapperEvent> listenersList,
									DelaunayJPanelsEventBuilderBase eventBuilder,
									boolean init) {
		super(appliConf, false, false);

		_listenersList = listenersList;
		_eventBuilder = eventBuilder;
		if( init )
			init();
	}

	@Override
	protected void init( ) //PdfViewerMaster controller )
	{
		super.init();

//		_controller = controller;

		initComponents();

		initPanels();

		setWindowConfiguration();
	}

	protected GenComponentWrapperEventBuilder<GenComponentWrapperEvent> createEventBuilder()
	{
		return( getEventBuilder() );
	}

	protected ListOfGenComponentWrapperEventContextListenersBase<JPanel, GenComponentWrapperEvent>
		createListenersList()
	{
		return( getListenersList() );
	}

	protected void initPanels()
	{
		try
		{
//			SwingUtilities.invokeLater( () -> {

				_navigatorPanel = new NavigatorJPanel( this );
				_navigatorPanel.setBounds(0, 0, 220, 50);
				_navigatorPanel.setBorder( jPanel3.getBorder() );
				jPanel3.add( _navigatorPanel );
//			});
		}
		catch( Throwable th )
		{
			th.printStackTrace();
		}
	}

	protected void setWindowConfiguration( )
	{
		_resizeRelocateInfo = new MapResizeRelocateComponentItem();
		MapResizeRelocateComponentItem mapRRCI = _resizeRelocateInfo;
		try
		{
			mapRRCI.putResizeRelocateComponentItem( jPanel1, ResizeRelocateItem.FILL_WHOLE_PARENT );
			mapRRCI.putResizeRelocateComponentItem( jPanel3, 0 );

			mapRRCI.putResizeRelocateComponentItem( _navigatorPanel, ResizeRelocateItem.RESIZE_TO_RIGHT +
																	ResizeRelocateItem.RESIZE_TO_BOTTOM );
		}
		catch( Throwable th )
		{
			th.printStackTrace();
		}

//		ExecutionFunctions.instance().safeMethodExecution( () -> registerInternationalizedStrings() );
	}

    private void jCBTransition(ActionEvent e) {
        // TODO add your code here

		notifyEvt(createChangeFollowingTransitionEvt());
    }

	@Override
	public MapResizeRelocateComponentItem getResizeRelocateInfo()
	{
		return( _resizeRelocateInfo );
	}

	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	@SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    // Generated using JFormDesigner non-commercial license
    private void initComponents() {
        jPanel1 = new JPanel();
        jLZoomFactor = new JLabel();
        jCBZoomFactor = new JComboBox();
        jPanel3 = new JPanel();
        jB_boundary = new JButton();
        jB_clear = new JButton();
        jB_makeMasterImage = new JButton();
        jCB_isMaster = new JCheckBox();
        jP_FadeToMaster = new JPanel();
        jB_Apply = new JButton();
        jS_fadingFraction = new JSlider();
        jCBTransition = new JComboBox();
        jL_followingTransition = new JLabel();

        //======== this ========
        setLayout(null);

        //======== jPanel1 ========
        {
            jPanel1.setName("jPanel1");
            jPanel1.setPreferredSize(new Dimension(510, 50));
            jPanel1.setLayout(null);

            //---- jLZoomFactor ----
            jLZoomFactor.setText("Zoom factor:");
            jLZoomFactor.setName("jLZoomFactor");
            jPanel1.add(jLZoomFactor);
            jLZoomFactor.setBounds(10, 10, 100, jLZoomFactor.getPreferredSize().height);

            //---- jCBZoomFactor ----
            jCBZoomFactor.setMinimumSize(new Dimension(40, 15));
            jCBZoomFactor.setName("jCBZoomFactor");
            jCBZoomFactor.setPreferredSize(new Dimension(33, 20));
            jCBZoomFactor.addActionListener(e -> jCBZoomFactorActionPerformed(e));
            jPanel1.add(jCBZoomFactor);
            jCBZoomFactor.setBounds(10, 30, 90, jCBZoomFactor.getPreferredSize().height);

            //======== jPanel3 ========
            {
                jPanel3.setBorder(LineBorder.createBlackLineBorder());
                jPanel3.setName("jPanel3_test");
                jPanel3.setLayout(null);

                {
                    // compute preferred size
                    Dimension preferredSize = new Dimension();
                    for(int i = 0; i < jPanel3.getComponentCount(); i++) {
                        Rectangle bounds = jPanel3.getComponent(i).getBounds();
                        preferredSize.width = Math.max(bounds.x + bounds.width, preferredSize.width);
                        preferredSize.height = Math.max(bounds.y + bounds.height, preferredSize.height);
                    }
                    Insets insets = jPanel3.getInsets();
                    preferredSize.width += insets.right;
                    preferredSize.height += insets.bottom;
                    jPanel3.setMinimumSize(preferredSize);
                    jPanel3.setPreferredSize(preferredSize);
                }
            }
            jPanel1.add(jPanel3);
            jPanel3.setBounds(115, 10, 220, 50);

            //---- jB_boundary ----
            jB_boundary.setMaximumSize(new Dimension(1000, 1000));
            jB_boundary.setMinimumSize(new Dimension(20, 20));
            jB_boundary.setName("name=jB_boundary,icon=com/frojasg1/generic/resources/othericons/boundary.png");
            jB_boundary.setPreferredSize(new Dimension(35, 35));
            jB_boundary.addActionListener(e -> jB_boundaryActionPerformed(e));
            jPanel1.add(jB_boundary);
            jB_boundary.setBounds(360, 70, 35, 35);

            //---- jB_clear ----
            jB_clear.setMaximumSize(new Dimension(1000, 1000));
            jB_clear.setMinimumSize(new Dimension(20, 20));
            jB_clear.setName("name=jB_clear,icon=com/frojasg1/generic/resources/othericons/clear.png");
            jB_clear.setPreferredSize(new Dimension(35, 35));
            jB_clear.addActionListener(e -> jB_clearActionPerformed(e));
            jPanel1.add(jB_clear);
            jB_clear.setBounds(410, 70, 35, 35);

            //---- jB_makeMasterImage ----
            jB_makeMasterImage.setText("Make master image");
            jB_makeMasterImage.setName("jB_makeMasterImage");
            jB_makeMasterImage.addActionListener(e -> jB_makeMasterImageActionPerformed(e));
            jPanel1.add(jB_makeMasterImage);
            jB_makeMasterImage.setBounds(360, 30, 175, jB_makeMasterImage.getPreferredSize().height);

            //---- jCB_isMaster ----
            jCB_isMaster.setText("is master");
            jCB_isMaster.setEnabled(false);
            jCB_isMaster.setName("jCB_isMaster");
            jPanel1.add(jCB_isMaster);
            jCB_isMaster.setBounds(360, 5, 175, jCB_isMaster.getPreferredSize().height);

            //======== jP_FadeToMaster ========
            {
                jP_FadeToMaster.setBorder(new TitledBorder(LineBorder.createBlackLineBorder(), "Fade to master"));
                jP_FadeToMaster.setName("jP_FadeToMaster");
                jP_FadeToMaster.setLayout(null);

                //---- jB_Apply ----
                jB_Apply.setText("Apply");
                jB_Apply.setName("jB_Apply");
                jB_Apply.addActionListener(e -> jB_ApplyActionPerformed(e));
                jP_FadeToMaster.add(jB_Apply);
                jB_Apply.setBounds(250, 15, jB_Apply.getPreferredSize().width, 30);
                jP_FadeToMaster.add(jS_fadingFraction);
                jS_fadingFraction.setBounds(5, 25, 245, jS_fadingFraction.getPreferredSize().height);

                {
                    // compute preferred size
                    Dimension preferredSize = new Dimension();
                    for(int i = 0; i < jP_FadeToMaster.getComponentCount(); i++) {
                        Rectangle bounds = jP_FadeToMaster.getComponent(i).getBounds();
                        preferredSize.width = Math.max(bounds.x + bounds.width, preferredSize.width);
                        preferredSize.height = Math.max(bounds.y + bounds.height, preferredSize.height);
                    }
                    Insets insets = jP_FadeToMaster.getInsets();
                    preferredSize.width += insets.right;
                    preferredSize.height += insets.bottom;
                    jP_FadeToMaster.setMinimumSize(preferredSize);
                    jP_FadeToMaster.setPreferredSize(preferredSize);
                }
            }
            jPanel1.add(jP_FadeToMaster);
            jP_FadeToMaster.setBounds(10, 60, 325, 50);

            //---- jCBTransition ----
            jCBTransition.setMinimumSize(new Dimension(84, 20));
            jCBTransition.setPreferredSize(new Dimension(79, 20));
            jCBTransition.addActionListener(e -> jCBTransition(e));
            jPanel1.add(jCBTransition);
            jCBTransition.setBounds(360, 115, 175, jCBTransition.getPreferredSize().height);

            //---- jL_followingTransition ----
            jL_followingTransition.setText("Following transition:");
            jL_followingTransition.setName("jL_followingTransition");
            jL_followingTransition.setHorizontalAlignment(SwingConstants.TRAILING);
            jPanel1.add(jL_followingTransition);
            jL_followingTransition.setBounds(188, 117, 170, jL_followingTransition.getPreferredSize().height);

            {
                // compute preferred size
                Dimension preferredSize = new Dimension();
                for(int i = 0; i < jPanel1.getComponentCount(); i++) {
                    Rectangle bounds = jPanel1.getComponent(i).getBounds();
                    preferredSize.width = Math.max(bounds.x + bounds.width, preferredSize.width);
                    preferredSize.height = Math.max(bounds.y + bounds.height, preferredSize.height);
                }
                Insets insets = jPanel1.getInsets();
                preferredSize.width += insets.right;
                preferredSize.height += insets.bottom;
                jPanel1.setMinimumSize(preferredSize);
                jPanel1.setPreferredSize(preferredSize);
            }
        }
        add(jPanel1);
        jPanel1.setBounds(0, 0, 550, 145);

        {
            // compute preferred size
            Dimension preferredSize = new Dimension();
            for(int i = 0; i < getComponentCount(); i++) {
                Rectangle bounds = getComponent(i).getBounds();
                preferredSize.width = Math.max(bounds.x + bounds.width, preferredSize.width);
                preferredSize.height = Math.max(bounds.y + bounds.height, preferredSize.height);
            }
            Insets insets = getInsets();
            preferredSize.width += insets.right;
            preferredSize.height += insets.bottom;
            setMinimumSize(preferredSize);
            setPreferredSize(preferredSize);
        }
    }// </editor-fold>//GEN-END:initComponents

    private void jCBZoomFactorActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jCBZoomFactorActionPerformed
        // TODO add your handling code here:

        if( ! _flagComboBoxItemSelectedByProgram )
        {
			
            DoubleReference newFactor = getSelectedZoomFactor();
			this.notifyEvt( createZoomEvt( _previousZoomFactor, newFactor ) );
        }
		
		_previousZoomFactor = getSelectedZoomFactorDouble();
    }//GEN-LAST:event_jCBZoomFactorActionPerformed

    private void jB_boundaryActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jB_boundaryActionPerformed
        // TODO add your handling code here:

		notifyEvt( createNewBoundaryButtonPressedEvt() );
    }//GEN-LAST:event_jB_boundaryActionPerformed

    private void jB_clearActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jB_clearActionPerformed
        // TODO add your handling code here:

		notifyEvt( createClearTriangulationEventData() );
    }//GEN-LAST:event_jB_clearActionPerformed

    private void jB_ApplyActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jB_ApplyActionPerformed
        // TODO add your handling code here:

		notifyEvt( createFadedMasterImageButtonPressedEvt() );
    }//GEN-LAST:event_jB_ApplyActionPerformed

    private void jB_makeMasterImageActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jB_makeMasterImageActionPerformed
        // TODO add your handling code here:

		notifyEvt( createMakeMasterImageEvt() );
    }//GEN-LAST:event_jB_makeMasterImageActionPerformed


    // Variables declaration - do not modify//GEN-BEGIN:variables
    // Generated using JFormDesigner non-commercial license
    private JPanel jPanel1;
    private JLabel jLZoomFactor;
    private JComboBox jCBZoomFactor;
    private JPanel jPanel3;
    private JButton jB_boundary;
    private JButton jB_clear;
    private JButton jB_makeMasterImage;
    private JCheckBox jCB_isMaster;
    private JPanel jP_FadeToMaster;
    private JButton jB_Apply;
    private JSlider jS_fadingFraction;
    private JComboBox jCBTransition;
    private JLabel jL_followingTransition;
    // End of variables declaration//GEN-END:variables

	protected double getPreviousZoomFactor()
	{
		return( _previousZoomFactor );
	}

	@Override
	public DoubleReference getSelectedZoomFactor()
	{
        DoubleReference newFactor = null;
        Object obj = jCBZoomFactor.getSelectedItem();
        if( obj instanceof DoubleReference )
        {
            newFactor = (DoubleReference) obj;
        }
		return( newFactor );
	}

	protected double getSelectedZoomFactorDouble()
	{
		double factor = 1.0D;

		DoubleReference dr = getSelectedZoomFactor();
		if( dr != null )
			factor = dr._value;

		return( factor );
	}

	@Override
	public void updateZoomComboBoxValues( List<DoubleReference> listOfZoomFactors )
	{
		if( listOfZoomFactors != null )
		{
			DefaultComboBoxModel<DoubleReference> dcbm = new DefaultComboBoxModel<DoubleReference>( new Vector<DoubleReference>( listOfZoomFactors ) );
			jCBZoomFactor.setModel( dcbm );
		}
	}
/*
	@Override
	public void updateCurrentPageTexts( int currentPage, int numPages )
	{
		jTF_currentPage.setText( String.valueOf( currentPage ) );
		jTFTotalPages.setText( String.valueOf( numPages ) );
	}
*/
	@Override
	public void setZoomSelectedItem(DoubleReference selectedItem) {
		_flagComboBoxItemSelectedByProgram = true;
		jCBZoomFactor.setSelectedItem( selectedItem );
		_flagComboBoxItemSelectedByProgram = false;
	}

	@Override
	public Dimension getInternalSize()
	{
		return( jPanel1.getSize() );
	}

	@Override
	public void setComponentMapper(ComponentMapper compMapper)
	{
		jCBZoomFactor = compMapper.mapComponent( jCBZoomFactor );
		jLZoomFactor = compMapper.mapComponent( jLZoomFactor );
		jPanel1 = compMapper.mapComponent( jPanel1 );
		jPanel3 = compMapper.mapComponent( jPanel3 );
		jCBTransition = compMapper.mapComponent(jCBTransition);
		jL_followingTransition = compMapper.mapComponent(jL_followingTransition);

		super.setComponentMapper(compMapper);

		fillTransitionCombo();
	}

	protected void fillTransitionCombo() {
		ComboBoxFunctions.instance().fillComboBoxGen(jCBTransition, MorphingTransition.NORMAL,
				MorphingTransition.NORMAL, MorphingTransition.FACE_DETECTION);
	}

	@Override
	public void navigator_start(InformerInterface panel)
	{
		notifyNavigatorEvt( NavigatorEvtType.START );
	}

	@Override
	public void navigator_end(InformerInterface panel)
	{
		notifyNavigatorEvt( NavigatorEvtType.END );
	}

	@Override
	public void navigator_previous(InformerInterface panel)
	{
		notifyNavigatorEvt( NavigatorEvtType.PREVIOUS );
	}

	@Override
	public void navigator_next(InformerInterface panel)
	{
		notifyNavigatorEvt( NavigatorEvtType.NEXT );
	}

	protected void notifyNavigatorEvt( NavigatorEvtType type )
	{
		notifyEvt( createNavigatorEvt( type ) );
	}

	public DelaunayJPanelsEventBuilderBase getEventBuilder() {
		return ( (DelaunayJPanelsEventBuilderBase) super.getEventBuilder() );
	}

	protected NavigatorEvt createNavigatorEvt( NavigatorEvtType type )
	{
		return( getEventBuilder().createNavigatorEvt(type) );
	}

	protected NewBoundaryButtonPressedEvt createNewBoundaryButtonPressedEvt()
	{
		return( getEventBuilder().createNewBoundaryButtonPressedEvt() );
	}

	protected ClearTriangulationEvent createClearTriangulationEventData()
	{
		return( getEventBuilder().createClearTriangulationEvent() );
	}

	protected FadedMasterImageButtonPressedEvt createFadedMasterImageButtonPressedEvt()
	{
		return( getEventBuilder().createFadedMasterImageButtonPressedEvt( getFadingFraction() ) );
	}	

	protected MakeMasterImageEvt createMakeMasterImageEvt()
	{
		return( getEventBuilder().createMakeMasterImageEvt( getCurrentImageName() ) );
	}	

	protected ZoomEvt createZoomEvt( double previousZoomFactor, DoubleReference newZoomFactor )
	{
		return( getEventBuilder().createZoomEvt(previousZoomFactor, newZoomFactor) );
	}

	protected ChangeFollowingTransitionEvt createChangeFollowingTransitionEvt() {
		return getEventBuilder().createChangeFollowingTransitionEvt(getCurrentImageName(), getTransition());
	}

	@Override
	public Rectangle getInternalBounds()
	{
		return( jPanel1.getBounds() );
	}

	public double getFadingFraction()
	{
		return( ((double) jS_fadingFraction.getValue() ) / (jS_fadingFraction.getMaximum() - jS_fadingFraction.getMinimum() ) );
	}

	public String getCurrentImageName() {
		return _currentImageName;
	}

	public void setCurrentImageData(String currentImageName,
									boolean isMaster ) {
		this._currentImageName = currentImageName;
		setCurrentImageIsMaster( isMaster );
	}

	public void setMorphingTransition(MorphingTransition morphingTransition, boolean enabled) {
		jCBTransition.setSelectedItem(morphingTransition);
		jCBTransition.setEnabled(enabled);
	}

	protected void setCurrentImageIsMaster( boolean isMaster )
	{
		jCB_isMaster.setSelected(isMaster);
	}

	protected MorphingTransition getTransition() {
		return (MorphingTransition) jCBTransition.getSelectedItem();
	}
}
