/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.applications.libmorphing.view.panels.evt;

import com.frojasg1.applications.libmorphing.api.facedetector.pojo.FaceDetectionMeshData;
import com.frojasg1.applications.libmorphing.frames.framebuilder.impl.MorphingTransition;
import com.frojasg1.applications.libmorphing.view.painters.TriangulationPainterLayerTypeEnum;
import com.frojasg1.applications.libmorphing.view.panels.evt.enums.FaceDetectionSelectionType;
import com.frojasg1.applications.libmorphing.view.panels.evt.enums.RecalculateTriangulationType;
import com.frojasg1.applications.libmorphing.view.panels.evt.enums.VertexEventType;
import com.frojasg1.libgeometry.types.GeometricRectangle;
import com.frojasg1.libgeometry.types.Point2d;
import com.frojasg1.general.view.evt.wrapper.impl.GenComponentWrapperEvent;
import com.frojasg1.general.view.evt.wrapper.impl.GenComponentWrapperEventBuilderBase;
import com.frojasg1.general.view.evt.wrapper.impl.MultiMapForBuildingJComponentWrapperEvents;
import com.frojasg1.general.number.DoubleReference;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class DelaunayJPanelsEventBuilderBase<PP>
	extends GenComponentWrapperEventBuilderBase<PP, GenComponentWrapperEvent> {

	public DelaunayJPanelsEventBuilderBase(PP panel)
	{
		this( panel, true );
	}

	protected DelaunayJPanelsEventBuilderBase(PP panel, boolean init)
	{
		super( panel );

		if( init )
			init();
	}

	@Override
	protected void fillMultiMap(MultiMapForBuildingJComponentWrapperEvents multiMap) {
        multiMap.put( ZoomEvt.EVENT_TYPE, ZoomEvt.class, ZoomEvt::new );
        multiMap.put( NavigatorEvt.EVENT_TYPE, NavigatorEvt.class, NavigatorEvt::new );
        multiMap.put( FadedMasterImageButtonPressedEvt.EVENT_TYPE, FadedMasterImageButtonPressedEvt.class, FadedMasterImageButtonPressedEvt::new );
        multiMap.put( VertexEvent.EVENT_TYPE, VertexEvent.class, VertexEvent::new );
        multiMap.put( ClearTriangulationEvent.EVENT_TYPE, ClearTriangulationEvent.class, ClearTriangulationEvent::new );
        multiMap.put( NewOnTheFlyBoundsEvt.EVENT_TYPE, NewOnTheFlyBoundsEvt.class, NewOnTheFlyBoundsEvt::new );
        multiMap.put( NewConfirmedBoundsEvt.EVENT_TYPE, NewConfirmedBoundsEvt.class, NewConfirmedBoundsEvt::new );
        multiMap.put( TriangleSelectedEvent.EVENT_TYPE, TriangleSelectedEvent.class, TriangleSelectedEvent::new );
        multiMap.put( MakeMasterImageEvt.EVENT_TYPE, MakeMasterImageEvt.class, MakeMasterImageEvt::new );

		multiMap.put( FaceDetectionSelectionEvent.EVENT_TYPE, FaceDetectionSelectionEvent.class, FaceDetectionSelectionEvent::new );
		multiMap.put( LayerSelectionEvent.EVENT_TYPE, LayerSelectionEvent.class, LayerSelectionEvent::new );
		multiMap.put( RunFaceDetectionEvent.EVENT_TYPE, RunFaceDetectionEvent.class, RunFaceDetectionEvent::new );
		multiMap.put( RecalculateTriangulationEvent.EVENT_TYPE, RecalculateTriangulationEvent.class, RecalculateTriangulationEvent::new );

		multiMap.put( ResetDetectedFaceMeshFrameEvent.EVENT_TYPE, ResetDetectedFaceMeshFrameEvent.class, ResetDetectedFaceMeshFrameEvent::new );
		multiMap.put( ResetModifiedDetectedFaceMeshEvent.EVENT_TYPE, ResetModifiedDetectedFaceMeshEvent.class, ResetModifiedDetectedFaceMeshEvent::new );

		multiMap.put( ChangeFollowingTransitionEvt.EVENT_TYPE, ChangeFollowingTransitionEvt.class, ChangeFollowingTransitionEvt::new );
		multiMap.put(Insert68DetectedPointsEvt.EVENT_TYPE, Insert68DetectedPointsEvt.class, Insert68DetectedPointsEvt::new);
	}

	protected ZoomEvtData createZoomEvtData( double previousZoomFactor, DoubleReference newZoomFactor )
	{
		ZoomEvtData result = new ZoomEvtData();
		result.setPreviousFactor(previousZoomFactor);
		result.setNewFactor(newZoomFactor);

		return( result );
	}

	protected NewBoundsEventData createNewBoundsEventData(String imageName,
														GeometricRectangle newBounds)
	{
		NewBoundsEventData result = new NewBoundsEventData();
		result.setImageName(imageName);
		result.setNewBounds(newBounds);

		return( result );
	}

	public NewOnTheFlyBoundsEvt<PP> createNewOnTheFlyBoundsEvt(String imageName,
															GeometricRectangle newBounds)
	{
		NewOnTheFlyBoundsEvt result = buildEvent( NewOnTheFlyBoundsEvt.class );
		result.setEventData( createNewBoundsEventData( imageName, newBounds ) );

		return( result );
	}

	public NewConfirmedBoundsEvt<PP> createNewConfirmedBoundsEvt(String imageName,
															GeometricRectangle newBounds)
	{
		NewConfirmedBoundsEvt result = buildEvent( NewConfirmedBoundsEvt.class );
		result.setEventData( createNewBoundsEventData( imageName, newBounds ) );

		return( result );
	}

	public NewBoundaryButtonPressedEvt<PP> createNewBoundaryButtonPressedEvt()
	{
		NewBoundaryButtonPressedEvt result = new NewBoundaryButtonPressedEvt();

		return( result );
	}

	public ZoomEvt<PP> createZoomEvt( double previousZoomFactor, DoubleReference newZoomFactor )
	{
		ZoomEvt result = buildEvent( ZoomEvt.class );
		result.setEventData(createZoomEvtData(previousZoomFactor, newZoomFactor) );

		return( result );
	}

	protected NavigatorEvtData createNavigatorEvtData( NavigatorEvtType type )
	{
		NavigatorEvtData result = new NavigatorEvtData( );
		result.setNavigatorEvtType(type);

		return( result );
	}

	protected MakeMasterImageEvtData createMakeMasterImageEvtData( String imageName )
	{
		MakeMasterImageEvtData result = new MakeMasterImageEvtData();
		result.setImageName(imageName);

		return( result );
	}
	protected Insert68DetectedPointsData createInsert68DetectedPointsEvtData(String imageName)
	{
		Insert68DetectedPointsData result = new Insert68DetectedPointsData();
		result.setImageName(imageName);

		return( result );
	}

	protected ChangeFollowingTransitionEvtData createChangeFollowingTransitionEvtData(String imageName,
																					  MorphingTransition transition)
	{
		ChangeFollowingTransitionEvtData result = new ChangeFollowingTransitionEvtData();
		result.setImageName(imageName);
		result.setTransition(transition);

		return( result );
	}

	protected FadedMasterImageButtonPressedEvtData createFadedMasterImageButtonPressedEvtData(
													double fadingPercentage )
	{
		FadedMasterImageButtonPressedEvtData result = new FadedMasterImageButtonPressedEvtData();
		result.setFadingPercentage(fadingPercentage);

		return( result );
	}

	public MakeMasterImageEvt<PP> createMakeMasterImageEvt( String imageName )
	{
		MakeMasterImageEvt<PP> result = buildEvent( MakeMasterImageEvt.class );
		result.setEventData( createMakeMasterImageEvtData( imageName ) );

		return( result );
	}

	public ChangeFollowingTransitionEvt<PP> createChangeFollowingTransitionEvt(String imageName, MorphingTransition transition)
	{
		ChangeFollowingTransitionEvt<PP> result = buildEvent(ChangeFollowingTransitionEvt.class);
		result.setEventData(createChangeFollowingTransitionEvtData(imageName, transition));

		return( result );
	}

	public Insert68DetectedPointsEvt<PP> createInsert68DetectedPointsEvt(String imageName)
	{
		Insert68DetectedPointsEvt<PP> result = buildEvent( Insert68DetectedPointsEvt.class );
		result.setEventData( createInsert68DetectedPointsEvtData(imageName) );

		return( result );
	}

	public FadedMasterImageButtonPressedEvt<PP> createFadedMasterImageButtonPressedEvt(
			double fadingPercentage )
	{
		FadedMasterImageButtonPressedEvt<PP> result = buildEvent( FadedMasterImageButtonPressedEvt.class );
		result.setEventData( createFadedMasterImageButtonPressedEvtData( fadingPercentage ) );

		return( result );
	}

	public NavigatorEvt<PP> createNavigatorEvt( NavigatorEvtType type )
	{
		NavigatorEvt result = buildEvent( NavigatorEvt.class );
		result.setEventData(createNavigatorEvtData(type));

		return( result );
	}

	protected RecalculateTriangulationData createRecalculateTriangulationData(RecalculateTriangulationType type,
																			  String imageName) {
		RecalculateTriangulationData result = new RecalculateTriangulationData();
		result.setType(type);
		result.setImageName(imageName);

		return result;
	}

	protected FaceDetectionSelectionData createFaceDetectionSelectionData(FaceDetectionSelectionType type,
																		  String imageName,
																		  FaceDetectionMeshData faceDetectionMeshData) {
		FaceDetectionSelectionData result = new FaceDetectionSelectionData();
		result.setType(type);
		result.setImageName(imageName);
		result.setFaceDetectionMeshData(faceDetectionMeshData);

		return result;
	}

	protected RunFaceDetectionData createRunFaceDetectionData(String imageName) {
		RunFaceDetectionData result = new RunFaceDetectionData();
		result.setImageName(imageName);

		return result;
	}

	protected LayerSelectionData createLayerSelectionData(String imageName, TriangulationPainterLayerTypeEnum layerType) {
		LayerSelectionData result = new LayerSelectionData();
		result.setImageName(imageName);
		result.setSelectedLayer(layerType);

		return result;
	}

	protected VertexEventData createVertexEventData( String imageName, Point2d originalVertex,
												Point2d vertex, boolean isConfirmed,
												VertexEventType type )
	{
		VertexEventData result = new VertexEventData();
		result.setImageName(imageName);
		result.setIsConfirmed(isConfirmed);
		result.setType(type);
		result.setVertex(vertex);
		result.setOriginalVertex(originalVertex);

		return( result );
	}

	protected ResetDetectedFaceMeshFrameData createResetDetectedFaceMeshFrameData(String imageName) {
		ResetDetectedFaceMeshFrameData result = new ResetDetectedFaceMeshFrameData();
		result.setImageName(imageName);

		return result;
	}

	protected ResetModifiedDetectedFaceMeshData createResetModifiedDetectedFaceMeshData(String imageName) {
		ResetModifiedDetectedFaceMeshData result = new ResetModifiedDetectedFaceMeshData();
		result.setImageName(imageName);

		return result;
	}

	public FaceDetectionSelectionEvent<PP> createFaceDetectionSelectionEvent(FaceDetectionSelectionType type,
																			 String imageName,
																			 FaceDetectionMeshData faceDetectionMeshData) {
		FaceDetectionSelectionEvent result = buildEvent( FaceDetectionSelectionEvent.class );
		result.setEventData(createFaceDetectionSelectionData(type, imageName, faceDetectionMeshData) );

		return( result );
	}

	public RecalculateTriangulationEvent<PP> createRecalculateTriangulationEvent(RecalculateTriangulationType type,
																				 String imageName) {
		RecalculateTriangulationEvent result = buildEvent( RecalculateTriangulationEvent.class );
		result.setEventData(createRecalculateTriangulationData(type, imageName) );

		return( result );
	}

	public RunFaceDetectionEvent<PP> createRunFaceDetectionEvent(String imageName) {
		RunFaceDetectionEvent result = buildEvent( RunFaceDetectionEvent.class );
		result.setEventData(createRunFaceDetectionData(imageName) );

		return( result );
	}

	public LayerSelectionEvent<PP> createLayerSelectionEvent(String imageName, TriangulationPainterLayerTypeEnum layerType)
	{
		LayerSelectionEvent result = buildEvent( LayerSelectionEvent.class );
		result.setEventData(createLayerSelectionData(imageName, layerType));

		return( result );
	}

	public VertexEvent<PP> createVertexEvent( String imageName, Point2d originalVertex,
											  Point2d vertex, boolean isConfirmed,
											  VertexEventType type )
	{
		VertexEvent result = buildEvent( VertexEvent.class );
		result.setEventData(createVertexEventData(imageName, originalVertex,
				vertex, isConfirmed,
				type) );

		return( result );
	}

	public ResetDetectedFaceMeshFrameEvent<PP> createResetDetectedFaceMeshFrameEvent(String imageName)
	{
		ResetDetectedFaceMeshFrameEvent result = buildEvent( ResetDetectedFaceMeshFrameEvent.class );
		result.setEventData(createResetDetectedFaceMeshFrameData(imageName));

		return result;
	}

	public ResetModifiedDetectedFaceMeshEvent<PP> createResetModifiedDetectedFaceMeshEvent(String imageName)
	{
		ResetModifiedDetectedFaceMeshEvent result = buildEvent( ResetModifiedDetectedFaceMeshEvent.class );
		result.setEventData(createResetModifiedDetectedFaceMeshData(imageName));

		return result;
	}
/*
	protected ClearTriangulationEventData createClearTriangulationEventData()
	{
		ClearTriangulationEventData result = new ClearTriangulationEventData();

		return( result );
	}
*/
	public ClearTriangulationEvent<PP> createClearTriangulationEvent()
	{
		ClearTriangulationEvent result = buildEvent( ClearTriangulationEvent.class );
//		result.setEventData(createClearTriangulationEventData(imageName));

		return( result );
	}

	protected TriangleSelectedEventData createTriangleSelectedEventData( Point2d pt,
																		String imageName)
	{
		TriangleSelectedEventData result = new TriangleSelectedEventData();
		result.setVertex(pt);
		result.setImageName(imageName);

		return( result );
	}

	public TriangleSelectedEvent createTriangleSelectedEvent( Point2d pt, String imageName )
	{
		TriangleSelectedEvent result = buildEvent( TriangleSelectedEvent.class );
		result.setEventData( createTriangleSelectedEventData(pt, imageName) );

		return( result );
	}
}
