/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.applications.libmorphing.xml.facedetector;

import com.frojasg1.applications.libmorphing.model.facedetector.ImageFaceDetectionPersistenceModel;
import com.frojasg1.applications.libmorphing.model.facedetector.ImageFaceDetectionPersistenceModelKey;
import com.frojasg1.applications.libmorphing.model.facedetector.ImageFaceDetectionPersistenceModelMap;
import com.frojasg1.applications.libmorphing.model.images.color.ColorsMap;
import com.frojasg1.applications.libmorphing.model.images.configuration.MorphingProjectConfigurationModel;
import com.frojasg1.applications.libmorphing.model.images.image.MorphingImageModel;
import com.frojasg1.applications.libmorphing.model.images.image.MorphingImageModelList;
import com.frojasg1.applications.libmorphing.model.images.transitions.TransitionConfigurationModelMap;
import com.frojasg1.applications.libmorphing.model.images.video.MorphingVideoConfigurationModelMapWrapper;
import com.frojasg1.general.FileFunctions;
import com.frojasg1.general.copyable.DefaultConstructorInitCopier;
import com.frojasg1.general.xml.model.NameKeyWithFileNameModelBase;

import java.util.Map;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class FaceDetectionWholeFileModelBase extends NameKeyWithFileNameModelBase implements FaceDetectionWholeFileModel {
	protected DefaultConstructorInitCopier _copier = DefaultConstructorInitCopier.instance();


	protected ImageFaceDetectionPersistenceModelMap imageFaceDetectionPersistenceModelMap = new ImageFaceDetectionPersistenceModelMap();

	// function for DefaultConstructorInitCopier
	public FaceDetectionWholeFileModelBase(String name)
	{
		super( name );
	}

	@Override
	public String getDirName()
	{
		return( FileFunctions.instance().getAbsoluteDirName(getFileName()) );
	}

	@Override
	public ImageFaceDetectionPersistenceModelMap getImageFaceDetectionPersistenceModelMap() {
		return imageFaceDetectionPersistenceModelMap;
	}

	@Override
	public void setImageFaceDetectionPersistenceModelMap(ImageFaceDetectionPersistenceModelMap imageFaceDetectionPersistenceModelMap) {
		this.imageFaceDetectionPersistenceModelMap = imageFaceDetectionPersistenceModelMap;
	}

	@Override
	public int addAll(ImageFaceDetectionPersistenceModelMap imageFaceDetectionPersistenceModelMap) {
		int numAddedImages = 0;
		if(imageFaceDetectionPersistenceModelMap != null) {
			for(Map.Entry<ImageFaceDetectionPersistenceModelKey, ImageFaceDetectionPersistenceModel> entry:
					imageFaceDetectionPersistenceModelMap.getMap().entrySet()) {
				if(!getImageFaceDetectionPersistenceModelMap().getMap().containsKey(entry.getKey())) {
					getImageFaceDetectionPersistenceModelMap().put(entry.getValue());
					numAddedImages++;
				}
			}
		}
		return numAddedImages;
	}
}
