/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.applications.libmorphing.xml.whole;

import com.frojasg1.applications.libmorphing.contexts.MorphingEditorApplicationContext;
import com.frojasg1.applications.libmorphing.model.images.color.ColorsMap;
import com.frojasg1.applications.libmorphing.model.images.configuration.MorphingProjectConfigurationModel;
import com.frojasg1.applications.libmorphing.model.facedetector.ImageFaceDetectionPersistenceModelMap;
import com.frojasg1.applications.libmorphing.model.images.image.MorphingImageModel;
import com.frojasg1.applications.libmorphing.model.images.image.MorphingImageModelList;
import com.frojasg1.applications.libmorphing.model.images.transitions.TransitionConfigurationModelMap;
import com.frojasg1.applications.libmorphing.model.images.video.MorphingVideoConfigurationModelMapWrapper;
import com.frojasg1.applications.libmorphing.xml.facedetector.FaceDetectionWholeFileModel;
import com.frojasg1.general.FileFunctions;
import com.frojasg1.general.copyable.DefaultConstructorInitCopier;
import com.frojasg1.general.xml.model.NameKeyWithFileNameModelBase;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class MorphingProjectWholeFileModelBase extends NameKeyWithFileNameModelBase implements MorphingProjectWholeFileModel {
	protected DefaultConstructorInitCopier _copier = DefaultConstructorInitCopier.instance();

	protected ColorsMap _colorsMap;
	protected MorphingProjectConfigurationModel _defaultConfiguration;

	protected MorphingImageModelList _imagesListWrapper;

	protected TransitionConfigurationModelMap _transitionConfigurationMap;

	protected MorphingVideoConfigurationModelMapWrapper _videoConfigurationMap;

	protected ImageFaceDetectionPersistenceModelMap imageFaceDetectionPersistenceModelMap = new ImageFaceDetectionPersistenceModelMap();

	protected MorphingEditorApplicationContext appliCtx;

	protected FaceDetectionWholeFileModel faceDetectionForXml;

	// function for DefaultConstructorInitCopier
	public MorphingProjectWholeFileModelBase(String name, MorphingEditorApplicationContext appliCtx)
	{
		this( name, appliCtx, true );
	}

	public MorphingProjectWholeFileModelBase(String name, MorphingEditorApplicationContext appliCtx, boolean init)
	{
		super( name );
		this.appliCtx = appliCtx;

		if( init )
			init();
	}

	// function for DefaultConstructorInitCopier
//	public void init( MorphingProjectWholeFileModel other )
//	{
//	}

	@Override
	public void init()
	{
		_imagesListWrapper = createMorphingImageModelList();
		_transitionConfigurationMap = createConfigurationMap();
		_colorsMap = createColorsMap();
		_defaultConfiguration = createDefaultConfiguration();
	}

	public MorphingEditorApplicationContext getAppliCtx() {
		return appliCtx;
	}

	protected MorphingImageModelList createMorphingImageModelList() {
		return( new MorphingImageModelList() );
	}

	protected TransitionConfigurationModelMap createConfigurationMap()
	{
		return( new TransitionConfigurationModelMap() );
	}

	protected ColorsMap createColorsMap()
	{
		return( new ColorsMap() );
	}

	protected MorphingProjectConfigurationModel createDefaultConfiguration()
	{
		return( new MorphingProjectConfigurationModel(null) );
	}

	@Override
	public ColorsMap getColorsMap() {
		return _colorsMap;
	}

	public void setColorsMap(ColorsMap _colorsMap) {
		this._colorsMap = _colorsMap;
	}

	@Override
	public MorphingProjectConfigurationModel getDefaultConfiguration() {
		return _defaultConfiguration;
	}

	public void setDefaultConfiguration(MorphingProjectConfigurationModel _defaultConfiguration) {
		this._defaultConfiguration = _defaultConfiguration;
	}

	@Override
	public TransitionConfigurationModelMap getTransitionConfigurationMap() {
		return _transitionConfigurationMap;
	}

	public void setTransitionConfigurationMap(TransitionConfigurationModelMap _transitionConfigurationMap) {
		this._transitionConfigurationMap = _transitionConfigurationMap;
	}

	@Override
	public MorphingVideoConfigurationModelMapWrapper getVideoConfigurationMap() {
		return _videoConfigurationMap;
	}

	public void setVideoConfigurationMap(MorphingVideoConfigurationModelMapWrapper _videoConfigurationMap) {
		this._videoConfigurationMap = _videoConfigurationMap;
	}

	@Override
	public MorphingImageModelList getImagesListWrapper() {
		return _imagesListWrapper;
	}

	public void setImagesListWrapper(MorphingImageModelList _imagesListWrapper) {
		this._imagesListWrapper = _imagesListWrapper;
	}

	@Override
	public MorphingImageModel getMasterImageModel()
	{
		return( getImagesListWrapper().getList().stream()
			.filter( im -> im.isMaster() ).findFirst()
			.orElse(null) );
	}

	@Override
	public MorphingImageModel getImageModel(String imageName)
	{
		return( getImagesListWrapper().getList().stream()
			.filter( im -> im.getName().equals(imageName) ).findFirst()
			.orElse(null) );
	}

	@Override
	public String getMasterImageName()
	{
		MorphingImageModel masterImageModel = getMasterImageModel();
		return( (masterImageModel == null) ? null : masterImageModel.getName() );
	}

	@Override
	public String getDirName()
	{
		return( FileFunctions.instance().getAbsoluteDirName(getFileName()) );
	}

	@Override
	public FaceDetectionWholeFileModel getFaceDetectionForXml() {
		return faceDetectionForXml;
	}

	@Override
	public void setFaceDetectionForXml(FaceDetectionWholeFileModel faceDetectionReadFromXml) {
		this.faceDetectionForXml = faceDetectionReadFromXml;
	}
//	@Override
//	public ImageFaceDetectionPersistenceModelMap getImageFaceDetectionPersistenceModelMap() {
//		return imageFaceDetectionPersistenceModelMap;
//	}

//	@Override
//	public void setImageFaceDetectionPersistenceModelMap(ImageFaceDetectionPersistenceModelMap imageFaceDetectionPersistenceModelMap) {
//		this.imageFaceDetectionPersistenceModelMap = imageFaceDetectionPersistenceModelMap;
//	}
}
