/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.applications.libmorphing.xml.whole.loader;

import com.frojasg1.applications.libmorphing.contexts.MorphingEditorApplicationContext;
import com.frojasg1.applications.libmorphing.model.images.image.MorphingImageModel;
import com.frojasg1.applications.libmorphing.xml.whole.MorphingProjectWholeFileModel;
import com.frojasg1.general.FileFunctions;
import com.frojasg1.general.xml.persistency.XmlSimpleFilePersistency;
import com.frojasg1.general.xml.persistency.loader.ModelToXml;
import com.frojasg1.general.xml.persistency.loader.XmlToModel;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class MorphingProjectWholeFilePersistency extends XmlSimpleFilePersistency<MorphingProjectWholeFileModel>
{
	protected MorphingEditorApplicationContext appliCtx;


	public MorphingProjectWholeFilePersistency(MorphingEditorApplicationContext appliCtx)//( BaseApplicationConfigurationInterface appConf )
	{
		this.appliCtx = appliCtx;
	}

	protected MorphingEditorApplicationContext getAppliCtx() {
		return appliCtx;
	}

	@Override
	protected ModelToXml<MorphingProjectWholeFileModel> createModelToXmlTranslator()
	{
		return( new MorphingProjectWholeFileModelToXml(getAppliCtx()) );
	}

	@Override
	protected XmlToModel<MorphingProjectWholeFileModel> createXmlToModelTranslator()
	{
		return( new XmlToMorphingProjectWholeFileModel(getAppliCtx()) );
	}

	@Override
	public void registerInternationalString(String label, String value) {
		throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
	}

	@Override
	public String getInternationalString(String label) {
		throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
	}

	@Override
	public String createCustomInternationalString(String label, Object... args) {
		throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
	}

	@Override
	protected void postProcess( MorphingProjectWholeFileModel result )
	{
		String projectModelDirName = result.getDirName();
		for( MorphingImageModel mim: result.getImagesListWrapper().getList() )
			mim.setFileName( getRealFileName( mim, projectModelDirName ) );
	}

	protected String getRealFileName( MorphingImageModel mim, String projectModelDirName )
	{
		String longFileName = FileFunctions.instance().getLongFileName(projectModelDirName, mim.getFileName());
		return( FileFunctions.instance().getAbsolutePathName(longFileName) );
	}
}
