/*
 * Copyright (c) 2023. Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 3.0 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */

package com.frojasg1.applications.libmorphing.xml.whole.manager;

import com.frojasg1.applications.libmorphing.controller.MorphingVerticesMapManager;
import com.frojasg1.applications.libmorphing.model.images.configuration.MorphingProjectConfigurationModel;
import com.frojasg1.applications.libmorphing.model.images.image.MorphingImageModel;
import com.frojasg1.applications.libmorphing.xml.whole.MorphingProjectWholeFileModel;
import com.frojasg1.libgeometry.types.GeometricRectangle;
import org.xml.sax.SAXException;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;
import java.io.IOException;
import java.util.List;

public interface MorphingProjectWholeFileModelManager {
    MorphingProjectWholeFileModel loadMorphingProjectModel(String fileName) throws IOException, SAXException, ParserConfigurationException;

    void saveMorphingProjectModel() throws IOException, TransformerException;

    void saveMorphingProjectModelAs(String fileName) throws IOException, TransformerException;

    MorphingImageModel addImage(String imageName, String imageFileName,
                                GeometricRectangle bounds,
                                MorphingProjectConfigurationModel configurationModel,
                                boolean isMaster,
                                int index);

    MorphingProjectWholeFileModel loadProject(String fileName) throws IOException, ParserConfigurationException, SAXException;

    /*
            public boolean removeImageModel( int index )
            {
                return( getVerticesManager().removeImageModel( index ) );
            }

            public boolean removeImageModel( String imageName )
            {
                return( getVerticesManager().removeImageModel( imageName ) );
            }

            public void relocateImage( String imageName, int toIndex )
            {
                getVerticesManager().relocateImage(imageName, toIndex);
            }

            public void relocateImage( int fromIndex, int toIndex )
            {
                getVerticesManager().relocateImage(toIndex, toIndex);
            }
        */
    void relocateImages(List<MorphingImageModel> resultingList);

    void deleteImages(int[] indicesToBeDeleted, List<MorphingImageModel> resultingList);

    MorphingVerticesMapManager getVerticesManager();

    void setVerticesManager(MorphingVerticesMapManager _verticesManager);

    MorphingProjectWholeFileModel getMorphingProjectModel();

    void setMorphingProjectModel(MorphingProjectWholeFileModel morphingProjectModel);

    void changeImageModel(MorphingImageModel imageModel);

    void setMasterImageName(String masterImageName);
}
