/* 
 * Copyright (C) 2021 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/gpl-3.0.txt
 *
 */
package com.frojasg1.general.jersey.queries;

import com.frojasg1.general.desktop.queries.*;
import org.json.JSONArray;
import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.ws.rs.client.WebTarget;
import javax.ws.rs.core.MultivaluedMap;
import javax.ws.rs.core.Response;

/**
 *
 * @author fjavier.rojas
 */
public abstract class InetQueryBaseWithResult<RR, CC extends InetQueryResultWithResultBase<RR>> extends InetQueryBase2<CC> implements InetQuery<CC>
{
	private static final Logger LOGGER = LoggerFactory.getLogger(InetQueryBaseWithResult.class);

	@Override
	public void init( String uri, String path )
	{
		super.init(uri, path);
	}

	public abstract void init( String uri );

	protected RR convertInternalResult(JSONObject parentJsonObject) {
		RR result = null;
		JSONObject jsonObject1 = getJsonObject(parentJsonObject, "result");
		if(jsonObject1 != null) {
			result = convertInternalResultInternal(jsonObject1);
		} else {
			JSONArray jsonArray = getJsonArray(parentJsonObject, "result");
			if(jsonArray != null) {
				result = convertInternalResultInternal(jsonArray);
			}
		}
		return result;
	}

	protected abstract RR convertInternalResultInternal(JSONObject jsonObject);
	protected abstract RR convertInternalResultInternal(JSONArray jsonArray);


	protected boolean isAsuccessInternal(Integer resultCode) {
		return (resultCode != null) && (resultCode == InetQueryResultBase.SUCCESS_RESULT_CODE);
	}

	protected boolean isAsuccess(JSONObject jsonObject) {
		Integer resultCode = getInt(jsonObject, "result_code");
		return isAsuccessInternal(resultCode);
	}

	protected CC convert( String jsonStr ) {
		JSONObject jsonObject = getJsonObject(jsonStr);

		CC result = createInetQueryResult();
		if(isAsuccess(jsonObject)) {
			result.setResult(convertInternalResult(jsonObject));
		} else {
			result.setException(createInetQueryExceptionFromNotSuccessfulResult(jsonObject));
		}
		return result;
	}

	protected InetQueryException createInetQueryExceptionFromNotSuccessfulResult(JSONObject jsonObject)
	{
		return( new InetQueryException( getInt(jsonObject, "result_code"),
										getString(jsonObject, "text") ) );
	}
}
