/* 
 * Copyright (C) 2021 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/gpl-3.0.txt
 *
 */
package com.frojasg1.general.jersey.queries;

import com.frojasg1.general.BooleanFunctions;
import com.frojasg1.general.ExecutionFunctions;
import com.frojasg1.general.UtilFunctions;
import com.frojasg1.general.desktop.queries.InetQueryException;
import com.frojasg1.general.desktop.queries.InetQueryResult;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import javax.ws.rs.core.MultivaluedHashMap;
import javax.ws.rs.core.MultivaluedMap;

/**
 *
 * @author fjavier.rojas
 */
public abstract class JsonFunctionsBase<CC extends InetQueryResult >
{
	protected JSONObject getJsonObject( String jsonString )
	{
		return( new JSONObject( jsonString ) );
	}

	protected JSONArray getJsonArray( String jsonString ) throws JSONException
	{
		return( new JSONArray( jsonString ) );
	}
	protected JSONObject getJsonObject( JSONObject jsonObject, String field )
	{
		return safeSilentFunctionExecution(() -> jsonObject.getJSONObject(field));
	}

	protected JSONArray getJsonArray( JSONObject jsonObject, String field )
	{
		return safeSilentFunctionExecution(() -> jsonObject.getJSONArray(field));
	}
	protected JSONObject getJsonObject( JSONArray jsonArray, int index )
	{
		return safeSilentFunctionExecution(() -> jsonArray.getJSONObject(index));
	}

	protected JSONArray getJsonArray( JSONArray jsonArray, int index ) throws JSONException
	{
		return safeSilentFunctionExecution(() -> jsonArray.getJSONArray(index));
	}

	protected <CC> CC safeSilentFunctionExecution( ExecutionFunctions.UnsafeFunction<CC> run ) {
		return ExecutionFunctions.instance().safeSilentFunctionExecution(run);
	}


	protected Integer getInt( JSONObject jsonObject, String fieldName )
	{
		return(getIntWithDefault(jsonObject, fieldName, null));
	}

	protected Integer getIntWithDefault( JSONObject jsonObject, String fieldName, Integer defaultValue )
	{
		return( getValueWithDefault( () -> jsonObject.getInt( fieldName ) , defaultValue ));
	}

	protected String getString( JSONObject jsonObject, String fieldName )
	{
		return( getStringWithDefault(jsonObject, fieldName, null) );
	}

	protected String getStringWithDefault( JSONObject jsonObject, String fieldName, String defaultValue )
	{
		return( getValueWithDefault( () -> jsonObject.getString( fieldName ), defaultValue ) );
	}

	protected <C> C getValueWithDefault(ExecutionFunctions.UnsafeFunction<C> supplier, C defaultValue) {
		return UtilFunctions.instance().getValueWithDefault(supplier, defaultValue);
	}

	protected boolean getBooleanWithDefault( JSONObject jsonObject, String fieldName, boolean defaultValue )
	{
		return(getValueWithDefault(() -> jsonObject.getBoolean( fieldName ), defaultValue));
	}

	protected String booleanToString( boolean value )
	{
		return( BooleanFunctions.instance().booleanToString( value ) );
	}

	protected MultivaluedMap<String, String> createMultimap( String ... strArr )
	{
		MultivaluedMap<String, String> result = new MultivaluedHashMap<>();

		for( int ii=0; ii<strArr.length-1; ii+=2 )
		{
			result.add( strArr[ii], strArr[ii+1] );
		}

		return( result );
	}

	protected CC createInetQueryResult( Exception exception )
	{
		CC result = createInetQueryResult();

		result.setException(exception);

		return( result );
	}

	protected abstract CC convert( String jsonStr ) throws InetQueryException;
	protected abstract CC createInetQueryResult();
}
