package com.frojasg1.weka.clustering.factory;


import com.frojasg1.libgenai.clustering.model.ClustererModel;
import com.frojasg1.libgenai.clustering.model.ClustererTypeContext;
import com.frojasg1.libgenai.clustering.model.impl.gaussianmixture.GaussianMixtureConfiguration;
import com.frojasg1.libgenai.clustering.model.impl.gaussianmixture.GaussianMixtureConfigurationBase;
import com.frojasg1.weka.clustering.helper.WekaHelper;
import com.frojasg1.weka.clustering.model.gaussianmixture.WekaGaussianMixtureContext;
import com.frojasg1.weka.clustering.model.gaussianmixture.WekaGaussianMixtureModel;
import com.frojasg1.weka.clustering.model.gaussianmixture.WekaGaussianMixtureTrainer;
import com.frojasg1.weka.clustering.model.gaussianmixture.io.WekaGaussianMixtureValues;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import weka.clusterers.EM;

public class LibWekaFactoryImpl implements LibWekaFactory {

    protected static LibWekaFactoryImpl INSTANCE = new LibWekaFactoryImpl();

    public static LibWekaFactoryImpl instance() {
        return INSTANCE;
    }

    @Override
    public WekaGaussianMixtureModel createModel(EM expectationMaximization,
                                                GaussianMixtureConfiguration configuration) {
        WekaGaussianMixtureModel result = new WekaGaussianMixtureModel(expectationMaximization);
        result.setConfiguration(configuration);

        return result;
    }

    @Override
    public ClustererModel<GaussianMixtureConfiguration> createModel(WekaGaussianMixtureValues wekaGaussianMixtureValues) {
        return createModel(createExpectationMaximization(wekaGaussianMixtureValues), wekaGaussianMixtureValues.getConfiguration());
    }

    @Override
    public WekaGaussianMixtureTrainer createTrainer(GaussianMixtureConfiguration configuration) {
        return new WekaGaussianMixtureTrainer(this, configuration);
    }

    public WekaGaussianMixtureValues createWekaGaussianMixtureValues() {
        return new WekaGaussianMixtureValues();
    }

    @Override
    public WekaGaussianMixtureValues createWekaGaussianMixtureValues(ClustererModel<GaussianMixtureConfiguration> clustererModel) {
        WekaGaussianMixtureModel model = (WekaGaussianMixtureModel) clustererModel;

        WekaGaussianMixtureValues result = createWekaGaussianMixtureValues();
        result.setModelValues(model.getWekaExpectationMaximization().getClusterModelsNumericAtts());
        result.setConfiguration(clustererModel.getConfiguration());

        return result;
    }

    protected WekaFactory getWekaFactory() {
        return WekaFactory.instance();
    }

    protected WekaHelper getWekaHelper() {
        return WekaHelper.instance();
    }

    protected double[] array(double ... array) {
        return array;
    }

    protected List<double[]> createDefaultInstances(GaussianMixtureConfiguration configuration) {
        List<double[]> result = new ArrayList<>();
        for(int ii = 0; ii < configuration.getNumClusters(); ii++) {
            result.add(Arrays.copyOf(array(ii), configuration.getAttributeNames().length));
        }
        return result;
    }

    protected EM createExpectationMaximization(WekaGaussianMixtureValues wekaGaussianMixtureValues) {
        GaussianMixtureConfiguration configuration = wekaGaussianMixtureValues.getConfiguration();
        EM result = getWekaFactory()
                .createExpectationMaximizationAndTrain(configuration, createDefaultInstances(configuration));
        getWekaHelper().copyEmModel(wekaGaussianMixtureValues.getModelValues(), result.getClusterModelsNumericAtts());

        return result;
    }

    @Override
    public ClustererTypeContext<GaussianMixtureConfiguration> createContext() {
        return new WekaGaussianMixtureContext(this);
    }

    @Override
    public GaussianMixtureConfiguration createConfiguration() {
        return new GaussianMixtureConfigurationBase();
    }
}
