package com.frojasg1.weka.clustering.factory;

import com.frojasg1.general.ExecutionFunctions;
import com.frojasg1.libgenai.clustering.model.impl.gaussianmixture.GaussianMixtureConfiguration;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;
import weka.clusterers.EM;
import weka.core.Attribute;
import weka.core.DenseInstance;
import weka.core.Instance;
import weka.core.Instances;

public class WekaFactory {

    protected static WekaFactory INSTANCE = new WekaFactory();

    public static WekaFactory instance() {
        return INSTANCE;
    }

    public Attribute createAttribute(String attributeName) {
        return new Attribute(attributeName);
    }

    public Instance createInstance(double weight, double ... fields) {
        return new DenseInstance(weight, fields);
    }

    protected ArrayList<Attribute> createAttributeList(String[] attributeNames) {
        return new ArrayList(
                Arrays.stream(attributeNames).map(this::createAttribute)
                        .collect(Collectors.toList())
        );
    }

    public EM createExpectationMaximization() {
        return new EM();
    }

    protected ExecutionFunctions getExecutionFunctions() {
        return ExecutionFunctions.instance();
    }

    protected Exception runtimeExceptionMethodExecution(ExecutionFunctions.UnsafeMethod run) {
        return getExecutionFunctions().runtimeExceptionMethodExecution(run);
    }


    public EM createExpectationMaximization(GaussianMixtureConfiguration configuration) {
        EM result = createExpectationMaximization();
        result.setSeed(configuration.getSeed());
        runtimeExceptionMethodExecution(() -> result.setNumClusters(configuration.getNumClusters()));
        Integer maxIterations = configuration.getMaxIterations();
        if(maxIterations != null) {
            runtimeExceptionMethodExecution(() -> result.setMaxIterations(maxIterations));
        }

        return result;
    }

    public Instances createInstances(String name, String[] attributeNames, int capacity) {
        return new Instances(name, createAttributeList(attributeNames), capacity);
    }

    protected Instances createInstances(GaussianMixtureConfiguration configuration) {
        return createInstances(
                configuration.getModelName(),
                configuration.getAttributeNames(),
                33);
    }

    protected Instances createInstances(GaussianMixtureConfiguration configuration,
                                        List<double[]> dataset) {
        Instances result = createInstances(configuration);
        for(double[] instance: dataset) {
            result.add(createInstance(1.0d, instance));
        }

        return result;
    }

    public EM createExpectationMaximizationAndTrain(GaussianMixtureConfiguration configuration,
                                                    List<double[]> dataset) {
        EM result = createExpectationMaximization(configuration);
        runtimeExceptionMethodExecution(() -> result.buildClusterer(createInstances(configuration, dataset)));

        return result;
    }
}
