package com.frojasg1.weka.clustering.model.gaussianmixture;

import com.frojasg1.general.ExecutionFunctions;
import com.frojasg1.general.xml.JavaXmlFunctions;
import com.frojasg1.general.xml.XmlElement;
import com.frojasg1.general.xml.XmlFunctions;
import com.frojasg1.libgenai.clustering.model.ClustererModel;
import com.frojasg1.libgenai.clustering.model.ClustererTrainer;
import com.frojasg1.libgenai.clustering.model.ClustererTypeContext;
import com.frojasg1.libgenai.clustering.model.impl.gaussianmixture.GaussianMixtureConfiguration;
import com.frojasg1.libgenai.clustering.model.type.ClusteringAlgorithm;
import com.frojasg1.libgenai.clustering.model.type.ClusteringAlgorithmEnum;
import com.frojasg1.weka.clustering.factory.LibWekaFactory;
import com.frojasg1.weka.clustering.model.gaussianmixture.io.WekaGaussianMixtureValues;
import com.frojasg1.weka.clustering.model.gaussianmixture.io.WekaGaussianMixtureValuesToXml;
import com.frojasg1.weka.clustering.model.gaussianmixture.io.XmlToWekaGaussianMixtureValues;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;

public class WekaGaussianMixtureContext implements ClustererTypeContext<GaussianMixtureConfiguration> {

    protected LibWekaFactory libWekaFactory;

    public WekaGaussianMixtureContext(LibWekaFactory libWekaFactory) {
        this.libWekaFactory = libWekaFactory;
    }

    protected LibWekaFactory getLibWekaFactory() {
        return libWekaFactory;
    }

    @Override
    public ClusteringAlgorithm getType() {
        return ClusteringAlgorithmEnum.GAUSSIAN_MIXTURE;
    }

    @Override
    public ClustererModel<GaussianMixtureConfiguration> load(String fileName) {
        XmlElement elem = runtimeExceptionFunctionExecution(
                () -> loadDocumentCustomFileName( fileName )
        );

        ClustererModel<GaussianMixtureConfiguration> result = createFileModel( elem );

        return( result );
    }

    @Override
    public void save(ClustererModel<GaussianMixtureConfiguration> clustererModel, String fileName) {
        runtimeExceptionMethodExecution(() -> saveDocumentCustomFileName(getXmlElement(clustererModel), fileName));
    }

    @Override
    public ClustererTrainer<GaussianMixtureConfiguration> createTrainer(GaussianMixtureConfiguration configuration) {
        return getLibWekaFactory().createTrainer(configuration);
    }

    protected ExecutionFunctions getExecutionFunctions() {
        return ExecutionFunctions.instance();
    }

    protected <CC> CC runtimeExceptionFunctionExecution(ExecutionFunctions.UnsafeFunction<CC> run) {
        return getExecutionFunctions().runtimeExceptionFunctionExecution(run);
    }

    protected Exception runtimeExceptionMethodExecution(ExecutionFunctions.UnsafeMethod run) {
        return getExecutionFunctions().runtimeExceptionMethodExecution(run);
    }

    protected XmlElement loadDocumentCustomFileName(String longFileName ) throws IOException, ParserConfigurationException, SAXException
    {
        return( loadDocument( new FileInputStream( longFileName ) ) );
    }

    protected XmlElement loadDocument( InputStream is ) throws IOException, ParserConfigurationException, SAXException
    {
        XmlElement result = null;
        try( InputStream is1 = is; ) {
            Document doc = JavaXmlFunctions.instance().parseXml( is1 );
            result = XmlFunctions.instance().getXmlTree(doc);
        }

        return result;
    }

    protected WekaGaussianMixtureValues loadWekaGaussianMixtureValues(XmlElement xe) {
        return new XmlToWekaGaussianMixtureValues().build(xe);
    }

    protected ClustererModel<GaussianMixtureConfiguration> createFileModel( XmlElement xe ) {
        WekaGaussianMixtureValues wekaGaussianMixtureValues = loadWekaGaussianMixtureValues(xe);

        return getLibWekaFactory().createModel(wekaGaussianMixtureValues);
    }

    protected void saveDocumentCustomFileName( XmlElement xmlElement, String fileName ) throws IOException, TransformerException
    {
        saveDocument( xmlElement, new FileOutputStream( fileName ) );
    }

    protected void saveDocument( XmlElement xmlElement, OutputStream outputStream ) throws IOException, TransformerException
    {
        Document xmlDoc = XmlFunctions.instance().createDocument(xmlElement);

        JavaXmlFunctions.instance().writeDocument(xmlDoc, outputStream );
    }

    protected WekaGaussianMixtureValues createWekaGaussianMixtureValues(ClustererModel<GaussianMixtureConfiguration> clustererModel) {
        return getLibWekaFactory().createWekaGaussianMixtureValues(clustererModel);
    }

    protected XmlElement getXmlElement(ClustererModel<GaussianMixtureConfiguration> clustererModel) {
        return new WekaGaussianMixtureValuesToXml().build(createWekaGaussianMixtureValues(clustererModel));
    }
}
