package com.frojasg1.weka.clustering.model.gaussianmixture;

import com.frojasg1.general.ExecutionFunctions;
import com.frojasg1.libgenai.clustering.model.ClustererModel;
import com.frojasg1.libgenai.clustering.model.impl.gaussianmixture.GaussianMixtureConfiguration;
import com.frojasg1.libgenai.clustering.model.impl.gaussianmixture.GaussianMixtureModelBase;
import com.frojasg1.weka.clustering.factory.WekaFactory;
import weka.clusterers.EM;

public class WekaGaussianMixtureModel
    extends GaussianMixtureModelBase
        implements ClustererModel<GaussianMixtureConfiguration> {

    protected EM wekaExpectationMaximization;

    public WekaGaussianMixtureModel(EM wekaExpectationMaximization) {
        this.wekaExpectationMaximization = wekaExpectationMaximization;
    }

    public EM getWekaExpectationMaximization() {
        return wekaExpectationMaximization;
    }

    protected WekaFactory getWekaFactory() {
        return WekaFactory.instance();
    }

    protected ExecutionFunctions getExecutionFunctions() {
        return ExecutionFunctions.instance();
    }

    public <CC> CC runtimeExceptionFunctionExecution(ExecutionFunctions.UnsafeFunction<CC> run) {
        return getExecutionFunctions().runtimeExceptionFunctionExecution(run);
    }

    @Override
    public double[] getClusterProbabilities(double[] instance) {
        double[] result = runtimeExceptionFunctionExecution(
                () -> getWekaExpectationMaximization()
                        .logDensityPerClusterForInstance(
                                getWekaFactory().createInstance(1.0, instance)
                        )
        );
        return result;
    }
}
