package com.frojasg1.weka.clustering.model.gaussianmixture;

import com.frojasg1.general.ExecutionFunctions;
import com.frojasg1.libgenai.clustering.model.ClustererModel;
import com.frojasg1.libgenai.clustering.model.ClustererTrainer;
import com.frojasg1.libgenai.clustering.model.impl.gaussianmixture.GaussianMixtureConfiguration;
import com.frojasg1.weka.clustering.factory.LibWekaFactory;
import com.frojasg1.weka.clustering.factory.WekaFactory;
import java.util.List;
import weka.clusterers.EM;
import weka.core.Instances;

public class WekaGaussianMixtureTrainer implements ClustererTrainer<GaussianMixtureConfiguration> {

    protected LibWekaFactory libWekaFactory;
    protected GaussianMixtureConfiguration configuration;

    public WekaGaussianMixtureTrainer(LibWekaFactory libWekaFactory,
                                      GaussianMixtureConfiguration configuration) {
        this.libWekaFactory = libWekaFactory;
        this.configuration = configuration;
    }

    protected GaussianMixtureConfiguration getConfiguration() {
        return configuration;
    }

    protected WekaFactory getWekaFactory() {
        return WekaFactory.instance();
    }

    protected LibWekaFactory getLibWekaFactory() {
        return libWekaFactory;
    }

    protected EM createExpectationMaximizationAndTrain(List<double[]> dataset) {
        return getWekaFactory().createExpectationMaximizationAndTrain(getConfiguration(), dataset);
    }

    protected ClustererModel<GaussianMixtureConfiguration> createModel(EM expectationMaximization) {
        return getLibWekaFactory().createModel(expectationMaximization, getConfiguration());
    }

    @Override
    public ClustererModel<GaussianMixtureConfiguration> train(List<double[]> dataset) {
        EM expectationMaximization = createExpectationMaximizationAndTrain(dataset);
        ClustererModel<GaussianMixtureConfiguration> result = createModel(expectationMaximization);

        return result;
    }
}
