package com.frojasg1.weka.clustering.model.gaussianmixture.io;

import com.frojasg1.general.xml.XmlElement;
import com.frojasg1.general.xml.persistency.loader.impl.ModelToXmlBase;
import com.frojasg1.libgenai.clustering.model.impl.gaussianmixture.GaussianMixtureConfiguration;
import com.frojasg1.libgenai.clustering.model.impl.gaussianmixture.io.GaussianMixtureConfigurationToXml;

public class WekaGaussianMixtureValuesToXml extends ModelToXmlBase<WekaGaussianMixtureValues> {

    protected GaussianMixtureConfigurationToXml configurationToXml;

    public WekaGaussianMixtureValuesToXml() {
        this(new GaussianMixtureConfigurationToXml());
    }

    public WekaGaussianMixtureValuesToXml(GaussianMixtureConfigurationToXml configurationToXml) {
        this.configurationToXml = configurationToXml;
    }

    protected GaussianMixtureConfigurationToXml getConfigurationToXml() {
        return configurationToXml;
    }

    @Override
    public XmlElement build(WekaGaussianMixtureValues model) {
        XmlElement result = createElement("WekaGaussianMixtureModelXml");
        result.addChild(configToXml(model.getConfiguration()));
        fillModelValues(createElement(result, "model"), model.getModelValues());

        return result;
    }

    protected XmlElement configToXml(GaussianMixtureConfiguration configuration) {
        return getConfigurationToXml().build(configuration);
    }

    protected void fillModelValues(XmlElement xeResult, double[][][] array) {
        XmlElement dimenXe = createElement(xeResult, "dimensions");
        createLeafElementGen(dimenXe, "i_dimen", array.length);
        createLeafElementGen(dimenXe, "j_dimen", array[0].length);
        createLeafElementGen(dimenXe, "k_dimen", array[0][0].length);

        XmlElement valuesXe = createElement(xeResult, "values");
        for(int ii=0; ii<array.length; ii++) {
            XmlElement iiXe = createElement(valuesXe, "ii");
            iiXe.putAttribute("index", String.valueOf(ii));
            for(int jj=0; jj<array[0].length; jj++) {
                XmlElement jjXe = createElement(iiXe, "jj");
                jjXe.putAttribute("index", String.valueOf(jj));
                for(int kk=0; kk<array[0][0].length; kk++) {
                    createLeafElementGen(jjXe, "kk", array[ii][jj][kk])
                            .putAttribute("index", String.valueOf(kk));
                }
            }
        }
    }
}
