package com.frojasg1.weka.clustering.model.gaussianmixture.io;

import com.frojasg1.general.xml.XmlElement;
import com.frojasg1.general.xml.persistency.loader.impl.XmlToModelBase;
import com.frojasg1.libgenai.clustering.model.impl.gaussianmixture.GaussianMixtureConfiguration;
import com.frojasg1.libgenai.clustering.model.impl.gaussianmixture.io.XmlToGaussianMixtureConfiguration;

public class XmlToWekaGaussianMixtureValues extends XmlToModelBase<WekaGaussianMixtureValues> {

    protected XmlToGaussianMixtureConfiguration xmlToConfiguration;

    public XmlToWekaGaussianMixtureValues() {
        this(new XmlToGaussianMixtureConfiguration());
    }

    public XmlToWekaGaussianMixtureValues(XmlToGaussianMixtureConfiguration xmlToConfiguration) {
        this.xmlToConfiguration = xmlToConfiguration;
    }

    protected XmlToGaussianMixtureConfiguration getXmlToConfiguration() {
        return xmlToConfiguration;
    }

    @Override
    public WekaGaussianMixtureValues build(XmlElement xmlElement) {
        WekaGaussianMixtureValues result = new WekaGaussianMixtureValues();
        result.setConfiguration(loadConfiguration(getMandatoryChild(xmlElement, "configuration")));
        result.setModelValues(loadModelValues(getMandatoryChild(xmlElement, "model")));

        return result;
    }

    protected GaussianMixtureConfiguration loadConfiguration(XmlElement xe) {
        return getXmlToConfiguration().build(xe);
    }

    protected double[][][] loadModelValues(XmlElement xe) {
        XmlElement dimenXe = getMandatoryChild(xe, "dimensions");
        int totalIi = getMandatoryChildInt(dimenXe, "i_dimen");
        int totalJj = getMandatoryChildInt(dimenXe, "j_dimen");
        int totalKk = getMandatoryChildInt(dimenXe, "k_dimen");

        double[][][] result = new double[totalIi][totalJj][totalKk];

        int ii = 0;
        for(XmlElement iiXe: getMandatoryChild(xe, "values").getChildrenByName("ii").getCollection()) {
            int jj=0;
            for(XmlElement jjXe: iiXe.getChildrenByName("jj").getCollection()) {
                int kk=0;
                for(XmlElement kkXe: jjXe.getChildrenByName("kk").getCollection()) {
                    result[ii][jj][kk++] = getObject(kkXe, Double.class);
                }
                jj++;
            }
            ii++;
        }
        return result;
    }
}
