package com.frojasg1.weka.clustering;

import java.util.ArrayList;
import java.util.Arrays;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import weka.clusterers.EM;
import weka.core.Attribute;
import weka.core.DenseInstance;
import weka.core.Instance;
import weka.core.Instances;

public class WekaEmTest {
    private static final Logger LOGGER = LoggerFactory.getLogger(WekaEmTest.class);

    public static void main(String[] args) {
        new WekaEmTest().mainInternal(args);
    }

    protected Attribute attribute(String attributeName) {
        return new Attribute(attributeName);
    }

    protected Instance instance(double weight, double ... fields) {
        return new DenseInstance(weight, fields);
    }

    protected void mainInternal(String[] args) {
        try {
            EM em = new EM();

            em.setNumClusters(3);
            em.setSeed(0);

            ArrayList<Attribute> attributes = new ArrayList(
                    Arrays.asList( attribute("field-1"),
                            attribute("field-2"), attribute("field-3"))
            );

            Instances instances = new Instances("myGaussianMixtureModel", attributes, 0);
            instances.add(instance(1.0d, 1.0d, 0d, 0d));
            instances.add(instance(1.0d, 0d, 1.0d, 0d));
            instances.add(instance(1.0d, 0d, 0d, 1.0d));
            instances.add(instance(1.0d, 0.99d, 0.01d, -0.01d));
            instances.add(instance(1.0d, 0.01d, 1.01d, 0.01d));
            instances.add(instance(1.0d, -0.01d, -0.01d, 1.0d));
            instances.add(instance(1.0d, 1.01d, -0.01d, 0.01d));
            instances.add(instance(1.0d, -0.01d, 0.99d, -0.01d));
            instances.add(instance(1.0d, 0.01d, 0.01d, 0.99d));

            em.buildClusterer(instances);

            Instance i1 = instance(1.0d, 1.0d, 1.0d, 0d);
            Instance i2 = instance(1.0d, 0.98d, 0.01d, -0.01d);
            double[] w1 = em.logDensityPerClusterForInstance(i1);
            double[] w2 = em.logDensityPerClusterForInstance(i2);

            LOGGER.info("Model1");
            logFittingResult(i1, w1);
            logFittingResult(i2, w2);

            double[][][] myModel = em.getClusterModelsNumericAtts();

            EM em2 = new EM();
            em2.setNumClusters(3);
            Instances instances2 = new Instances("myGaussianMixtureModel", attributes, 0);
            em2.buildClusterer(instances2);

            copyModel(myModel, em2.getClusterModelsNumericAtts());

            double[] w11 = em2.logDensityPerClusterForInstance(i1);
            double[] w12 = em2.logDensityPerClusterForInstance(i2);

            LOGGER.info("Model2");
            logFittingResult(i1, w11);
            logFittingResult(i2, w12);

        } catch(Exception ex) {
            LOGGER.error("Error when doing clustering", ex);
        }
    }

    protected void logFittingResult(Instance instance, double[] logWeights) {
        LOGGER.info("{} -----> {}", instance, Arrays.toString(logWeights));
    }

    protected void copyModel(double[][][] input, double[][][] output) {
        for(int ii=0; ii<input.length; ii++) {
            for(int jj=0; jj<input[0].length; jj++) {
                for(int kk=0; kk<input[0][0].length; kk++) {
                    output[ii][jj][kk] = input[ii][jj][kk];
                }
            }
        }
    }
}
