/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.libgenai.image;

import com.frojasg1.general.number.DoubleFunctions;

public class NormalizedRange {

    protected double lowerValue;
    protected double upperValue;

    public NormalizedRange(double lowerValue, double upperValue) {
        this.lowerValue = lowerValue;
        this.upperValue = upperValue;
    }

    public double getRangeLength() {
        return getUpperValue() - getLowerValue();
    }

    public double getLowerValue() {
        return lowerValue;
    }

    public double getUpperValue() {
        return upperValue;
    }

    public double getMean() {
        return (getLowerValue() + getUpperValue()) / 2d;
    }

    public double limit(double value) {
        return DoubleFunctions.instance().limit(value, getLowerValue(), getUpperValue());
    }

    @Override
    public String toString() {
        return "NormalizedRange{" +
                "lowerValue=" + lowerValue +
                ", upperValue=" + upperValue +
                '}';
    }
}
