/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.libgenai.metrics;

import com.frojasg1.libgenai.image.NormalizedImage;

import java.util.function.BinaryOperator;

public class ImageMetrics {

    protected static ImageMetrics instance = new ImageMetrics();

    public static ImageMetrics instance() {
        return instance;
    }

    public double mae(NormalizedImage first, NormalizedImage second) {
        return averageMetric(first, second, ScalarMetrics.L1);
    }

    public double mse(NormalizedImage first, NormalizedImage second) {
        return averageMetric(first, second, ScalarMetrics.L2);
    }

    public double averageMetric(NormalizedImage first, NormalizedImage second, ScalarMetrics scalarMetrics) {
        first.checkCompatibility(second);

        BinaryOperator<Double> termToAddCalculator = scalarMetrics.getTermToAddCalculator();
        double partial;
        double result = 0;
        int width = first.getWidth();
        int height = first.getHeight();
        for(int xx=0; xx<width; xx++) {
            partial = 0;
            for(int yy=0; yy<height; yy++) {
                partial += termToAddCalculator.apply(first.get(xx, yy), second.get(xx, yy));
            }
            result += partial / height;
        }
        result /= width;

        return result;
    }
}
