/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.libgenai.metrics;

import com.frojasg1.general.number.DoubleFunctions;

import java.util.function.BinaryOperator;

public enum ScalarMetrics {

    L1(ScalarMetrics::absoluteError),
    L2(ScalarMetrics::squaredError);

    BinaryOperator<Double> termToAddCalculator;

    ScalarMetrics(BinaryOperator<Double> termToAddCalculator) {
        this.termToAddCalculator = termToAddCalculator;
    }

    protected static double squaredError(double v1, double v2) {
        double delta = v1 - v2;
        return delta * delta;
    }

    protected static double absoluteError(double v1, double v2) {
        return DoubleFunctions.instance().abs(v1 -v2);
    }

    public BinaryOperator<Double> getTermToAddCalculator() {
        return termToAddCalculator;
    }
}
