/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general;

import java.util.concurrent.atomic.AtomicReference;
import java.util.function.Function;
import java.util.function.UnaryOperator;

/**
 *
 * @author fjavier.rojas
 */
public class UtilFunctions
{
	protected static UtilFunctions _instance = new UtilFunctions();

	public static void changeInstance( UtilFunctions inst )
	{
		_instance = inst;
	}

	public static UtilFunctions instance()
	{
		return( _instance );
	}

	public <I, I2, O2, O> O functionWithCodec(Function<I2, O2> function, I input, Function<I, I2> encoder, Function<O2, O> decoder) {
		O result = null;
		if(input != null) {
			I2 i2 = encoder.apply(input);
			O2 o2 = function.apply(i2);
			result = decoder.apply(o2);
		}
		return result;
	}

	protected <C> C safeFunction( ExecutionFunctions.UnsafeFunction<C> run )
	{
		return( ExecutionFunctions.instance().safeFunctionExecution(run) );
	}

	public <C> C getValueWithDefault(ExecutionFunctions.UnsafeFunction<C> supplier, C defaultValue) {
		C result = safeFunction(supplier);
		if(result == null) {
			result = defaultValue;
		}
		return result;
	}
}
