/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.analysers.impl;

import com.frojasg1.general.analysers.GenListSplitterBase;
import com.frojasg1.general.analysers.GenListSplitterBase;
import java.util.Objects;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class DefaultListQuotedDataSplitter extends GenListSplitterBase
{
	protected static final String QUOTE_CHARS = "\"'";
	
	protected String _notableChars;

	public DefaultListQuotedDataSplitter( String string, String separator )
	{
		super( string, separator );

		_notableChars = QUOTE_CHARS + separator;
	}

	protected void passWholeAttribute()
	{
		String ch = null;
		loop: while( ! hasEnded() )
		{
			ch = getNextNotableChar();
			if( ch == null )
				break;

			_pos++;

			switch( ch )
			{
				case "'":
				case "\"":
//					String ch2 = incPosAndGetMatchingChar(ch);
					String ch2 = getMatchingChar(ch);
					if( !Objects.equals( ch2, ch) )
						throw( new RuntimeException( "Matching quote not found at pos: " + _pos ) );
					_pos++;

				break;

				default:
					if( isSeparator( ch ) )
					{
						break loop;
					}
			}
		}
	}

	protected String getNextNotableChar()
	{
		return( getNextNotableChar(_notableChars) );
	}
}
