/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.codec.impl;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class DefaultGenericStringStringDecoderImpl extends GenericLeafStringDecoderBase<String>
{
	private static final Logger LOGGER = LoggerFactory.getLogger(DefaultGenericStringStringDecoderImpl.class);

	public DefaultGenericStringStringDecoderImpl( GenericStringDecoderBuilder builder )
	{
		super( String.class, builder );
	}

	protected String getTransformedSimpleValueString( String valueStr )
	{
		String result = removeQuotes( valueStr );

		return( result );
	}

	protected boolean hasQuotes( String str )
	{
		return( hasQuotes( str, '\'' ) || hasQuotes( str, '"' ) );
	}

	protected boolean hasQuotes( String str, char quote )
	{
		boolean result = ( str != null ) && ( str.length() >= 2 );

		if( result )
			result = ( str.charAt(0) == quote ) && ( str.charAt( str.length() - 1 ) == quote );

		return( result );
	}

	protected String removeQuotes( String str )
	{
		if( ! hasQuotes(str) )
			throw( new RuntimeException( String.format( "Lack of quotation: '%s'", str ) ) );

		return( str.substring( 1, str.length() - 1 ) );
	}
}
