/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.codec.impl;

import com.frojasg1.general.codec.GenericCodec;
import com.frojasg1.general.codec.GenericDecoder;
import com.frojasg1.general.codec.GenericEncoder;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class GenericCodecBase<DD, EE> implements GenericCodec<DD, EE> {
	
	protected GenericEncoder<DD, EE> _encoder;
	protected GenericDecoder<DD, EE> _decoder;

	public GenericCodecBase( GenericEncoder<DD, EE> encoder, GenericDecoder<DD, EE> decoder )
	{
		_encoder = encoder;
		_decoder = decoder;
	}

	@Override
	public EE encode(DD record) {
		return( _encoder.encode(record) );
	}

	@Override
	public DD decode(EE serializedValue) {
		return( _decoder.decode(serializedValue) );
	}

	@Override
	public void decode(EE serializedValue, DD result) {
		_decoder.decode( serializedValue, result );
	}
}
