/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.codec.impl;

import com.frojasg1.general.codec.GenericStringDecoderBase;
import com.frojasg1.general.string.translator.GenericStringTranslator;
import java.util.Objects;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public abstract class GenericLeafStringDecoderBase<RR> extends GenericStringDecoderBase<RR>
{
	private static final Logger LOGGER = LoggerFactory.getLogger(GenericLeafStringDecoderBase.class);

	protected Class<RR> _class;

	public GenericLeafStringDecoderBase( Class<RR> clazz, GenericStringDecoderBuilder builder )
	{
		super( builder );

		_class = clazz;
	}

	@Override
	public RR decode( String string )
	{
		return( super.decode( string ) );
	}

	@Override
	public RR decodeInternal( String string, String className )
	{
		String valueStr = removeClassNameAndBraces( string, className );

		RR result = createSimpleObject( valueStr );

		return( result );
	}

	@Override
	protected boolean isClassExpectedInternal( String className )
	{
		return( Objects.equals( className, _class.getName() ) );
	}

	@Override
	protected String getExpectedClassName()
	{
		return( _class.getName() );
	}

	@Override
	public void decodeInternalWithResult( String string, String className, RR result )
	{
		throw( new RuntimeException( "This functions should never be called, as we have to create the leaf object, instead of modifying it" ) );
	}

	protected abstract String getTransformedSimpleValueString( String valueStr );

	protected RR createSimpleObject( String valueStr )
	{
		return( createSimpleObjectInternal( getTransformedSimpleValueString( valueStr ) ) );
	}

	protected RR createSimpleObjectInternal( String valueStr )
	{
		return( GenericStringTranslator.instance().fromString(valueStr, _class) );
	}
}
