/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.codec.impl;

import com.frojasg1.general.codec.GenericStringDecoder;
import com.frojasg1.general.codec.SelfStringCodec;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public abstract class SelfStringCodecBase implements SelfStringCodec
{
	protected GenericStringDecoderBuilder _builder;
	
	public SelfStringCodecBase()
	{
		this( GenericStringDecoderBuilder.instance() );
	}

	public SelfStringCodecBase( GenericStringDecoderBuilder builder )
	{
		_builder = builder;
	}

	@Override
	public String encode()
	{
		return( DefaultGenericStringEncoderImpl.instance().encode(this) );
	}

	@Override
	public void decode(String encodedValue)
	{
		getGenericStringDecoder().decode(encodedValue, this);
	}

	protected GenericStringDecoder getGenericStringDecoder( )
	{
		return( getGenericStringDecoderBuilder().get( getClass() ) );
	}

	protected GenericStringDecoderBuilder getGenericStringDecoderBuilder()
	{
		return( _builder );
	}
}
