/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.collection;

import java.util.Collection;
import java.util.List;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 * @param <RR>	- Record
 * @param <CC>	- Container
 */
public interface ThreadSafeListWrapper<RR, CC extends List<RR>> {
	
	public CC getList();
	public CC getListCopy();
	public void setList( CC list );

	public void clear();

	public void add( RR elem );
	public void add( int index, RR elem );

	public void addAll( Collection<? extends RR> list );
	public void addAll( int index, Collection<? extends RR> list);
	public <R extends RR> void addAll( ThreadSafeListWrapper<R, ? extends List<? extends R>> list );
	public <R extends RR> void addAll( int index,
									ThreadSafeListWrapper<R, ? extends List<? extends R>> list );

	public RR get(int index);

	public CC remove( int index, int numElems );
	public RR remove( int index );
	public boolean remove( RR elem );

	public int size();

	public boolean contains( RR record );
}
