/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.collection.codec;

import com.frojasg1.general.ExecutionFunctions;
import com.frojasg1.general.analysers.GenListSplitterBase;
import com.frojasg1.general.analysers.impl.DefaultListQuotedDataSplitter;
import com.frojasg1.general.codec.GenericStringDecoder;
import com.frojasg1.general.codec.GenericStringDecoderBase;
import com.frojasg1.general.codec.impl.GenericStringDecoderBuilder;
import com.frojasg1.general.collection.impl.ThreadSafeGenListWrapper;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class ThreadSafeArrayListWrapperStringDecoder
	extends GenericStringDecoderBase<ThreadSafeGenListWrapper>
{
	protected String _separator;

	public ThreadSafeArrayListWrapperStringDecoder( GenericStringDecoderBuilder decoderBuilder )
	{
		this( decoderBuilder, ";" );
	}

	public ThreadSafeArrayListWrapperStringDecoder( GenericStringDecoderBuilder decoderBuilder, String separator )
	{
		super( decoderBuilder );
		_separator = separator;
	}

	public String getSeparator() {
		return _separator;
	}

	@Override
	public ThreadSafeGenListWrapper decodeInternal( String string, String className )
	{
		ThreadSafeGenListWrapper result = createEmptyObject((Class<? extends ThreadSafeGenListWrapper>) classForName( className ) );

		decode( string, result );

		return( result );
	}

	@Override
	public void decodeInternalWithResult( String string, String className, ThreadSafeGenListWrapper result )
	{
		String valueStr = removeClassNameAndBraces(string, className);

		result.clear();

		GenListSplitterBase splitter = splitter( valueStr, getSeparator() );
		String elemStr;
		do
		{
			elemStr = splitter.next();
			if( elemStr != null )
				result.add( decodeElem( elemStr ) );
		}
		while( elemStr != null );
	}

	protected Object decodeElem( String elemStr )
	{
		String elemClassName = getFirstClassName(elemStr);

		GenericStringDecoder decoder = getGenericStringDecoder(elemClassName);

		return( decoder.decode( elemStr ) );
	}

	protected GenListSplitterBase splitter( String string, String separator )
	{
		return( new DefaultListQuotedDataSplitter( string, separator ) );
	}


	protected ThreadSafeGenListWrapper createEmptyObject( Class<? extends ThreadSafeGenListWrapper> clazz)
	{
		return( ExecutionFunctions.instance().runtimeExceptionFunctionExecution( () -> clazz.newInstance() ) );
	}

	@Override
	protected boolean isClassExpectedInternal( String className )
	{
		return( ThreadSafeGenListWrapper.class.isAssignableFrom( classForName(className) ) );
	}

	@Override
	protected String getExpectedClassName()
	{
		return( "derived from: " + ThreadSafeGenListWrapper.class.getName() );
	}
}
