/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.comparators;

import java.util.Comparator;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class ComparatorFunctions
{
	protected static ComparatorFunctions INSTANCE = new ComparatorFunctions();

	public static ComparatorFunctions instance()
	{
		return( INSTANCE );
	}

	public <CC> Comparator<CC> notNullFirstComparator( Comparator<CC> comparator )
	{
		boolean notNullFirst = true;
		return( dealWithNullComparator( comparator, notNullFirst ) );
	}

	public <CC> Comparator<CC> dealWithNullComparator( Comparator<CC> comparator,
														boolean notNullFirst )
	{
		return( (c1, c2) -> {
			int result = 0;
			if( c1 != c2 ) {
				if( c1 == null )
				{
					if( notNullFirst )
						result = 1;
					else
						result = -1;
				}
				else if( c2 == null )
				{
					if( notNullFirst )
						result = -1;
					else
						result = 1;
				}
				else
					result = comparator.compare(c1, c2);
			}
			return( result );
		});
	}
}
