/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.comparators;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.function.Function;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class ComparatorOfAttributes<RR> implements Comparator<RR> {

	protected List< Function<RR, ? extends Comparable<?>> > _getters = new ArrayList<>();

	public <C extends Comparable<C>> void add( Function<RR, C> getter )
	{
		_getters.add( getter );
	}

	@Override
	public int compare(RR o1, RR o2) {
		int result = 0;
		for( Function getter: _getters )
		{
			result = compareAtt( get( o1, getter ), get( o2, getter ) );
			if( result != 0 )
				break;
		}

		return( result );
	}

	protected <C extends Comparable<C>> C get( RR obj, Function<RR, C> getter )
	{
		C result = null;
		if( obj != null )
			result = getter.apply(obj);

		return( result );
	}


	public <C extends Comparable<C>> int compareAtt(C arg0, C arg1) {
		int result = 0;
		if( ( arg0 == null ) && ( arg1 == null ) )
		{
			
		}
		else if( arg0 == null )
			result = -1;
		else if( arg1 == null )
			result = 1;
		else result = arg0.compareTo( arg1 );

		return( result );
	}
}
