/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.dialogs.filefilter;

import com.frojasg1.general.ExecutionFunctions;
import java.lang.reflect.Constructor;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class GenericFileFilterTranslatorBase<FF extends GenericFileFilter> {
	protected Map<GenericFileFilter, FF> _map;

	protected Class<FF> _outputClass;

	protected Constructor<FF> _constructor;

	public GenericFileFilterTranslatorBase( Class<FF> outputClass )
	{
		_outputClass = outputClass;
		_map = new ConcurrentHashMap<>();
	}

	public FF translateFileFilter( GenericFileFilter filter )
	{
		FF result = null;
		if(filter != null) {
			if (_outputClass.isInstance(filter))
				result = (FF) filter;
			else
				result = translateFileFilterInternal(filter);
		}

		return( result );
	}

	protected FF translateFileFilterInternal( GenericFileFilter filter )
	{
		return( _map.computeIfAbsent( filter, this::createTranslatedFilter ) );
	}

	public <CC> CC safeFunctionExecution( ExecutionFunctions.UnsafeFunction<CC> run )
	{
		return( ExecutionFunctions.instance().safeFunctionExecution( run ) );
	}

	protected synchronized Constructor<FF> getConstructor()
	{
		if( _constructor == null )
			_constructor = safeFunctionExecution( () -> _outputClass.getConstructor( GenericFileFilter.class ) );

		return( _constructor );
	}

	protected FF createTranslatedFilter( GenericFileFilter filter )
	{
		return( safeFunctionExecution( () -> getConstructor().newInstance(filter) ) );
	}
}
