/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.dialogs.filefilter.impl;

import com.frojasg1.applications.common.configuration.InternationalizedStringConf;
import com.frojasg1.general.dialogs.DialogsWrapperFunctions;
import com.frojasg1.general.dialogs.filefilter.GenericFileFilter;
import com.frojasg1.generic.GenericFunctions;
import java.util.ArrayList;
import java.util.List;
import java.util.function.Supplier;
import java.util.stream.Collectors;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public abstract class GenericFileFilterArrayBase
{
	protected InternationalizedStringConf _internationalizedStringConf = null;

	protected GenericFileFilter[] _array;

	public GenericFileFilterArrayBase(
		InternationalizedStringConf internationalizedStringConf )
	{
		_internationalizedStringConf = internationalizedStringConf;
	}

	public GenericFileFilterArrayBase init()
	{
		_array = createArray();

		return this;
	}

	protected abstract void fillListAndSort( List<GenericFileFilter> list );

	protected GenericFileFilter[] createArray()
	{
		List<GenericFileFilter> list = new ArrayList<>();
		fillListAndSort( list );
		list = list.stream()
			.map( this::translateFileFilter )
			.collect( Collectors.toList() );

		return( list.toArray( new GenericFileFilter[list.size()] ) );
	}

	public GenericFileFilter[] getArray()
	{
		return( _array );
	}

	protected GenericFileFilter translateFileFilter( GenericFileFilter filter )
	{
		return( GenericFunctions.instance().getDialogsWrapper().translateGenericFileFilter(filter) );
	}

	public String getInternationalString(String label)
	{
		return( _internationalizedStringConf.getInternationalString(label ) );
	}

	public String createCustomInternationalString(String label, Object... args) {
		return( _internationalizedStringConf.createCustomInternationalString( label, args ) );
	}

	public GenericFileFilterIsAnyExtension createExtensionFilter( Supplier<String> descriptionFunction,
																String ... extensions )
	{
		return( DialogsWrapperFunctions.instance().createExtensionFilter(descriptionFunction, extensions ) );
	}

	protected String getSingleExtension(GenericFileFilter filter) {
		String result = null;
		if(filter instanceof GenericFileFilterIsAnyExtension) {
			String[] array = ((GenericFileFilterIsAnyExtension) filter).getExtensions();
			if(array.length == 1) {
				result = array[0];
			}
		}
		return result;
	}

	public GenericFileFilter getFileFilter(String extension) {
		GenericFileFilter result = null;
		if(extension != null) {
			GenericFileFilter[] array = getArray();
			String filterExtension;
			for (GenericFileFilter filter : array) {
				filterExtension = getSingleExtension(filter);
				if(filterExtension != null) {
					if (extension.compareToIgnoreCase(filterExtension) == 0) {
						result = filter;
						break;
					}
				}
			}
		}
		return result;
	}
}
