/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.dialogs.filefilter.impl;

import com.frojasg1.general.dialogs.filefilter.impl.helpers.GenericFileChooserHelper;
import java.util.Arrays;
import java.util.function.Supplier;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class GenericFileFilterIsAnyExtension extends GenericFileFilterBase {

	protected String[] _extensions;

	public GenericFileFilterIsAnyExtension( Supplier<String> descriptionSupplier,
											String ... extensions )
	{
		super( file -> GenericFileChooserHelper.instance().isAnyExtension(file, extensions),
			descriptionSupplier );

		_extensions = extensions;
	}

	@Override
	public int hashCode() {
		int hash = 3;
		hash = 37 * hash + Arrays.deepHashCode(this._extensions);
		return hash;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		final GenericFileFilterIsAnyExtension other = (GenericFileFilterIsAnyExtension) obj;
		if (!Arrays.deepEquals(this._extensions, other._extensions)) {
			return false;
		}
		return true;
	}

	public String[] getExtensions() {
		return _extensions;
	}
}
