/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.dialogs.filefilter.impl.gen;

import com.frojasg1.applications.common.configuration.imp.InternationalizedStringConfImp;
import com.frojasg1.general.GenericConstants;
import com.frojasg1.general.dialogs.filefilter.GenericFileFilter;
import com.frojasg1.general.dialogs.filefilter.impl.GenericFileFilterChooserBase;
import com.frojasg1.general.dialogs.filefilter.impl.executable.GenericFileFilterForExecutable;
import java.util.Map;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class GenericFileFilterChooserGeneral extends GenericFileFilterChooserBase
{
	public static final String GLOBAL_CONF_FILE_NAME = "GenericFileFilterChooserBase.properties";
	protected static final String CONF_GENERIC_FILE_FILTER_FOR_EXECUTABLE_BY_ATTRIBUTES_DESCRIPTION = "CONF_GENERIC_FILE_FILTER_FOR_EXECUTABLE_BY_ATTRIBUTES_DESCRIPTION";

	public static final int GENERIC_FILE_FILTER_FOR_EXECUTABLE_BY_ATTRIBUTES = 0;
	public static final int XML_FILE_FILTER = 1;
	public static final int PDF_FILE_FILTER = 2;
	public static final int PGN_FILE_FILTER = 3;
	public static final int MP3_FILE_FILTER = 4;
	public static final int M3U_FILE_FILTER = 5;
	public static final int ALL_IMAGE_FORMATS_FILE_FILTER = 6;

	protected static GenericFileFilterChooserGeneral INSTANCE = new GenericFileFilterChooserGeneral();

	public static GenericFileFilterChooserGeneral instance()
	{
		return( INSTANCE );
	}

	protected GenericFileFilterChooserGeneral()
	{
		this(true);
	}

	protected GenericFileFilterChooserGeneral(boolean init)
	{
		super( new InternationalizedStringConfImp( GLOBAL_CONF_FILE_NAME,
					GenericConstants.sa_PROPERTIES_PATH_IN_JAR ) );
		
		if( init )
			init();
	}

	@Override
	protected void fillMap(Map<Integer, GenericFileFilter> map)
	{
		map.put( GENERIC_FILE_FILTER_FOR_EXECUTABLE_BY_ATTRIBUTES, createGenericFileFilterForExecutable() );
		map.put( XML_FILE_FILTER, createExtensionFilter(() -> ".xml : Extensible markup language", "xml") );
		map.put( PDF_FILE_FILTER, createExtensionFilter(() -> ".pdf : Portable document format", "pdf") );
		map.put( PGN_FILE_FILTER, createExtensionFilter(() -> ".pgn : Chess game notation", "pgn") );
		map.put( MP3_FILE_FILTER, createExtensionFilter(() -> ".mp3 : Mpeg Audio Layer 3", "mp3") );
		map.put( M3U_FILE_FILTER, createExtensionFilter(() -> ".m3u : m3u playlist file", "m3u") );
		map.put( ALL_IMAGE_FORMATS_FILE_FILTER, createExtensionFilter(() -> "Images", "jpg", "jpeg", "bmp", "gif", "png", "tiff", "tif") );
	}

	public GenericFileFilter createGenericFileFilterForExecutable()
	{
		GenericFileFilterForExecutable result = new GenericFileFilterForExecutable(
			() -> getInternationalString( CONF_GENERIC_FILE_FILTER_FOR_EXECUTABLE_BY_ATTRIBUTES_DESCRIPTION ) );

		return( result );
	}
}
