/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.executor;

import com.frojasg1.general.ExecutionFunctions;
import com.frojasg1.general.executor.imp.ExecutorBase;
import com.frojasg1.general.threads.ThreadFunctions;
import java.util.function.Consumer;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public interface TaskExecutor {
	
	public void addPendingExecutor( ExecutorInterface executor );
	public void addPendingNonStopableExecutor( Runnable executor );

	public <RR> void addPendingExecutor( ExecutionFunctions.UnsafeFunction<RR> function, Consumer<RR> callback );

	default public void executeTask( Runnable runnable )
	{
		addPendingNonStopableExecutor( runnable );
	}

	default public void executeDelayedInvokeEventDispatchThread( Runnable runnable, int delayMs )
	{
		addPendingNonStopableExecutor( () -> ThreadFunctions.instance().delayedInvokeEventDispatchThread(runnable, delayMs) );
	}

	default public void executeDelayedTask( Runnable runnable, int delayMs )
	{
		addPendingNonStopableExecutor( () -> ThreadFunctions.instance().invokeWithDelay(runnable, delayMs) );
	}
}
