/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.executor.background;

import com.frojasg1.general.ExecutionFunctions;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicReference;
import java.util.function.BiFunction;
import java.util.function.Consumer;
import java.util.function.Function;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class BackgroundProcessor<RR> {
	protected AtomicReference<RR> _resultElemRef = new AtomicReference<>();

	protected AtomicBoolean _hasToCancel = new AtomicBoolean();

	protected BiFunction<ExecutionFunctions.UnsafeFunction<RR>, Consumer<RR>, Boolean> executeBackgroundFunction;

	public RR getResultElem() {
		return _resultElemRef.get();
	}

	public synchronized void setResultElem(RR resultElemRef) {
////		if(resultElemRef == null) {
//			cancel();
////		}
		_resultElemRef.set(resultElemRef);
	}

	public BiFunction<ExecutionFunctions.UnsafeFunction<RR>, Consumer<RR>, Boolean> getExecuteBackgroundFunction() {
		return executeBackgroundFunction;
	}

	public void setExecuteBackgroundFunction(BiFunction<ExecutionFunctions.UnsafeFunction<RR>, Consumer<RR>, Boolean> executeBackgroundFunction) {
		this.executeBackgroundFunction = executeBackgroundFunction;
	}

	public synchronized void reset()
	{
		cancel();
		setResultElem(null);
	}

	public synchronized void cancel()
	{
		_hasToCancel.set( true );
	}

	protected AtomicBoolean replaceHasToCancel()
	{
		cancel();

		_hasToCancel = new AtomicBoolean();

		return( _hasToCancel );
	}

	public boolean process( Function<AtomicBoolean, RR> backgroundFunction,
							Consumer<RR> callback )
	{
		boolean result = false;

		if( backgroundFunction != null )
		{
			AtomicBoolean hasToCancel = replaceHasToCancel();

			result = addPendingExecutor(() -> backgroundFunction.apply( hasToCancel ), callback );
		}

		return( result );
	}

	protected void callback( RR result, Consumer<RR> callback )
	{
		setResultElem(result);
		if( callback != null )
			callback.accept(result);
	}

	protected boolean addPendingExecutor( ExecutionFunctions.UnsafeFunction<RR> function, Consumer<RR> callback )
	{
		boolean result = false;
		BiFunction<ExecutionFunctions.UnsafeFunction<RR>, Consumer<RR>, Boolean> addPendingExecutorFunction =
			getExecuteBackgroundFunction();

		if( addPendingExecutorFunction != null )
			result = addPendingExecutorFunction.apply(function, res -> callback( res, callback ) );

		return( result );
	}
}
