/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.executor.imp;

import com.frojasg1.general.ExecutionFunctions;
import java.util.function.Consumer;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class FunctionWithCallbackExecutor<RR> extends ExecutorBase
{
	protected ExecutionFunctions.UnsafeFunction<RR> _function;
	protected Consumer<RR> _callback;

	public FunctionWithCallbackExecutor( ExecutionFunctions.UnsafeFunction<RR> function, Consumer<RR> callback )
	{
		_function = function;
		_callback = callback;
	}

	@Override
	public void run()
	{
		RR result = ExecutionFunctions.instance().safeFunctionExecution( _function::run );

		if( ( _callback != null ) && !getHasToStop() )
			_callback.accept(result);
	}
}
