/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.getters;

import com.frojasg1.general.NullFunctions;
import com.frojasg1.general.copy.CopyFacilities;
import com.frojasg1.general.copy.CopyFacilitiesFunctions;
import java.util.function.Function;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class GetterDelegator<CC>
{
	protected CC _parentToDelegateTo;

	public GetterDelegator(CC _parentToDelegateTo) {
		this._parentToDelegateTo = _parentToDelegateTo;
	}

	public void setParentToDelegateTo(CC _parentToDelegateTo) {
		this._parentToDelegateTo = _parentToDelegateTo;
	}

	public CC getParentToDelegateTo() {
		return _parentToDelegateTo;
	}

	protected boolean canDelegate()
	{
		return( getParentToDelegateTo() != null );
	}

	protected <R> R getDelegating( Function<CC, R> directGetter,
									Function<CC, R> delegatedGetter )
	{
		R result = directGetter.apply((CC) this);
		if( ( result == null ) && canDelegate() )
			result = delegatedGetter.apply( getParentToDelegateTo() );

		return( result );
	}

	protected <C extends CopyFacilities<C>> C copy( C obj, C that )
	{
		return( CopyFacilitiesFunctions.instance().copy(obj, that) );
	}

	protected <CC, RR> RR getIfNotNull( CC obj, Function<CC,RR> getter )
	{
		return( NullFunctions.instance().getIfNotNull(obj, getter) );
	}
}
