/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.getters;

import java.util.function.Function;
import java.util.function.Supplier;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class OneParameterLazyHolder<I1, CC> extends LazyHolderBase<CC>
{
	protected Function<I1, CC> _getterFunction;

	public OneParameterLazyHolder( Function<I1, CC> getterFunction )
	{
		_getterFunction = getterFunction;
	}

	public CC get(I1 input)
	{
		if( _value.get() == null )
			calculate(input);

		return( _value.get() );
	}

	protected synchronized CC calculate(I1 input)
	{
		if( _value.get() == null )
			set(_getterFunction.apply(input));

		return( _value.get() );
	}
}
