/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.io.model;

import com.frojasg1.general.collection.ModifiedStatus;
import com.frojasg1.general.FileFunctions;
import java.util.Objects;
import java.util.concurrent.atomic.AtomicReference;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class GenericIOModelContextBase<MM extends ModifiedStatus> implements GenericIOModelContext<MM> {

	protected String _fileName;
	protected String _charsetName;
	protected MM _model;

	@Override
	public String getCharsetName() {
		return( _charsetName );
	}

	@Override
	public GenericIOModelContextBase<MM> setCharsetName(String charsetName) {
		_charsetName = charsetName;
		return( this );
	}

	@Override
	public String getFileName() {
		return( _fileName );
	}

	@Override
	public GenericIOModelContextBase<MM> setFileName(String fileName) {
		_fileName = fileName;

		return( this );
	}

	@Override
	public MM getModel() {
		return( _model );
	}

	@Override
	public GenericIOModelContextBase<MM> setModel(MM model) {
		_model = model;

		return( this );
	}

	@Override
	public boolean hasBeenModfied()
	{
		boolean result = false;
		if( getModel() != null )
			result = getModel().hasBeenModified();

		return( result );
	}

	protected String getDirName( String fileName )
	{
		String result = null;
		if( fileName != null )
			result = FileFunctions.instance().getDirName(fileName);

		return( result );
	}
}
