/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.io.model;

import com.frojasg1.general.CollectionFunctions;
import com.frojasg1.general.codec.GenericSelfCodec;
import java.util.Enumeration;
import java.util.List;
import java.util.function.Function;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 * @param <RR>	- Record
 */
public class ListEncoderToStringBase {
	
	protected String _separator;
	
	public ListEncoderToStringBase( String separator )
	{
		_separator = separator;
	}

	protected <RR> List<RR> copyToList( Enumeration<RR> source )
	{
		return( CollectionFunctions.instance().copyToList(source) );
	}

	protected <RR> String encode( List<RR> list, Function<RR, String> translator )
	{
		String separator = "";
		StringBuilder result = new StringBuilder();
		if( list != null )
			for( RR elem: list )
			{
				result.append(separator).append(translator.apply(elem));
				separator = _separator;
			}

		return( result.toString() );
	}

	protected <RR extends GenericSelfCodec<String>> String encode( List<RR> list )
	{
		return( encode( list, GenericSelfCodec::encode ) );
	}
}
