/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.list;

import java.util.ArrayList;
import java.util.List;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class ListWrapper<VV>
{
	protected List<VV> _list;

	public ListWrapper()
	{
		this( true );
	}

	protected ListWrapper( boolean init )
	{
		if( init )
			init();
	}

	protected void init()
	{
		_list = createList();
	}

	protected List<VV> createList()
	{
		return( new ArrayList<>() );
	}

	public void add( VV value )
	{
		_list.add(value);
	}

	public void add( int index, VV value )
	{
		_list.add(index, value);
	}

	public boolean remove( VV value )
	{
		return( _list.remove(value) );
	}

	public void clear()
	{
		_list.clear();
	}

	public int size()
	{
		return( _list.size() );
	}

	public List<VV> getList()
	{
		return( _list );
	}

	public List<VV> getListCopy()
	{
		List<VV> result = createList();
		for( VV value: getList() )
			result.add(value);

		return( result );
	}

	public boolean contains( VV value )
	{
		return( getList().contains(value) );
	}

	public void addAll( List<VV> that )
	{
		getList().addAll(that);
	}

	public void replace( List<VV> that )
	{
		getList().clear();
		addAll(that);
	}

	public VV get( int index )
	{
		return( getList().get(index) );
	}

	public int indexOf( VV elem )
	{
		return( getList().indexOf(elem) );
	}
}
