/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.listeners;

import java.util.function.Consumer;
import java.util.function.Function;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 * @param <LT>	- Listener
 * @param <EV>	- Event
 */
public class ListOfListenersNotifyEvtBase <LT, EV> extends ListOfListenersBase<LT>
{
	protected Function<LT, Consumer<? super EV>> _consumerGetter;

	public ListOfListenersNotifyEvtBase(Function<LT, Consumer<? super EV>> _consumerGetter) {
		this._consumerGetter = _consumerGetter;
	}

	public ListOfListenersNotifyEvtBase(ListOfListenersNotifyEvtBase<LT, EV> that) {
		super(that);

		this._consumerGetter = that._consumerGetter;
	}

	public Function<LT, Consumer<? super EV>> getConsumerGetter() {
		return _consumerGetter;
	}

	protected Consumer<? super EV> getEvtConsumer( LT listener )
	{
		return( getConsumerGetter().apply(listener) );
	}

	protected void notifyListener( LT listener, EV evt )
	{
		if( hasToSend(listener, evt) )
			getEvtConsumer(listener).accept(evt);
	}

	protected boolean hasToSend( LT listener, EV evt )
	{
		return( true );
	}

	public void notifyEvt( EV evt )
	{
		for( LT listener: getList() )
		{
			safeMethodExecution( () -> notifyListener(listener, evt) );
		}
	}
}
