/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.listeners.filterevents;

import com.frojasg1.general.listeners.ListOfListenersNotifyEvtBase;
import java.util.ListIterator;
import java.util.function.Consumer;
import java.util.function.Function;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 * @param <EV>	- Event
 * @param <ET>	- Event type
 * @param <LT>	- Listener
 */
public abstract class ListOfEventContextListenersBase<EV, ET, LT extends EventListenerContextBase<EV, ET>>
	extends ListOfListenersNotifyEvtBase<LT, EV> {

	protected Function<ET, Long> _eventTypeToLongFunction;
	protected Function<EV, ET> _eventToEventTypeFunction;

	public ListOfEventContextListenersBase( Function<ET, Long> eventTypeToLongFunction,
											Function<EV, ET> eventToEventTypeFunction)
	{
		super( EventListenerContextBase::getListenerFunction );

		_eventTypeToLongFunction = eventTypeToLongFunction;
		_eventToEventTypeFunction = eventToEventTypeFunction;
	}

	protected Function<ET, Long> getEventTypeToLongFunction()
	{
		return _eventTypeToLongFunction;
	}

	protected Function<EV, ET> getEventToEventTypeFunction()
	{
		return _eventToEventTypeFunction;
	}

	public LT addListenerContext( Consumer<? super EV> consumer, ET ... eventTypesToRegister )
	{
		LT listener = createListenerContext( consumer, eventTypesToRegister );
		add( listener );

		return listener;
	}

	protected ET getEventType( EV evt )
	{
		return( getEventToEventTypeFunction().apply(evt) );
	}

	protected abstract LT createListenerContextInternal( Consumer<? super EV> consumer, ET ... eventTypesToRegister );

	protected LT createListenerContext( Consumer<? super EV> consumer, ET ... eventTypesToRegister )
	{
		return( createListenerContextInternal( consumer, eventTypesToRegister ) );
	}

	@Override
	protected boolean hasToSend( LT listener, EV evt )
	{
		return( listener.isActivated( getEventType(evt) ) );
	}

	public void remove( Consumer<? super EV> listenerFunction, ET ... eventTypesToRemove )
	{
		ListIterator<LT> it = _list.listIterator();
		while( it.hasNext() )
		{
			LT elem = it.next();
			if( elem.getListenerFunction().equals( listenerFunction ) )
			{
				if( ( eventTypesToRemove != null ) && ( eventTypesToRemove.length > 0 ) )
					elem.deactivateEventTypes(eventTypesToRemove);
				else
					it.remove();
			}
		}
	}
}
