/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.map.multimap.dynamic;

import java.util.function.Function;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 * @param <EV>	- element value (key)
 * @param <VV>	- value (multi-element value)
 * @param <CC>	- element Context
 * @param <EE>	- Map Entry
 */
public abstract class DynamicMultiMap<EV,
									VV extends DynamicMultiMapElementBase<EV>,
									CC >
	extends DynamicMultiMapGen<EV, VV, CC, DynamicMultiMapMapEntry<EV, VV, CC>>
{
	protected DynamicMultiMap(Class<EV> keyClass)
	{
		this( keyClass, false );
	}

	protected DynamicMultiMap(Class<EV> keyClass, boolean init)
	{
		super( keyClass );

		if( init )
			init();
	}

	@Override
    protected void init() {
        super.init();
    }

	@Override
	protected <K> DynamicMultiMapMapEntry<EV, VV, CC> createEmptyMultiMapMapEntry( Class<K> keyClass, Function<VV, K> keyGetter )
	{
		return( new DynamicMultiMapMapEntry<>( (Class<EV>) keyClass, (Function<VV, EV>) keyGetter) );
	}
}
