/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.map.multimap.dynamic;

import com.frojasg1.general.map.MapFunctions;
import com.frojasg1.general.map.multimap.MultiMapGen;
import java.util.Map;
import java.util.function.Function;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 * @param <EV>	- element value (key)
 * @param <VV>	- value (multi-element value)
 * @param <CC>	- element Context
 * @param <EE>	- Map Entry
 */
public abstract class DynamicMultiMapGen<EV,
										VV extends DynamicMultiMapElementBase<EV>,
										CC,
										EE extends DynamicMultiMapMapEntry<EV, VV, CC>>
	extends MultiMapGen<VV, EE>
{
	protected Class<EV> _keyClass;

	protected DynamicMultiMapGen(Class<EV> keyClass)
	{
		this( keyClass, false );
	}

	protected DynamicMultiMapGen(Class<EV> keyClass, boolean init)
	{
		_keyClass = keyClass;

		if( init )
			init();
	}

	@Override
    protected void init() {
        super.init();
    }

	public Class<EV> getKeyClass() {
		return _keyClass;
	}

	@Override
	protected abstract <K> EE createEmptyMultiMapMapEntry( Class<K> keyClass, Function<VV, K> keyGetter );

	protected EE createEmptyMultiMapMapEntry( String name )
	{
		return( createMultiMapMapEntry( name, _keyClass, value -> value.get(name) ) );
	}

	protected EE addMultiMapMapEntry( Map<String, EE> map, String name, CC elementContext)
	{
		if( mapByName.get(name) != null )
			throw( new IllegalArgumentException( "Cannot add " + name +
						" MultiMapMapElement. Already existed." ) );

		EE result = createEmptyMultiMapMapEntry(name);
		result.setElementContext(elementContext);

		addMultiMapMapEntry( map, name, result );

		return( result );
	}

	@Override
	protected void renameMapEntryName( EE mme, String oldName, String newName )
	{
		super.renameMapEntryName(mme, oldName, newName);

		DynamicMultiMapMapEntry<EV, VV, CC> dmme = (DynamicMultiMapMapEntry<EV, VV, CC>) mme;

		Map<EV, VV> map = dmme.getMap();
		MapFunctions.instance().browseMap( map, (k, v) -> renameElementOfValue(v, oldName, newName) );
	}

	protected void renameElementOfValue( VV value, String oldName, String newName )
	{
		value.put( newName, value.remove(oldName) );
	}
}
