/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.map.multimap.dynamic;

import com.frojasg1.general.map.multimap.MultiMapMapEntry;
import java.util.Map;
import java.util.function.Function;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 * @param <KK>	- Key
 * @param <VV>	- Value
 * @param <CC>	- Entry Context
 */
public class DynamicMultiMapMapEntry<KK, VV extends DynamicMultiMapElementBase<? extends KK>, CC>
	extends MultiMapMapEntry<KK, VV>
{
	protected CC _elementContext;

	public DynamicMultiMapMapEntry( Class<KK> keyClass, Function<VV, KK> keyGetter )
	{
		super( keyClass, keyGetter );
	}

	public CC getElementContext() {
		return _elementContext;
	}

	public void setElementContext(CC _elementContext) {
		this._elementContext = _elementContext;
	}

	@Override
	public void removeName(String name)
	{
		Map<KK, VV> map = getMap();
		VV value;
		for( KK key: map.keySet() )
		{
			value = map.get(key);
			if( value != null )
				value.remove(name);
		}
	}
}
