/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.map.multimap.dynamic.deductible;

import com.frojasg1.general.map.multimap.dynamic.DynamicMultiMapMapEntry;
import java.util.concurrent.atomic.AtomicReference;
import java.util.function.Function;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 * @param <EV>	- element value (key)
 * @param <VV>	- value (multi-element value)
 * @param <CC>	- element Context
 */
public abstract class DeductibleDynamicMultiMap<EV,
												VV extends DeductibleDynamicMultiMapElementBase<EV>,
												CC extends DeductibleDynamicMultiMapMapEntryContext<EV>>
	extends DeductibleDynamicMultiMapGen<EV, VV, CC, DynamicMultiMapMapEntry<EV, VV, CC>>
{
	protected AtomicReference<String> _masterNameRef;

	public DeductibleDynamicMultiMap(Class<EV> keyClass, String masterName) {
		this(keyClass, masterName, true);
	}

	protected DeductibleDynamicMultiMap(Class<EV> keyClass, String masterName,
										boolean init) {
		super(keyClass, masterName);
		_masterNameRef = new AtomicReference<>(masterName);

		if( init )
			init();
	}

	@Override
	protected <K> DynamicMultiMapMapEntry<EV, VV, CC> createEmptyMultiMapMapEntry( Class<K> keyClass, Function<VV, K> keyGetter )
	{
		return( new DynamicMultiMapMapEntry<>( (Class<EV>) keyClass, (Function<VV, EV>) keyGetter) );
	}

//	protected abstract UnaryOperator<EV> getFromMasterDirectTranslator( DynamicMultiMapMapEntry<EV, VV, CC> dmme );

//	protected abstract UnaryOperator<EV> getToMasterMapper( DynamicMultiMapMapEntry<EV, VV, CC> dmme );
//	protected abstract UnaryOperator<EV> getFromMasterMapper( DynamicMultiMapMapEntry<EV, VV, CC> dmme );

	protected abstract VV createEmptyMultiValue();

	public VV getMasterElement( EV elementValue )
	{
		return( get( getMasterName(), elementValue ) );
	}
}
