/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.process;

import java.util.ArrayList;
import java.util.List;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class ProcExecResult {
	protected static final int SUCCESS_CODE = 0;

	protected String[] _commandAndParams;

	protected List<String> _standardOutput = new ArrayList<>();
	protected List<String> _standardError = new ArrayList<>();;
	protected int _exitValue;

	public void setCommand( String[] commandAndParams )
	{
		_commandAndParams = commandAndParams;
	}

	public String getCommand( )
	{
		StringBuilder sb = new StringBuilder();

		for( String line: _commandAndParams )
			sb.append(line).append( " " );

		return( sb.toString() );
	}

	public int getExitValue() {
		return _exitValue;
	}

	public void setExitValue(int _exitValue) {
		this._exitValue = _exitValue;
	}

	public List<String> getStandardOutput() {
		return _standardOutput;
	}

	public List<String> getStandardError() {
		return _standardError;
	}
	
	public void addStandardOutputLine( String line )
	{
		_standardOutput.add( line );
	}
	
	public void addStandardErrorLine( String line )
	{
		_standardError.add( line );
	}

	public boolean wasSuccessful()
	{
		return( getExitValue() == SUCCESS_CODE );
	}

	public String getAllOutputInAString()
	{
		return( concat( getStandardOutput() ) );
	}

	public String getAllErrorInAString()
	{
		return( concat( getStandardError() ) );
	}

	protected String concat( List<String> list )
	{
		StringBuilder sb = new StringBuilder();

		for( String line: list )
			sb.append(line).append( "\n" );

		return( sb.toString() );
	}
}
