/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.proxy;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public abstract class InvocationHandlerBase<OO, CC extends ProxyContextBase<OO>>
	implements InvocationHandler, ProxyInterface<OO, CC>
{
	protected CC _proxyContext;

	public InvocationHandlerBase( CC proxyContext )
	{
		_proxyContext = proxyContext;
	}

	public OO getObject()
	{
		return( getProxyContext().getObj() );
	}

	public CC getProxyContext()
	{
		return( _proxyContext );
	}

	public Object invoke(Method method, Object[] args) throws Throwable {
		return( method.invoke( getObject(), args ) );
	}

	protected boolean isProxyInterfaceFunction( Method method )
	{
		return( "getProxyContext".equals( method.getName() ) &&
				( method.getParameterTypes().length == 0 ) );
	}

	@Override
	public abstract Object invoke(Object proxy, Method method, Object[] args) throws Throwable;
}
