/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.proxy;

import java.lang.reflect.Proxy;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class ProxyFunctions {
	
	protected static class LazyHolder
	{
		public static final ProxyFunctions INSTANCE = new ProxyFunctions();
	}

	public static ProxyFunctions instance()
	{
		return( LazyHolder.INSTANCE );
	}

	protected <OO> ProxyContextBase<OO> createProxyContextBase( OO obj )
	{
		return( new ProxyContextBase().setObj(obj) );
	}

	public <CC> CC createReadOnlyProxy( CC obj, Class<CC> interfaceToImplement )
	{
		return( (CC) Proxy.newProxyInstance( ProxyFunctions.class.getClassLoader(), 
											new Class[] { interfaceToImplement, ProxyInterface.class }, 
											new ReadOnlyInvocationHandler<CC>(createProxyContextBase(obj)) ) );
	}
}
