/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.result;

import java.util.function.Consumer;

public class GenericResult {

    protected Successfulness successfulness;
    protected String message;
    protected Throwable throwable;
    protected Consumer<GenericResult> callback;

    public GenericResult() {
        this(null);
    }

    public GenericResult(Successfulness successfulness) {
        this.successfulness = successfulness;
    }

    public Successfulness getSuccessfulness() {
        return successfulness;
    }

    public void setSuccessfulness(Successfulness successfulness) {
        this.successfulness = successfulness;
    }

    public String getMessage() {
        return message;
    }

    public void setMessage(String message) {
        this.message = message;
    }

    public Throwable getThrowable() {
        return throwable;
    }

    public void setThrowable(Throwable throwable) {
        this.throwable = throwable;
    }

    public boolean isSuccess() {
        return (successfulness != null) && successfulness.isSuccess();
    }

    public Consumer<GenericResult> getCallback() {
        return callback;
    }

    public void setCallback(Consumer<GenericResult> callback) {
        this.callback = callback;
    }

    public void updateResult(Successfulness successfulness, String message, Throwable throwable) {
        setSuccessfulness(successfulness);
        setMessage(message);
        setThrowable(throwable);
    }

    public void doCallback() {
        callback.accept(this);
    }

    public void exceptionToCallback(Throwable th) {
        updateResult(Successfulness.FAILURE, th.getMessage(), th);

        doCallback();
    }

    public void successToCallback() {
        updateResult(Successfulness.SUCCESS, null, null);
        doCallback();
    }
}
