/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.threads;

import java.util.function.Supplier;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public abstract class LockAndConditionOwner {
	protected LockAndConditionBase _lock;

	protected void init()
	{
		_lock = createLockAndCondition();
	}

	protected abstract int getNumConditionsForLock();

	protected LockAndConditionBase createLockAndCondition()
	{
		return( new LockAndConditionBase( getNumConditionsForLock() ) );
	}

	public LockAndConditionBase getLock() {
		return _lock;
	}

	protected boolean conditionHasWaiters( int conditionIndex ) {
		return getLock().hasWaiters( conditionIndex );
	}

	protected void lockProcedure( Runnable procedure )
	{
		getLock().lockProcedure(procedure);
	}

	protected boolean awaitInterrupted( int conditionIndex, Integer timeoutMs )
	{
		return( getLock().awaitInterrupted( conditionIndex, timeoutMs) );
	}

	protected void ifHasWaitersSignal( int conditionIndex )
	{
		getLock().ifHasWaitersSignal(conditionIndex);
	}

	protected boolean hasWaiters( int conditionIndex )
	{
		return( getLock().hasWaiters( conditionIndex ) );
	}

	protected void signalCondition( int conditionIndex )
	{
		getLock().signalCondition( conditionIndex );
	}

	protected void signalAllCondition( int conditionIndex )
	{
		getLock().signalAllCondition(conditionIndex);
	}

	public void taskToSignalGen( Runnable runnable,
								int conditionIndex)
	{
		getLock().taskToSignalGen( runnable, conditionIndex );
	}

	protected void signalAll()
	{
		getLock().signalAll();
	}

	protected <CC> CC lockFunction( Supplier<CC> function )
	{
		return( getLock().lockFunction( function ) );
	}

	protected boolean isInterrupted()
	{
		return( Thread.currentThread().isInterrupted() );
	}
}
