/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.threads;

import java.util.function.Consumer;
import java.util.function.Function;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class SynchronizedFunctions {

	protected static class LazyHolder
	{
		public static final SynchronizedFunctions INSTANCE = new SynchronizedFunctions();
	}

	public static SynchronizedFunctions instance()
	{
		return( LazyHolder.INSTANCE );
	}

	public <CC> void threadSafeMethod( CC source, Consumer<CC> method )
	{
		if( source != null )
		{
			synchronized( source )
			{
				method.accept(source);
			}
		}
	}

	/*
		<CC> - Class of source
		<RR> - Result
	*/
	public <CC, RR> RR threadSafeFunction( CC source, Function<CC, RR> function )
	{
		RR result = null;
		if( source != null )
		{
			synchronized( source )
			{
				result = function.apply(source);
			}
		}

		return( result );
	}
}
