/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.threads;

import com.frojasg1.general.ExecutionFunctions;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class ThreadAndRunnableBase<RR extends RunnableBase> {
	
	protected Thread _thread;
	protected RR _runnable;

	protected String _name;

	public ThreadAndRunnableBase()
	{
		this(null);
	}

	public ThreadAndRunnableBase( String name )
	{
		_name = name;
	}

	public Thread getThread() {
		return _thread;
	}

	public RR getRunnable() {
		return _runnable;
	}

	public synchronized void replaceAndStartRunnable(RR runnable) {
		stopCurrent();

		_thread = null;
		setRunnable( runnable );

		start();
	}

	public synchronized void start()
	{
		if( ( _thread != null ) && _thread.isAlive() )
			throw( new IllegalStateException( "Attempting to start another thread when current is alive" ) );

		if( getRunnable() == null )
			throw( new IllegalStateException( "Attempting to start thread when runnable is null" ) );

		( _thread = createThread() ).start();
	}

	protected Thread createThread()
	{
		Thread result = null;
		if( _name != null )
			result = new Thread( getRunnable(), _name );
		else
			result = new Thread( getRunnable() );
		return( result );
	}

	public void setRunnable(RR _runnable) {
		this._runnable = _runnable;
	}

	public boolean isAlive()
	{
		Thread thread = getThread();
		return( ( thread != null ) && thread.isAlive() );
	}

	public synchronized void stopCurrent()
	{
		if( isAlive() )
			stop( getThread(), getRunnable() );

		_thread = null;
	}

	protected void stop( Thread thread, RR runnable )
	{
		new Thread( () -> {
			if( runnable != null )
				runnable.setHasToStop(true);

			ThreadFunctions.instance().sleep(10000);
			
			if( thread.isAlive() )
				ExecutionFunctions.instance().safeMethodExecution( thread::stop );
		}).start();
	}
}
