/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.threads.runnablebasewrapper;

import com.frojasg1.general.threads.RunnableBase;

import java.util.function.Consumer;
import java.util.function.Supplier;

public class RunnableBaseWrapper extends RunnableBase {

    protected Supplier<Boolean> doInitFunction;
    protected Supplier<Boolean> externalHasToStopFunction;
    protected Supplier<Boolean> particularHasEnded;
    protected Supplier<Boolean> doRoutineFunction;
    protected Runnable doEndFunction;

    public RunnableBaseWrapper(Supplier<Boolean> doInitFunction, Supplier<Boolean> externalHasToStopFunction,
                               Supplier<Boolean> particularHasEnded, Supplier<Boolean> doRoutineFunction,
                               Runnable doEndFunction, Consumer<Exception> onExceptionFunction) {
        this.doInitFunction = doInitFunction;
        this.externalHasToStopFunction = externalHasToStopFunction;
        this.particularHasEnded = particularHasEnded;
        this.doRoutineFunction = doRoutineFunction;
        this.doEndFunction = doEndFunction;

        setOnExceptionFunction(onExceptionFunction);
    }

    @Override
    protected boolean doInit() {
        return booleanSupplier(getDoInitFunction(), true);
    }

    protected boolean hasEnded() {
        return booleanSupplier(getParticularHasEnded(), false);
    }

    protected boolean booleanSupplier(Supplier<Boolean> supplier, boolean defaultValue) {
        boolean result = defaultValue;
        if(supplier != null) {
            Boolean result2 = supplier.get();
            if(result2 != null) {
                result = result2;
            }
        }

        return result;
    }

    @Override
    protected boolean doRoutine() {
        boolean result = getDoRoutineFunction() != null;
        if(result) {
            result = getDoRoutineFunction().get();
        }

        return result;
    }

    @Override
    public void doEnd() {
        if(getDoEndFunction() != null) {
            getDoEndFunction().run();
        }
    }

    @Override
    protected void hasToStopAssociatedTasks() {

    }

    @Override
    protected void doBeforePause() {

    }

    @Override
    protected void doAfterPause() {

    }

    @Override
    protected boolean getHasToStop()
    {
        return super.getHasToStop() || externalHasToStop();
    }

    protected Supplier<Boolean> getDoInitFunction() {
        return doInitFunction;
    }

    protected Supplier<Boolean> getExternalHasToStopFunction() {
        return externalHasToStopFunction;
    }

    protected Supplier<Boolean> getParticularHasEnded() {
        return particularHasEnded;
    }

    public Supplier<Boolean> getDoRoutineFunction() {
        return doRoutineFunction;
    }

    protected Runnable getDoEndFunction() {
        return doEndFunction;
    }

    protected boolean externalHasToStop() {
        return booleanSupplier(getExternalHasToStopFunction(), false);
    }
}
