/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.view;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

public class ReleaseResourceableList implements ReleaseResourcesable {
    protected List<ReleaseResourcesable> elemsToRelease = new ArrayList<>();

    public synchronized <C> C add(C elem) {
        if(elem instanceof ReleaseResourcesable) {
            elemsToRelease.add((ReleaseResourcesable) elem);
        }
        return elem;
    }

    protected synchronized List<ReleaseResourcesable> emptyElemsToRelease() {
        List<ReleaseResourcesable> result = new ArrayList<>(elemsToRelease);
        elemsToRelease.clear();

        return result;
    }

    @Override
    public void releaseResources() {
        List<ReleaseResourcesable> list = emptyElemsToRelease();
        Collections.reverse(list);

        for(ReleaseResourcesable releaseResourcesable: list) {
            releaseResourcesable.releaseResources();
        }
        list.clear();
        list = null;
    }
}
