/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.view.evt.wrapper.impl;

import com.frojasg1.general.ExecutionFunctions;
import java.util.function.Supplier;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public abstract class GenComponentWrapperEventBuilderBase<GC, EV extends GenComponentWrapperEvent>
	implements GenComponentWrapperEventBuilder<EV>
{
	protected GC genComponent;

	protected MultiMapForBuildingJComponentWrapperEvents _multiMap;

	protected GenComponentWrapperEventBuilderBase(GC genComponent )
	{
		genComponent = genComponent;
	}

	protected void init()
	{
		_multiMap = createMultiMap();
	}

	protected MultiMapForBuildingJComponentWrapperEvents createMultiMap()
	{
		MultiMapForBuildingJComponentWrapperEvents result = createEmptyMultiMap();
		fillMultiMap(result);

		return( result );
	}

	protected MultiMapForBuildingJComponentWrapperEvents createEmptyMultiMap()
	{
		return( new MultiMapForBuildingJComponentWrapperEvents() );
	}

	protected abstract void fillMultiMap( MultiMapForBuildingJComponentWrapperEvents multiMap );

	@Override
	public EV buildEvent(long eventType)
	{
		return( (EV) buildEvent( _multiMap.getSupplier(eventType) ) );
	}

	@Override
	public <EVT extends EV> EVT buildEvent(Class<EVT> eventClass )
	{
		return( buildEvent( _multiMap.getSupplier(eventClass) ) );
	}

	public <EVT extends EV> EVT buildEvent(Supplier<EVT> constructor)
	{
		EVT result = null;
		if( constructor != null )
		{
			result = constructor.get();
			result.setSource(genComponent);
		}

		return( result );
	}

	protected <CC> CC safeFunctionExecution(ExecutionFunctions.UnsafeFunction<CC> run)
	{
		return( ExecutionFunctions.instance().safeFunctionExecution(run) );
	}
}
