/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.view.evt.wrapper.impl;

import com.frojasg1.general.map.multimap.MultiMap;
import com.frojasg1.general.map.multimap.MultiMapMapEntry;
import java.util.HashMap;
import java.util.Map;
import java.util.function.Supplier;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class MultiMapForBuildingJComponentWrapperEvents
	extends MultiMap<GenComponentWrapperEventBuildingContext>
{
	protected static final String EVENT_TYPE = "EVENT_TYPE";
	protected static final String EVENT_CLASS = "EVENT_CLASS";

	public MultiMapForBuildingJComponentWrapperEvents() {
		this( true );
	}

	public MultiMapForBuildingJComponentWrapperEvents(boolean hasToInit) {
		if( hasToInit )
			init();
	}

	@Override
	protected Map<String, MultiMapMapEntry<?, GenComponentWrapperEventBuildingContext>> fillMultiMapMapEntryMap()
	{
		Map<String, MultiMapMapEntry<?, GenComponentWrapperEventBuildingContext>> result = new HashMap<>();

		addMultiMapMapEntry( result, EVENT_TYPE, Long.class,
							GenComponentWrapperEventBuildingContext::getEventType );
		addMultiMapMapEntry( result, EVENT_CLASS, Class.class,
							GenComponentWrapperEventBuildingContext::getEventClass );

		return( result );
	}

	protected GenComponentWrapperEventBuildingContext get(Long eventType )
	{
		return( get(EVENT_TYPE, eventType) );
	}

	protected <EVT> GenComponentWrapperEventBuildingContext<EVT> get(Class<EVT> eventClass )
	{
		return( get(EVENT_CLASS, eventClass) );
	}

	public <EVT> Supplier<EVT> getSupplier( Class<EVT> eventClass )
	{
		Supplier<EVT> result = null;
		GenComponentWrapperEventBuildingContext<EVT> ctx = get(eventClass);
		if( ctx != null )
			result = ctx.getConstructor();

		return( result );
	}

	public Supplier getSupplier( Long eventType )
	{
		Supplier result = null;
		GenComponentWrapperEventBuildingContext ctx = get(eventType);
		if( ctx != null )
			result = ctx.getConstructor();

		return( result );
	}

	protected <EVT> GenComponentWrapperEventBuildingContext<EVT> createContext(
		Long eventType, Class<EVT> eventClass, Supplier<EVT> constructor )
	{
		GenComponentWrapperEventBuildingContext<EVT> result = new GenComponentWrapperEventBuildingContext<>();
		result.setEventType(eventType);
		result.setEventClass(eventClass);
		result.setConstructor(constructor);

		return( result );
	}

	public <EVT> void put( Long eventType, Class<EVT> eventClass,
							Supplier<EVT> constructor )
	{
		put( createContext( eventType, eventClass, constructor ) );
	}
}
