/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.view.evt.wrapper.impl.combobox.impl;

import com.frojasg1.general.view.evt.wrapper.impl.GenComponentWrapperEventBuilderBase;
import com.frojasg1.general.view.evt.wrapper.impl.combobox.GenComboBoxWrapperEventBase;
import com.frojasg1.general.view.evt.wrapper.impl.combobox.GenComboBoxWrapperEventType;
import com.frojasg1.general.view.evt.wrapper.impl.MultiMapForBuildingJComponentWrapperEvents;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class ComboBoxWrapperEventBuilderBase<RR, CB>
	extends GenComponentWrapperEventBuilderBase<CB, GenComboBoxWrapperEventBase> {

	protected Class<RR> _recordClass;

	public ComboBoxWrapperEventBuilderBase(CB combo, Class<RR> recordClass,
										   boolean withInit)
	{
		super( combo );
		_recordClass = recordClass;

		if( withInit )
			init();
	}

	@Override
	protected void fillMultiMap(MultiMapForBuildingJComponentWrapperEvents multiMap) {
        multiMap.put( GenComboBoxWrapperEventType.NEW_ITEM_SELECTED, ComboBoxWrapperNewItemSelectedEvt.class, ComboBoxWrapperNewItemSelectedEvt::new );
	}

	public Class<RR> getRecordClass() {
		return _recordClass;
	}

	public ComboBoxWrapperNewItemSelectedEvt<CB, RR> createJComboBoxWrapperNewItemSelected(RR value )
	{
		ComboBoxWrapperNewItemSelectedEvt<CB, RR> evt = buildEvent(ComboBoxWrapperNewItemSelectedEvt.class);
		evt.setNewItemSelected(value);

		return( evt );
	}
}
