/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.view.evt.wrapper.impl.table.impl;

import com.frojasg1.general.collection.functions.ListOperationFunctions;
import com.frojasg1.general.collection.functions.ListStateContextBase;
import com.frojasg1.general.view.evt.wrapper.impl.GenComponentWrapperEventBuilderBase;
import com.frojasg1.general.view.evt.wrapper.impl.table.GenTableWrapperEventBase;
import com.frojasg1.general.collection.functions.ListRecordMoveContext;
import com.frojasg1.general.view.evt.wrapper.impl.MultiMapForBuildingJComponentWrapperEvents;
import com.frojasg1.general.view.evt.wrapper.impl.table.impl.deleterecords.GenTableWrapperRecordsDeletedEvent;
import com.frojasg1.general.view.evt.wrapper.impl.table.impl.moverecords.GenTableWrapperRecordsMovedEvent;
import com.frojasg1.general.view.evt.wrapper.impl.table.impl.sortedrecords.GenTableWrapperRecordsSortedEvent;
import java.io.File;
import java.util.List;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class GenTableWrapperEventBuilderBase<RR, TT>
	extends GenComponentWrapperEventBuilderBase<TT, GenTableWrapperEventBase> {

	protected Class<RR> _recordClass;

	public GenTableWrapperEventBuilderBase(TT table, Class<RR> recordClass,
										   boolean withInit)
	{
		super( table );
		_recordClass = recordClass;

		if( withInit )
			init();
	}

	@Override
	protected void fillMultiMap(MultiMapForBuildingJComponentWrapperEvents multiMap) {
        multiMap.put( GenTableWrapperDropFilesEvent.EVENT_TYPE, GenTableWrapperDropFilesEvent.class, GenTableWrapperDropFilesEvent::new );
        multiMap.put( GenTableWrapperHasBeenModifiedEvent.EVENT_TYPE, GenTableWrapperHasBeenModifiedEvent.class, GenTableWrapperHasBeenModifiedEvent::new );
        multiMap.put( GenTableWrapperNewCurrentEvent.EVENT_TYPE, GenTableWrapperNewCurrentEvent.class, GenTableWrapperNewCurrentEvent::new );
        multiMap.put( GenTableWrapperRecordsMovedEvent.EVENT_TYPE, GenTableWrapperRecordsMovedEvent.class, GenTableWrapperRecordsMovedEvent::new );
        multiMap.put( GenTableWrapperRecordsDeletedEvent.EVENT_TYPE, GenTableWrapperRecordsDeletedEvent.class, GenTableWrapperRecordsDeletedEvent::new );
        multiMap.put( GenTableWrapperFilterHasChangedEvent.EVENT_TYPE, GenTableWrapperFilterHasChangedEvent.class, () -> new GenTableWrapperFilterHasChangedEvent( _recordClass ) );
        multiMap.put( GenTableWrapperSelectionHasChangedEvent.EVENT_TYPE, GenTableWrapperSelectionHasChangedEvent.class, GenTableWrapperSelectionHasChangedEvent::new );
        multiMap.put( GenTableWrapperRecordsSortedEvent.EVENT_TYPE, GenTableWrapperRecordsSortedEvent.class, GenTableWrapperRecordsSortedEvent::new );
	}

	public Class<RR> getRecordClass() {
		return _recordClass;
	}

	public ListRecordMoveContext<RR> createListRecordMoveContext(
		int selectedIndex, List<RR> list, int[] selection, int indexWhereToMove)
	{
		return( ListOperationFunctions.instance().createListRecordMoveContext(
				selectedIndex, list, selection, indexWhereToMove ) );
	}

	public GenTableWrapperRecordsMovedEvent<RR, TT> createJTableWrapperRecordsMovedEvent(
		int previouslySelectedIndex, List<RR> originalList, int[] previousSelection, int indexWhereToMove,
		int newSelectedIndex, List<RR> newList, int[] newSelection, int indexWhereMoved )
	{
		ListRecordMoveContext<RR> previous = createListRecordMoveContext(previouslySelectedIndex,
			originalList, previousSelection, indexWhereToMove );
		ListRecordMoveContext<RR> after = createListRecordMoveContext(newSelectedIndex,
			newList, newSelection, indexWhereMoved );

		return( createJTableWrapperRecordsMovedEvent( previous, after ) );
	}

	protected <CC extends ListStateContextBase<RR>>
	GenTableWrapperRecordsModifiedEventBaseData<RR, CC>
		createRecordsModifiedEvtData( CC previous, CC after )
	{
		GenTableWrapperRecordsModifiedEventBaseData<RR, CC> result =
			new GenTableWrapperRecordsModifiedEventBaseData<>();

		result.setPrevious(previous);
		result.setAfter(after);

		return( result );
	}

	public GenTableWrapperRecordsMovedEvent<RR, TT> createJTableWrapperRecordsMovedEvent(
		ListRecordMoveContext<RR> previous, ListRecordMoveContext<RR> after )
	{
		GenTableWrapperRecordsMovedEvent<RR, TT> result = new GenTableWrapperRecordsMovedEvent<>();
		result.setEventData( createRecordsModifiedEvtData(previous, after) );

		return( result );
	}

	public ListStateContextBase<RR> createListStateContextBase(
		Integer selectedIndex, List<RR> list, int[] selection)
	{
		return( ListOperationFunctions.instance().createListStateContextBase(
				selectedIndex, list, selection ) );
	}

	protected <CC extends GenTableWrapperRecordsModifiedEventBase<RR, TT, ListStateContextBase<RR>>>
			CC createJTableWrapperRecordsModifiedGenEvent(
				Class<CC> clazz,
				int previouslySelectedIndex, List<RR> originalList, int[] previousSelection,
				int newSelectedIndex, List<RR> newList, int[] newSelection )
	{
		ListStateContextBase<RR> previous = createListStateContextBase(previouslySelectedIndex,
			originalList, previousSelection );
		ListStateContextBase<RR> after = createListStateContextBase(newSelectedIndex,
			newList, newSelection );

		return( createJTableWrapperRecordsModifiedGenEvent( clazz, previous, after ) );
	}

	public <CC extends GenTableWrapperRecordsModifiedEventBase<RR, TT, ListStateContextBase<RR>>>
			CC createJTableWrapperRecordsModifiedGenEvent(
				Class<CC> clazz,
				ListStateContextBase<RR> previous, ListStateContextBase<RR> after )
	{
		CC evt = buildEvent(clazz);
		evt.setEventData( createRecordsModifiedEvtData(previous, after) );

		return( evt );
	}

	public GenTableWrapperRecordsDeletedEvent<RR, TT> createJTableWrapperRecordsDeletedEvent(
		int previouslySelectedIndex, List<RR> originalList, int[] previousSelection,
		int newSelectedIndex, List<RR> newList, int[] newSelection )
	{
		return( createJTableWrapperRecordsModifiedGenEvent( GenTableWrapperRecordsDeletedEvent.class,
			previouslySelectedIndex, originalList, previousSelection,
			newSelectedIndex, newList, newSelection) );
	}

	public GenTableWrapperRecordsDeletedEvent<RR, TT> createJTableWrapperRecordsDeletedEvent(
		ListStateContextBase<RR> previous, ListStateContextBase<RR> after )
	{
		return( createJTableWrapperRecordsModifiedGenEvent( GenTableWrapperRecordsDeletedEvent.class,
					previous, after ) );
	}

	public GenTableWrapperRecordsSortedEvent<RR, TT> createJTableWrapperRecordsSortedEvent(
		int previouslySelectedIndex, List<RR> originalList, int[] previousSelection,
		int newSelectedIndex, List<RR> newList, int[] newSelection )
	{
		return( createJTableWrapperRecordsModifiedGenEvent( GenTableWrapperRecordsSortedEvent.class,
			previouslySelectedIndex, originalList, previousSelection,
			newSelectedIndex, newList, newSelection) );
	}

	public GenTableWrapperRecordsSortedEvent<RR, TT> createJTableWrapperRecordsSortedEvent(
		ListStateContextBase<RR> previous, ListStateContextBase<RR> after )
	{
		return( createJTableWrapperRecordsModifiedGenEvent( GenTableWrapperRecordsSortedEvent.class,
					previous, after ) );
	}

	protected GenTableWrapperFilterHasChangedEventData<RR> createJTableWrapperFilterHasChangedEventData(List<RR> filteredList )
	{
		GenTableWrapperFilterHasChangedEventData<RR> result = new GenTableWrapperFilterHasChangedEventData<>();
		result.setNewFilteredList( filteredList );

		return( result );
	}

	public GenTableWrapperFilterHasChangedEvent createFilterChangedEvt(List<RR> filteredList)
	{
		GenTableWrapperFilterHasChangedEvent result = buildEvent( GenTableWrapperFilterHasChangedEvent.class );

		result.setEventData( createJTableWrapperFilterHasChangedEventData( filteredList ) );

		return( result );
	}

	protected GenTableWrapperNewCurrentEventData createJTableWrapperNewCurrentEventData(int newCurrentRow )
	{
		GenTableWrapperNewCurrentEventData result = new GenTableWrapperNewCurrentEventData();

		result.setNewCurrentRow( newCurrentRow );

		return( result );
	}

	public GenTableWrapperNewCurrentEvent<RR, TT> createNewCurrentEvt(int newCurrentRow )
	{
		GenTableWrapperNewCurrentEvent<RR, TT> result = buildEvent( GenTableWrapperNewCurrentEvent.class );

		result.setEventData( createJTableWrapperNewCurrentEventData(newCurrentRow) );

		return( result );
	}

	protected GenTableWrapperDropFilesEventData createFilesDroppedData(List<File> fileList,
																	   int indexToDropFiles)
	{
		GenTableWrapperDropFilesEventData result = new GenTableWrapperDropFilesEventData();
		result.setFileList( fileList );
		result.setIndexForInsertion( indexToDropFiles );

		return( result );
	}

	public GenTableWrapperDropFilesEvent createFilesDropped(List<File> fileList,
															int indexToDropFiles)
	{
		GenTableWrapperDropFilesEvent result = buildEvent( GenTableWrapperDropFilesEvent.class );
		result.setEventData( createFilesDroppedData( fileList, indexToDropFiles ) );

		return( result );
	}
}
