/*
 * Copyright (c) 2023. Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 3.0 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */

package com.frojasg1.general.view.gencomponentwrapper;

import com.frojasg1.general.update.Updateable;
import com.frojasg1.general.view.ReleaseResourcesable;
import com.frojasg1.general.view.evt.wrapper.impl.GenComponentWrapperEvent;
import com.frojasg1.general.view.evt.wrapper.impl.GenComponentWrapperEventBuilder;

import java.util.List;
import java.util.function.Consumer;

public interface GenComponentWrapper<GC, EV extends GenComponentWrapperEvent> extends ReleaseResourcesable, Updateable {
    GC getGenComponent();

    void setGenComponent(GC genComponent);

    GenComponentWrapperEventBuilder<EV> getEventBuilder();

    void addWrapperListener(Consumer<? super EV> listener, Long... eventTypes);

    void removeWrapperListener(Consumer<? super EV> listener);

    void notifyEvt(EV evt);

    @Override
    void releaseResources();

    <EVT extends EV> EVT createEvt(Class<EVT> clazz);

    void updateInternal();

    void addCallStack(String message, List<Exception> callStacks);

    void logCallStacks(String message, List<Exception> callStacks);

    GuiSystemDependingFunctions getGuiSystemDependingFunctions();
}
