/*
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.view.gencomponentwrapper.combobox;

import com.frojasg1.general.ArrayFunctions;
import com.frojasg1.general.view.evt.wrapper.impl.combobox.GenComboBoxWrapperEventBase;
import com.frojasg1.general.view.evt.wrapper.impl.combobox.impl.ComboBoxWrapperEventBuilderBase;
import com.frojasg1.general.view.evt.wrapper.impl.combobox.impl.ComboBoxWrapperNewItemSelectedEvt;
import com.frojasg1.general.view.gencomponentwrapper.GenComponentWrapperBase;
import com.frojasg1.general.view.gencomponentwrapper.GuiSystemDependingFunctions;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.function.Function;
import java.util.function.Supplier;


/**
 * @param <CB> - ComboBox
 * @param <RR> - Record
 */
public abstract class GenComboBoxWrapper<CB, RR> extends GenComponentWrapperBase<CB, GenComboBoxWrapperEventBase> {

    protected Supplier<RR[]> _arrayBuilderForFillingCombo;

    protected Class<RR> _itemClass;

    protected Function<RR, ?> _functionForEquals;

    public GenComboBoxWrapper(CB combo, Class<RR> itemClass, Supplier<RR[]> arrayBuilderForFillingCombo,
                              GuiSystemDependingFunctions guiSystemDependingFunctions) {
        this(combo, itemClass, arrayBuilderForFillingCombo, Function.identity(), guiSystemDependingFunctions);
    }

    public GenComboBoxWrapper(CB combo, Class<RR> itemClass,
                              Supplier<RR[]> arrayBuilderForFillingCombo,
                              Function<RR, ?> functionForEquals,
                              GuiSystemDependingFunctions guiSystemDependingFunctions) {
        this(combo, itemClass, arrayBuilderForFillingCombo, functionForEquals, guiSystemDependingFunctions, true);
    }

    protected GenComboBoxWrapper(CB combo, Class<RR> itemClass,
                                 Supplier<RR[]> arrayBuilderForFillingCombo,
                                 Function<RR, ?> functionForEquals,
                                 GuiSystemDependingFunctions guiSystemDependingFunctions,
                                 boolean init) {
        super(combo, new ComboBoxWrapperEventBuilderBase<RR, CB>(combo, itemClass, true), guiSystemDependingFunctions);

        _functionForEquals = functionForEquals;
        _itemClass = itemClass;
        _arrayBuilderForFillingCombo = arrayBuilderForFillingCombo;

        if (init)
            init();
    }

    @Override
    protected abstract void addListeners();

    @Override
    protected abstract void removeListeners();

    @Override
    public void updateInternal() {
        fillCombo();
    }

    protected ActionListener createActionListener() {
        return (this::fireNewItemSelected);
    }

    protected void fireNewItemSelected(ActionEvent evt) {
        notifyEvt(createNewItemSelectedEvent());
    }

    public ComboBoxWrapperEventBuilderBase<RR, JComboBox<RR>> getEventBuilder() {
        return ((ComboBoxWrapperEventBuilderBase<RR, JComboBox<RR>>) super.getEventBuilder());
    }

    protected ComboBoxWrapperNewItemSelectedEvt createNewItemSelectedEvent() {
        return (getEventBuilder().createJComboBoxWrapperNewItemSelected(getSelectedItem()));
    }

    protected RR[] getItems() {
        RR[] result = null;

        if (_arrayBuilderForFillingCombo != null)
            result = _arrayBuilderForFillingCombo.get();

        return (result);
    }

    public CB getCombo() {
        return (getGenComponent());
    }

    public abstract RR getSelectedItem();

    public abstract void setSelectedItem(RR newSelectedItem);

    public abstract void setSelectedIndex(int newSelectedIndex);

    protected abstract void addItems(RR[] items);
//	{
//		ComboBoxModel<RR> model = createModel( items );
//		getCombo().setModel(model);
//	}

    protected void fillCombo() {
        RR[] items = getItems();
        if (hasChanged(items)) {
            RR selectedItem = getSelectedItem();

            addItems(items);

            int newSelectedIndex = (selectedItem == null) ?
                    -1 :
                    ArrayFunctions.instance().getFirstIndexOfEquals(items, selectedItem,
                            _functionForEquals);

            if (newSelectedIndex >= 0)
                setSelectedIndex(newSelectedIndex);
            else if (items.length > 0)
                setSelectedIndex(0);
        }
    }

    protected abstract boolean hasChanged(RR[] items);
//	{
//		ComboBoxModel model = getCombo().getModel();
//		boolean result = ( items.length != model.getSize() );
//		for( int ii = 0; !result && (ii < items.length ); ii++ )
//			result = !Objects.equals( items[ii], model.getElementAt(ii) );
//
//		return( result );
//	}

    @Override
    public void releaseResources() {
        super.releaseResources();
    }
}
