/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.view.gencomponentwrapper.jtable;

import com.frojasg1.general.update.Updateable;

import java.util.function.Function;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class ViewAndModelSelectionValue implements Updateable {
	protected Function<Integer, Integer> _viewToModelTranslator;
	protected Function<Integer, Integer> _modelToViewTranslator;

	protected Integer _modelSelectedIndex;
	protected Integer _viewSelectedIndex;

	public ViewAndModelSelectionValue( Function<Integer, Integer> viewToModelTranslator,
										Function<Integer, Integer> modelToViewTranslator )
	{
		_viewToModelTranslator = viewToModelTranslator;
		_modelToViewTranslator = modelToViewTranslator;
	}

	public Integer getModelSelectedIndex()
	{
		return( _modelSelectedIndex );
	}

	public Integer getViewSelectedIndex()
	{
		return( _viewSelectedIndex );
	}

	public void setViewSelectedIndex( Integer value )
	{
		_viewSelectedIndex = value;
		int tmp = translate( value, _viewToModelTranslator );
		if( tmp != -1 )
			_modelSelectedIndex = tmp;
	}

	public void setModelSelectedIndex( Integer value )
	{
		_modelSelectedIndex = value;
		_viewSelectedIndex = translate( value, _modelToViewTranslator );
	}

	protected Integer translate( Integer viewIndex, Function<Integer, Integer> translator )
	{
		Integer result = null;
		if( viewIndex != null )
			result = translator.apply( viewIndex );
		return( result );
	}

	@Override
	public void update()
	{
		setModelSelectedIndex( getModelSelectedIndex() );
	}
}
