/*
 * Copyright (c) 2023. Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 3.0 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */

package com.frojasg1.general.desktop.color.colorfunctions;

import com.frojasg1.general.desktop.image.ImageFunctions;
import org.apache.http.annotation.NotThreadSafe;

import java.util.function.UnaryOperator;

@NotThreadSafe
public abstract class ColorFunctionBase implements UnaryOperator<Integer> {

    protected double[] components;

    protected ColorFunctionBase init() {
        setComponents(createEmptyComponents());

        return this;
    }

    protected double[] createEmptyComponents() {
        return getImageFunctions().createEmptyComponents();
    }

    protected double[] getComponents() {
        return components;
    }

    protected void setComponents(double[] components) {
        this.components = components;
    }

    @Override
    public abstract Integer apply(Integer origColor);

    protected short  getGrayScale(int rgb) {
        return getImageFunctions().getGrayScale(rgb);
    }

    protected ImageFunctions getImageFunctions() {
        return ImageFunctions.instance();
    }

    protected int getARGB( double[] components ) {
        return getImageFunctions().getARGB(components);
    }

    protected double[] getComponents(int argb) {
        double[] components = getComponents();
        fillComponents(argb, components);

        return components;
    }

    protected void fillComponents(int argb, double[] components) {
        getImageFunctions().resetComponents(components);
        getImageFunctions().addARGB(components, argb, 1.0d);
    }
}
