/*
 * Copyright (c) 2023. Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 3.0 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */

package com.frojasg1.general.desktop.color.colorfunctions.impl;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Arrays;
import java.util.List;
import java.util.function.UnaryOperator;

public class ColorFunctionsConcat implements UnaryOperator<Integer> {
    private static final Logger LOGGER = LoggerFactory.getLogger(ColorFunctionsConcat.class);

    protected List<UnaryOperator<Integer>> colorFunctions;

    public ColorFunctionsConcat(UnaryOperator<Integer> ... colorFunctions) {
        this.colorFunctions = Arrays.asList(colorFunctions);
    }

    @Override
    public Integer apply(Integer origColor) {
        int color = origColor;
        for(UnaryOperator<Integer> colorFunction: colorFunctions) {
            color = colorFunction.apply(color);
        }
        return color;
    }
}
