/*
 * Copyright (c) 2023. Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 3.0 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */

package com.frojasg1.general.desktop.generic.dialogs.helper.contexts;

import com.frojasg1.general.desktop.generic.dialogs.helper.DesktopDialogsWrapperHelper;
import com.frojasg1.general.desktop.view.ViewFunctions;
import java.awt.Component;
import java.util.HashSet;
import java.util.Set;

public abstract class BrowseComponentHierarchyZoomProcessingContextBase implements BrowseComponentHierarchyProcessingContext {

    protected Set<Component> processedComponents;
    protected double zoomFactor;

    public BrowseComponentHierarchyZoomProcessingContextBase(Set<Component> processedComponents,
                                                             double zoomFactor) {
        this.processedComponents = processedComponents != null ? processedComponents : new HashSet<>();
        this.zoomFactor = zoomFactor;
    }

    protected Set<Component> getProcessedComponents() {
        return processedComponents;
    }

    protected double getZoomFactor() {
        return zoomFactor;
    }

    public boolean hasAlreadyBeenInvoked(Component comp) {
        boolean result = getProcessedComponents().contains(comp);
        if(!result) {
            addProcessedComponent(comp);
        }

        return result;
    }

    protected void addProcessedComponent(Component comp) {
        getProcessedComponents().add(comp);
    }

    protected ViewFunctions getViewFunctions() {
        return ViewFunctions.instance();
    }

    protected DesktopDialogsWrapperHelper getHelper()
    {
        return( DesktopDialogsWrapperHelper.instance() );
    }

    protected void emptyComponentConsumer(Component comp) {
        // intentionally left blank
    }
}
