/*
 * Copyright (c) 2023. Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 3.0 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */

package com.frojasg1.general.desktop.generic.dialogs.helper.contexts.impl;

import com.frojasg1.general.desktop.generic.dialogs.helper.contexts.BrowseComponentHierarchyZoomProcessingContextBase;
import com.frojasg1.general.desktop.view.ComponentFunctions;
import com.frojasg1.general.number.IntegerFunctions;
import java.awt.Component;
import java.awt.Container;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.util.Set;
import java.util.function.Consumer;
import java.util.function.Predicate;

public class BrowseComponentHierarchyResizeComponentContext extends BrowseComponentHierarchyZoomProcessingContextBase {

    protected Predicate<Component> specialComponentProcessing;


    public BrowseComponentHierarchyResizeComponentContext(Predicate<Component> specialComponentProcessing,
                                                          double zoomFactor) {
        this(specialComponentProcessing, null, zoomFactor);
    }

    public BrowseComponentHierarchyResizeComponentContext(Predicate<Component> specialComponentProcessing,
                                                          Set<Component> processedComponents, double zoomFactor) {
        super(processedComponents, zoomFactor);
        this.specialComponentProcessing = specialComponentProcessing;
    }

    @Override
    public void processComponent(Component comp) {
        if (hasToProcess(comp, getSpecialComponentProcessing())) {
            getHelper().zoomSizedSize(comp, getZoomFactor(), false);
        }
    }

    @Override
    public Consumer<Component> getProcessChildFunction(Container parent) {
        GridBagLayout gridBagLayout = ComponentFunctions.instance().getGridBagLayout(parent);

        return child -> processChild(gridBagLayout, child);
    }

    public Predicate<Component> getSpecialComponentProcessing() {
        return specialComponentProcessing;
    }

    protected boolean hasToProcess(Component comp, Predicate<Component> specialComponentProcessing) {
        return (specialComponentProcessing == null) || !((comp == null) || specialComponentProcessing.test(comp));
    }

    protected void processChild(GridBagLayout gridBagLayout, Component child) {
        if (gridBagLayout != null) {
            GridBagConstraints constraints = gridBagLayout.getConstraints(child);

            constraints.ipadx = zoomIpad(constraints.ipadx, zoomFactor);
            constraints.ipady = zoomIpad(constraints.ipady, zoomFactor);

            gridBagLayout.setConstraints(child, constraints);
        }
    }

    protected int zoomIpad(int value, double zoomFactor) {
        return (value > 0) ? IntegerFunctions.zoomValueRound(value, zoomFactor) : value;
    }
}
