/*
 * Copyright (c) 2023. Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 3.0 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */

package com.frojasg1.general.desktop.generic.dialogs.helper.contexts.impl;

import com.frojasg1.general.desktop.generic.dialogs.helper.contexts.BrowseComponentHierarchyZoomProcessingContextBase;
import com.frojasg1.general.desktop.view.ComponentFunctions;
import java.awt.Component;
import java.awt.Container;
import java.awt.Font;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.function.Consumer;
import javax.swing.JComponent;
import javax.swing.JTable;
import javax.swing.border.Border;
import javax.swing.border.TitledBorder;

public class BrowseComponentHierarchyZoomFontContext extends BrowseComponentHierarchyZoomProcessingContextBase {

    protected List<Object> processedObjs = new ArrayList<>();

    public BrowseComponentHierarchyZoomFontContext(double zoomFactor) {
        this(null, zoomFactor);
    }

    public BrowseComponentHierarchyZoomFontContext(Set<Component> processedComponents, double zoomFactor) {
        super(processedComponents, zoomFactor);
        this.processedObjs = processedObjs;
    }

    @Override
    public void processComponent(Component comp) {
        if(!(comp instanceof JTable))
            getHelper().zoomFont(comp, zoomFactor, getProcessedObjs());

        zoomBorderFont(comp, getZoomFactor());
    }

    @Override
    public Consumer<Component> getProcessChildFunction(Container parent) {
        return this::emptyComponentConsumer;
    }

    protected Border[] getDerivedBorders(Border border) {
        return ComponentFunctions.instance().getDerivedBorders(border);
    }

    protected List<Object> getProcessedObjs() {
        return processedObjs;
    }

    protected void zoomBorderFont(Border border, double zoomFactor) {
        if(border instanceof TitledBorder) {
            TitledBorder titledBorder = (TitledBorder) border;
            Font newFont = getHelper().zoomFont(titledBorder.getTitleFont(), zoomFactor, getProcessedObjs());
            if(newFont != null) {
                titledBorder.setTitleFont(newFont);
            }
        }
        for(Border derivedBorder: getDerivedBorders(border)) {
            zoomBorderFont(derivedBorder, zoomFactor);
        }
    }

    protected void zoomBorderFont(Component comp, double zoomFactor) {
        if(comp instanceof JComponent) {
            JComponent jcomp = (JComponent) comp;
            zoomBorderFont(jcomp.getBorder(), zoomFactor);
        }
    }
}
