/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.desktop.image.filefilters;

import com.frojasg1.applications.common.configuration.imp.InternationalizedStringConfImp;
import com.frojasg1.general.desktop.GenericDesktopConstants;
import com.frojasg1.general.desktop.image.ImageFunctions;
import com.frojasg1.general.dialogs.filefilter.GenericFileFilter;
import com.frojasg1.general.dialogs.filefilter.impl.GenericFileFilterArrayBase;
import java.util.List;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class FileFiltersForImages extends GenericFileFilterArrayBase
{
	public static final String GLOBAL_CONF_FILE_NAME = "FileFiltersForImages.properties";

	// $1 image file
	protected static final String CONF_EXTENSION_IMAGE_FILE = "EXTENSION_IMAGE_FILE";

	// All supported image files
	protected static final String CONF_ALL_SUPPORTED_IMAGE_FILES = "ALL_SUPPORTED_IMAGE_FILES";


	protected static FileFiltersForImages INSTANCE = new FileFiltersForImages().init();


	public static FileFiltersForImages instance()
	{
		return( INSTANCE );
	}

	protected FileFiltersForImages()
	{
		super( new InternationalizedStringConfImp( GLOBAL_CONF_FILE_NAME,
					GenericDesktopConstants.sa_PROPERTIES_PATH_IN_JAR ) );
	}

	@Override
	public FileFiltersForImages init() {
		super.init();
		return this;
	}

	@Override
	protected void fillListAndSort(List<GenericFileFilter> list) {
		List<String> imagesExtensionsList = ImageFunctions.instance().getReaderImageFormatNamesSortedList();

		list.add( createExtensionFilter( () -> getInternationalString( CONF_ALL_SUPPORTED_IMAGE_FILES ),
			imagesExtensionsList.toArray( new String[imagesExtensionsList.size()]) ) );

		for( String extension: imagesExtensionsList )
			list.add( createExtensionFilter( () -> getDescriptionForImageExtension( extension ),
				extension ) );
	}

	protected String getDescriptionForImageExtension( String extension )
	{
		return( createCustomInternationalString( CONF_EXTENSION_IMAGE_FILE, extension ) );
	}
}
