/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.desktop.init;

import com.frojasg1.generic.GenericFunctions;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class GenericDesktopDefaultExceptionHandler implements Thread.UncaughtExceptionHandler {

	protected static final String OUT_OF_MEMORY = "Out of memory!!";
	private static Logger LOGGER = LoggerFactory.getLogger(GenericDesktopDefaultExceptionHandler.class);

	protected int _outOfMemoryShown = 0;

	@Override
	public void uncaughtException(Thread t, Throwable e) {
		LOGGER.info("Unhandled exception caught!", e);

		if( (_outOfMemoryShown < 7) && ( e instanceof OutOfMemoryError ) )
		{
			_outOfMemoryShown++;
			GenericFunctions.instance().getDialogsWrapper().showError(null, OUT_OF_MEMORY );
		}
	}
}
