/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.desktop.view.about;

import com.frojasg1.general.desktop.view.zoom.components.ZoomJScrollPane;
import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;
import com.frojasg1.desktop.liblens.graphics.Coordinate2D;
import com.frojasg1.general.desktop.view.painter.PaintEnabled;
import com.frojasg1.general.desktop.view.panels.*;
import com.frojasg1.general.string.StringFunctions;
import com.frojasg1.general.desktop.view.editorkits.WrapEditorKit;
import com.frojasg1.general.desktop.view.pdf.ImageJPanel;
import com.frojasg1.desktop.liblens.graphics.lens.LensJPanel;
import com.frojasg1.general.ExecutionFunctions;
import com.frojasg1.general.desktop.image.ImageFunctions;
import com.frojasg1.general.desktop.view.FontFunctions;
import com.frojasg1.general.desktop.view.editorpane.FastColorInversorStaticDocumentJEditorPane;
import com.frojasg1.general.desktop.view.icons.IconResources;
import com.frojasg1.general.lib3d.components.api.about.animation.AnimationForAbout;
import com.frojasg1.general.desktop.view.text.link.imp.ScrollableJTextComponentUrlLauncher;
import com.frojasg1.general.desktop.view.zoom.mapper.ComponentMapper;
import com.frojasg1.general.desktop.view.zoom.mapper.InternallyMappedComponent;
import com.frojasg1.general.number.IntegerFunctions;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.image.BufferedImage;
import javax.swing.JToggleButton;
import javax.swing.event.ChangeEvent;
import javax.swing.text.Document;
import javax.swing.text.JTextComponent;

/**
 *
 * @author Usuario
 */
public class GenericAboutJPanel extends com.frojasg1.general.desktop.view.panels.CustomJPanelBase  implements InternallyMappedComponent	//,MouseListener, MouseMotionListener
{
//	protected static final String TORUS_IMAGE_RESOURCE = "com/frojasg1/generic/resources/images/torus.png";

	protected static final String SPACES_LINE = "                                                                      " + StringFunctions.RETURN;

	protected AboutJPanelControllerInterface _controller = null;

	protected ImageJPanel _imageJPanel = null;

	protected BufferedImage _image = null;

	protected FastColorInversorStaticDocumentJEditorPane _rtfEditorPane = null;

	protected LensJPanel _lensJPanel = null;

//	protected String _lastURL = null;

	protected GenericAboutJPanel _this = null;

	protected double _zoomFactor = 1.0D;

	protected ScrollableJTextComponentUrlLauncher _scrollableJTextForUrl = new ScrollableJTextComponentUrlLauncher();

	protected Dimension _originalImageJPanelSize = null;

	protected String _downloadFileName = null;

	protected AnimationForAbout _animation = null;

//	protected int _oldScrollYYValue = -1;

	protected BufferedImage _osborneImage = null;
	protected boolean _isOsborne = false;

	/**
	 * Creates new form GenericAboutJPanel
	 */
	public GenericAboutJPanel( AboutJPanelControllerInterface controller,
								BufferedImage image,
								String downloadFileName,
								double zoomFactor )
	{
		super.init();

		_zoomFactor = zoomFactor;
		_this = this;
		_controller = controller;
		_downloadFileName = downloadFileName;
		setImage( image );

		initComponents();

		addListeners();

		initOwnComponents();
	}

	public void setSendEmailButtonText( String text )
	{
		jBtn_sendEmail.setText( text );
	}
/*
	public void setHomePageLabelText( String text )
	{
		jL_homeWebPage.setText( text );
	}
*/
	public void setLensJPanel( LensJPanel ljp )
	{
		_lensJPanel = ljp;

		if( _lensJPanel != null )
		{
			addComponentNotToPaint( jTa_text );
//			_lensJPanel.M_addComponentNotToPaint( jBtn_exit );
//			_lensJPanel.M_addComponentNotToPaint( jBtn_sendEmail );
			addComponentNotToPaint( getJTextComponent() );
			addComponentNotToPaint(jScrollPane1.getViewport());
		}
	}

	protected void addComponentNotToPaint(JComponent comp) {
		_lensJPanel.M_addComponentNotToPaint(comp);
//		RepaintManager.currentManager(comp).markCompletelyClean(comp);

		if(comp instanceof PaintEnabled) {
			((PaintEnabled) comp).setCanPaint(false);
		}
	}

	public void setImage( BufferedImage image )
	{
		if( ( image == null ) || ( image.getHeight() == 0 ) )
		{
			_image = new BufferedImage( 245, 120, BufferedImage.TYPE_INT_RGB );
			Graphics gc = _image.getGraphics();
			gc.setColor( Color.WHITE );
			gc.fillRect( 0, 0, 245, 120 );
		}
		else
			_image = image;
	}

	public void setText( String text )
	{
		jTa_text.setText( text );
		jTa_text.setSelectionStart(0);
		jTa_text.setSelectionEnd(0);
	}

	public JTextComponent getJTextComponent()
	{
		JTextComponent result = _rtfEditorPane;
		if( result == null )
			result = jTa_text;

		return( result );
	}

	public void setRtfDocument( Document doc )
	{
		if( doc != null )
		{
			jTa_text.setFocusable( false );
			if( _rtfEditorPane != null )
				_lensJPanel.removeComponentNotToPaint( _rtfEditorPane );

			_rtfEditorPane = new FastColorInversorStaticDocumentJEditorPane( "text/rtf", "" );
			_rtfEditorPane.setEditorKit( new WrapEditorKit() );
			_rtfEditorPane.setEditable( false );
			_rtfEditorPane.setDocument( doc );
			jScrollPane1.setViewportView( _rtfEditorPane );
			_rtfEditorPane.setSelectionStart(0);
			_rtfEditorPane.setSelectionEnd(0);

			getColorInversor().setDarkMode(_rtfEditorPane, isDarkMode(), null);

			try
			{
				doc.insertString( doc.getLength(), StringFunctions.RETURN, null);
				doc.insertString( doc.getLength(), SPACES_LINE, null);
				doc.insertString( doc.getLength(), SPACES_LINE, null);
				doc.insertString( doc.getLength(), SPACES_LINE, null);
			}
			catch( Throwable th )
			{
				th.printStackTrace();
			}

			if( _lensJPanel != null )
			{
				_lensJPanel.M_addComponentNotToPaint( _rtfEditorPane );
				_rtfEditorPane.requestFocusInWindow();
			}
		}
		else
		{
			_rtfEditorPane = null;
			jScrollPane1.setViewportView( jTa_text );
			jTa_text.setFocusable( true );
			jTa_text.requestFocusInWindow();
			jTa_text.setSelectionStart(0);
			jTa_text.setSelectionEnd(0);
		}

		_scrollableJTextForUrl.setJTextComponent( _rtfEditorPane );
		_lensJPanel.repaint();
	}

	protected void zoomImage()
	{
		_imageJPanel.zoomImageWithSize( _zoomFactor );

//		_imageJPanel.repaint();
	}

	protected void setAnimation( AnimationForAbout animation )
	{
		_animation = animation;
		jTB_showTorus.setVisible( _animation != null );
		if( _animation != null )
			_animation.animationSetEnclosingDimension( getPreferredSize() );
	}

	public AnimationForAbout getAnimation()
	{
		return( _animation );
	}

	public void initOwnComponents()
	{
		jB_osborne.setVisible(false);
		jL_downloadFileName.setText( _downloadFileName );

		jTa_text.setWrapStyleWord(true);
		jTa_text.setLineWrap( true );

		_imageJPanel = new ImageJPanel( _image );
//		_imageJPanel.zoomImageAndSetPreferredSize( _zoomFactor );

		setFrameThick( 3 );

		jPanelImage.add( _imageJPanel );

		int width = (int) ( jPanelImage.getHeight() * _image.getWidth() / _image.getHeight() );
		int height = (int) jPanelImage.getHeight();

		_originalImageJPanelSize = new Dimension( width, height );

		Rectangle jpiBounds = jPanelImage.getBounds();
		jPanelImage.setBounds( (int) ( jpiBounds.getX() + ( jpiBounds.getWidth() - width ) / 2 ),
								(int) ( jpiBounds.getY() + ( jpiBounds.getHeight() - height ) / 2 ),
								width, height );
		_imageJPanel.setBounds( 0, 0, width, height );
		_imageJPanel.setPreferredSize( _originalImageJPanelSize );

		jL_homeWebPage.setFont( FontFunctions.instance().getUnderlinedFont( jL_homeWebPage.getFont() ) );
//		jTa_text.addMouseListener(this);
//		jTa_text.addMouseMotionListener(this);

//		jScrollPane1.getViewport().addMouseListener( this );
//		jScrollPane1.getViewport().addMouseMotionListener( this );

		zoomImage();
	}

	public void setForegroundColorOfHomeWebPage( Color color )
	{
		jL_homeWebPage.setForeground( color );
	}

	protected int getZoomedValue( double value )
	{
		int result = IntegerFunctions.roundToInt( value * _zoomFactor );

		return( result );
	}

	public void setFrameThick( int value )
	{
		int newThick = getZoomedValue( value );
		
		_imageJPanel.setFrameThick(newThick);
	}
	
	public void setOriginalFontForTextLines( Font font )
	{
//		int newFontSize = getZoomedValue( font.getSize() );
//		Font zoomedFont = font.deriveFont( (float) newFontSize );

//		_imageJPanel.setOriginalFontForTextLines(zoomedFont);
		_imageJPanel.setOriginalFontForTextLines(font);
	}

	public void setForegroundColorForTextLines( Color color )
	{
		_imageJPanel.setForegroundColorForTextLines(color);
	}
	
	public void addTextLine( ImageJPanel.TextLine textLine )
	{
		_imageJPanel.addTextLine(textLine);
	}

	public void clearTextLines()
	{
		_imageJPanel.clearTextLines();
	}

	public Dimension getSizeOfOriginalImageJPanel()
	{
		return( _originalImageJPanelSize );
	}

	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	@SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    // Generated using JFormDesigner non-commercial license
    private void initComponents() {
        jPanel1 = new JPanel();
        jBtn_exit = new JButton();
        jScrollPane1 = new JScrollPane();
        jTa_text = new JTextArea();
        jBtn_sendEmail = new JButton();
        jPanelImage = new JPanel();
        jL_homeWebPage = new JLabel();
        jL_Version = new JLabel();
        jL_downloadFileName = new JLabel();
        jTB_showTorus = new JToggleButton();
        jB_osborne = new JButton();

        //======== this ========
        setPreferredSize(new Dimension(650, 405));
        setLayout(null);

        //======== jPanel1 ========
        {
            jPanel1.setLayout(null);

            //---- jBtn_exit ----
            jBtn_exit.setText("Exit");
            jBtn_exit.setName("jBtn_exit");
            jBtn_exit.addActionListener(e -> jBtn_exitActionPerformed(e));
            jPanel1.add(jBtn_exit);
            jBtn_exit.setBounds(540, 120, 100, 20);

            //======== jScrollPane1 ========
            {

                //---- jTa_text ----
                jTa_text.setEditable(false);
                jTa_text.setColumns(20);
                jTa_text.setRows(5);
                jScrollPane1.setViewportView(jTa_text);
            }
            jPanel1.add(jScrollPane1);
            jScrollPane1.setBounds(10, 160, 630, 230);

            //---- jBtn_sendEmail ----
            jBtn_sendEmail.setText("Send e-mail to:  frojasg1@hotmail.com");
            jBtn_sendEmail.setFocusable(false);
            jBtn_sendEmail.addActionListener(e -> jBtn_sendEmailActionPerformed(e));
            jPanel1.add(jBtn_sendEmail);
            jBtn_sendEmail.setBounds(10, 400, 630, 25);

            //======== jPanelImage ========
            {
                jPanelImage.setBorder(LineBorder.createBlackLineBorder());
                jPanelImage.setLayout(null);

                {
                    // compute preferred size
                    Dimension preferredSize = new Dimension();
                    for(int i = 0; i < jPanelImage.getComponentCount(); i++) {
                        Rectangle bounds = jPanelImage.getComponent(i).getBounds();
                        preferredSize.width = Math.max(bounds.x + bounds.width, preferredSize.width);
                        preferredSize.height = Math.max(bounds.y + bounds.height, preferredSize.height);
                    }
                    Insets insets = jPanelImage.getInsets();
                    preferredSize.width += insets.right;
                    preferredSize.height += insets.bottom;
                    jPanelImage.setMinimumSize(preferredSize);
                    jPanelImage.setPreferredSize(preferredSize);
                }
            }
            jPanel1.add(jPanelImage);
            jPanelImage.setBounds(230, 10, 128, 128);

            //---- jL_homeWebPage ----
            jL_homeWebPage.setFont(new Font("Tahoma", Font.BOLD, 13));
            jL_homeWebPage.setForeground(new Color(0x279b00));
            jL_homeWebPage.setHorizontalAlignment(SwingConstants.TRAILING);
            jL_homeWebPage.setText("https://frojasg1.com");
            jL_homeWebPage.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
            jL_homeWebPage.setName("name=jL_homeWebPage,url=https://frojasg1.com");
            jPanel1.add(jL_homeWebPage);
            jL_homeWebPage.setBounds(440, 10, 200, jL_homeWebPage.getPreferredSize().height);

            //---- jL_Version ----
            jL_Version.setHorizontalAlignment(SwingConstants.RIGHT);
            jL_Version.setText("Version :");
            jL_Version.setName("jL_Version");
            jPanel1.add(jL_Version);
            jL_Version.setBounds(10, 140, 60, jL_Version.getPreferredSize().height);

            //---- jL_downloadFileName ----
            jL_downloadFileName.setFont(new Font("Tahoma", Font.BOLD, 11));
            jL_downloadFileName.setForeground(new Color(0xdd870d));
            jL_downloadFileName.setText("Empty");
            jPanel1.add(jL_downloadFileName);
            jL_downloadFileName.setBounds(80, 140, 560, jL_downloadFileName.getPreferredSize().height);

            //---- jTB_showTorus ----
            jTB_showTorus.setText("press");
            jTB_showTorus.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
            jTB_showTorus.setMaximumSize(new Dimension(100000, 100000));
            jTB_showTorus.setMinimumSize(new Dimension(0, 0));
            jTB_showTorus.setName("name=jTB_showTorus,icon=com/frojasg1/generic/resources/images/torus.png");
            jTB_showTorus.addActionListener(e -> jTB_showTorusActionPerformed(e));
            jPanel1.add(jTB_showTorus);
            jTB_showTorus.setBounds(10, 10, 60, 60);

            //---- jB_osborne ----
            jB_osborne.setName("name=jB_osborne,icon=com/frojasg1/generic/resources/osborne/osborne.png");
            jB_osborne.addActionListener(e -> jB_osborneActionPerformed(e));
            jPanel1.add(jB_osborne);
            jB_osborne.setBounds(75, 10, 60, 60);

            {
                // compute preferred size
                Dimension preferredSize = new Dimension();
                for(int i = 0; i < jPanel1.getComponentCount(); i++) {
                    Rectangle bounds = jPanel1.getComponent(i).getBounds();
                    preferredSize.width = Math.max(bounds.x + bounds.width, preferredSize.width);
                    preferredSize.height = Math.max(bounds.y + bounds.height, preferredSize.height);
                }
                Insets insets = jPanel1.getInsets();
                preferredSize.width += insets.right;
                preferredSize.height += insets.bottom;
                jPanel1.setMinimumSize(preferredSize);
                jPanel1.setPreferredSize(preferredSize);
            }
        }
        add(jPanel1);
        jPanel1.setBounds(0, 0, 650, 435);

        {
            // compute preferred size
            Dimension preferredSize = new Dimension();
            for(int i = 0; i < getComponentCount(); i++) {
                Rectangle bounds = getComponent(i).getBounds();
                preferredSize.width = Math.max(bounds.x + bounds.width, preferredSize.width);
                preferredSize.height = Math.max(bounds.y + bounds.height, preferredSize.height);
            }
            Insets insets = getInsets();
            preferredSize.width += insets.right;
            preferredSize.height += insets.bottom;
            setMinimumSize(preferredSize);
            setPreferredSize(preferredSize);
        }
    }// </editor-fold>//GEN-END:initComponents

    private void jBtn_exitActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jBtn_exitActionPerformed
        // TODO add your handling code here:

        _controller.exit(this);

    }//GEN-LAST:event_jBtn_exitActionPerformed

    private void jBtn_sendEmailActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jBtn_sendEmailActionPerformed
        // TODO add your handling code here:

		_controller.sendEmailAddress();

    }//GEN-LAST:event_jBtn_sendEmailActionPerformed

    private void jTB_showTorusActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTB_showTorusActionPerformed
        // TODO add your handling code here:

		torusToggleButtonActionPerformed();

    }//GEN-LAST:event_jTB_showTorusActionPerformed

    private void jB_osborneActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jB_osborneActionPerformed
        // TODO add your handling code here:
		_isOsborne = !_isOsborne;
    }//GEN-LAST:event_jB_osborneActionPerformed


    // Variables declaration - do not modify//GEN-BEGIN:variables
    // Generated using JFormDesigner non-commercial license
    private JPanel jPanel1;
    private JButton jBtn_exit;
    private JScrollPane jScrollPane1;
    private JTextArea jTa_text;
    private JButton jBtn_sendEmail;
    private JPanel jPanelImage;
    private JLabel jL_homeWebPage;
    private JLabel jL_Version;
    private JLabel jL_downloadFileName;
    private JToggleButton jTB_showTorus;
    private JButton jB_osborne;
    // End of variables declaration//GEN-END:variables

	public void changeUrlCursor( int oldScrollYYValue )
	{
		_scrollableJTextForUrl.changeCursor( oldScrollYYValue );
	}

	public void setImageBackground( Color color )
	{
		_imageJPanel.setBackground(color);
	}

	@Override
	public void setComponentMapper(ComponentMapper mapper) {
		jBtn_exit = mapper.mapComponent(jBtn_exit);
		jBtn_sendEmail = mapper.mapComponent(jBtn_sendEmail);
		jL_homeWebPage = mapper.mapComponent(jL_homeWebPage);
		jPanelImage = mapper.mapComponent(jPanelImage);
		jScrollPane1 = mapper.mapComponent(jScrollPane1);
		jTa_text = mapper.mapComponent(jTa_text);
		jL_Version = mapper.mapComponent(jL_Version);
		jL_downloadFileName = mapper.mapComponent(jL_downloadFileName);
		jB_osborne = mapper.mapComponent(jB_osborne);

		JToggleButton jtb = mapper.mapComponent(jTB_showTorus);
		jTB_showTorus = jtb;
/*
		jTB_showTorus.addComponentListener(
			new ComponentListener() {
			@Override
			public void componentResized(ComponentEvent e) {
				addImageToShowTorusButton( TORUS_IMAGE_RESOURCE );
			}

			@Override
			public void componentMoved(ComponentEvent e) {
			}

			@Override
			public void componentShown(ComponentEvent e) {
			}

			@Override
			public void componentHidden(ComponentEvent e) {
			}
			
		});
*/
//		SwingUtilities.invokeLater( () -> ViewFunctions.instance().addImageToButtonFast( jtb,
//												TORUS_IMAGE_RESOURCE, new Insets(2,2,2,2) ) );

		super.setComponentMapper(mapper);
//
//		((ZoomJScrollPane) jScrollPane1).setNotComputeBlit(true);
	}
/*
	protected void addImageToShowTorusButton( String resourceName )
	{
		ViewFunctions.instance().addImageToButtonAccurate( jTB_showTorus,
												resourceName, new Insets(2,2,2,2) );
	}
*/
	protected void torusToggleButtonActionPerformed()
	{
		if( _animation != null )
		{
			boolean isSelected = jTB_showTorus.isSelected();
			if( isSelected )
			{
				_animation.animationReset();
			}
			else
			{
				_animation.animationStop();
				_lensJPanel.setSprite( null, null );
			}
			jB_osborne.setVisible(isSelected);
		}
	}

	protected BufferedImage getTorusImage()
	{
		BufferedImage result = _animation.animationGetFrame();
		if( _isOsborne )
		{
			if( _osborneImage == null )
				_osborneImage = createOsborneImage( result.getWidth(), result.getHeight() );
			result = _osborneImage;
		}

		return( result );
	}

	protected BufferedImage loadOsborne()
	{
		return( ExecutionFunctions.instance().safeFunctionExecution( () -> ImageFunctions.instance().loadImageFromJar(IconResources.OSBORNE_ICON_RESOURCE) ) );
	}

	protected BufferedImage createOsborneImage( int width, int height )
	{
		return( ImageFunctions.instance().resizeImageAccurately( loadOsborne(), width, height) );
	}

	protected void nextStepForAnimation()
	{
		if( _animation != null )
		{
			_animation.animationDoStep();

			if( _animation.animationGetPosition() != null )
				_lensJPanel.setSprite( getTorusImage(), point2Coordinate2D(_animation.animationGetPosition() ) );
		}
	}

	protected Coordinate2D point2Coordinate2D( Point pnt )
	{
		Coordinate2D result = null;
		if( pnt != null )
			result = new Coordinate2D(pnt.x, pnt.y);

		return( result );
	}

	protected void onScroll(ChangeEvent evt) {
		getWindow().repaint();
	}

	protected void addListeners() {
		jScrollPane1.getViewport().addChangeListener(this::onScroll);
	}

	protected void removeListeners() {
		jScrollPane1.getViewport().removeChangeListener(this::onScroll);
	}

	@Override
	public void releaseResources()
	{
		removeListeners();

		super.releaseResources();
	}
}
