/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.desktop.view.associatedcomponents;

import com.frojasg1.general.reflection.ReflectionFunctions;
import java.awt.Component;
import java.util.List;
import java.util.Set;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class ComponentEvtWrapperBuilderBase
{
	protected Class<? extends Component> getCompClass( Component component, Set<Class<? extends Component>> options )
	{
		Class<? extends Component> compClass = component.getClass();
		Class<? extends Component> result = null;
		for( Class<? extends Component> clazz: options )
			result = mostSuitable( compClass, clazz, result );

		return( result );
	}

	protected Class<? extends Component> mostSuitable(Class<?  extends Component> compClass,
														Class<? extends Component> clazz,
														Class<?  extends Component> result )
	{
		List< Class<?> > list = ReflectionFunctions.instance().getListOfClassesAndInterfacesOfClassFromTheMostGenericToTheMostParticular(compClass);

		int resultIndex = list.indexOf(result);
		int clazzIndex = list.indexOf(clazz);

		if( ( clazzIndex > -1 ) && ( clazzIndex < resultIndex ) )
			result = clazz;

		return( result );
	}
}
