/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.desktop.view.associatedcomponents;

import com.frojasg1.general.desktop.view.FrameworkComponentFunctions;
import com.frojasg1.general.desktop.view.zoom.mapper.ComponentMapper;
import java.awt.Component;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public abstract class ComponentEvtWrapperSimpleBase<CC extends Component, EV extends ComponentEvtEvent>
	extends ComponentEvtWrapperBase<EV>
	implements ComponentEvtSimpleWrapper<CC, EV>
{
	protected CC _component;
	protected Component _parent;

	public ComponentEvtWrapperSimpleBase( CC component, Component parent )
	{
		_component = component;
		if( parent != null )
			_parent = parent;
		else
			_parent = component;
	}

	@Override
	public void init()
	{
		super.init();
	}

	protected Component getParent()
	{
		return( _parent );
	}

	@Override
	protected void initChild()
	{
		addListener();

		addInternallyMappedComponentToParentWindow();
	}

	@Override
	public void setEnabled( boolean value )
	{
		getComponent().setEnabled(value);
	}

	protected void addInternallyMappedComponentToParentWindow()
	{
		FrameworkComponentFunctions.instance().addInternallyMappedComponentToParent( this, getParent() );
	}

	@Override
	protected EV createEvent()
	{
		EV result = super.createEvent();

		return( result );
	}

	protected boolean canNotify()
	{
		return( true );
	}

	protected void internalValueChangedListener()
	{
		if( canNotify() )
		{
			EV evt = createEvent();

			getListenersList().notifyEvt( evt );
		}
	}

	@Override
	public CC getComponent() {
		return _component;
	}

	@Override
	protected WrapperEventSource createSource()
	{
		WrapperEventSource result = super.createSource();
		result.setParentSource( createSource( getComponent() ) );

		return( result );
	}

	protected abstract void addListener();

	@Override
	protected abstract EV createEmptyEvent();

	@Override
	public void setComponentMapper( ComponentMapper mapper )
	{
		_component = mapper.mapComponent(_component);
	}
}
