/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.desktop.view.associatedcomponents.value.impl.integer;

import com.frojasg1.general.desktop.view.associatedcomponents.value.BoundedComponentValueWrapperSimpleDefault;
import java.awt.Component;
import javax.swing.JSlider;
import javax.swing.event.ChangeListener;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class JSliderWrapperDefault extends BoundedComponentValueWrapperSimpleDefault<JSlider, Integer>
{

	protected ChangeListener _componentListener;

	public JSliderWrapperDefault(JSlider component, Component parent ) {
		super(component, parent);
	}

	@Override
	protected void initChild()
	{
		_componentListener = createChangeListener();

		super.initChild();
	}

	protected ChangeListener createChangeListener()
	{
		return( evt -> internalValueChangedListener() );
	}

	public ChangeListener getComponentListener() {
		return _componentListener;
	}

	@Override
	protected void addListener() {
		getComponent().addChangeListener( getComponentListener() );
	}

	@Override
	protected void removeListeners() {
		getComponent().removeChangeListener( getComponentListener() );
	}

	@Override
	public Integer getValue() {
		return( getComponent().getValue() );
	}

	@Override
	public void setValue(Integer value) {
		getComponent().setValue(value);
	}

	@Override
	public void setBounds(Integer min, Integer max) {
		getComponent().setMinimum(min);
		getComponent().setMaximum(max);
	}

	@Override
	public Integer getMin() {
		return( getComponent().getMinimum() );
	}

	@Override
	public Integer getMax() {
		return( getComponent().getMaximum() );
	}
}
