/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.desktop.view.associatedcomponents.value.impl.integer;

import com.frojasg1.general.desktop.view.associatedcomponents.value.BoundedComponentValueWrapperSimpleDefault;
import java.awt.Component;
import javax.swing.JSpinner;
import javax.swing.SpinnerModel;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeListener;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class JSpinnerWrapperDefault extends BoundedComponentValueWrapperSimpleDefault<JSpinner, Integer>
{
	protected ChangeListener _componentListener;

	protected Integer _defaultValue;

	public JSpinnerWrapperDefault(JSpinner component, Component parent ) {
		super(component, parent);
	}

	@Override
	protected void initChild()
	{
		_componentListener = createChangeListener();

		super.initChild();
	}

	public Integer getDefaultValue() {
		return _defaultValue;
	}

	protected ChangeListener createChangeListener()
	{
		return( evt -> internalValueChangedListener() );
	}

	public ChangeListener getComponentListener() {
		return _componentListener;
	}

	@Override
	protected void addListener() {
		getComponent().addChangeListener( getComponentListener() );
	}

	@Override
	protected void removeListeners() {
		getComponent().removeChangeListener( getComponentListener() );
	}

	@Override
	public Integer getValue() {
		return( (Integer) getComponent().getValue() );
	}

	@Override
	public void setValue(Integer value) {
		_defaultValue = value;
		getComponent().setValue(value);
	}

	@Override
	public void setBounds(Integer min, Integer max) {
		SpinnerModel sm = new SpinnerNumberModel((Number) getDefaultValue(),
												min,
												max, 1); //default value,lower bound,upper bound,increment by
		getComponent().setModel(sm);
	}

	protected SpinnerNumberModel getNumberModel()
	{
		SpinnerNumberModel result = null;
		if( getComponent().getModel() instanceof SpinnerNumberModel )
			result = (SpinnerNumberModel) getComponent().getModel();

		return( result );
	}

	@Override
	public Integer getMin() {
		return( getIfNotNull( getNumberModel(), spinModel -> (Integer) spinModel.getMinimum() ) );
	}

	@Override
	public Integer getMax() {
		return( getIfNotNull( getNumberModel(), spinModel -> (Integer) spinModel.getMaximum() ) );
	}
}
