/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.desktop.view.buttons;

import com.frojasg1.applications.common.components.name.ComponentNameComponents;
import java.awt.Insets;
import javax.swing.AbstractButton;
import javax.swing.JButton;
import javax.swing.JToggleButton;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class ResizableImageJButtonBuilder
{
	protected static class LazyHolder
	{
		public static final ResizableImageJButtonBuilder INSTANCE = new ResizableImageJButtonBuilder();
	}

	public static ResizableImageJButtonBuilder instance()
	{
		return( LazyHolder.INSTANCE );
	}

	public String getImageResourceFromName( String name )
	{
		ComponentNameComponents cnc = new ComponentNameComponents( name );
		String result = cnc.getComponent( ComponentNameComponents.ICON_COMPONENT );

		return( result );
	}

	public AbstractButton createResizableImageJButton( AbstractButton jbutton )
	{
		Insets insets = new Insets( 1, 1, 1, 1 );
		AbstractButton result = null;
		String imageResource = getImageResourceFromName( jbutton.getName() );

		if( imageResource != null )
		{
			if( jbutton instanceof JToggleButton )
				result = new ResizableImageJToggleButton( imageResource, insets );
			else if( jbutton instanceof JButton )
				result = new ResizableImageJButton( imageResource, insets );
		}

		return( result );
	}
}
