/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.desktop.view.color.impl;

import com.frojasg1.general.desktop.image.ImageFunctions;
import com.frojasg1.general.desktop.view.color.ColorInversor;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.image.BufferedImage;
import java.util.function.Consumer;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class ColorThemeChangeableForCustomComponent extends ColorThemeChangeableBase
{
	protected Component _comp;
	protected Consumer<Graphics> _paintFunction;

	protected boolean _originallyWasDark;

	public ColorThemeChangeableForCustomComponent( Component comp,
												Consumer<Graphics> paintFunction,
												boolean originallyWasDark )
	{
		_comp = comp;
		_paintFunction = paintFunction;
		_originallyWasDark = originallyWasDark;
	}

	public boolean wasOriginallyDark()
	{
		return( _originallyWasDark );
	}

	public void paint( Graphics grp )
	{
		synchronized( this )
		{
			if( wasOriginallyDark() == isDarkMode() )
				_paintFunction.accept( grp );
			else
			{
				BufferedImage image = new BufferedImage( _comp.getWidth(),
														_comp.getHeight(),
														BufferedImage.TYPE_INT_ARGB );
				Graphics grp2 = image.createGraphics();
				grp2.setClip( grp.getClip() );
				_paintFunction.accept( grp2 );

				BufferedImage invertedImage = invertImage( image );
				grp.drawImage(invertedImage,
								0, 0,
								image.getWidth(), image.getHeight(),
								null );
				grp2.dispose();
			}
		}
	}

	public Consumer<Graphics> getPaintFunction() {
		return _paintFunction;
	}

	public void setPaintFunction(Consumer<Graphics> _paintFunction) {
		this._paintFunction = _paintFunction;
	}

	protected BufferedImage invertImage( BufferedImage image )
	{
		return( ImageFunctions.instance().invertImage( image ) );
	}

	protected Component getComponent()
	{
		return( _comp );
	}
}
