/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.desktop.view.color.renderers;

import com.frojasg1.general.desktop.view.color.ColorInversor;
import com.frojasg1.general.desktop.view.zoom.componentcopier.GenericCompCopier;
import java.awt.Component;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.table.TableCellRenderer;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class TableCellRendererColorInversor implements TableCellRenderer
{
	protected TableCellRenderer _originalRenderer;
	protected ColorInversor _colorInversor;
	protected String _name;

	public TableCellRendererColorInversor( TableCellRenderer originalRenderer,
										ColorInversor colorInversor )
	{
		_originalRenderer = originalRenderer;
		_colorInversor = colorInversor;
	}

	public TableCellRenderer getOriginalTableCellRenderer()
	{
		return( _originalRenderer );
	}

	public void setOriginalTableCellRenderer(TableCellRenderer tableCellRenderer)
	{
		_originalRenderer = tableCellRenderer;
	}

	public String getName()
	{
		return( _name );
	}

	public void setName( String name )
	{
		_name = name;
	}

	protected JLabel cloneLabel( JLabel label )
	{
		JLabel result = new JLabel();
		GenericCompCopier.instance().copyToNew(label, result);
		return( result );
	}

	@Override
	public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
		Component result = _originalRenderer.getTableCellRendererComponent( table, value,
														isSelected, hasFocus, row, column );
		if( result instanceof JLabel )
			result = cloneLabel((JLabel) result);

		invertColors( result, isSelected, hasFocus, row, column );

		return( result );
	}

	protected void invertColors(Component result, boolean isSelected, boolean hasFocus,
							int row, int column) {
//		_colorInversor.invertSingleColorsGen(result);
		_colorInversor.invertBackground(result);
		_colorInversor.invertForeground(result);
//		if( isSelected )
//		{
//			_colorInversor.invertSelectionBackground(result);
//			_colorInversor.invertSelectionForeground(result);
//		}
	}
}
