/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.desktop.view.combobox.chained.impl;

import com.frojasg1.applications.common.configuration.ParameterListConfiguration;
import com.frojasg1.applications.common.configuration.application.BaseApplicationConfigurationInterface;
import com.frojasg1.general.ExecutionFunctions;
import com.frojasg1.general.combohistory.TextComboBoxContent;
import com.frojasg1.general.combohistory.impl.TextComboBoxHistoryWithProperties;
import com.frojasg1.general.desktop.view.combobox.chained.ChainedParentComboBoxGroupManager;
import com.frojasg1.general.desktop.view.combobox.chained.ChainedParentForChildComboContentServer;
import java.util.List;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class ChainedParentChildComboBoxManagerAutomaticSave extends ChainedParentChildComboBoxManagerBase {
	

	public ChainedParentChildComboBoxManagerAutomaticSave( String key,
						BaseApplicationConfigurationInterface appliConf,
						String confBaseFileName,
						int maxItems,
						ChainedParentForChildComboContentServer contentServer )
	{
		super( key, null, contentServer, null );

		String language = null;
		ParameterListConfiguration conf = new ParameterListConfiguration( appliConf.getConfigurationMainFolder(),
															appliConf.getApplicationNameFolder(),
															appliConf.getApplicationGroup(), language,
															confBaseFileName + ".properties" );
		ExecutionFunctions.instance().safeMethodExecution( () -> conf.M_openConfiguration() );
		TextComboBoxHistoryWithProperties content = new TextComboBoxHistoryWithProperties( maxItems, conf );
		content.init( (List<String>)null);
		content.loadItems();

		setComboBoxContent( content );
	}

	public ChainedParentChildComboBoxManagerAutomaticSave( String key, TextComboBoxHistoryWithProperties contents,
						ChainedParentForChildComboContentServer contentServer )
	{
		this( key, contents, contentServer, null );
	}

	public ChainedParentChildComboBoxManagerAutomaticSave( String key, TextComboBoxHistoryWithProperties contents,
						ChainedParentForChildComboContentServer contentServer,
						ChainedParentComboBoxGroupManager parent )
	{
		super( key, contents, contentServer, parent );
	}

	@Override
	public void setComboBoxContent(TextComboBoxContent comboBoxContents)
	{
		if( !( comboBoxContents instanceof TextComboBoxHistoryWithProperties ) )
			throw( new RuntimeException( "comboBoxContents is not instanceof TextComboBoxHistoryWithProperties" ) );

		super.setComboBoxContent(comboBoxContents);
	}

	@Override
	public TextComboBoxHistoryWithProperties getComboBoxContent()
	{
		return( (TextComboBoxHistoryWithProperties) super.getComboBoxContent() );
	}

	@Override
	public void newItemSelected( String newItem )
	{
		super.newItemSelected( newItem );
		saveComboboxContents();
	}

	protected void saveComboboxContents()
	{
		ExecutionFunctions.instance().safeMethodExecution( () -> getComboBoxContent().save() );
	}
}
