/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.desktop.view.componentwrapper.combobox;

import com.frojasg1.general.desktop.edt.EventDispatchThreadFunctions;
import com.frojasg1.general.desktop.generic.view.DesktopViewWindow;
import com.frojasg1.general.desktop.view.ComponentFunctions;
import com.frojasg1.general.desktop.view.FrameworkComponentFunctions;
import com.frojasg1.general.desktop.view.componentwrapper.JComponentWrapper;
import com.frojasg1.general.desktop.view.componentwrapper.SwingSystemDependingFunctions;
import com.frojasg1.general.desktop.view.zoom.mapper.ComponentMapper;
import com.frojasg1.general.view.evt.wrapper.impl.combobox.GenComboBoxWrapperEventBase;
import com.frojasg1.general.ArrayFunctions;
import com.frojasg1.general.desktop.view.componentwrapper.JComponentWrapperCommon;
import com.frojasg1.general.view.evt.wrapper.impl.combobox.impl.ComboBoxWrapperEventBuilderBase;
import com.frojasg1.general.view.evt.wrapper.impl.combobox.impl.ComboBoxWrapperNewItemSelectedEvt;
import com.frojasg1.general.desktop.view.menus.BaseJPopupMenu;
import com.frojasg1.general.desktop.view.transfer.file.FileTransferHandler;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Locale;
import java.util.Objects;
import java.util.function.Function;
import java.util.function.Supplier;
import javax.swing.*;
import javax.swing.event.PopupMenuEvent;
import javax.swing.event.PopupMenuListener;
import javax.swing.plaf.ComboBoxUI;
import com.frojasg1.general.view.gencomponentwrapper.combobox.GenComboBoxWrapper;

/**
 *
 * @author fjavier.rojas
 */
public class JComboBoxWrapper<RR>
	extends GenComboBoxWrapper<JComboBox<RR>, RR>
		implements JComponentWrapper<JComboBox<RR>, ComboBoxUI, GenComboBoxWrapperEventBase, Object> {
//
//	protected FileTransferHandler<Object> _fileTransferHandler;
	protected ComboBoxUI _compUI;

//	protected BaseJPopupMenu _basePopupMenu = null;

	protected boolean _hasBeenAlreadyMapped;

	protected Supplier<RR[]> _arrayBuilderForFillingCombo;

	protected Class<RR> _itemClass;

	protected PopupMenuListener _popupMenuListener;

	protected ActionListener _actionListener;

	protected Function<RR, ?> _functionForEquals;

	public JComboBoxWrapper( JComboBox<RR> combo, Class<RR> itemClass, Supplier<RR[]> arrayBuilderForFillingCombo )
	{
		this( combo, itemClass, arrayBuilderForFillingCombo, Function.identity() );
	}

	public JComboBoxWrapper( JComboBox<RR> combo, Class<RR> itemClass,
							Supplier<RR[]> arrayBuilderForFillingCombo,
							Function<RR, ?> functionForEquals)
	{
		this( combo, itemClass, arrayBuilderForFillingCombo, functionForEquals, true );
	}

	protected JComboBoxWrapper( JComboBox<RR> combo, Class<RR> itemClass,
							Supplier<RR[]> arrayBuilderForFillingCombo,
							Function<RR, ?> functionForEquals,
							boolean init)
	{
		super(combo, itemClass, arrayBuilderForFillingCombo, functionForEquals, SwingSystemDependingFunctions.instance(),
				false);
//		super( combo, new ComboBoxWrapperEventBuilderBase<RR, JComboBox<RR>>(combo, itemClass, true),
//				SwingSystemDependingFunctions.instance());

		_functionForEquals = functionForEquals;
		_itemClass = itemClass;
		_arrayBuilderForFillingCombo = arrayBuilderForFillingCombo;

		FrameworkComponentFunctions.instance().addInternallyMappedComponentToParent(this, combo);

		if( init )
			init();
	}

	protected void init()
	{
//		setUI();

//		setFileTransfer();
//
//		_basePopupMenu = createBasePopupMenu();
////		_listenersList = createListenersList();
////
////		addListeners();
		super.init();

//		changeViewportUIToBlockable();
	}

//	protected void installUI(ComboBoxUI compUI) {
//
//	}
//	protected synchronized void setCompUI(ComboBoxUI compUI) {
//		this._compUI = compUI;
//		if( compUI != null )
//			installUI( compUI );
//	}
//
//	protected synchronized void setUI()
//	{
//		ComboBoxUI compUI = createUI();
//		setCompUI( compUI );
//		updateUI(compUI);
//	}
//
//	protected void updateUI(ComboBoxUI compUI) {
//
//	}
//
//	protected ComboBoxUI createUI()
//	{
//		return( null );
//	}

	@Override
	protected void addListeners() {
		getCombo().addPopupMenuListener( getPopupMenuListener() );
		getCombo().addActionListener( getActionListener() );
	}

	@Override
	protected void removeListeners() {
		getCombo().removePopupMenuListener( getPopupMenuListener() );
		getCombo().removeActionListener( getActionListener() );
	}

	@Override
	public void updateInternal() {
		fillCombo();
	}

	protected PopupMenuListener createPopupMenuListener()
	{
		return( new PopupMenuListenerForWrapper() );
	}

	protected synchronized PopupMenuListener getPopupMenuListener()
	{
		if( _popupMenuListener == null )
			_popupMenuListener = createPopupMenuListener();
		
		return( _popupMenuListener );
	}

//	protected ActionListener createActionListener()
//	{
//		return( this::fireNewItemSelected );
//	}
//
//	protected void fireNewItemSelected( ActionEvent evt )
//	{
//		notifyEvt( createNewItemSelectedEvent() );
//	}
//
//	public ComboBoxWrapperEventBuilderBase<RR, JComboBox<RR>> getEventBuilder() {
//		return( (ComboBoxWrapperEventBuilderBase<RR, JComboBox<RR>>) super.getEventBuilder() );
//	}
//
//	protected ComboBoxWrapperNewItemSelectedEvt createNewItemSelectedEvent()
//	{
//		return( getEventBuilder().createJComboBoxWrapperNewItemSelected( getSelectedItem() ) );
//	}

	protected synchronized ActionListener getActionListener()
	{
		if( _actionListener == null )
			_actionListener = createActionListener();
		
		return( _actionListener );
	}

//	protected RR[] getItems()
//	{
//		RR[] result = null;
//
//		if( _arrayBuilderForFillingCombo != null )
//			result = _arrayBuilderForFillingCombo.get();
//
//		return( result );
//	}

	protected ComboBoxModel<RR> createModel( RR[] items )
	{
		return( new DefaultComboBoxModel<>( items ) );
	}

//	public JComboBox<RR> getCombo()
//	{
//		return( (JComboBox<RR>) getJComponent() );
//	}

	@Override
	public JComboBox<RR> getJComponent() {
		return getGenComponent();
	}

	@Override
	public ComboBoxUI getCompUI() {
		return null;
	}

	@Override
	public FileTransferHandler getFileTransferHandler() {
		return null;
	}

	@Override
	public DesktopViewWindow getMainWindow()
	{
		return( FrameworkComponentFunctions.instance().getInternationalizedWindow( getJComponent()) );
	}

	@Override
	public RR getSelectedItem()
	{
		return( (RR) getCombo().getSelectedItem() );
	}

	@Override
	public void setSelectedItem(RR newSelectedItem) {
		getCombo().setSelectedItem(newSelectedItem);
	}


	@Override
	public void setSelectedIndex(int newSelectedIndex) {
		getCombo().setSelectedIndex(newSelectedIndex);
	}

	@Override
	protected void addItems(RR[] items) {
		ComboBoxModel<RR> model = createModel( items );
		getCombo().setModel(model);
	}

//	protected void fillCombo()
//	{
//		RR[] items = getItems();
//		if( hasChanged( items ) )
//		{
//			ComboBoxModel<RR> model = createModel( items );
//
//			RR selectedItem = getSelectedItem();
//
//			getCombo().setModel(model);
//
//			int newSelectedIndex = ( selectedItem == null ) ?
//									-1 :
//									ArrayFunctions.instance().getFirstIndexOfEquals(items, selectedItem,
//																					_functionForEquals);
//
//			if( newSelectedIndex >= 0 )
//				getCombo().setSelectedIndex( newSelectedIndex );
//			else if( items.length > 0 )
//				getCombo().setSelectedIndex(0);
//		}
//	}

	@Override
	protected boolean hasChanged( RR[] items )
	{
		ComboBoxModel model = getCombo().getModel();
		boolean result = ( items.length != model.getSize() );
		for( int ii = 0; !result && (ii < items.length ); ii++ )
			result = !Objects.equals( items[ii], model.getElementAt(ii) );

		return( result );
	}

	@Override
	public void releaseResources() {
		super.releaseResources();
	}

	@Override
	public JScrollPane getScrollPane(Component component) {
		return( ComponentFunctions.instance().getScrollPane( component ) );
	}

	@Override
	public Locale getOutputLocale() {
		return( FrameworkComponentFunctions.instance().getOutputLocale( getJComponent() ) );
	}

	@Override
	public void setComponentMapper(ComponentMapper mapper) {
		setGenComponent(mapper.mapComponent( getJComponent() ));

		_hasBeenAlreadyMapped = true;
	}

	@Override
	public boolean hasBeenAlreadyMapped()
	{
		return( _hasBeenAlreadyMapped );
	}

	@Override
	public void update()
	{
		EventDispatchThreadFunctions.instance().edtAsinchronousMethodExecution(this::updateInternal);
	}

	protected class PopupMenuListenerForWrapper implements PopupMenuListener
	{
		@Override
		public void popupMenuWillBecomeVisible(PopupMenuEvent e) {
			updateInternal();
		}

		@Override
		public void popupMenuWillBecomeInvisible(PopupMenuEvent e) {
		}

		@Override
		public void popupMenuCanceled(PopupMenuEvent e) {
		}
	}
}
