/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.desktop.view.componentwrapper.jtable.column;

import javax.swing.table.TableColumnModel;
import com.frojasg1.general.codec.SelfStringCodec;
import com.frojasg1.general.collection.impl.ThreadSafeArrayListWrapperSelfStringCodec;
import java.util.Enumeration;
import javax.swing.table.TableColumn;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class JTableArrangeableColumnsContext<RR>
	extends ThreadSafeArrayListWrapperSelfStringCodec<TableColumnData<RR>>
	implements SelfStringCodec
{
	public synchronized void update( TableColumnModel columnModel )
	{
		clear();
		
		Enumeration<TableColumn> columns = columnModel.getColumns();
		while( columns.hasMoreElements() )
		{
			TableColumnData tcd = getTableColumnData( columns.nextElement() );
			if( tcd != null )
				add( tcd );
		}
	}

	protected TableColumnData<RR> getTableColumnData( TableColumn tc )
	{
		TableColumnData result = null;
		if( ( tc != null ) && ( tc.getHeaderValue() instanceof TableColumnData ) )
			result = (TableColumnData) tc.getHeaderValue();

		return( result );
	}
}
