/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.desktop.view.painter;

import com.frojasg1.general.desktop.view.color.ColorInversor;
import java.awt.Component;
import java.awt.Graphics;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public abstract class ExternalPainterBase implements ExternalPainter
{
	protected boolean isActive = true;

	public boolean isActive() {
		return isActive;
	}

	public void setActive(boolean active) {
		isActive = active;
	}

	protected abstract void paintComponentInternal(Component comp, Graphics grp );

	@Override
	public void paintComponent(Component comp, Graphics grp)
	{
		if(isActive()) {
			synchronized (comp) {
				paintComponentInternal(comp, grp);
			}
		}
	}

	@Override
	public abstract void invertColors(ColorInversor colorInversor);
}
