/*
 * Copyright (c) 2023. Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 3.0 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */

package com.frojasg1.general.desktop.view.painter.impl;

import com.frojasg1.general.desktop.view.color.ColorInversor;
import com.frojasg1.general.desktop.view.painter.ExternalPainter;
import com.frojasg1.general.desktop.view.painter.ExternalPainterBase;

import java.awt.*;
import java.util.List;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

public class ComposedExternalPainter<KK> extends ExternalPainterBase {

    protected Map<KK, ExternalPainterBase> paintersMap = new ConcurrentHashMap<>();
    protected List<ExternalPainterBase> painters = Collections.synchronizedList(new ArrayList<>());

    public void put(KK key, ExternalPainterBase painter) {
        painters.add(painter);
        paintersMap.put(key, painter);
    }

    public ExternalPainterBase get(KK key) {
        return paintersMap.get(key);
    }

    public void setActive(KK key, boolean value) {
        ExternalPainterBase painter = get(key);
        if(painter != null) {
            painter.setActive(value);
        }
    }

    public void setOnlyActive(KK key) {
        deactivateAll();
        setActive(key, true);
    }

    public void deactivateAll() {
        for(ExternalPainterBase externalPainter: painters) {
            externalPainter.setActive(false);
        }
    }

    @Override
    protected void paintComponentInternal(Component comp, Graphics grp) {
        for(ExternalPainter externalPainter: painters) {
            externalPainter.paintComponent(comp, grp);
        }
    }

    @Override
    public void invertColors(ColorInversor colorInversor) {
        for(ExternalPainter externalPainter: painters) {
            externalPainter.invertColors(colorInversor);
        }
    }
}
