/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.desktop.view.panels;

import java.awt.*;
import javax.swing.*;
import com.frojasg1.applications.common.components.resizecomp.MapResizeRelocateComponentItem;
import com.frojasg1.applications.common.components.resizecomp.ResizeRelocateItem;
import com.frojasg1.general.desktop.view.ViewFunctions;
import com.frojasg1.general.desktop.view.zoom.mapper.ComponentMapper;
import com.frojasg1.general.desktop.view.zoom.mapper.InternallyMappedComponent;
import javax.swing.JButton;

/**
 *
 * @author Usuario
 */
public class AcceptCancelRevertPanel extends com.frojasg1.general.desktop.view.panels.CustomJPanelBase
									implements // ResizableComponentInterface,
												InformerInterface,
												AcceptCancelRevertControllerInterface,
												InternallyMappedComponent
{
	protected static final String _resourcePath = "com/frojasg1/generic/resources/acceptcancel";

	protected AcceptCancelRevertControllerInterface _controller = null;

	protected MapResizeRelocateComponentItem _mapRRCI = null;


	/**
	 * Creates new form AcceptCancelRevertPanel
	 */
	public AcceptCancelRevertPanel( AcceptCancelRevertControllerInterface controller )
	{
		init();

		_controller = controller;

		initComponents();

		setBounds( 0, 0, jPanel1.getWidth(), jPanel1.getHeight() );
		
//		initNavigatorButtons();

		_mapRRCI = createResizeRelocateInfo();
	}

	protected String getResourceName( String shortResourceName )
	{
		return( _resourcePath + "/" + shortResourceName );
	}

	public JButton getCancelButton()
	{
		return( jB_cancel );
	}
/*
	protected void initNavigatorButtons()
	{
		Insets insets = new Insets( 1, 1, 1, 1 );

		ViewFunctions.instance().addImageToButtonAccurate( jB_cancel, getResourceName( "erase.png" ), insets );
		ViewFunctions.instance().addImageToButtonAccurate( jB_acceptChanges, getResourceName( "accept.png" ), insets );
		ViewFunctions.instance().addImageToButtonAccurate( jB_revertChanges, getResourceName( "revert.png" ), insets );
	}
*/
	public MapResizeRelocateComponentItem getResizeRelocateInfo()
	{
		if( _mapRRCI == null )
		{
			_mapRRCI = createResizeRelocateInfo();
		}

		return( _mapRRCI );
	}

	protected MapResizeRelocateComponentItem createResizeRelocateInfo()
	{
		MapResizeRelocateComponentItem mapRRCI = new MapResizeRelocateComponentItem();

		try
		{
			mapRRCI.putResizeRelocateComponentItem( jPanel1, ResizeRelocateItem.FILL_WHOLE_PARENT );
			mapRRCI.putResizeRelocateComponentItem( jB_acceptChanges, ResizeRelocateItem.MOVE_LEFT_SIDE_PROPORTIONAL +
																		ResizeRelocateItem.MOVE_RIGHT_SIDE_PROPORTIONAL +
																		ResizeRelocateItem.MOVE_TOP_SIDE_PROPORTIONAL +
																		ResizeRelocateItem.MOVE_BOTTOM_SIDE_PROPORTIONAL );
			mapRRCI.putResizeRelocateComponentItem(jB_cancel, ResizeRelocateItem.MOVE_LEFT_SIDE_PROPORTIONAL +
																		ResizeRelocateItem.MOVE_RIGHT_SIDE_PROPORTIONAL +
																		ResizeRelocateItem.MOVE_TOP_SIDE_PROPORTIONAL +
																		ResizeRelocateItem.MOVE_BOTTOM_SIDE_PROPORTIONAL );
			mapRRCI.putResizeRelocateComponentItem( jB_revertChanges, ResizeRelocateItem.MOVE_LEFT_SIDE_PROPORTIONAL +
																		ResizeRelocateItem.MOVE_RIGHT_SIDE_PROPORTIONAL +
																		ResizeRelocateItem.MOVE_TOP_SIDE_PROPORTIONAL +
																		ResizeRelocateItem.MOVE_BOTTOM_SIDE_PROPORTIONAL );
/*
			mapRRCI.putResizeRelocateComponentItem( jB_acceptChanges, 0 );
			mapRRCI.putResizeRelocateComponentItem(jB_cancel, 0 );
			mapRRCI.putResizeRelocateComponentItem( jB_revertChanges, 0 );
*/

//			HintForComponent hfc = new HintForComponent( jB_acceptChanges, "Accept changes" );
//			HintForComponent hfc2 = new HintForComponent( jB_eraseTag, "Erase record" );
//			HintForComponent hfc3 = new HintForComponent( jB_revertChanges, "Revert changes" );
		}
		catch( Throwable th )
		{
			th.printStackTrace();
		}

		return( mapRRCI );
	}

	public void setAcceptButtonEnabled( boolean value )
	{
		jB_acceptChanges.setEnabled( value );
	}

	public void setRevertButtonEnabled( boolean value )
	{
		jB_revertChanges.setEnabled( value );
	}

	public void setEraseButtonEnabled( boolean value )
	{
		jB_cancel.setEnabled( value );
	}

	public void setCompoundNameForAcceptButton( String name )
	{
		this.jB_acceptChanges.setName( name );
	}

	public void setCompoundNameForRevertButton( String name )
	{
		this.jB_revertChanges.setName( name );
	}

	public void setCompoundNameForCancelButton( String name )
	{
		this.jB_cancel.setName( name );
	}

	public void setNameForAcceptButton( String name )
	{
		ViewFunctions.instance().setNameForComponent( jB_acceptChanges, name );
	}

	public void setNameForRevertButton( String name )
	{
		ViewFunctions.instance().setNameForComponent( jB_revertChanges, name );
	}

	public void setNameForCancelButton( String name )
	{
		ViewFunctions.instance().setNameForComponent( jB_cancel, name );
	}

	public String getCompoundNameForAcceptButton()
	{
		return( jB_acceptChanges.getName() );
	}

	public String getCompoundNameForRevertButton()
	{
		return( jB_revertChanges.getName() );
	}

	public String getCompoundNameForCancelButton()
	{
		return( jB_cancel.getName() );
	}

	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	@SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    // Generated using JFormDesigner non-commercial license
    private void initComponents() {
        jPanel1 = new JPanel();
        jB_cancel = new JButton();
        jB_acceptChanges = new JButton();
        jB_revertChanges = new JButton();

        //======== this ========
        setMinimumSize(new Dimension(130, 50));
        setPreferredSize(new Dimension(130, 50));
        setLayout(null);

        //======== jPanel1 ========
        {
            jPanel1.setLayout(null);

            //---- jB_cancel ----
            jB_cancel.setHorizontalTextPosition(SwingConstants.CENTER);
            jB_cancel.setMaximumSize(new Dimension(10000, 10000));
            jB_cancel.setMinimumSize(new Dimension(0, 0));
            jB_cancel.setName("name=jB_cancel,icon=com/frojasg1/generic/resources/acceptcancel/cancel.png");
            jB_cancel.setPreferredSize(new Dimension(30, 30));
            jB_cancel.addActionListener(e -> jB_cancelActionPerformed(e));
            jPanel1.add(jB_cancel);
            jB_cancel.setBounds(10, 10, jB_cancel.getPreferredSize().width, 30);

            //---- jB_acceptChanges ----
            jB_acceptChanges.setHorizontalTextPosition(SwingConstants.CENTER);
            jB_acceptChanges.setMaximumSize(new Dimension(10000, 10000));
            jB_acceptChanges.setMinimumSize(new Dimension(0, 0));
            jB_acceptChanges.setName("name=jB_acceptChanges,icon=com/frojasg1/generic/resources/acceptcancel/accept.png");
            jB_acceptChanges.setPreferredSize(new Dimension(30, 30));
            jB_acceptChanges.addActionListener(e -> jB_acceptChangesActionPerformed(e));
            jPanel1.add(jB_acceptChanges);
            jB_acceptChanges.setBounds(50, 10, 30, 30);

            //---- jB_revertChanges ----
            jB_revertChanges.setHorizontalTextPosition(SwingConstants.CENTER);
            jB_revertChanges.setMaximumSize(new Dimension(10000, 10000));
            jB_revertChanges.setMinimumSize(new Dimension(0, 0));
            jB_revertChanges.setName("name=jB_revertChanges,icon=com/frojasg1/generic/resources/acceptcancel/revert.png");
            jB_revertChanges.setPreferredSize(new Dimension(30, 30));
            jB_revertChanges.addActionListener(e -> jB_revertChangesActionPerformed(e));
            jPanel1.add(jB_revertChanges);
            jB_revertChanges.setBounds(90, 10, 30, 30);

            {
                // compute preferred size
                Dimension preferredSize = new Dimension();
                for(int i = 0; i < jPanel1.getComponentCount(); i++) {
                    Rectangle bounds = jPanel1.getComponent(i).getBounds();
                    preferredSize.width = Math.max(bounds.x + bounds.width, preferredSize.width);
                    preferredSize.height = Math.max(bounds.y + bounds.height, preferredSize.height);
                }
                Insets insets = jPanel1.getInsets();
                preferredSize.width += insets.right;
                preferredSize.height += insets.bottom;
                jPanel1.setMinimumSize(preferredSize);
                jPanel1.setPreferredSize(preferredSize);
            }
        }
        add(jPanel1);
        jPanel1.setBounds(0, 0, 130, 50);

        {
            // compute preferred size
            Dimension preferredSize = new Dimension();
            for(int i = 0; i < getComponentCount(); i++) {
                Rectangle bounds = getComponent(i).getBounds();
                preferredSize.width = Math.max(bounds.x + bounds.width, preferredSize.width);
                preferredSize.height = Math.max(bounds.y + bounds.height, preferredSize.height);
            }
            Insets insets = getInsets();
            preferredSize.width += insets.right;
            preferredSize.height += insets.bottom;
            setMinimumSize(preferredSize);
            setPreferredSize(preferredSize);
        }
    }// </editor-fold>//GEN-END:initComponents

    private void jB_cancelActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jB_cancelActionPerformed

		cancel( this );

    }//GEN-LAST:event_jB_cancelActionPerformed

    private void jB_acceptChangesActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jB_acceptChangesActionPerformed

		accept( this );

    }//GEN-LAST:event_jB_acceptChangesActionPerformed

    private void jB_revertChangesActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jB_revertChangesActionPerformed
        // TODO add your handling code here:

        revert( this );
    }//GEN-LAST:event_jB_revertChangesActionPerformed


    // Variables declaration - do not modify//GEN-BEGIN:variables
    // Generated using JFormDesigner non-commercial license
    private JPanel jPanel1;
    private JButton jB_cancel;
    private JButton jB_acceptChanges;
    private JButton jB_revertChanges;
    // End of variables declaration//GEN-END:variables

/*
	@Override
	public void doTasksAfterResizingComponent()
	{
//		initNavigatorButtons();		// to resize the images.
	}
*/
	@Override
	public void accept(InformerInterface panel)
	{
		if( _controller != null )
			_controller.accept( this );
	}

	@Override
	public void cancel(InformerInterface panel)
	{
		if( _controller != null )
			_controller.cancel( this );
	}

	@Override
	public void revert(InformerInterface panel)
	{
		if( _controller != null )
			_controller.revert( this );
	}

	@Override
	public void setComponentMapper(ComponentMapper mapper) {
		jB_acceptChanges = mapper.mapComponent(jB_acceptChanges);
		jB_cancel = mapper.mapComponent(jB_cancel);
		jB_revertChanges = mapper.mapComponent(jB_revertChanges);
		jPanel1 = mapper.mapComponent(jPanel1);

		super.setComponentMapper(mapper);
	}
}
