/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.desktop.view.panels;

import com.frojasg1.applications.common.components.resizecomp.MapResizeRelocateComponentItem;
import com.frojasg1.general.desktop.view.zoom.mapper.ComposedComponent;
import java.awt.Dimension;
import java.awt.Rectangle;
import javax.swing.JPanel;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public abstract class ComposedCustomJPanelBase extends CustomJPanelBase
									implements ComposedComponent
{
	protected MapResizeRelocateComponentItem _resizeRelocateInfo = null;

	protected abstract MapResizeRelocateComponentItem createMapResizeRelocateComponentItem();

	@Override
	public void init()
	{
		super.init();
		
		initComponentsChild();
		setWindowConfiguration();
	}

	protected abstract void initComponentsChild();

	protected void setWindowConfiguration( )
	{
		_resizeRelocateInfo = createMapResizeRelocateComponentItem();
		if( _resizeRelocateInfo != null )
			_resizeRelocateInfo.addAllPopupMenus( getJPopupMenuListCopy() );
	}

	@Override
	public MapResizeRelocateComponentItem getResizeRelocateInfo()
	{
		return( _resizeRelocateInfo );
	}

	@Override
	public Dimension getInternalSize()
	{
		return( getParentPanel().getSize() );
	}

	@Override
	public Rectangle getInternalBounds()
	{
		return( getParentPanel().getBounds() );
	}

	protected abstract JPanel getParentPanel();
}
