/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.desktop.view.panels.evt;

import com.frojasg1.applications.common.configuration.application.BaseApplicationConfigurationInterface;
import com.frojasg1.general.view.evt.wrapper.ListOfGenComponentWrapperEventContextListenersBase;
import com.frojasg1.general.view.evt.wrapper.impl.GenComponentWrapperEvent;
import com.frojasg1.general.view.evt.wrapper.impl.GenComponentWrapperEventBuilder;
import com.frojasg1.general.desktop.view.panels.CustomJPanelBase;
import java.util.function.Consumer;
import javax.swing.JPanel;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class CustomJPanelBaseWithJComponentEvt extends CustomJPanelBase
{
	protected ListOfGenComponentWrapperEventContextListenersBase<JPanel, GenComponentWrapperEvent> _listenersList;
	protected GenComponentWrapperEventBuilder<GenComponentWrapperEvent> _eventBuilder;

	protected CustomJPanelBaseWithJComponentEvt( boolean doNotInvertColors, boolean init )
	{
		this( null, doNotInvertColors, init );
	}

	protected CustomJPanelBaseWithJComponentEvt( BaseApplicationConfigurationInterface appliConf,
												boolean doNotInvertColors, boolean init )
	{
		super( appliConf, doNotInvertColors );

		if( init )
			init();
	}

	protected void init()
	{
		super.init();

		_listenersList = createListenersList();
		_eventBuilder = createEventBuilder();
	}

	protected GenComponentWrapperEventBuilder<GenComponentWrapperEvent> createEventBuilder()
	{
		// can be overriden by derived classes.
		return( null );
	}

	public GenComponentWrapperEventBuilder<GenComponentWrapperEvent> getEventBuilder() {
		return _eventBuilder;
	}

	protected ListOfGenComponentWrapperEventContextListenersBase<JPanel, GenComponentWrapperEvent>
		createListenersList()
	{
		return( new ListOfGenComponentWrapperEventContextListenersBase<>() );
	}

	public ListOfGenComponentWrapperEventContextListenersBase<JPanel, GenComponentWrapperEvent>
		getListenersList()
	{
		return( _listenersList );
	}

	public void addWrapperListener(Consumer<? super GenComponentWrapperEvent> listener, Long ... eventTypes )
	{
		getListenersList().addListenerContext(listener, eventTypes);
	}

	public void removeWrapperListener( Consumer<? super GenComponentWrapperEvent> listener )
	{
		getListenersList().remove(listener);
	}

	public void notifyEvt( GenComponentWrapperEvent evt )
	{
		getListenersList().notifyEvt(evt);
	}

	protected GenComponentWrapperEvent createEvt(long eventType )
	{
		return( getEventBuilder().buildEvent(eventType) );
	}

	public <EVT extends GenComponentWrapperEvent> EVT createEvt(Class<EVT> clazz )
	{
		return( getEventBuilder().buildEvent(clazz) );
	}
}
