/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.desktop.view.panels.evt.map;

import com.frojasg1.general.desktop.view.zoom.mapper.ComponentMapper;
import com.frojasg1.general.desktop.view.zoom.mapper.InternallyMappedComponent;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;
import javax.swing.AbstractButton;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class ButtonEventEntry<ET> implements InternallyMappedComponent {

	protected ET _eventType;
	protected AbstractButton _button;
	protected String _imageResource;

	protected List<AbstractButton> _associatedButtonsList = new ArrayList<>();

	public ET getEventType() {
		return _eventType;
	}

	public void setEventType(ET _eventType) {
		this._eventType = _eventType;
	}

	public AbstractButton getButton() {
		return _button;
	}

	public void setButton(AbstractButton _button) {
		this._button = _button;
	}

	public String getImageResource() {
		return _imageResource;
	}

	public void setImageResource(String _imageResource) {
		this._imageResource = _imageResource;
	}

	public List<AbstractButton> getAssociatedButtonList()
	{
		return( _associatedButtonsList );
	}

	public void addAssociatedButton( AbstractButton button )
	{
		if( !_associatedButtonsList.contains(button) )
			_associatedButtonsList.add( button );
	}

	@Override
	public void setComponentMapper(ComponentMapper mapper)
	{
		_button = mapper.mapComponent(_button);

		List<AbstractButton> newAssociatedList = getAssociatedButtonList().stream()
			.map( mapper::mapComponent ).collect( Collectors.toList() );

		_associatedButtonsList = newAssociatedList;
	}

	@Override
	public boolean hasBeenAlreadyMapped() {
		throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
	}
}
