/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.desktop.view.panels.impl.file;

import java.awt.*;
import com.frojasg1.general.desktop.view.panels.evt.BaseJPanelOfEvents;
import com.frojasg1.general.desktop.view.panels.evt.map.ButtonEventEntryMultiMap;
import com.frojasg1.general.desktop.view.zoom.mapper.ComponentMapper;
import com.frojasg1.general.desktop.view.zoom.mapper.ComposedComponent;
import java.util.List;
import javax.swing.JButton;
import javax.swing.JPanel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class FileHandllingJPanel
	extends BaseJPanelOfEvents<FileHandlingPanelEventType, FileHandlingPanelEvent>
	implements ComposedComponent
{
	private static final Logger LOGGER = LoggerFactory.getLogger(FileHandllingJPanel.class);

	/**
	 * Creates new form PlayerNavigatorJPanel
	 */
	public FileHandllingJPanel() {
		super( FileHandlingPanelEventType.class );
	}

	public void init()
	{
		super.init();
	}

	@Override
	protected void initComponentsChild() {
		initComponents();
	}

	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	@SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    // Generated using JFormDesigner non-commercial license
    private void initComponents() {
        jPanel1 = new JPanel();
        jB_saveAs = new JButton();
        jB_open = new JButton();
        jB_save = new JButton();

        //======== this ========
        setLayout(null);

        //======== jPanel1 ========
        {
            jPanel1.setLayout(null);

            //---- jB_saveAs ----
            jB_saveAs.setMaximumSize(new Dimension(20, 20));
            jB_saveAs.setMinimumSize(new Dimension(20, 20));
            jB_saveAs.setName("name=jB_saveAs,icon=com/frojasg1/generic/resources/file/saveAs.png");
            jB_saveAs.setPreferredSize(new Dimension(20, 20));
            jPanel1.add(jB_saveAs);
            jB_saveAs.setBounds(55, 5, 20, jB_saveAs.getPreferredSize().height);

            //---- jB_open ----
            jB_open.setMaximumSize(new Dimension(20, 20));
            jB_open.setMinimumSize(new Dimension(20, 20));
            jB_open.setName("name=jB_open,icon=com/frojasg1/generic/resources/file/openFile.png");
            jB_open.setPreferredSize(new Dimension(20, 20));
            jPanel1.add(jB_open);
            jB_open.setBounds(5, 5, 20, 20);

            //---- jB_save ----
            jB_save.setMaximumSize(new Dimension(20, 20));
            jB_save.setMinimumSize(new Dimension(20, 20));
            jB_save.setName("name=jB_save,icon=com/frojasg1/generic/resources/file/save.png");
            jB_save.setPreferredSize(new Dimension(20, 20));
            jPanel1.add(jB_save);
            jB_save.setBounds(30, 5, 20, jB_save.getPreferredSize().height);

            {
                // compute preferred size
                Dimension preferredSize = new Dimension();
                for(int i = 0; i < jPanel1.getComponentCount(); i++) {
                    Rectangle bounds = jPanel1.getComponent(i).getBounds();
                    preferredSize.width = Math.max(bounds.x + bounds.width, preferredSize.width);
                    preferredSize.height = Math.max(bounds.y + bounds.height, preferredSize.height);
                }
                Insets insets = jPanel1.getInsets();
                preferredSize.width += insets.right;
                preferredSize.height += insets.bottom;
                jPanel1.setMinimumSize(preferredSize);
                jPanel1.setPreferredSize(preferredSize);
            }
        }
        add(jPanel1);
        jPanel1.setBounds(-1, 0, 80, 30);

        {
            // compute preferred size
            Dimension preferredSize = new Dimension();
            for(int i = 0; i < getComponentCount(); i++) {
                Rectangle bounds = getComponent(i).getBounds();
                preferredSize.width = Math.max(bounds.x + bounds.width, preferredSize.width);
                preferredSize.height = Math.max(bounds.y + bounds.height, preferredSize.height);
            }
            Insets insets = getInsets();
            preferredSize.width += insets.right;
            preferredSize.height += insets.bottom;
            setMinimumSize(preferredSize);
            setPreferredSize(preferredSize);
        }
    }// </editor-fold>//GEN-END:initComponents


    // Variables declaration - do not modify//GEN-BEGIN:variables
    // Generated using JFormDesigner non-commercial license
    private JPanel jPanel1;
    private JButton jB_saveAs;
    private JButton jB_open;
    private JButton jB_save;
    // End of variables declaration//GEN-END:variables

	@Override
	protected JPanel getParentPanel()
	{
		return( jPanel1 );
	}

	@Override
	protected FileHandlingPanelEvent createEvent( FileHandlingPanelEventType type )
	{
		return( new FileHandlingPanelEvent( type ) );
	}

	@Override
	protected void fillMultiMap( ButtonEventEntryMultiMap<FileHandlingPanelEventType> result )
	{
		result.put("com/frojasg1/generic/resources/file/openFile.png", FileHandlingPanelEventType.OPEN, jB_open);
		result.put("com/frojasg1/generic/resources/file/saveAs.png", FileHandlingPanelEventType.SAVE_AS, jB_saveAs);
		result.put("com/frojasg1/generic/resources/file/save.png", FileHandlingPanelEventType.SAVE, jB_save);
	}

	@Override
	public void setComponentMapper(ComponentMapper compMapper)
	{
		jB_save = compMapper.mapComponent(jB_save );
		jB_open = compMapper.mapComponent(jB_open );
		jB_saveAs = compMapper.mapComponent(jB_saveAs );
		jPanel1 = compMapper.mapComponent( jPanel1 );

		super.setComponentMapper(compMapper);
	}
}
