/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.desktop.view.panels.impl.player;

import java.awt.*;
import javax.swing.*;
import com.frojasg1.general.desktop.view.icons.IconResources;
import com.frojasg1.general.desktop.view.panels.evt.BaseJPanelOfEvents;
import com.frojasg1.general.desktop.view.panels.evt.map.ButtonEventEntryMultiMap;
import com.frojasg1.general.desktop.view.zoom.mapper.ComponentMapper;
import com.frojasg1.general.desktop.view.zoom.mapper.ComposedComponent;
import java.awt.event.ActionEvent;
import java.util.Objects;
import javax.swing.JPanel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class PlayerNavigatorJPanel
	extends BaseJPanelOfEvents<PlayerNavigatorEventType, PlayerNavigatorEvent>
	implements ComposedComponent
{
	private static final Logger LOGGER = LoggerFactory.getLogger(PlayerNavigatorJPanel.class);

	protected boolean _isIndependent;

	/**
	 * Creates new form PlayerNavigatorJPanel
	 */
	public PlayerNavigatorJPanel() {
		this(true);
	}

	/**
	 * Creates new form PlayerNavigatorJPanel
	 */
	public PlayerNavigatorJPanel( boolean isIndependent ) {
		super( PlayerNavigatorEventType.class );
		_isIndependent = isIndependent;
	}

	public void init()
	{
		super.init();
	}

	@Override
	protected void initComponentsChild() {
		initComponents();
	}

	public boolean isIndependent() {
		return _isIndependent;
	}



	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	@SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    // Generated using JFormDesigner non-commercial license
    private void initComponents() {
        jPanel1 = new JPanel();
        jB_stop = new JButton();
        jB_previous = new JButton();
        jB_next = new JButton();
        jB_playPause = new JButton();

        //======== this ========
        setLayout(null);

        //======== jPanel1 ========
        {
            jPanel1.setLayout(null);

            //---- jB_stop ----
            jB_stop.setMaximumSize(new Dimension(20, 20));
            jB_stop.setMinimumSize(new Dimension(20, 20));
            jB_stop.setName("name=jB_stop,icon=com/frojasg1/generic/resources/othericons/stop.png");
            jB_stop.setPreferredSize(new Dimension(20, 20));
            jPanel1.add(jB_stop);
            jB_stop.setBounds(55, 5, 20, jB_stop.getPreferredSize().height);

            //---- jB_previous ----
            jB_previous.setMaximumSize(new Dimension(20, 20));
            jB_previous.setMinimumSize(new Dimension(20, 20));
            jB_previous.setName("name=jB_previous,icon=com/frojasg1/generic/resources/navigator/version2/controls.start.png");
            jB_previous.setPreferredSize(new Dimension(20, 20));
            jPanel1.add(jB_previous);
            jB_previous.setBounds(5, 5, 20, 20);

            //---- jB_next ----
            jB_next.setMaximumSize(new Dimension(20, 20));
            jB_next.setMinimumSize(new Dimension(20, 20));
            jB_next.setName("name=jB_next,icon=com/frojasg1/generic/resources/navigator/version2/controls.end.png");
            jB_next.setPreferredSize(new Dimension(20, 20));
            jPanel1.add(jB_next);
            jB_next.setBounds(80, 5, 20, jB_next.getPreferredSize().height);

            //---- jB_playPause ----
            jB_playPause.setMaximumSize(new Dimension(20, 20));
            jB_playPause.setMinimumSize(new Dimension(20, 20));
            jB_playPause.setName("name=jB_playPause,icon=com/frojasg1/generic/resources/othericons/play.png");
            jB_playPause.setPreferredSize(new Dimension(20, 20));
            jPanel1.add(jB_playPause);
            jB_playPause.setBounds(30, 5, 20, jB_playPause.getPreferredSize().height);

            {
                // compute preferred size
                Dimension preferredSize = new Dimension();
                for(int i = 0; i < jPanel1.getComponentCount(); i++) {
                    Rectangle bounds = jPanel1.getComponent(i).getBounds();
                    preferredSize.width = Math.max(bounds.x + bounds.width, preferredSize.width);
                    preferredSize.height = Math.max(bounds.y + bounds.height, preferredSize.height);
                }
                Insets insets = jPanel1.getInsets();
                preferredSize.width += insets.right;
                preferredSize.height += insets.bottom;
                jPanel1.setMinimumSize(preferredSize);
                jPanel1.setPreferredSize(preferredSize);
            }
        }
        add(jPanel1);
        jPanel1.setBounds(-1, 0, 105, 30);

        {
            // compute preferred size
            Dimension preferredSize = new Dimension();
            for(int i = 0; i < getComponentCount(); i++) {
                Rectangle bounds = getComponent(i).getBounds();
                preferredSize.width = Math.max(bounds.x + bounds.width, preferredSize.width);
                preferredSize.height = Math.max(bounds.y + bounds.height, preferredSize.height);
            }
            Insets insets = getInsets();
            preferredSize.width += insets.right;
            preferredSize.height += insets.bottom;
            setMinimumSize(preferredSize);
            setPreferredSize(preferredSize);
        }
    }// </editor-fold>//GEN-END:initComponents


    // Variables declaration - do not modify//GEN-BEGIN:variables
    // Generated using JFormDesigner non-commercial license
    private JPanel jPanel1;
    private JButton jB_stop;
    private JButton jB_previous;
    private JButton jB_next;
    private JButton jB_playPause;
    // End of variables declaration//GEN-END:variables

	@Override
	protected JPanel getParentPanel()
	{
		return( jPanel1 );
	}

	@Override
	protected PlayerNavigatorEvent createEvent( PlayerNavigatorEventType type )
	{
		return( new PlayerNavigatorEvent( type ) );
	}

	public void callAfterPlayPressed()
	{
		setNewImageResource( getButton(PlayerNavigatorEventType.PLAY), IconResources.PAUSE_ICON_RESOURCE );
		disableAllAssociated( PlayerNavigatorEventType.PLAY );
		enableAllAssociated( PlayerNavigatorEventType.PAUSE );
	}

	public void callAfterPausePressed()
	{
		setNewImageResource( getButton(PlayerNavigatorEventType.PAUSE), IconResources.PLAY_ICON_RESOURCE );
		disableAllAssociated( PlayerNavigatorEventType.PAUSE );
		enableAllAssociated( PlayerNavigatorEventType.PLAY );
	}

	@Override
	protected void actionPerformed( ActionEvent evt )
	{
		super.actionPerformed(evt);

		Object button = evt.getSource();
		String imageResourceName = getImageResourceName( button );

		if( isIndependent() )
		{
			if( Objects.equals( imageResourceName, IconResources.PLAY_ICON_RESOURCE ) )
				callAfterPlayPressed();
			else if( Objects.equals( imageResourceName, IconResources.PAUSE_ICON_RESOURCE ) )
				callAfterPausePressed();
			else if( Objects.equals( imageResourceName, IconResources.STOP_ICON_RESOURCE ) )
				callAfterPausePressed();
		}
	}

	@Override
	protected void fillMultiMap( ButtonEventEntryMultiMap<PlayerNavigatorEventType> result )
	{
		result.put( IconResources.START_ICON_RESOURCE, PlayerNavigatorEventType.PREVIOUS, jB_previous);
		result.put( IconResources.STOP_ICON_RESOURCE, PlayerNavigatorEventType.STOP, jB_stop);
		result.put( IconResources.END_ICON_RESOURCE, PlayerNavigatorEventType.NEXT, jB_next);
		result.put( IconResources.PLAY_ICON_RESOURCE, PlayerNavigatorEventType.PLAY, jB_playPause);
		result.put( IconResources.PAUSE_ICON_RESOURCE, PlayerNavigatorEventType.PAUSE, jB_playPause);
	}

	@Override
	public void setComponentMapper(ComponentMapper compMapper)
	{
		jB_next = compMapper.mapComponent( jB_next );
		jB_playPause = compMapper.mapComponent( jB_playPause );
		jB_previous = compMapper.mapComponent( jB_previous );
		jB_stop = compMapper.mapComponent( jB_stop );
		jPanel1 = compMapper.mapComponent( jPanel1 );

		super.setComponentMapper(compMapper);
	}
}
