/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.desktop.view.text.listeners;

import com.frojasg1.general.desktop.view.text.utils.TextViewFunctions;
import java.util.function.Consumer;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.JTextComponent;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class SimpleUpdateDocumentListener implements DocumentListener
{
	protected JTextComponent _textComponent;
	protected Consumer<String> _updateFunction;

	public SimpleUpdateDocumentListener( JTextComponent textComponent,
										Consumer<String> updateFunction)
	{
		_textComponent = textComponent;
		_updateFunction = updateFunction;
	}

	protected String getText()
	{
		return( TextViewFunctions.instance().getText(_textComponent) );
	}

	public void update()
	{
		_updateFunction.accept( getText() );
	}

	@Override
	public void insertUpdate(DocumentEvent e) {
		update();
	}

	@Override
	public void removeUpdate(DocumentEvent e) {
		update();
	}

	@Override
	public void changedUpdate(DocumentEvent e) {
		
	}
}
