/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.desktop.view.ui;

import com.frojasg1.general.view.Blockable;
import java.awt.Graphics;
import javax.swing.JComponent;
import javax.swing.plaf.basic.BasicViewportUI;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class BasicViewportBlockableUI extends BasicViewportUI implements Blockable
{
	protected BlockablePainter _blockablePainter;

	public BasicViewportBlockableUI()
	{
		super();
	}

	public BasicViewportBlockableUI init()
	{
		_blockablePainter = new BlockablePainter( this::superPaint );

		return( this );
	}

	@Override
	public void paint( Graphics grp, JComponent jComponent )
	{
		_blockablePainter.paint( grp, jComponent );
	}

	protected void superPaint( Graphics grp, JComponent jComponent )
	{
		super.paint( grp, jComponent );
	}

	@Override
	public void block()
	{
		_blockablePainter.block();
	}

	@Override
	public void unblock()
	{
		_blockablePainter.unblock();
	}

	@Override
	public void setBlocked(boolean value)
	{
		_blockablePainter.setBlocked( value );
	}

	@Override
	public boolean isBlocked() {
		return( _blockablePainter.isBlocked() );
	}
}
