/*
 * Copyright (c) 2023. Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 3.0 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */

package com.frojasg1.general.desktop.view.zoom.colorchooser;

import com.frojasg1.applications.common.configuration.application.BaseApplicationConfigurationInterface;
import com.frojasg1.general.desktop.screen.ScreenFunctions;
import com.frojasg1.general.desktop.view.ViewFunctions;
import com.frojasg1.general.desktop.view.zoom.colorchooser.ui.ZoomBasicColorChooserUI;
import com.frojasg1.generic.GenericFunctions;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.HeadlessException;
import java.awt.Rectangle;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.io.Serializable;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Locale;
import java.util.Set;
import javax.swing.JColorChooser;
import javax.swing.JDialog;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class ZoomJColorChooser extends JColorChooser {
    private static final String uiClassID = "ZoomBasicColorChooserUI";

    private static final Logger LOGGER = LoggerFactory.getLogger(ZoomJColorChooser.class);

//    protected Set<Component> processedComps = new HashSet<>();

    protected Component _parent;

    protected String title;

    protected JDialog jDialog;
    protected ColorTracker ok;

    protected ZoomBasicColorChooserUI myUi;

    public ZoomJColorChooser(Component parent, String title, Color defaultColor) {
        this(parent, title, defaultColor, null);
    }

    public ZoomJColorChooser(Component parent, String title, Color defaultColor,
                             Locale locale) {
        super(defaultColor);

        _parent = parent;
        this.title = title;
        System.out.println("Locale: " + locale);

        if (locale == null)
            locale = Locale.US;
        setLocale(locale);

        System.out.println("Locale: " + locale);
//
//        ZoomMetalFileChooserUI ui = new ZoomMetalFileChooserUI( this, isDarkMode );
//        ui.setZoomFactor(conf.getZoomFactor());
//
//        setUI( ui );
    }

//    public Set<Component> getProcessedComps() {
//        return processedComps;
//    }

    public List<Color> getRecentColors() {
        return myUi.getRecentColors();
    }

    public void setRecentColors(List<Color> colors) {
        myUi.setRecentColors(colors);
    }

    protected ZoomBasicColorChooserUI createUi() {
        return new ZoomBasicColorChooserUI();
    }


    @Override
    public void updateUI() {
        myUi = createUi();
        setUI(myUi);
    }

    @Override
    public String getUIClassID() {
        return uiClassID;
    }

    public ZoomJColorChooser init(Dimension new100percentDimension) {
        jDialog = createDialog_internal();

        setNewBounds(jDialog, new100percentDimension);
//        getProcessedComps().add(jDialog);

        return this;
    }

    protected ViewFunctions getViewFunctions() {
        return ViewFunctions.instance();
    }

    protected Rectangle calculateNewBounds(Rectangle bounds, Dimension new100percentDimension) {
        Rectangle result = null;
        if(new100percentDimension != null) {
            result = getViewFunctions().calculateNewBoundsWithTheSameCenter(bounds, new100percentDimension,
                    getAppliConf().getZoomFactor());
        } else {
            result = getViewFunctions().zoomRectangleWithSameCenter(bounds,
                    getAppliConf().getZoomFactor());
        }
        return result;
    }

    protected Rectangle getNewBounds(Component comp, Dimension new100percentDimension) {
        return getViewFunctions().applyLimits(
                calculateNewBounds(comp.getBounds(), new100percentDimension),
                ScreenFunctions.getScreenSize(comp, false));
    }

    protected void setNewBounds(JDialog jDialog, Dimension new100percentDimension) {
        jDialog.setBounds(getNewBounds(jDialog, new100percentDimension));
    }

    public JDialog getJDialog() {
        return jDialog;
    }

    protected BaseApplicationConfigurationInterface getAppliConf() {
        return (GenericFunctions.instance().getAppliConf());
    }

    public Color showZoomDialog() throws HeadlessException {
        jDialog.show(); // blocks until user brings dialog down...

        return ok.getColor();
    }

    protected JDialog createDialog_internal() throws HeadlessException {
        JColorChooser pane = this;
        ok = new ColorTracker(pane);
        JDialog dialog = createDialog(_parent, title, true, pane, ok, null);

        dialog.addComponentListener(new DisposeOnClose());

        return dialog;
    }

    protected static class DisposeOnClose extends ComponentAdapter implements Serializable {
        public void componentHidden(ComponentEvent e) {
            Window w = (Window) e.getComponent();
            w.dispose();
        }
    }

    protected static class ColorTracker implements ActionListener, Serializable {
        JColorChooser chooser;
        Color color;

        public ColorTracker(JColorChooser c) {
            chooser = c;
        }

        public void actionPerformed(ActionEvent e) {
            color = chooser.getColor();
        }

        public Color getColor() {
            return color;
        }
    }
}
