/*
 * Copyright (c) 2023. Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 3.0 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */

package com.frojasg1.general.desktop.view.zoom.colorchooser.swatch;

import java.awt.Color;
import java.awt.Dimension;
import javax.swing.UIManager;

public class ZoomMainSwatchPanel  extends ZoomSwatchPanel {


    protected void initValues() {
        swatchSize = UIManager.getDimension("ColorChooser.swatchesSwatchSize", getLocale());
        numSwatches = new Dimension( 31, 9 );
        gap = new Dimension(1, 1);
    }

    protected void initColors() {
        int[] rawValues = initRawValues();
        int numColors = rawValues.length / 3;

        colors = new Color[numColors];
        for (int i = 0; i < numColors ; i++) {
            colors[i] = new Color( rawValues[(i*3)], rawValues[(i*3)+1], rawValues[(i*3)+2] );
        }
    }

    private int[] initRawValues() {

        int[] rawValues = {
                255, 255, 255, // first row.
                204, 255, 255,
                204, 204, 255,
                204, 204, 255,
                204, 204, 255,
                204, 204, 255,
                204, 204, 255,
                204, 204, 255,
                204, 204, 255,
                204, 204, 255,
                204, 204, 255,
                255, 204, 255,
                255, 204, 204,
                255, 204, 204,
                255, 204, 204,
                255, 204, 204,
                255, 204, 204,
                255, 204, 204,
                255, 204, 204,
                255, 204, 204,
                255, 204, 204,
                255, 255, 204,
                204, 255, 204,
                204, 255, 204,
                204, 255, 204,
                204, 255, 204,
                204, 255, 204,
                204, 255, 204,
                204, 255, 204,
                204, 255, 204,
                204, 255, 204,
                204, 204, 204,  // second row.
                153, 255, 255,
                153, 204, 255,
                153, 153, 255,
                153, 153, 255,
                153, 153, 255,
                153, 153, 255,
                153, 153, 255,
                153, 153, 255,
                153, 153, 255,
                204, 153, 255,
                255, 153, 255,
                255, 153, 204,
                255, 153, 153,
                255, 153, 153,
                255, 153, 153,
                255, 153, 153,
                255, 153, 153,
                255, 153, 153,
                255, 153, 153,
                255, 204, 153,
                255, 255, 153,
                204, 255, 153,
                153, 255, 153,
                153, 255, 153,
                153, 255, 153,
                153, 255, 153,
                153, 255, 153,
                153, 255, 153,
                153, 255, 153,
                153, 255, 204,
                204, 204, 204,  // third row
                102, 255, 255,
                102, 204, 255,
                102, 153, 255,
                102, 102, 255,
                102, 102, 255,
                102, 102, 255,
                102, 102, 255,
                102, 102, 255,
                153, 102, 255,
                204, 102, 255,
                255, 102, 255,
                255, 102, 204,
                255, 102, 153,
                255, 102, 102,
                255, 102, 102,
                255, 102, 102,
                255, 102, 102,
                255, 102, 102,
                255, 153, 102,
                255, 204, 102,
                255, 255, 102,
                204, 255, 102,
                153, 255, 102,
                102, 255, 102,
                102, 255, 102,
                102, 255, 102,
                102, 255, 102,
                102, 255, 102,
                102, 255, 153,
                102, 255, 204,
                178, 178, 178, // fourth row
                51, 255, 255,
                51, 204, 255,
                51, 153, 255,
                51, 102, 255,
                51, 51, 255,
                51, 51, 255,
                51, 51, 255,
                102, 51, 255,
                153, 51, 255,
                204, 51, 255,
                255, 51, 255,
                255, 51, 204,
                255, 51, 153,
                255, 51, 102,
                255, 51, 51,
                255, 51, 51,
                255, 51, 51,
                255, 102, 51,
                255, 153, 51,
                255, 204, 51,
                255, 255, 51,
                204, 255, 51,
                153, 255, 51,
                102, 255, 51,
                51, 255, 51,
                51, 255, 51,
                51, 255, 51,
                51, 255, 102,
                51, 255, 153,
                51, 255, 204,
                153, 153, 153, // Fifth row
                0, 255, 255,
                0, 204, 255,
                0, 153, 255,
                0, 102, 255,
                0, 51, 255,
                0, 0, 255,
                51, 0, 255,
                102, 0, 255,
                153, 0, 255,
                204, 0, 255,
                255, 0, 255,
                255, 0, 204,
                255, 0, 153,
                255, 0, 102,
                255, 0, 51,
                255, 0 , 0,
                255, 51, 0,
                255, 102, 0,
                255, 153, 0,
                255, 204, 0,
                255, 255, 0,
                204, 255, 0,
                153, 255, 0,
                102, 255, 0,
                51, 255, 0,
                0, 255, 0,
                0, 255, 51,
                0, 255, 102,
                0, 255, 153,
                0, 255, 204,
                128, 128, 128, // sixth row
                0, 204, 204,
                0, 204, 204,
                0, 153, 204,
                0, 102, 204,
                0, 51, 204,
                0, 0, 204,
                51, 0, 204,
                102, 0, 204,
                153, 0, 204,
                204, 0, 204,
                204, 0, 204,
                204, 0, 204,
                204, 0, 153,
                204, 0, 102,
                204, 0, 51,
                204, 0, 0,
                204, 51, 0,
                204, 102, 0,
                204, 153, 0,
                204, 204, 0,
                204, 204, 0,
                204, 204, 0,
                153, 204, 0,
                102, 204, 0,
                51, 204, 0,
                0, 204, 0,
                0, 204, 51,
                0, 204, 102,
                0, 204, 153,
                0, 204, 204,
                102, 102, 102, // seventh row
                0, 153, 153,
                0, 153, 153,
                0, 153, 153,
                0, 102, 153,
                0, 51, 153,
                0, 0, 153,
                51, 0, 153,
                102, 0, 153,
                153, 0, 153,
                153, 0, 153,
                153, 0, 153,
                153, 0, 153,
                153, 0, 153,
                153, 0, 102,
                153, 0, 51,
                153, 0, 0,
                153, 51, 0,
                153, 102, 0,
                153, 153, 0,
                153, 153, 0,
                153, 153, 0,
                153, 153, 0,
                153, 153, 0,
                102, 153, 0,
                51, 153, 0,
                0, 153, 0,
                0, 153, 51,
                0, 153, 102,
                0, 153, 153,
                0, 153, 153,
                51, 51, 51, // eigth row
                0, 102, 102,
                0, 102, 102,
                0, 102, 102,
                0, 102, 102,
                0, 51, 102,
                0, 0, 102,
                51, 0, 102,
                102, 0, 102,
                102, 0, 102,
                102, 0, 102,
                102, 0, 102,
                102, 0, 102,
                102, 0, 102,
                102, 0, 102,
                102, 0, 51,
                102, 0, 0,
                102, 51, 0,
                102, 102, 0,
                102, 102, 0,
                102, 102, 0,
                102, 102, 0,
                102, 102, 0,
                102, 102, 0,
                102, 102, 0,
                51, 102, 0,
                0, 102, 0,
                0, 102, 51,
                0, 102, 102,
                0, 102, 102,
                0, 102, 102,
                0, 0, 0, // ninth row
                0, 51, 51,
                0, 51, 51,
                0, 51, 51,
                0, 51, 51,
                0, 51, 51,
                0, 0, 51,
                51, 0, 51,
                51, 0, 51,
                51, 0, 51,
                51, 0, 51,
                51, 0, 51,
                51, 0, 51,
                51, 0, 51,
                51, 0, 51,
                51, 0, 51,
                51, 0, 0,
                51, 51, 0,
                51, 51, 0,
                51, 51, 0,
                51, 51, 0,
                51, 51, 0,
                51, 51, 0,
                51, 51, 0,
                51, 51, 0,
                0, 51, 0,
                0, 51, 51,
                0, 51, 51,
                0, 51, 51,
                0, 51, 51,
                51, 51, 51 };
        return rawValues;
    }
}
