/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.desktop.view.zoom.componentcopier.imp;

import com.frojasg1.general.desktop.view.zoom.componentcopier.CompCopier;
import com.frojasg1.general.desktop.view.zoom.componentcopier.CompCopierBase;
import java.awt.Component;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.List;
import javax.swing.JScrollPane;
import javax.swing.JViewport;
import javax.swing.ScrollPaneConstants;
import javax.swing.event.ChangeListener;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class JViewportCopier extends CompCopierBase<JViewport>
{
	@Override
	protected List<CompCopier<JViewport>> createCopiers() {

		List<CompCopier<JViewport>> result = new ArrayList<>();

		result.add(this::copyPropertyChangeListenerList);
		result.add(this::copyExtentSize);
		result.add(this::copyScrollMode);
		result.add(this::copyView);
		result.add(this::copyViewPosition);
		result.add(this::copyViewSize);

		return( result );
	}

	protected void copyViewSize(JViewport originalComponent, JViewport newComponent)
	{
		newComponent.setViewSize(originalComponent.getViewSize());
	}

	protected void copyViewPosition(JViewport originalComponent, JViewport newComponent)
	{
		newComponent.setViewPosition(originalComponent.getViewPosition());
	}

	protected void copyView(JViewport originalComponent, JViewport newComponent)
	{
		newComponent.setView(originalComponent.getView());
	}

	protected void copyScrollMode(JViewport originalComponent, JViewport newComponent)
	{
		newComponent.setScrollMode(originalComponent.getScrollMode());
	}

	protected void copyExtentSize(JViewport originalComponent, JViewport newComponent)
	{
		newComponent.setExtentSize(originalComponent.getExtentSize());
	}

	protected void copyPropertyChangeListenerList(JViewport originalComponent, JViewport newComponent)
	{
		copyListeners( originalComponent, newComponent,
				ChangeListener.class,
				JViewport::getChangeListeners,
				JViewport::addChangeListener,
				JViewport::removeChangeListener);
	}

	@Override
	public Class<JViewport> getParameterClass() {
		return( JViewport.class );
	}
}
