/*
 * Copyright (c) 2023. Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 3.0 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */

package com.frojasg1.general.desktop.view.zoom.components;

import com.frojasg1.general.desktop.view.FrameworkComponentFunctions;
import com.frojasg1.general.desktop.view.painter.PaintEnabled;
import javax.swing.*;
import java.awt.*;

public class ZoomJViewport extends JViewport implements PaintEnabled {

//    protected boolean notComputeBlit;

    protected boolean canPaint = true;


//    public boolean getNotComputeBlit() {
//        return notComputeBlit;
//    }
//
//    public void setNotComputeBlit(Boolean value) {
//        this.notComputeBlit = value;
//    }
//
//    @Override
//    protected boolean computeBlit(int dx,
//                                  int dy,
//                                  Point blitFrom,
//                                  Point blitTo,
//                                  Dimension blitSize,
//                                  Rectangle blitPaint) {
//        boolean result = super.computeBlit(dx, dy, blitFrom, blitTo, blitSize, blitPaint);
//        return !notComputeBlit && result;
//    }

    @Override
    public boolean canPaint() {
        return canPaint;
    }

    @Override
    public void setCanPaint(boolean canPaint) {
        this.canPaint = canPaint;
    }

    protected void setViewPositionInternal(Point p)
    {
        Component view = getView();
        if (view == null) {
            return;
        }

        int oldX, oldY, x = p.x, y = p.y;

        /* Collect the old x,y values for the views location
         * and do the song and dance to avoid allocating
         * a Rectangle object if we don't have to.
         */
        if (view instanceof JComponent) {
            JComponent c = (JComponent)view;
            oldX = c.getX();
            oldY = c.getY();
        }
        else {
            Rectangle r = view.getBounds();
            oldX = r.x;
            oldY = r.y;
        }

        /* The view scrolls in the opposite direction to mouse
         * movement.
         */
        int newX = -x;
        int newY = -y;

        if ((oldX != newX) || (oldY != newY)) {
            // This calls setBounds(), and then repaint().
            view.setLocation(newX, newY);
        }
    }

    @Override
    public void setViewPosition(Point p)
    {
        if(!canPaint()) {
            setViewPositionInternal(p);
            FrameworkComponentFunctions.instance().getInternationalizedWindow(this).repaint();
        } else {
            super.setViewPosition(p);
        }
    }
}
