/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.desktop.view.zoom.filechooser;

import com.frojasg1.general.desktop.generic.dialogs.impl.StaticDesktopDialogsWrapper;
import java.io.File;
import javax.swing.filechooser.FileFilter;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class ProxyFileFilter extends FileFilter
{
		protected FileFilter originalFilter;
		public ProxyFileFilter( FileFilter filter )
		{
			this.originalFilter = filter;
		}

		@Override
		public boolean accept(File f)
		{
			return( originalFilter.accept(f) );
		}

		@Override
		public String getDescription()
		{
			return( originalFilter.getDescription() );
		}

		@Override
		public String toString()
		{
			return getDescription();
		}

		protected FileFilter getAncestor( Object fileFilter )
		{
			FileFilter result = null;
			if( fileFilter instanceof FileFilter )
			{
				result = (FileFilter) fileFilter;
				while( result instanceof ProxyFileFilter )
					result = ( (ProxyFileFilter) result ).originalFilter;
			}

			return( result );
		}

		@Override
		public boolean equals( Object that )
		{
			return getAncestor(this).equals( getAncestor(that) );
		}
}
