/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.desktop.view.zoom.helpers;

import com.frojasg1.general.desktop.view.FrameworkComponentFunctions;
import com.frojasg1.general.desktop.view.color.ColorInversor;
import java.awt.Color;
import javax.swing.JComponent;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class ComponentSimpleColorInversor {
	
	protected static class LazyHolder
	{
		protected static final ComponentSimpleColorInversor INSTANCE = new ComponentSimpleColorInversor();
	}

	public static ComponentSimpleColorInversor instance()
	{
		return( LazyHolder.INSTANCE );
	}

	protected ColorInversor getColorInversor(JComponent comp)
	{
		return( FrameworkComponentFunctions.instance().getColorInversor(comp) );
	}

    public Color invertColorIfNecessary( JComponent comp, Color color ) {
		Color result = color;
		ColorInversor ci = getColorInversor(comp);
		if( ( ci != null ) && ( ci.isDarkMode(comp) ) )
			result = ci.invertColor(result);

		return result;
    }
}
