/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.libgeometry.algebra;

import com.frojasg1.general.structures.Pair;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class AlgebraFunctions {
	protected static AlgebraFunctions INSTANCE = new AlgebraFunctions();
	
	public static AlgebraFunctions instance()
	{
		return( INSTANCE );
	}

	public Pair<Double, Double> solve2x2LinearSystem( double a11, double a12,
								double c1, double a21, double a22, double c2 )
	{
		Pair<Double, Double> result = null;
		double determinant = determinant( a11, a12, a21, a22 );
		
		if( !isZero( determinant ) )
		{
			double xx = determinant( c1, a12, c2, a22 ) / determinant;
			double yy = determinant( a11, c1, a21, c2 ) / determinant;

			result = new Pair( xx, yy );
		}

		return( result );
	}

	public boolean isZero( double value )
	{
		return( value < 1.0e-13d );
	}

	public double determinant( double a11, double a12,
								double a21, double a22 )
	{
		return( a11 * a22 - a12 * a21 );
	}
}
