package com.frojasg1.libgeometry.helper;

import com.frojasg1.libgeometry.types.GeometricRectangle;
import java.util.function.BiPredicate;

public class GenericCropper {

    protected static GenericCropper INSTANCE = new GenericCropper();

    public static GenericCropper instance() {
        return INSTANCE;
    }

    public GeometricRectangle crop(int width, int height, BiPredicate<Integer, Integer> isVoidFunction) {
        GeometricRectangle result = new GeometricRectangle(0, 0, 0, 0);

        result.x = limitLeft(width, height, isVoidFunction);
        if(result.x != width) {
            int right = limitRight(width, height, isVoidFunction);
            result.y = limitTop(width, height, isVoidFunction);
            if(result.y != height) {
                int bottom = limitBottom(width, height, isVoidFunction);

                result.width = right - result.x + 1;
                result.height = bottom - result.y + 1;
            }
        }

        return result;
    }

    protected int limitLeft(int width, int height, BiPredicate<Integer, Integer> isVoidFunction) {
        return limitGen(0, 0, 1, 0, width, height, isVoidFunction);
    }

    protected int limitRight(int width, int height, BiPredicate<Integer, Integer> isVoidFunction) {
        return limitGen(width - 1, 0, -1, 0, width, height, isVoidFunction);
    }

    protected int limitTop(int width, int height, BiPredicate<Integer, Integer> isVoidFunction) {
        return limitGen(0, 0, 0, 1, width, height, isVoidFunction);
    }

    protected int limitBottom(int width, int height, BiPredicate<Integer, Integer> isVoidFunction) {
        return limitGen(0, height - 1, 0, -1, width, height, isVoidFunction);
    }

    protected int limitGen(int x0, int y0, int dx, int dy, int width, int height,
                           BiPredicate<Integer, Integer> isVoidFunction) {
        int xx = x0;
        int yy = y0;
        while(!endCondition(xx, yy, dx, dy, width, height, isVoidFunction)) {
            xx += dx;
            yy += dy;
        }

        return (dx == 0) ? yy : xx;
    }

    protected boolean endCondition(int xx, int yy, int dx, int dy, int width, int height,
                                   BiPredicate<Integer, Integer> isVoidFunction) {
        boolean result = false;
        if(dx == 0) {
            result = (yy < 0) || (yy >= height) || horizontalScan(yy, width, isVoidFunction);
        } else if(dy == 0) {
            result = (xx < 0) || (xx >= width) || verticalScan(xx, height, isVoidFunction);
        } else {
            throw new IllegalArgumentException("either dx or dy must be equal to 0");
        }
        return result;
    }

    protected boolean horizontalScan(int yy, int width, BiPredicate<Integer, Integer> isVoidFunction) {
        boolean result = false;
        for(int xx = 0; xx < width; xx++) {
            if(!isVoidFunction.test(xx, yy)) {
                result = true;
                break;
            }
        }
        return result;
    }

    protected boolean verticalScan(int xx, int height, BiPredicate<Integer, Integer> isVoidFunction) {
        boolean result = false;
        for(int yy = 0; yy < height; yy++) {
            if(!isVoidFunction.test(xx, yy)) {
                result = true;
                break;
            }
        }
        return result;
    }
}
