/* 
 * Copyright (C) 2023 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.libgeometry.transformation.impl;

import com.frojasg1.libgeometry.transformation.GeometricTransformation;
import com.frojasg1.libgeometry.types.GeometricRectangle;
import com.frojasg1.libgeometry.types.Point2d;

public class AffineTransformation implements GeometricTransformation {

    protected double deltax;
    protected double deltay;
    protected double factorx;
    protected double factory;

    public AffineTransformation(GeometricRectangle originalBounds, GeometricRectangle transformedBounds) {
        deltax = transformedBounds.x - originalBounds.x;
        deltay = transformedBounds.y - originalBounds.y;
        factorx = (transformedBounds.width) / (originalBounds.width - 1);
        factory = (transformedBounds.height) / (originalBounds.height - 1);
    }

    protected double transform(double value, double delta, double factor) {
        return (value + 0.5d) * factor + delta - 0.5d;
    }

    @Override
    public Point2d transform(Point2d pt) {
        Point2d result = pt.createCopy();
        result.x = transform(pt.x, deltax, factorx);
        result.y = transform(pt.y, deltay, factory);

        return result;
    }
}
