package com.frojasg1.libgeometry.algebra.helpers;

import com.frojasg1.libgeometry.types.Point2d;
import com.frojasg1.libgeometry.types.factory.GeometryCommonFactory;
import java.util.function.BiConsumer;
import org.junit.Before;
import org.junit.Test;


import static org.junit.Assert.*;

public class GeometryHelperTest {
    protected GeometryHelper instance;


    @Before
    public void setUp() throws Exception {
        instance = new GeometryHelper();
    }

    @Test
    public void browseMappedTriangle() {
        Point2d pt11 = createPoint2d(0, 20);
        Point2d pt12 = createPoint2d(0, 41);
        Point2d pt13 = createPoint2d(10, 30);

        Point2d pt21 = createPoint2d(0, 0);
        Point2d pt22 = createPoint2d(0, 18);
        Point2d pt23 = createPoint2d(9, 9);

        browseMappedTriangle(pt11, pt12, pt13, pt21, pt22, pt23);
    }

    @Test
    public void browseMappedTriangle2() {
        Point2d pt11 = createPoint2d(0, 140);
        Point2d pt12 = createPoint2d(0, 141);
        Point2d pt13 = createPoint2d(9, 134);

        Point2d pt21 = createPoint2d(10, 9);
        Point2d pt22 = createPoint2d(0, 0);
        Point2d pt23 = createPoint2d(0, 18);

        browseMappedTriangle(pt11, pt12, pt13, pt21, pt22, pt23);
    }

    @Test
    public void mapPoint() {
        Point2d pt11 = createPoint2d(0, 10);
        Point2d pt12 = createPoint2d(0, 0);
        Point2d pt13 = createPoint2d(10, 0);

        Point2d pt21 = pt11;
        Point2d pt22 = createPoint2d(10, 10);
        Point2d pt23 = pt13;

        BiConsumer<Point2d, Point2d> myChecker = getMappingChecker(pt11, pt12, pt13, pt21, pt22, pt23);

        myChecker.accept(pt11, pt21);
        myChecker.accept(pt12, pt22);
        myChecker.accept(pt13, pt23);
    }

    protected void checkPoints(Point2d expectedMappedPt, Point2d expectedOriginalPoint, Point2d mappedPoint, Point2d originalPoint) {
        assertEquals(expectedMappedPt, mappedPoint);
        assertEquals(expectedOriginalPoint, originalPoint);
    }

    protected void mapPointAndCheck(Point2d expectedMappedPt, Point2d expectedOriginalPoint, Point2d pt11, Point2d pt12, Point2d pt13,
                                    Point2d pt21, Point2d pt22, Point2d pt23) {
        instance.processMappedTrianglePoint( expectedOriginalPoint,
                (mapped, orig) -> checkPoints(expectedMappedPt, expectedOriginalPoint, mapped, orig), pt11, pt12, pt13,
                pt21, pt22, pt23);
    }

    protected BiConsumer<Point2d, Point2d> getMappingChecker(Point2d pt11, Point2d pt12, Point2d pt13,
                                                             Point2d pt21, Point2d pt22, Point2d pt23) {
        return (expectedMappedPt, expectedOriginalPoint) -> mapPointAndCheck(expectedMappedPt, expectedOriginalPoint, pt11, pt12, pt13, pt21, pt22, pt23);
    }

    public void browseMappedTriangle(Point2d pt11, Point2d pt12, Point2d pt13,
                                     Point2d pt21, Point2d pt22, Point2d pt23) {

        System.out.println("Original\t\tMapped");
        paintPts(pt11, pt21);
        paintPts(pt12, pt22);
        paintPts(pt13, pt23);
        System.out.println("=========");
        instance.browseMappedTriangle((pt, mappedPt) -> paintPts(pt, mappedPt), pt11, pt12, pt13, pt21, pt22, pt23, null);
    }

    protected void paintPts(Point2d pt, Point2d mappedPt) {
        System.out.println(String.format("%.0f\t%.0f\t%.0f\t%.0f", pt.x, pt.y, mappedPt.x, mappedPt.y));
    }

    protected Point2d createPoint2d(double x, double y) {
        return GeometryCommonFactory.instance().createPoint2d(x, y);
    }

}