package com.frojasg1.libgeometry.helper;

import com.frojasg1.libgeometry.types.GeometricRectangle;
import java.util.Arrays;
import java.util.Objects;
import java.util.Set;
import java.util.stream.Collectors;
import org.junit.Before;
import org.junit.Test;


import static org.junit.Assert.*;

public class GenericCropperTest {

    protected GenericCropper instance;

    @Before
    public void setUp() throws Exception {
        instance = new GenericCropper();
    }

    @Test
    public void cropAll() {
        GeometricRectangle rect = instance.crop(25, 45, (x, y) -> true);

        assertEquals(0.0d, rect.width, 0.0d);
        assertEquals(0.0d, rect.height, 0.0d);
    }

    @Test
    public void cropNothing() {
        GeometricRectangle rect = instance.crop(25, 45, (x, y) -> false);

        GeometricRectangle expectedRect = rect(0, 0, 25, 45);

        assertEquals(expectedRect, rect);
    }

    @Test
    public void cropPoint() {
        Set<Coord> notEmptyCoords = set(coord(16, 33));

        GeometricRectangle rect = instance.crop(25, 45, (x, y) -> isEmpty(x, y, notEmptyCoords));

        GeometricRectangle expectedRect = rect(16, 33, 1, 1);

        assertEquals(expectedRect, rect);
    }

    @Test
    public void cropFrame() {
        Set<Coord> notEmptyCoords = set(coord(16, 33), coord(7, 38));

        GeometricRectangle rect = instance.crop(25, 45, (x, y) -> isEmpty(x, y, notEmptyCoords));

        GeometricRectangle expectedRect = rect(7, 33, 10, 6);

        assertEquals(expectedRect, rect);
    }

    @Test
    public void cropFrame2() {
        Set<Coord> notEmptyCoords = set(coord(16, 35), coord(7, 36),
                coord(10, 33), coord(15, 38),
                coord(10, 35));

        GeometricRectangle rect = instance.crop(25, 45, (x, y) -> isEmpty(x, y, notEmptyCoords));

        GeometricRectangle expectedRect = rect(7, 33, 10, 6);

        assertEquals(expectedRect, rect);
    }

    protected <C> Set<C> set(C ... coords) {
        return Arrays.stream(coords).collect(Collectors.toSet());
    }

    protected Coord coord(int xx, int yy) {
        return new Coord(xx, yy);
    }

    protected boolean isEmpty(int xx, int yy, Set<Coord> notEmptyCoords) {
        return !notEmptyCoords.contains(coord(xx, yy));
    }

    protected GeometricRectangle rect(double xx, double yy, double width, double height) {
        return new GeometricRectangle(xx, yy, width, height);
    }

    protected static class Coord {
        protected int xx;
        protected int yy;

        public Coord(int xx, int yy) {
            this.xx = xx;
            this.yy = yy;
        }

        @Override
        public boolean equals(Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;
            Coord coord = (Coord) o;
            return xx == coord.xx && yy == coord.yy;
        }

        @Override
        public int hashCode() {
            return Objects.hash(xx, yy);
        }

        @Override
        public String toString() {
            return "Coord{" +
                    "xx=" + xx +
                    ", yy=" + yy +
                    '}';
        }
    }
}
