/* 
 * Copyright (C) 2021 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.desktop.queries;

import com.frojasg1.general.NullFunctions;

import java.util.function.Function;

/**
 *
 * @author fjavier.rojas
 */
public class InetQueryResultBase implements InetQueryResult
{
	public static final int SUCCESS_RESULT_CODE = 0;
	public static final int INTERNAL_ERROR_RESULT_CODE = -1;

	protected Exception _exception = null;
	public InetQueryResultBase( )
	{
		this( null );
	}

	public InetQueryResultBase( InetQueryException exception )
	{
		_exception = exception;
	}

	public boolean isSuccessful()
	{
		return( getResultCode() == SUCCESS_RESULT_CODE );
	}

	protected <CC, RR> RR getIfNotNull(CC obj, Function<CC,RR> getter, RR defaultValue ) {
		return NullFunctions.instance().getIfNotNull(obj, getter, defaultValue);
	}

	public <CC, RR> RR getIfNotNull( CC obj, Function<CC,RR> getter ) {
		return NullFunctions.instance().getIfNotNull(obj, getter);
	}

	protected InetQueryException getInetQueryException() {
		InetQueryException result = null;
		Exception ex = getException();
		if(ex instanceof InetQueryException) {
			result = (InetQueryException) ex;
		}
		return result;
	}

	public int getResultCode() {
		int result =  getIfNotNull(getInetQueryException(), InetQueryException::getResultCode, SUCCESS_RESULT_CODE);
		if((result == SUCCESS_RESULT_CODE) && (getException() != null)) {
			result = INTERNAL_ERROR_RESULT_CODE;
		}
		return result;
	}

	public String getErrorMessage() {
		return getIfNotNull(getInetQueryException(), InetQueryException::getMessage);
	}

	@Override
	public Exception getException()
	{
		return( _exception );
	}
	
	@Override
	public void setException( Exception exception )
	{
		_exception = exception;
	}
}
